/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/button.h")]
namespace OSLib.Button
{
  [CCode (cname = "button_gadget")]
  public struct Gadget
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "value")]
    public unowned Toolbox.MsgReference Value;
    [CCode (cname = "value_limit")]
    public unowned int ValueLimit;
    [CCode (cname = "validation")]
    public unowned Toolbox.StringReference Validation;
    [CCode (cname = "validation_limit")]
    public unowned int ValidationLimit;
  }

  [CCode (cname = "button_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "button_flags")]
    public unowned bits ButtonFlags;
    [CCode (cname = "value")]
    public unowned Toolbox.MsgReference Value;
    [CCode (cname = "value_limit")]
    public unowned int ValueLimit;
    [CCode (cname = "validation")]
    public unowned Toolbox.StringReference Validation;
    [CCode (cname = "validation_limit")]
    public unowned int ValidationLimit;
  }

  [CCode (cname="class_BUTTON")]
  public const Toolbox.Class Class_Button;
  [CCode (cname="button_TASK_SPRITE_AREA")]
  public const bits TaskSpriteArea;
  [CCode (cname="button_ALLOW_MENU_CLICKS")]
  public const bits AllowMenuClicks;

  [CCode (cname = "xbutton_get_flags")]
  public static OS.Error* XGetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     out bits icon_flags);

  [CCode (cname = "button_get_flags")]
  public static bits GetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C button);

  [CCode (cname = "xbutton_set_flags")]
  public static OS.Error* XSetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     int clear_word,
     int eor_word);

  [CCode (cname = "button_set_flags")]
  public static void SetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     int clear_word,
     int eor_word);

  [CCode (cname = "xbutton_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string value);

  [CCode (cname = "button_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string value);

  [CCode (cname = "xbutton_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "button_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string buffer,
     int size);

  [CCode (cname = "xbutton_set_validation")]
  public static OS.Error* XSetValidation
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string validation);

  [CCode (cname = "button_set_validation")]
  public static void SetValidation
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string validation);

  [CCode (cname = "xbutton_get_validation")]
  public static OS.Error* XGetValidation
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "button_get_validation")]
  public static int GetValidation
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string buffer,
     int size);

  [CCode (cname = "xbutton_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string font_name,
     int xsize,
     int ysize);

  [CCode (cname = "button_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C button,
     string font_name,
     int xsize,
     int ysize);

}
