/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.SCSI;

[CCode (cheader_filename = "oslib/cd.h")]
namespace OSLib.CD
{
  [CCode (cname = "cd_address")]
  public enum Address
  {
    Default = 0
  }

  [CCode (cname = "cd_address_mode")]
  public enum AddressMode
  {
    [CCode (cname = "cd_LOGICAL_BLOCK_ADDRESSING")]
    LogicalBlockAddressing,
    [CCode (cname = "cd_RED_BOOK_ADDRESSING")]
    RedBookAddressing,
    [CCode (cname = "cd_PHYSICAL_BLOCK_ADDRESSING")]
    PhysicalBlockAddressing
  }

  [CCode (cname = "cd_control_block")]
  public struct ControlBlock
  {
    [CCode (cname = "reserved")]
    public unowned int Reserved[5];
  }

  [CCode (cname = "cd_audio_flags")]
  public enum AudioFlags
  {
    [CCode (cname = "cd_AUDIO")]
    AUDIO,
    [CCode (cname = "cd_DATA")]
    Data,
    [CCode (cname = "cd_FOUR_CHANNEL")]
    FourChannel,
    [CCode (cname = "cd_TWO_CHANNEL")]
    TwoChannel
  }

  [CCode (cname = "cd_version_block")]
  public struct VersionBlock
  {
    [CCode (cname = "version_number")]
    public unowned int VersionNumber;
    [CCode (cname = "version_string")]
    public unowned char VersionString[256];
  }

  [CCode (cname = "cd_status")]
  public enum Status
  {
    [CCode (cname = "cd_DRIVE_STATUS_OK")]
    DriveStatusOK,
    [CCode (cname = "cd_DRIVE_STATUS_BUSY")]
    DriveStatusBusy,
    [CCode (cname = "cd_DRIVE_STATUS_NOT_READY")]
    DriveStatusNotReady,
    [CCode (cname = "cd_DRIVE_STATUS_UNAVAILABLE")]
    DriveStatusUnavailable
  }

  [CCode (cname = "cd_readiness")]
  public enum Readiness
  {
    [CCode (cname = "cd_DRIVE_READY")]
    DriveREADY,
    [CCode (cname = "cd_DRIVE_NOT_READY")]
    DriveNotReady
  }

  [CCode (cname = "cd_multiplier")]
  public enum Multiplier
  {
    [CCode (cname = "cd_MULTIPLIER_DEFAULT")]
    Default,
    [CCode (cname = "cd_MULTIPLIER_EIGHTH_SEC")]
    EighthSec,
    [CCode (cname = "cd_MULTIPLIER_QUARTER_SEC")]
    QuarterSec,
    [CCode (cname = "cd_MULTIPLIER_HALF_SEC")]
    HalfSec,
    [CCode (cname = "cd_MULTIPLIER_ONE_SEC")]
    OneSec,
    [CCode (cname = "cd_MULTIPLIER_TWO_SEC")]
    TwoSec,
    [CCode (cname = "cd_MULTIPLIER_FOUR_SEC")]
    FourSec,
    [CCode (cname = "cd_MULTIPLIER_EIGHT_SEC")]
    EightSec,
    [CCode (cname = "cd_MULTIPLIER_SIXTEEN_SEC")]
    SixteenSec,
    [CCode (cname = "cd_MULTIPLIER_THIRTY_TWO_SEC")]
    ThirtyTwoSec,
    [CCode (cname = "cd_MULTIPLIER_ONE_MIN")]
    OneMin,
    [CCode (cname = "cd_MULTIPLIER_TWO_MIN")]
    TwoMin,
    [CCode (cname = "cd_MULTIPLIER_FOUR_MIN")]
    FourMin,
    [CCode (cname = "cd_MULTIPLIER_EIGHT_MIN")]
    EightMin,
    [CCode (cname = "cd_MULTIPLIER_SIXTEEN_MIN")]
    SixteenMin,
    [CCode (cname = "cd_MULTIPLIER_THIRTY_TWO_MIN")]
    ThirtyTwoMin
  }

  [CCode (cname = "cd_data_mode")]
  public enum DataMode
  {
    [CCode (cname = "cd_MODE_AUDIO")]
    ModeAudio,
    [CCode (cname = "cd_MODE1")]
    Mode1,
    [CCode (cname = "cd_MODE2_FORM2")]
    Mode2Form2,
    [CCode (cname = "cd_MODE2_FORM1")]
    Mode2Form1
  }

  [CCode (cname = "cd_speed")]
  public enum Speed
  {
    Default = 0
  }

  [CCode (cname = "cd_parameter_block")]
  public struct ParameterBlock
  {
    [CCode (cname = "inactivity_timer")]
    public unowned Multiplier InactivityTimer;
    [CCode (cname = "retry_count")]
    public unowned int RetryCount;
    [CCode (cname = "mode")]
    public unowned DataMode Mode;
    [CCode (cname = "speed")]
    public unowned Speed Speed;
  }

  [CCode (cname = "cd_play_to_what")]
  public enum PlayToWhat
  {
    [CCode (cname = "cd_PLAY_TO_END_OF_TRACK")]
    PlayToEndOfTrack,
    [CCode (cname = "cd_PLAY_TO_END_OF_DISC")]
    PlayToEndOfDisc
  }

  [CCode (cname = "cd_enquire_all_tracks_block")]
  public struct EnquireAllTracksBlock
  {
    [CCode (cname = "first_track")]
    public unowned byte FirstTrack;
    [CCode (cname = "last_track")]
    public unowned byte LastTrack;
    [CCode (cname = "reserved0")]
    public unowned byte Reserved0;
    [CCode (cname = "reserved1")]
    public unowned byte Reserved1;
    [CCode (cname = "reserved2")]
    public unowned byte Reserved2;
  }

  [CCode (cname = "cd_enquire_one_track_block")]
  public struct EnquireOneTrackBlock
  {
    [CCode (cname = "address")]
    public unowned Address Address;
    [CCode (cname = "audio_control")]
    public unowned AudioFlags AudioControl;
  }

  [CCode (cname = "cd_sub_channel_block")]
  public struct SubChannelBlock
  {
    [CCode (cname = "relative_lba")]
    public unowned Address RelativeLba;
    [CCode (cname = "absolute_lba")]
    public unowned Address AbsoluteLba;
    [CCode (cname = "audio_control")]
    public unowned AudioFlags AudioControl;
    [CCode (cname = "track_number")]
    public unowned byte TrackNumber;
    [CCode (cname = "index_number")]
    public unowned byte IndexNumber;
  }

  [CCode (cname = "cd_disc_used_block")]
  public struct DiscUsedBlock
  {
    [CCode (cname = "block_count")]
    public unowned int BlockCount;
    [CCode (cname = "block_size")]
    public unowned int BlockSize;
  }

  [CCode (cname = "cd_audio")]
  public enum Audio
  {
    [CCode (cname = "cd_AUDIO_PLAYING")]
    Playing,
    [CCode (cname = "cd_AUDIO_PAUSED")]
    Paused,
    [CCode (cname = "cd_AUDIO_COMPLETED")]
    Completed,
    [CCode (cname = "cd_AUDIO_ERROR")]
    Error,
    [CCode (cname = "cd_AUDIO_STOPPED")]
    Stopped
  }

  [CCode (cname = "cd_inquiry_block")]
  public struct InquiryBlock
  {
    [CCode (cname = "i")]
    public unowned byte I[36];
  }

  [CCode (cname = "cd_error_response_level")]
  public enum ErrorResponseLevel
  {
    [CCode (cname = "cd_CHECK_ERROR")]
    CheckError,
    [CCode (cname = "cd_CHECK_SENSE")]
    CheckSense,
    [CCode (cname = "cd_CHECK_RETRY")]
    CheckRetry
  }

  [CCode (cname = "cd_flags")]
  public enum Flags
  {
    [CCode (cname = "cd_SUPPORTED_AUDIO_CONTROL")]
    SupportedAudioControl,
    [CCode (cname = "cd_SUPPORTED_PREFETCH")]
    SupportedPrefetch,
    [CCode (cname = "cd_SUPPORTED_CLOSE_DRAWER")]
    SupportedCloseDrawer,
    [CCode (cname = "cd_SUPPORTED_AUDIO_LEVEL")]
    SupportedAudioLevel,
    [CCode (cname = "cd_SUPPORTED_SPEED_MULTIPLIER")]
    SupportedSpeedMultiplier,
    [CCode (cname = "cd_SUPPORTED_READ_AUDIO")]
    SupportedReadAudio,
    [CCode (cname = "cd_SUPPORTED_AUDIO_PARAMS")]
    SupportedAudioParams
  }

  [CCode (cname = "cd_register_flags")]
  public enum RegisterFlags
  {
    [CCode (cname = "cd_USE_OLD_READ")]
    UseOldRead,
    [CCode (cname = "cd_USE_OLD_SEEK")]
    UseOldSeek,
    [CCode (cname = "cd_USE_OLD_INQUIRY")]
    UseOldInquiry,
    [CCode (cname = "cd_USE_OLD_CAPACITY")]
    UseOldCapacity,
    [CCode (cname = "cd_USE_OLD_READY")]
    UseOldReady,
    [CCode (cname = "cd_USE_OLD_STOP_OPEN")]
    UseOldStopOpen,
    [CCode (cname = "cd_USE_OLD_CHECK")]
    UseOldCheck,
    [CCode (cname = "cd_USE_OLD_STATUS")]
    UseOldStatus,
    [CCode (cname = "cd_USE_OLD_CONTROL")]
    UseOldControl,
    [CCode (cname = "cd_USE_OLD_PREFETCH")]
    UseOldPrefetch,
    [CCode (cname = "cd_USE_OLD_RESET")]
    UseOldReset,
    [CCode (cname = "cd_USE_READ_DATA_PROPRIETARY")]
    UseReadDataProprietary,
    [CCode (cname = "cd_USE_READ_DATA_SCATTER")]
    UseReadDataScatter,
    [CCode (cname = "cd_USE_SEEK_DATA_PROPRIETARY")]
    UseSeekDataProprietary,
    [CCode (cname = "cd_USE_SEEK_DATA_SCATTER")]
    UseSeekDataScatter,
    [CCode (cname = "cd_USE_SCSI_OP")]
    UseSCSIOp
  }

  [CCode (cname = "cd_register_block")]
  public struct RegisterBlock
  {
    [CCode (cname = "flags")]
    public unowned RegisterFlags Flags;
    [CCode (cname = "registration_count")]
    public unowned int RegistrationCount;
    [CCode (cname = "drive_type_count")]
    public unowned int DriveTypeCount;
  }

  [CCode (cname = "cd_drive_type")]
  public enum DriveType
  {
    [CCode (cname = "cd_DRIVE_TYPE_UNKNOWN")]
    Unknown
  }

  [CCode (cname = "cd_audio_completion")]
  public enum AudioCompletion
  {
    [CCode (cname = "cd_AUDIO_DONE")]
    AudioDone
  }

  [CCode (cname = "cd_volume_block")]
  public struct VolumeBlock
  {
    [CCode (cname = "volume")]
    public unowned int Volume[2];
  }

  [CCode (cname="error_CD_BASE")]
  public const bits Error_CDBase;
  [CCode (cname="error_CD_BAD_ALIGNMENT")]
  public const bits Error_CDBadAlignment;
  [CCode (cname="error_CD_DRIVE_NOT_SUPPORTED")]
  public const bits Error_CDDriveNotSupported;
  [CCode (cname="error_CD_BAD_MODE")]
  public const bits Error_CDBadMode;
  [CCode (cname="error_CD_INVALID_PARAMETER")]
  public const bits Error_CDInvalidParameter;
  [CCode (cname="error_CD_NOT_AUDIO_TRACK")]
  public const bits Error_CDNotAudioTrack;
  [CCode (cname="error_CD_NO_CADDY")]
  public const bits Error_CDNoCaddy;
  [CCode (cname="error_CD_NO_DRIVE")]
  public const bits Error_CDNoDrive;
  [CCode (cname="error_CD_INVALID_FORMAT")]
  public const bits Error_CDInvalidFormat;
  [CCode (cname="error_CD_BAD_MINUTES")]
  public const bits Error_CDBadMinutes;
  [CCode (cname="error_CD_BAD_SECONDS")]
  public const bits Error_CDBadSeconds;
  [CCode (cname="error_CD_BAD_BLOCKS")]
  public const bits Error_CDBadBlocks;
  [CCode (cname="error_CD_PHYSICAL_BLOCK_BAD")]
  public const bits Error_CDPhysicalBlockBad;
  [CCode (cname="error_CD_DRAWER_LOCKED")]
  public const bits Error_CDDrawerLocked;
  [CCode (cname="error_CD_WRONG_DATA_MODE")]
  public const bits Error_CDWrongDataMode;
  [CCode (cname="error_CD_CHANNEL_NOT_SUPPORTED")]
  public const bits Error_CDChannelNotSupported;
  [CCode (cname="error_CD_BAD_DEVICE_ID")]
  public const bits Error_CDBadDeviceID;
  [CCode (cname="error_CD_BAD_CARD_NUMBER")]
  public const bits Error_CDBadCardNumber;
  [CCode (cname="error_CD_BAD_LUN_NUMBER")]
  public const bits Error_CDBadLUNNumber;
  [CCode (cname="error_CD_NO_SUCH_TRACK")]
  public const bits Error_CDNoSuchTrack;
  [CCode (cname="error_CD_FAULTY_DISC")]
  public const bits Error_CDFaultyDisc;
  [CCode (cname="error_CD_NO_SUCH_BLOCK")]
  public const bits Error_CDNoSuchBlock;
  [CCode (cname="error_CD_NOT_SUPPORTED")]
  public const bits Error_CDNotSupported;
  [CCode (cname="error_CD_DRIVER_NOT_PRESENT")]
  public const bits Error_CDDriverNotPresent;
  [CCode (cname="error_CD_SWI_NOT_SUPPORTED")]
  public const bits Error_CD_SWINotSupported;
  [CCode (cname="error_CD_TOO_MANY_DRIVERS")]
  public const bits Error_CDTooManyDrivers;
  [CCode (cname="error_CD_NOT_REGISTERED")]
  public const bits Error_CDNotRegistered;
  [CCode (cname="cd_READ_AUDIO_READ")]
  public const byte ReadAudioRead;
  [CCode (cname="cd_READ_AUDIO_REASON_MASK")]
  public const bits ReadAudioReasonMask;
  [CCode (cname="cd_SPEED_UNCHANGED")]
  public const int SpeedUnchanged;
  [CCode (cname="cd_SPEED_STANDARD")]
  public const int SpeedStandard;
  [CCode (cname="cd_SPEED_DOUBLE")]
  public const int SpeedDouble;
  [CCode (cname="cd_SPEED_MAX")]
  public const int SpeedMax;
  [CCode (cname="cd_VOLUME_LIMIT")]
  public const int VolumeLimit;

  [CCode (cname = "xcd_version")]
  public static OS.Error* XVersion
    (out VersionBlock version);

  [CCode (cname = "cd_version")]
  public static VersionBlock Version ();

  [CCode (cname = "xcd_read_data")]
  public static OS.Error* XReadData
    (AddressMode address_mode,
     Address start_address,
     int block_count,
     char* buffer,
     int size,
     ControlBlock control);

  [CCode (cname = "cd_read_data")]
  public static void ReadData
    (AddressMode address_mode,
     Address start_address,
     int block_count,
     char* buffer,
     int size,
     ControlBlock control);

  [CCode (cname = "xcd_seek_to")]
  public static OS.Error* XSeekTo
    (AddressMode address_mode,
     Address seek,
     ControlBlock control);

  [CCode (cname = "cd_seek_to")]
  public static void SeekTo
    (AddressMode address_mode,
     Address seek,
     ControlBlock control);

  [CCode (cname = "xcd_drive_status")]
  public static OS.Error* XDriveStatus
    (ControlBlock control,
     out Status status);

  [CCode (cname = "cd_drive_status")]
  public static Status DriveStatus
    (ControlBlock control);

  [CCode (cname = "xcd_drive_ready")]
  public static OS.Error* XDriveReady
    (ControlBlock control,
     out Readiness ready);

  [CCode (cname = "cd_drive_ready")]
  public static Readiness DriveReady
    (ControlBlock control);

  [CCode (cname = "xcd_get_parameters")]
  public static OS.Error* XGetParameters
    (ParameterBlock params,
     ControlBlock control);

  [CCode (cname = "cd_get_parameters")]
  public static void GetParameters
    (ParameterBlock params,
     ControlBlock control);

  [CCode (cname = "xcd_set_parameters")]
  public static OS.Error* XSetParameters
    (ParameterBlock params,
     ControlBlock control);

  [CCode (cname = "cd_set_parameters")]
  public static void SetParameters
    (ParameterBlock params,
     ControlBlock control);

  [CCode (cname = "xcd_open_drawer")]
  public static OS.Error* XOpenDrawer
    (ControlBlock control);

  [CCode (cname = "cd_open_drawer")]
  public static void OpenDrawer
    (ControlBlock control);

  [CCode (cname = "xcd_eject_button")]
  public static OS.Error* XEjectButton
    (bool disable,
     ControlBlock control);

  [CCode (cname = "cd_eject_button")]
  public static void EjectButton
    (bool disable,
     ControlBlock control);

  [CCode (cname = "xcd_enquire_address")]
  public static OS.Error* XEnquireAddress
    (AddressMode address_mode,
     ControlBlock control,
     out Address address);

  [CCode (cname = "cd_enquire_address")]
  public static Address EnquireAddress
    (AddressMode address_mode,
     ControlBlock control);

  [CCode (cname = "xcd_enquire_data_mode")]
  public static OS.Error* XEnquireDataMode
    (AddressMode address_mode,
     Address address,
     ControlBlock control,
     out DataMode data_mode);

  [CCode (cname = "cd_enquire_data_mode")]
  public static DataMode EnquireDataMode
    (AddressMode address_mode,
     Address address,
     ControlBlock control);

  [CCode (cname = "xcd_play_audio")]
  public static OS.Error* XPlayAudio
    (AddressMode address_mode,
     Address start,
     Address end,
     ControlBlock control);

  [CCode (cname = "cd_play_audio")]
  public static void PlayAudio
    (AddressMode address_mode,
     Address start,
     Address end,
     ControlBlock control);

  [CCode (cname = "xcd_play_track")]
  public static OS.Error* XPlayTrack
    (int track,
     PlayToWhat play_to,
     ControlBlock control);

  [CCode (cname = "cd_play_track")]
  public static void PlayTrack
    (int track,
     PlayToWhat play_to,
     ControlBlock control);

  [CCode (cname = "xcd_audio_pause")]
  public static OS.Error* XAudioPause
    (bool pause,
     ControlBlock control);

  [CCode (cname = "cd_audio_pause")]
  public static void AudioPause
    (bool pause,
     ControlBlock control);

  [CCode (cname = "xcd_enquire_all_tracks")]
  public static OS.Error* XEnquireAllTracks
    (EnquireAllTracksBlock info,
     ControlBlock control);

  [CCode (cname = "cd_enquire_all_tracks")]
  public static void EnquireAllTracks
    (EnquireAllTracksBlock info,
     ControlBlock control);

  [CCode (cname = "xcd_enquire_one_track")]
  public static OS.Error* XEnquireOneTrack
    (int track,
     EnquireOneTrackBlock info,
     ControlBlock control);

  [CCode (cname = "cd_enquire_one_track")]
  public static void EnquireOneTrack
    (int track,
     EnquireOneTrackBlock info,
     ControlBlock control);

  [CCode (cname = "xcd_read_sub_channel")]
  public static OS.Error* XReadSubChannel
    (SubChannelBlock block,
     ControlBlock control);

  [CCode (cname = "cd_read_sub_channel")]
  public static void ReadSubChannel
    (SubChannelBlock block,
     ControlBlock control);

  [CCode (cname = "xcd_check_drive")]
  public static OS.Error* XCheckDrive
    (ControlBlock control,
     out bits status);

  [CCode (cname = "cd_check_drive")]
  public static bits CheckDrive
    (ControlBlock control);

  [CCode (cname = "xcd_disc_changed")]
  public static OS.Error* XDiscChanged
    (ControlBlock control,
     out bool changed);

  [CCode (cname = "cd_disc_changed")]
  public static bool DiscChanged
    (ControlBlock control);

  [CCode (cname = "xcd_stop_disc")]
  public static OS.Error* XStopDisc
    (ControlBlock control);

  [CCode (cname = "cd_stop_disc")]
  public static void StopDisc
    (ControlBlock control);

  [CCode (cname = "xcd_disc_used")]
  public static OS.Error* XDiscUsed
    (AddressMode address_mode,
     DiscUsedBlock info,
     ControlBlock control);

  [CCode (cname = "cd_disc_used")]
  public static void DiscUsed
    (AddressMode address_mode,
     DiscUsedBlock info,
     ControlBlock control);

  [CCode (cname = "xcd_audio_status")]
  public static OS.Error* XAudioStatus
    (ControlBlock control,
     out Audio status);

  [CCode (cname = "cd_audio_status")]
  public static Audio AudioStatus
    (ControlBlock control);

  [CCode (cname = "xcd_inquiry")]
  public static OS.Error* XInquiry
    (InquiryBlock data,
     ControlBlock control);

  [CCode (cname = "cd_inquiry")]
  public static void Inquiry
    (InquiryBlock data,
     ControlBlock control);

  [CCode (cname = "xcd_disc_has_changed")]
  public static OS.Error* XDiscHasChanged
    (ControlBlock control);

  [CCode (cname = "cd_disc_has_changed")]
  public static void DiscHasChanged
    (ControlBlock control);

  [CCode (cname = "xcd_control")]
  public static OS.Error* XControl
    (ErrorResponseLevel level,
     ControlBlock control);

  [CCode (cname = "cd_control")]
  public static void Control
    (ErrorResponseLevel level,
     ControlBlock control);

  [CCode (cname = "xcd_supported")]
  public static OS.Error* XSupported
    (ControlBlock control,
     out Flags flags);

  [CCode (cname = "cd_supported")]
  public static Flags Supported
    (ControlBlock control);

  [CCode (cname = "xcd_prefetch")]
  public static OS.Error* XPrefetch
    (AddressMode address_mode,
     Address address,
     ControlBlock control);

  [CCode (cname = "cd_prefetch")]
  public static void Prefetch
    (AddressMode address_mode,
     Address address,
     ControlBlock control);

  [CCode (cname = "xcd_reset")]
  public static OS.Error* XReset
    (ControlBlock control);

  [CCode (cname = "cd_reset")]
  public static void Reset
    (ControlBlock control);

  [CCode (cname = "xcd_close_drawer")]
  public static OS.Error* XCloseDrawer
    (ControlBlock control);

  [CCode (cname = "cd_close_drawer")]
  public static void CloseDrawer
    (ControlBlock control);

  [CCode (cname = "xcd_is_drawer_locked")]
  public static OS.Error* XIsDrawerLocked
    (ControlBlock control,
     out bool locked);

  [CCode (cname = "cd_is_drawer_locked")]
  public static bool IsDrawerLocked
    (ControlBlock control);

  [CCode (cname = "xcd_audio_control")]
  public static OS.Error* XAudioControl
    (int control,
     ControlBlock control_block);

  [CCode (cname = "cd_audio_control")]
  public static void AudioControl
    (int control,
     ControlBlock control_block);

  [CCode (cname = "xcd_last_error")]
  public static OS.Error* XLastError
    (out int errnum);

  [CCode (cname = "cd_last_error")]
  public static int LastError ();

  [CCode (cname = "xcd_audio_level")]
  public static OS.Error* XAudioLevel
    (ControlBlock control,
     out OS.Error error);

  [CCode (cname = "cd_audio_level")]
  public static void AudioLevel
    (ControlBlock control,
     out OS.Error error);

  [CCode (cname = "xcd_register")]
  public static OS.Error* XRegister
    (RegisterBlock info,
     AsmRoutine driver_code,
     char* driver_workspace);

  [CCode (cname = "cd_register")]
  public static void Register
    (RegisterBlock info,
     AsmRoutine driver_code,
     char* driver_workspace);

  [CCode (cname = "xcd_unregister")]
  public static OS.Error* XUnregister
    (RegisterBlock info,
     AsmRoutine driver_code);

  [CCode (cname = "cd_unregister")]
  public static void Unregister
    (RegisterBlock info,
     AsmRoutine driver_code);

  [CCode (cname = "xcd_byte_copy")]
  public static OS.Error* XByteCopy
    (byte* source,
     byte* destination,
     int size);

  [CCode (cname = "cd_byte_copy")]
  public static void ByteCopy
    (byte* source,
     byte* destination,
     int size);

  [CCode (cname = "xcd_identify")]
  public static OS.Error* XIdentify
    (ControlBlock control,
     out DriveType drive_type);

  [CCode (cname = "cd_identify")]
  public static DriveType Identify
    (ControlBlock control);

  [CCode (cname = "xcd_convert_to_lba")]
  public static OS.Error* XConvertToLBA
    (AddressMode address_mode,
     Address address,
     out Address new_address);

  [CCode (cname = "cd_convert_to_lba")]
  public static Address ConvertToLBA
    (AddressMode address_mode,
     Address address);

  [CCode (cname = "xcd_convert_to_msf")]
  public static OS.Error* XConvertToMSF
    (AddressMode address_mode,
     Address address,
     out Address new_address);

  [CCode (cname = "cd_convert_to_msf")]
  public static Address ConvertToMSF
    (AddressMode address_mode,
     Address address);

  [CCode (cname = "xcd_read_audio")]
  public static OS.Error* XReadAudio
    (AddressMode address_mode,
     Address address,
     int block_count,
     char* buffer,
     bits flags,
     ControlBlock control,
     out DriveType drive_type);

  [CCode (cname = "cd_read_audio")]
  public static DriveType ReadAudio
    (AddressMode address_mode,
     Address address,
     int block_count,
     char* buffer,
     bits flags,
     ControlBlock control);

  [CCode (cname = "xcd_read_user_data")]
  public static OS.Error* XReadUserData
    (AddressMode address_mode,
     Address address,
     int size,
     char* buffer,
     int offset,
     ControlBlock control,
     out Address last,
     out int next_offset);

  [CCode (cname = "cd_read_user_data")]
  public static Address ReadUserData
    (AddressMode address_mode,
     Address address,
     int size,
     char* buffer,
     int offset,
     ControlBlock control,
     out int next_offset);

  [CCode (cname = "xcd_seek_user_data")]
  public static OS.Error* XSeekUserData
    (AddressMode address_mode,
     Address address,
     int offset,
     ControlBlock control,
     out Address found,
     out int offset_out);

  [CCode (cname = "cd_seek_user_data")]
  public static Address SeekUserData
    (AddressMode address_mode,
     Address address,
     int offset,
     ControlBlock control,
     out int offset_out);

  [CCode (cname = "xcdgetaudioparms_volume_block")]
  public static OS.Error* XCDGetAudioParms_VolumeBlock
    (VolumeBlock block,
     ControlBlock control,
     out AudioCompletion cmp);

  [CCode (cname = "cdgetaudioparms_volume_block")]
  public static AudioCompletion CDGetAudioParms_VolumeBlock
    (VolumeBlock block,
     ControlBlock control);

  [CCode (cname = "xcd_scsi_user_op")]
  public static OS.Error* XSCSIUserOp
    (bits op,
     int block_size,
     SCSI.OpBlock block,
     char* start_addr,
     int length,
     int timeout,
     ControlBlock control,
     out char* next_addr,
     out int undone);

  [CCode (cname = "cd_scsi_user_op")]
  public static int SCSIUserOp
    (bits op,
     int block_size,
     SCSI.OpBlock block,
     char* start_addr,
     int length,
     int timeout,
     ControlBlock control,
     out char* next_addr);

}
