/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.Window;
using OSLib.ColourPicker;

[CCode (cheader_filename = "oslib/colourdbox.h")]
namespace OSLib.ColourDbox
{
  [CCode (cname = "colourdbox_flags")]
  public enum Flags
  {
    [CCode (cname = "colourdbox_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "colourdbox_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "colourdbox_INCLUDE_NONE_BUTTON")]
    IncludeNoneButton,
    [CCode (cname = "colourdbox_SELECT_NONE_BUTTON")]
    SelectNoneButton
  }

  [CCode (cname = "colourdbox_colour_flags")]
  public enum ColourFlags
  {
    [CCode (cname = "colourdbox_COLOUR_TRANSPARENT")]
    ColourTransparent
  }

  [CCode (cname = "colourdbox_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
  }

  [CCode (cname = "colourdbox_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
  }

  [CCode (cname = "colourdbox_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "colourdbox_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "colourdbox_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "colourdbox_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "colourdbox_action_colour_selected_block")]
  public struct ActionColourSelectedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "block_size")]
    public unowned int BlockSize;
    [CCode (cname = "info")]
    public unowned int Info[50];
  }

  [CCode (cname = "colourdbox_colour_model_base")]
  public struct ColourModelBase
  {
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "colourdbox_colour_model", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ColourModel
  {
    public ColourModel(int numEntries);
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourdbox_dialogue_completed_flags")]
  public enum DialogueCompletedFlags
  {
    [CCode (cname = "colourdbox_DIALOGUE_COMPLETED_COLOUR_CHOICE")]
    DialogueCompletedColourChoice
  }

  [CCode (cname = "colourdbox_action_dialogue_completed")]
  public struct ActionDialogueCompleted
  {
    [CCode (cname = "flags")]
    public unowned DialogueCompletedFlags Flags;
  }

  [CCode (cname = "colourdbox_action_colour_selected")]
  public struct ActionColourSelected
  {
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info")]
    public unowned int Info[50];
  }

  [CCode (cname="class_COLOUR_DBOX")]
  public const Toolbox.Class Class_ColourDbox;
  [CCode (cname="error_COLOUR_DBOX_TASKS_ACTIVE")]
  public const bits Error_ColourDboxTasksActive;
  [CCode (cname="error_COLOUR_DBOX_ALLOC_FAILED")]
  public const bits Error_ColourDboxAllocFailed;
  [CCode (cname="error_COLOUR_DBOX_SHORT_BUFFER")]
  public const bits Error_ColourDboxShortBuffer;
  [CCode (cname="error_COLOUR_DBOX_NO_SUCH_TASK")]
  public const bits Error_ColourDboxNoSuchTask;
  [CCode (cname="error_COLOUR_DBOX_NO_SUCH_METHOD")]
  public const bits Error_ColourDboxNoSuchMethod;
  [CCode (cname="error_COLOUR_DBOX_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_ColourDboxNoSuchMiscOpMethod;
  [CCode (cname="action_COLOUR_DBOX_ABOUT_TO_BE_SHOWN")]
  public const bits Action_ColourDboxAboutToBeShown;
  [CCode (cname="action_COLOUR_DBOX_DIALOGUE_COMPLETED")]
  public const bits Action_ColourDboxDialogueCompleted;
  [CCode (cname="action_COLOUR_DBOX_COLOUR_SELECTED")]
  public const bits Action_ColourDboxColourSelected;

  [CCode (cname = "xcolourdbox_get_wimp_handle")]
  public static OS.Error* XGetWimpHandle
    (bits flags,
     Toolbox.O colour_dbox,
     out Wimp.W w);

  [CCode (cname = "colourdbox_get_wimp_handle")]
  public static Wimp.W GetWimpHandle
    (bits flags,
     Toolbox.O colour_dbox);

  [CCode (cname = "xcolourdbox_get_dialogue_handle")]
  public static OS.Error* XGetDialogueHandle
    (bits flags,
     Toolbox.O colour_dbox,
     out ColourPicker.D d);

  [CCode (cname = "colourdbox_get_dialogue_handle")]
  public static ColourPicker.D GetDialogueHandle
    (bits flags,
     Toolbox.O colour_dbox);

  [CCode (cname = "xcolourdbox_set_colour")]
  public static OS.Error* XSetColour
    (ColourFlags flags,
     Toolbox.O colour_dbox,
     ColourPicker.Colour colour);

  [CCode (cname = "colourdbox_set_colour")]
  public static void SetColour
    (ColourFlags flags,
     Toolbox.O colour_dbox,
     ColourPicker.Colour colour);

  [CCode (cname = "xcolourdbox_get_colour")]
  public static OS.Error* XGetColour
    (bits flags,
     Toolbox.O colour_dbox,
     ColourPicker.Colour buffer,
     int size,
     out ColourFlags flags_out,
     out int used);

  [CCode (cname = "colourdbox_get_colour")]
  public static int GetColour
    (bits flags,
     Toolbox.O colour_dbox,
     ColourPicker.Colour buffer,
     int size,
     out ColourFlags flags_out);

  [CCode (cname = "xcolourdbox_set_colour_model")]
  public static OS.Error* XSetColourModel
    (bits flags,
     Toolbox.O colour_dbox,
     ColourModel model);

  [CCode (cname = "colourdbox_set_colour_model")]
  public static void SetColourModel
    (bits flags,
     Toolbox.O colour_dbox,
     ColourModel model);

  [CCode (cname = "xcolourdbox_get_colour_model")]
  public static OS.Error* XGetColourModel
    (bits flags,
     Toolbox.O colour_dbox,
     ColourModel buffer,
     int size,
     out int used);

  [CCode (cname = "colourdbox_get_colour_model")]
  public static int GetColourModel
    (bits flags,
     Toolbox.O colour_dbox,
     ColourModel buffer,
     int size);

  [CCode (cname = "xcolourdbox_set_none_available")]
  public static OS.Error* XSetNoneAvailable
    (bits flags,
     Toolbox.O colour_dbox,
     bool none);

  [CCode (cname = "colourdbox_set_none_available")]
  public static void SetNoneAvailable
    (bits flags,
     Toolbox.O colour_dbox,
     bool none);

  [CCode (cname = "xcolourdbox_get_none_available")]
  public static OS.Error* XGetNoneAvailable
    (bits flags,
     Toolbox.O colour_dbox,
     out bool none);

  [CCode (cname = "colourdbox_get_none_available")]
  public static bool GetNoneAvailable
    (bits flags,
     Toolbox.O colour_dbox);

}
