/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/colourmap.h")]
namespace OSLib.ColourMap
{

  [CCode (cname = "xcolourmap_start")]
  public static OS.Error* XStart
    (bits flags,
     char* workspace,
     int size,
     out int size_used);

  [CCode (cname = "colourmap_start")]
  public static int Start
    (bits flags,
     char* workspace,
     int size);

  [CCode (cname = "xcolourmapapplymapping_blend")]
  public static OS.Error* XColourMapApplyMapping_Blend
    (bits flags,
     char* workspace,
     int size,
     OS.Colour colour,
     int factor,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_blend")]
  public static int ColourMapApplyMapping_Blend
    (bits flags,
     char* workspace,
     int size,
     OS.Colour colour,
     int factor);

  [CCode (cname = "xcolourmapapplymapping_user")]
  public static OS.Error* XColourMapApplyMapping_User
    (bits flags,
     char* workspace,
     int size,
     OSSpriteOp.ColourMapping mapping,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_user")]
  public static int ColourMapApplyMapping_User
    (bits flags,
     char* workspace,
     int size,
     OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "xcolourmapapplymapping_identity")]
  public static OS.Error* XColourMapApplyMapping_Identity
    (bits flags,
     char* workspace,
     int size,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_identity")]
  public static int ColourMapApplyMapping_Identity
    (bits flags,
     char* workspace,
     int size);

  [CCode (cname = "xcolourmapapplymapping_invert")]
  public static OS.Error* XColourMapApplyMapping_Invert
    (bits flags,
     char* workspace,
     int size,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_invert")]
  public static int ColourMapApplyMapping_Invert
    (bits flags,
     char* workspace,
     int size);

  [CCode (cname = "xcolourmapapplymapping_monochrome")]
  public static OS.Error* XColourMapApplyMapping_Monochrome
    (bits flags,
     char* workspace,
     int size,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_monochrome")]
  public static int ColourMapApplyMapping_Monochrome
    (bits flags,
     char* workspace,
     int size);

  [CCode (cname = "xcolourmapapplymapping_monochrome_and_scale")]
  public static OS.Error* XColourMapApplyMapping_MonochromeAndScale
    (bits flags,
     char* workspace,
     int size,
     OS.Colour black,
     OS.Colour white,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_monochrome_and_scale")]
  public static int ColourMapApplyMapping_MonochromeAndScale
    (bits flags,
     char* workspace,
     int size,
     OS.Colour black,
     OS.Colour white);

  [CCode (cname = "xcolourmapapplymapping_gamma")]
  public static OS.Error* XColourMapApplyMapping_Gamma
    (bits flags,
     char* workspace,
     int size,
     int gamma_level,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_gamma")]
  public static int ColourMapApplyMapping_Gamma
    (bits flags,
     char* workspace,
     int size,
     int gamma_level);

  [CCode (cname = "xcolourmapapplymapping_contrast")]
  public static OS.Error* XColourMapApplyMapping_Contrast
    (bits flags,
     char* workspace,
     int size,
     int factor,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_contrast")]
  public static int ColourMapApplyMapping_Contrast
    (bits flags,
     char* workspace,
     int size,
     int factor);

  [CCode (cname = "xcolourmapapplymapping_brightness")]
  public static OS.Error* XColourMapApplyMapping_Brightness
    (bits flags,
     char* workspace,
     int size,
     int offset,
     out int size_required);

  [CCode (cname = "colourmapapplymapping_brightness")]
  public static int ColourMapApplyMapping_Brightness
    (bits flags,
     char* workspace,
     int size,
     int offset);

  [CCode (cname = "xcolourmap_end")]
  public static OS.Error* XEnd
    (bits flags,
     char* workspace,
     int size,
     out OSSpriteOp.ColourMapping mapping);

  [CCode (cname = "colourmap_end")]
  public static OSSpriteOp.ColourMapping End
    (bits flags,
     char* workspace,
     int size);

}
