/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/colourpicker.h")]
namespace OSLib.ColourPicker
{
  [CCode (cname = "colourpicker_d", has_type_id = false)]
  public struct D : bits
  {
  }

  [CCode (cname = "colourpicker_colour_base")]
  public struct ColourBase
  {
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "colourpicker_colour", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Colour
  {
    public Colour(int numEntries);
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_dialogue_flags")]
  public enum DialogueFlags
  {
    [CCode (cname = "colourpicker_DIALOGUE_OFFERS_TRANSPARENT")]
    DialogueOffersTransparent,
    [CCode (cname = "colourpicker_DIALOGUE_TRANSPARENT")]
    DialogueTransparent,
    [CCode (cname = "colourpicker_DIALOGUE_TYPE")]
    DialogueType,
    [CCode (cname = "colourpicker_DIALOGUE_IGNORE_HELP")]
    DialogueIgnoreHelp,
    [CCode (cname = "colourpicker_DIALOGUE_IGNORE_KEY_PRESSED")]
    DialogueIgnoreKeyPressed
  }

  [CCode (cname = "colourpicker_dialogue_base")]
  public struct DialogueBase
  {
    [CCode (cname = "flags")]
    public unowned DialogueFlags Flags;
    [CCode (cname = "title")]
    public unowned string Title;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "colourpicker_dialogue", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Dialogue
  {
    public Dialogue(int numEntries);
    [CCode (cname = "flags")]
    public unowned DialogueFlags Flags;
    [CCode (cname = "title")]
    public unowned string Title;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_model_flags")]
  public enum ModelFlags
  {
    Default = 0
  }

  [CCode (cname = "colourpicker_model")]
  public struct Model
  {
    [CCode (cname = "flags")]
    public unowned ModelFlags Flags;
    [CCode (cname = "name")]
    public unowned string Name;
    [CCode (cname = "description")]
    public unowned string Description;
    [CCode (cname = "info_size")]
    public unowned int InfoSize;
    [CCode (cname = "pane_size")]
    public unowned OS.Coord PaneSize;
    [CCode (cname = "entries")]
    public unowned AsmRoutine Entries[8];
  }

  [CCode (cname = "colourpicker_colour_flags")]
  public enum ColourFlags
  {
    [CCode (cname = "colourpicker_COLOUR_TRANSPARENT")]
    ColourTransparent,
    [CCode (cname = "colourpicker_COLOUR_DRAGGING")]
    ColourDragging
  }

  [CCode (cname = "colourpicker_message_colour_choice_base")]
  public struct MessageColourChoiceBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "colourpicker_message_colour_choice", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class MessageColourChoice
  {
    public MessageColourChoice(int numEntries);
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_message_colour_changed_base")]
  public struct MessageColourChangedBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [Compact]
  [CCode (cname = "colourpicker_message_colour_changed", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class MessageColourChanged
  {
    public MessageColourChanged(int numEntries);
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_message_open_parent_request")]
  public struct MessageOpenParentRequest
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_message_close_dialogue_request")]
  public struct MessageCloseDialogueRequest
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_message_reset_colour_request")]
  public struct MessageResetColourRequest
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_full_message_colour_choice_base")]
  public struct FullMessageColourChoiceBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "array_size")]
    public unowned int ArraySize;
  }

  [CCode (cname = "colourpicker_full_message_colour_choice")]
  public struct FullMessageColourChoice : Wimp.MessageHeaderBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "array_size")]
    public unowned int ArraySize;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_full_message_colour_changed_base")]
  public struct FullMessageColourChangedBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "array_size")]
    public unowned int ArraySize;
  }

  [CCode (cname = "colourpicker_full_message_colour_changed")]
  public struct FullMessageColourChanged : Wimp.MessageHeaderBase
  {
    [CCode (cname = "d")]
    public unowned D D;
    [CCode (cname = "flags")]
    public unowned ColourFlags Flags;
    [CCode (cname = "colour")]
    public unowned OS.Colour Colour;
    [CCode (cname = "array_size")]
    public unowned int ArraySize;
    [CCode (cname = "info", array_length = false)]
    public unowned int[] Info;
  }

  [CCode (cname = "colourpicker_full_message_open_parent_request")]
  public struct FullMessageOpenParentRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_full_message_close_dialogue_request")]
  public struct FullMessageCloseDialogueRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_full_message_reset_colour_request")]
  public struct FullMessageResetColourRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "d")]
    public unowned D D;
  }

  [CCode (cname = "colourpicker_open_flags")]
  public enum OpenFlags
  {
    [CCode (cname = "colourpicker_OPEN_TRANSIENT")]
    OpenTransient,
    [CCode (cname = "colourpicker_OPEN_SUB_MENU")]
    OpenSubMenu,
    [CCode (cname = "colourpicker_OPEN_TOOLBOX")]
    OpenToolbox
  }

  [CCode (cname = "colourpicker_close_flags")]
  public enum CloseFlags
  {
    Default = 0
  }

  [CCode (cname = "colourpicker_update_flags")]
  public enum UpdateFlags
  {
    [CCode (cname = "colourpicker_UPDATE_OFFERS_TRANSPARENT")]
    UpdateOffersTransparent,
    [CCode (cname = "colourpicker_UPDATE_TRANSPARENT")]
    UpdateTransparent,
    [CCode (cname = "colourpicker_UPDATE_TYPE")]
    UpdateType,
    [CCode (cname = "colourpicker_UPDATE_VISIBLE")]
    UpdateVisible,
    [CCode (cname = "colourpicker_UPDATE_SCROLL")]
    UpdateScroll,
    [CCode (cname = "colourpicker_UPDATE_TITLE")]
    UpdateTitle,
    [CCode (cname = "colourpicker_UPDATE_COLOUR")]
    UpdateColour,
    [CCode (cname = "colourpicker_UPDATE_MODEL")]
    UpdateModel,
    [CCode (cname = "colourpicker_UPDATE_IGNORE_HELP")]
    UpdateIgnoreHelp,
    [CCode (cname = "colourpicker_UPDATE_IGNORE_KEY_PRESSED")]
    UpdateIgnoreKeyPressed
  }

  [CCode (cname = "colourpicker_read_flags")]
  public enum ReadFlags
  {
    Default = 0
  }

  [CCode (cname = "colourpicker_set_flags")]
  public enum SetFlags
  {
    Default = 0
  }

  [CCode (cname = "colourpicker_help_flags")]
  public enum HelpFlags
  {
    Default = 0
  }

  [CCode (cname="error_COLOUR_PICKER_UNINIT")]
  public const bits Error_ColourPickerUninit;
  [CCode (cname="error_COLOUR_PICKER_BAD_MODEL")]
  public const bits Error_ColourPickerBadModel;
  [CCode (cname="error_COLOUR_PICKER_BAD_HANDLE")]
  public const bits Error_ColourPickerBadHandle;
  [CCode (cname="error_COLOUR_PICKER_BAD_FLAGS")]
  public const bits Error_ColourPickerBadFlags;
  [CCode (cname="error_COLOUR_PICKER_IN_USE")]
  public const bits Error_ColourPickerInUse;
  [CCode (cname="error_COLOUR_PICKER_MODEL_IN_USE")]
  public const bits Error_ColourPickerModelInUse;
  [CCode (cname="error_COLOUR_PICKER_BAD_REASON")]
  public const bits Error_ColourPickerBadReason;
  [CCode (cname="colourpicker_ENTRY_DIALOGUE_STARTING")]
  public const int EntryDialogueStarting;
  [CCode (cname="colourpicker_ENTRY_DIALOGUE_FINISHING")]
  public const int EntryDialogueFinishing;
  [CCode (cname="colourpicker_ENTRY_REDRAW_AREA")]
  public const int EntryRedrawArea;
  [CCode (cname="colourpicker_ENTRY_UPDATE_AREA")]
  public const int EntryUpdateArea;
  [CCode (cname="colourpicker_ENTRY_READ_VALUES")]
  public const int EntryReadValues;
  [CCode (cname="colourpicker_ENTRY_SET_VALUES")]
  public const int EntrySetValues;
  [CCode (cname="colourpicker_ENTRY_PROCESS_EVENT")]
  public const int EntryProcessEvent;
  [CCode (cname="colourpicker_ENTRY_SET_COLOUR")]
  public const int EntrySetColour;
  [CCode (cname="colourpicker_ENTRY_LIMIT")]
  public const int EntryLimit;
  [CCode (cname="colourpicker_DIALOGUE_TYPE_SHIFT")]
  public const int DialogueTypeShift;
  [CCode (cname="colourpicker_DIALOGUE_TYPE_NEVER")]
  public const bits DialogueTypeNever;
  [CCode (cname="colourpicker_DIALOGUE_TYPE_CLICK")]
  public const bits DialogueTypeClick;
  [CCode (cname="colourpicker_DIALOGUE_TYPE_CLICK_DRAG")]
  public const bits DialogueTypeClickDrag;
  [CCode (cname="colourpicker_MODEL_SIZE_RGB")]
  public const int ModelSizeRGB;
  [CCode (cname="colourpicker_MODEL_SIZE_CMYK")]
  public const int ModelSizeCMYK;
  [CCode (cname="colourpicker_MODEL_SIZE_HSV")]
  public const int ModelSizeHSV;
  [CCode (cname="colourpicker_MODEL_RGB")]
  public const int ModelRGB;
  [CCode (cname="colourpicker_MODEL_CMYK")]
  public const int ModelCMYK;
  [CCode (cname="colourpicker_MODEL_HSV")]
  public const int ModelHSV;
  [CCode (cname="message_COLOUR_PICKER_COLOUR_CHOICE")]
  public const bits Message_ColourPickerColourChoice;
  [CCode (cname="message_COLOUR_PICKER_COLOUR_CHANGED")]
  public const bits Message_ColourPickerColourChanged;
  [CCode (cname="message_COLOUR_PICKER_CLOSE_DIALOGUE_REQUEST")]
  public const bits Message_ColourPickerCloseDialogueRequest;
  [CCode (cname="message_COLOUR_PICKER_OPEN_PARENT_REQUEST")]
  public const bits Message_ColourPickerOpenParentRequest;
  [CCode (cname="message_COLOUR_PICKER_RESET_COLOUR_REQUEST")]
  public const bits Message_ColourPickerResetColourRequest;

  [CCode (cname = "xcolourpicker_register_model")]
  public static OS.Error* XRegisterModel
    (int model_no,
     Model model,
     char* workspace);

  [CCode (cname = "colourpicker_register_model")]
  public static void RegisterModel
    (int model_no,
     Model model,
     char* workspace);

  [CCode (cname = "xcolourpicker_deregister_model")]
  public static OS.Error* XDeregisterModel
    (int model_no);

  [CCode (cname = "colourpicker_deregister_model")]
  public static void DeregisterModel
    (int model_no);

  [CCode (cname = "xcolourpicker_open_dialogue")]
  public static OS.Error* XOpenDialogue
    (OpenFlags flags,
     Dialogue dialogue,
     out D d,
     out Wimp.W w);

  [CCode (cname = "colourpicker_open_dialogue")]
  public static D OpenDialogue
    (OpenFlags flags,
     Dialogue dialogue,
     out Wimp.W w);

  [CCode (cname = "xcolourpicker_close_dialogue")]
  public static OS.Error* XCloseDialogue
    (CloseFlags flags,
     D d);

  [CCode (cname = "colourpicker_close_dialogue")]
  public static void CloseDialogue
    (CloseFlags flags,
     D d);

  [CCode (cname = "xcolourpicker_update_dialogue")]
  public static OS.Error* XUpdateDialogue
    (UpdateFlags flags,
     D d,
     Dialogue dialogue);

  [CCode (cname = "colourpicker_update_dialogue")]
  public static void UpdateDialogue
    (UpdateFlags flags,
     D d,
     Dialogue dialogue);

  [CCode (cname = "xcolourpicker_read_dialogue")]
  public static OS.Error* XReadDialogue
    (ReadFlags flags,
     D d,
     Dialogue dialogue,
     out Wimp.W w,
     out int size);

  [CCode (cname = "colourpicker_read_dialogue")]
  public static void ReadDialogue
    (ReadFlags flags,
     D d,
     Dialogue dialogue,
     out Wimp.W w,
     out int size);

  [CCode (cname = "xcolourpicker_set_colour")]
  public static OS.Error* XSetColour
    (SetFlags flags,
     Colour colour);

  [CCode (cname = "colourpicker_set_colour")]
  public static void SetColour
    (SetFlags flags,
     Colour colour);

  [CCode (cname = "xcolourpicker_help_reply")]
  public static OS.Error* XHelpReply
    (HelpFlags flags,
     Wimp.Message help_request);

  [CCode (cname = "colourpicker_help_reply")]
  public static void HelpReply
    (HelpFlags flags,
     Wimp.Message help_request);

  [CCode (cname = "xcolourpickermodelswi_colour_changed")]
  public static OS.Error* XColourPickerModelSWI_ColourChanged
    (Colour colour);

  [CCode (cname = "colourpickermodelswi_colour_changed")]
  public static void ColourPickerModelSWI_ColourChanged
    (Colour colour);

  [CCode (cname = "xcolourpickermodelswi_colour_changed_by_dragging")]
  public static OS.Error* XColourPickerModelSWI_ColourChangedByDragging
    (Colour colour);

  [CCode (cname = "colourpickermodelswi_colour_changed_by_dragging")]
  public static void ColourPickerModelSWI_ColourChangedByDragging
    (Colour colour);

  [CCode (cname = "xcolourpickermodelswi_claim_event")]
  public static OS.Error* XColourPickerModelSWI_ClaimEvent
    (int event,
     Colour colour);

  [CCode (cname = "colourpickermodelswi_claim_event")]
  public static void ColourPickerModelSWI_ClaimEvent
    (int event,
     Colour colour);

  [CCode (cname = "xcolourpickermodelswi_release_event")]
  public static OS.Error* XColourPickerModelSWI_ReleaseEvent
    (int event,
     Colour colour);

  [CCode (cname = "colourpickermodelswi_release_event")]
  public static void ColourPickerModelSWI_ReleaseEvent
    (int event,
     Colour colour);

  [CCode (cname = "xcolourpickermodelswi_process_key")]
  public static OS.Error* XColourPickerModelSWI_ProcessKey
    (int c,
     Colour colour);

  [CCode (cname = "colourpickermodelswi_process_key")]
  public static void ColourPickerModelSWI_ProcessKey
    (int c,
     Colour colour);

  [CCode (cname = "xservice_colour_picker_loaded")]
  public static OS.Error* XService_ColourPickerLoaded
    (AsmRoutine loaded_service,
     char* workspace);

  [CCode (cname = "service_colour_picker_loaded")]
  public static void Service_ColourPickerLoaded
    (AsmRoutine loaded_service,
     char* workspace);

}
