/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.Font;

[CCode (cheader_filename = "oslib/colourtrans.h")]
namespace OSLib.ColourTrans
{
  [CCode (cname = "colourtrans_gcol_flags")]
  public enum GCOLFlags
  {
    [CCode (cname = "colourtrans_SET_FG_GCOL")]
    SetFG_GCol,
    [CCode (cname = "colourtrans_SET_BG_GCOL")]
    SetBG_GCol,
    [CCode (cname = "colourtrans_USE_ECFS_GCOL")]
    UseECFS_GCol,
    [CCode (cname = "colourtrans_SET_TEXT_GCOL")]
    SetText_GCol
  }

  [CCode (cname = "colourtrans_table_flags")]
  public enum TableFlags
  {
    [CCode (cname = "colourtrans_GIVEN_SPRITE")]
    GivenSprite,
    [CCode (cname = "colourtrans_CURRENT_IF_ABSENT")]
    CurrentIfAbsent,
    [CCode (cname = "colourtrans_GIVEN_TRANSFER_FN")]
    GivenTransferFn,
    [CCode (cname = "colourtrans_RETURN_GCOL_LIST")]
    ReturnGCOLList,
    [CCode (cname = "colourtrans_RETURN_WIDE_ENTRIES")]
    ReturnWideEntries,
    [CCode (cname = "colourtrans_RETURN_PALETTE_TABLE")]
    ReturnPaletteTable
  }

  [CCode (cname = "colourtrans_calibration_flags")]
  public enum CalibrationFlags
  {
    [CCode (cname = "colourtrans_WRITE_DEFAULT_CALIBRATION")]
    WriteDefaultCalibration
  }

  [CCode (cname = "colourtrans_palette_flags")]
  public enum PaletteFlags
  {
    [CCode (cname = "colourtrans_PALETTE_FOR_SPRITE")]
    PaletteForSprite,
    [CCode (cname = "colourtrans_FLASHING_PALETTE")]
    FlashingPalette
  }

  [CCode (cname = "colourtrans_colour_flags")]
  public enum ColourFlags
  {
    [CCode (cname = "colourtrans_SET_FG_COL")]
    SetFGCol,
    [CCode (cname = "colourtrans_SET_BG_COL")]
    SetBGCol,
    [CCode (cname = "colourtrans_SET_TEXT_COL")]
    SetTextCol
  }

  [CCode (cname = "colourtrans_text_colour_flags")]
  public enum TextColourFlags
  {
    [CCode (cname = "colourtrans_SET_FG_TEXT_COL")]
    SetFGTextCol,
    [CCode (cname = "colourtrans_SET_BG_TEXT_COL")]
    SetBGTextCol
  }

  [CCode (cname = "colourtrans_gcol_list")]
  public struct GCOLList
  {
    [CCode (cname = "gcol", array_length = false)]
    public unowned OS.GCOL[] Gcol;
  }

  [CCode (cname = "colourtrans_calibration_table_base")]
  public struct CalibrationTableBase
  {
    [CCode (cname = "rcount")]
    public unowned int Rcount;
    [CCode (cname = "gcount")]
    public unowned int Gcount;
    [CCode (cname = "bcount")]
    public unowned int Bcount;
  }

  [Compact]
  [CCode (cname = "colourtrans_calibration_table", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class CalibrationTable
  {
    public CalibrationTable(int numEntries);
    [CCode (cname = "rcount")]
    public unowned int Rcount;
    [CCode (cname = "gcount")]
    public unowned int Gcount;
    [CCode (cname = "bcount")]
    public unowned int Bcount;
    [CCode (cname = "standard_colour", array_length = false)]
    public unowned OS.Colour[] StandardColour;
  }

  [CCode (cname="colourtrans_COLOUR_RANGE")]
  public const int ColourRange;
  [CCode (cname="colourtrans_CURRENT_PALETTE")]
  public static OS.Palette* CurrentPalette;
  [CCode (cname="colourtrans_DEFAULT_PALETTE")]
  public static OS.Palette* DefaultPalette;
  [CCode (cname="error_COLOUR_TRANS_BAD_CALIB")]
  public const bits Error_ColourTransBadCalib;
  [CCode (cname="error_COLOUR_TRANS_CONV_OVER")]
  public const bits Error_ColourTransConvOver;
  [CCode (cname="error_COLOUR_TRANS_BAD_HSV")]
  public const bits Error_ColourTransBadHSV;
  [CCode (cname="error_COLOUR_TRANS_SWITCHED")]
  public const bits Error_ColourTransSwitched;
  [CCode (cname="error_COLOUR_TRANS_BAD_MISC_OP")]
  public const bits Error_ColourTransBadMiscOp;
  [CCode (cname="error_COLOUR_TRANS_BAD_FLAGS")]
  public const bits Error_ColourTransBadFlags;
  [CCode (cname="error_COLOUR_TRANS_BUFF_OVER")]
  public const bits Error_ColourTransBuffOver;
  [CCode (cname="error_COLOUR_TRANS_BAD_DEPTH")]
  public const bits Error_ColourTransBadDepth;

  [CCode (cname = "xcolourtrans_select_table")]
  public static OS.Error* XSelectTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "colourtrans_select_table")]
  public static void SelectTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "xcolourtrans_select_table_for_sprite")]
  public static OS.Error* XSelectTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags);

  [CCode (cname = "colourtrans_select_table_for_sprite")]
  public static void SelectTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags);

  [CCode (cname = "xcolourtrans_select_gcol_table")]
  public static OS.Error* XSelectGCOLTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     GCOLList gcol_list);

  [CCode (cname = "colourtrans_select_gcol_table")]
  public static void SelectGCOLTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     GCOLList gcol_list);

  [CCode (cname = "xcolourtrans_select_gcol_table_for_sprite")]
  public static OS.Error* XSelectGCOLTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     GCOLList gcol_list,
     TableFlags flags);

  [CCode (cname = "colourtrans_select_gcol_table_for_sprite")]
  public static void SelectGCOLTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     GCOLList gcol_list,
     TableFlags flags);

  [CCode (cname = "xcolourtrans_return_gcol")]
  public static OS.Error* XReturnGCOL
    (OS.Colour colour,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_return_gcol")]
  public static OS.GCOL ReturnGCOL
    (OS.Colour colour);

  [CCode (cname = "xcolourtrans_set_gcol")]
  public static OS.Error* XSetGCOL
    (OS.Colour colour,
     GCOLFlags flags,
     OS.Action action,
     out OS.GCOL gcol,
     out int log2_bpp);

  [CCode (cname = "colourtrans_set_gcol")]
  public static OS.GCOL SetGCOL
    (OS.Colour colour,
     GCOLFlags flags,
     OS.Action action,
     out int log2_bpp);

  [CCode (cname = "xcolourtrans_return_colour_number")]
  public static OS.Error* XReturnColourNumber
    (OS.Colour colour,
     out OS.ColourNumber colour_number);

  [CCode (cname = "colourtrans_return_colour_number")]
  public static OS.ColourNumber ReturnColourNumber
    (OS.Colour colour);

  [CCode (cname = "xcolourtrans_return_gcol_for_mode")]
  public static OS.Error* XReturnGCOLForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_return_gcol_for_mode")]
  public static OS.GCOL ReturnGCOLForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette);

  [CCode (cname = "xcolourtrans_return_colour_number_for_mode")]
  public static OS.Error* XReturnColourNumberForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette,
     out OS.ColourNumber colour_number);

  [CCode (cname = "colourtrans_return_colour_number_for_mode")]
  public static OS.ColourNumber ReturnColourNumberForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette);

  [CCode (cname = "xcolourtrans_return_opp_gcol")]
  public static OS.Error* XReturnOppGCOL
    (OS.Colour colour,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_return_opp_gcol")]
  public static OS.GCOL ReturnOppGCOL
    (OS.Colour colour);

  [CCode (cname = "xcolourtrans_set_opp_gcol")]
  public static OS.Error* XSetOppGCOL
    (OS.Colour colour,
     GCOLFlags flags,
     OS.Action action,
     out OS.GCOL gcol,
     out int log2_bpp);

  [CCode (cname = "colourtrans_set_opp_gcol")]
  public static OS.GCOL SetOppGCOL
    (OS.Colour colour,
     GCOLFlags flags,
     OS.Action action,
     out int log2_bpp);

  [CCode (cname = "xcolourtrans_return_opp_colour_number")]
  public static OS.Error* XReturnOppColourNumber
    (OS.Colour colour,
     out OS.ColourNumber colour_number);

  [CCode (cname = "colourtrans_return_opp_colour_number")]
  public static OS.ColourNumber ReturnOppColourNumber
    (OS.Colour colour);

  [CCode (cname = "xcolourtrans_return_opp_gcol_for_mode")]
  public static OS.Error* XReturnOppGCOLForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_return_opp_gcol_for_mode")]
  public static OS.GCOL ReturnOppGCOLForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette);

  [CCode (cname = "xcolourtrans_return_opp_colour_number_for_mode")]
  public static OS.Error* XReturnOppColourNumberForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette,
     out OS.ColourNumber colour_number);

  [CCode (cname = "colourtrans_return_opp_colour_number_for_mode")]
  public static OS.ColourNumber ReturnOppColourNumberForMode
    (OS.Colour colour,
     OS.Mode mode,
     OS.Palette dest_palette);

  [CCode (cname = "xcolourtrans_gcol_to_colour_number")]
  public static OS.Error* XGCOLToColourNumber
    (OS.GCOL gcol,
     out OS.ColourNumber colour_number);

  [CCode (cname = "colourtrans_gcol_to_colour_number")]
  public static OS.ColourNumber GCOLToColourNumber
    (OS.GCOL gcol);

  [CCode (cname = "xcolourtrans_colour_number_to_gcol")]
  public static OS.Error* XColourNumberToGCOL
    (OS.ColourNumber colour_number,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_colour_number_to_gcol")]
  public static OS.GCOL ColourNumberToGCOL
    (OS.ColourNumber colour_number);

  [CCode (cname = "xcolourtrans_return_font_colours")]
  public static OS.Error* XReturnFontColours
    (Font.F font,
     OS.Colour bg_hint,
     OS.Colour fill,
     int offset,
     out OS.Colour bg_hint_out,
     out OS.Colour fill_out,
     out int offset_out);

  [CCode (cname = "colourtrans_return_font_colours")]
  public static void ReturnFontColours
    (Font.F font,
     OS.Colour bg_hint,
     OS.Colour fill,
     int offset,
     out OS.Colour bg_hint_out,
     out OS.Colour fill_out,
     out int offset_out);

  [CCode (cname = "xcolourtrans_set_font_colours")]
  public static OS.Error* XSetFontColours
    (Font.F font,
     OS.Colour bg_hint,
     OS.Colour fill,
     int offset,
     out OS.Colour bg_hint_out,
     out OS.Colour fill_out,
     out int offset_out);

  [CCode (cname = "colourtrans_set_font_colours")]
  public static void SetFontColours
    (Font.F font,
     OS.Colour bg_hint,
     OS.Colour fill,
     int offset,
     out OS.Colour bg_hint_out,
     out OS.Colour fill_out,
     out int offset_out);

  [CCode (cname = "xcolourtrans_invalidate_cache")]
  public static OS.Error* XInvalidateCache ();

  [CCode (cname = "colourtrans_invalidate_cache")]
  public static void InvalidateCache ();

  [CCode (cname = "xcolourtrans_set_calibration")]
  public static OS.Error* XSetCalibration
    (CalibrationTable calibration_table);

  [CCode (cname = "colourtrans_set_calibration")]
  public static void SetCalibration
    (CalibrationTable calibration_table);

  [CCode (cname = "xcolourtrans_read_calibration")]
  public static OS.Error* XReadCalibration
    (CalibrationTable calibration_table,
     out int size);

  [CCode (cname = "colourtrans_read_calibration")]
  public static int ReadCalibration
    (CalibrationTable calibration_table);

  [CCode (cname = "xcolourtrans_convert_device_colour")]
  public static OS.Error* XConvertDeviceColour
    (OS.Colour colour,
     CalibrationTable calibration_table,
     out OS.Colour colour_out);

  [CCode (cname = "colourtrans_convert_device_colour")]
  public static OS.Colour ConvertDeviceColour
    (OS.Colour colour,
     CalibrationTable calibration_table);

  [CCode (cname = "xcolourtrans_convert_device_palette")]
  public static OS.Error* XConvertDevicePalette
    (int colour_count,
     OS.Colour list,
     OS.Colour buffer,
     CalibrationTable calibration_table);

  [CCode (cname = "colourtrans_convert_device_palette")]
  public static void ConvertDevicePalette
    (int colour_count,
     OS.Colour list,
     OS.Colour buffer,
     CalibrationTable calibration_table);

  [CCode (cname = "xcolourtrans_convert_rgb_to_cie")]
  public static OS.Error* XConvertRGBToCIE
    (int r,
     int g,
     int b,
     out int x,
     out int y,
     out int z);

  [CCode (cname = "colourtrans_convert_rgb_to_cie")]
  public static void ConvertRGBToCIE
    (int r,
     int g,
     int b,
     out int x,
     out int y,
     out int z);

  [CCode (cname = "xcolourtrans_convert_cie_to_rgb")]
  public static OS.Error* XConvertCIEToRGB
    (int x,
     int y,
     int z,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "colourtrans_convert_cie_to_rgb")]
  public static void ConvertCIEToRGB
    (int x,
     int y,
     int z,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "xcolourtrans_write_calibration_to_file")]
  public static OS.Error* XWriteCalibrationToFile
    (CalibrationFlags flags,
     OS.F file);

  [CCode (cname = "colourtrans_write_calibration_to_file")]
  public static void WriteCalibrationToFile
    (CalibrationFlags flags,
     OS.F file);

  [CCode (cname = "xcolourtrans_write_calibration_to_filew")]
  public static OS.Error* XWriteCalibrationToFileW
    (CalibrationFlags flags,
     OS.FW file);

  [CCode (cname = "colourtrans_write_calibration_to_filew")]
  public static void WriteCalibrationToFileW
    (CalibrationFlags flags,
     OS.FW file);

  [CCode (cname = "xcolourtrans_convert_rgb_to_hsv")]
  public static OS.Error* XConvertRGBToHSV
    (int r,
     int g,
     int b,
     out int h,
     out int s,
     out int v);

  [CCode (cname = "colourtrans_convert_rgb_to_hsv")]
  public static void ConvertRGBToHSV
    (int r,
     int g,
     int b,
     out int h,
     out int s,
     out int v);

  [CCode (cname = "xcolourtrans_convert_hsv_to_rgb")]
  public static OS.Error* XConvertHSVToRGB
    (int h,
     int s,
     int v,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "colourtrans_convert_hsv_to_rgb")]
  public static void ConvertHSVToRGB
    (int h,
     int s,
     int v,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "xcolourtrans_convert_rgb_to_cmyk")]
  public static OS.Error* XConvertRGBToCMYK
    (int r,
     int g,
     int b,
     out int c,
     out int m,
     out int y,
     out int k);

  [CCode (cname = "colourtrans_convert_rgb_to_cmyk")]
  public static void ConvertRGBToCMYK
    (int r,
     int g,
     int b,
     out int c,
     out int m,
     out int y,
     out int k);

  [CCode (cname = "xcolourtrans_convert_cmyk_to_rgb")]
  public static OS.Error* XConvertCMYKToRGB
    (int c,
     int m,
     int y,
     int k,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "colourtrans_convert_cmyk_to_rgb")]
  public static void ConvertCMYKToRGB
    (int c,
     int m,
     int y,
     int k,
     out int r,
     out int g,
     out int b);

  [CCode (cname = "xcolourtrans_read_palette")]
  public static OS.Error* XReadPalette
    (OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Palette palette,
     int size,
     PaletteFlags flags,
     out int spare);

  [CCode (cname = "colourtrans_read_palette")]
  public static int ReadPalette
    (OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Palette palette,
     int size,
     PaletteFlags flags);

  [CCode (cname = "xcolourtrans_write_palette")]
  public static OS.Error* XWritePalette
    (OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Palette palette,
     PaletteFlags flags);

  [CCode (cname = "colourtrans_write_palette")]
  public static void WritePalette
    (OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Palette palette,
     PaletteFlags flags);

  [CCode (cname = "xcolourtrans_set_colour")]
  public static OS.Error* XSetColour
    (OS.GCOL gcol,
     ColourFlags flags,
     OS.Action action);

  [CCode (cname = "colourtrans_set_colour")]
  public static void SetColour
    (OS.GCOL gcol,
     ColourFlags flags,
     OS.Action action);

  [CCode (cname = "xcolourtransmiscop_set_weights")]
  public static OS.Error* XColourTransMiscOp_SetWeights
    (int rweight,
     int gweight,
     int bweight);

  [CCode (cname = "colourtransmiscop_set_weights")]
  public static void ColourTransMiscOp_SetWeights
    (int rweight,
     int gweight,
     int bweight);

  [CCode (cname = "xcolourtrans_write_loadings_to_file")]
  public static OS.Error* XWriteLoadingsToFile
    (OS.F file);

  [CCode (cname = "colourtrans_write_loadings_to_file")]
  public static void WriteLoadingsToFile
    (OS.F file);

  [CCode (cname = "xcolourtrans_write_loadings_to_filew")]
  public static OS.Error* XWriteLoadingsToFileW
    (OS.FW file);

  [CCode (cname = "colourtrans_write_loadings_to_filew")]
  public static void WriteLoadingsToFileW
    (OS.FW file);

  [CCode (cname = "xcolourtrans_set_text_colour")]
  public static OS.Error* XSetTextColour
    (OS.Colour colour,
     TextColourFlags flags,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_set_text_colour")]
  public static OS.GCOL SetTextColour
    (OS.Colour colour,
     TextColourFlags flags);

  [CCode (cname = "xcolourtrans_set_opp_text_colour")]
  public static OS.Error* XSetOppTextColour
    (OS.Colour colour,
     TextColourFlags flags,
     out OS.GCOL gcol);

  [CCode (cname = "colourtrans_set_opp_text_colour")]
  public static OS.GCOL SetOppTextColour
    (OS.Colour colour,
     TextColourFlags flags);

  [CCode (cname = "xcolourtrans_generate_table")]
  public static OS.Error* XGenerateTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags,
     char* workspace,
     AsmRoutine transfer_fn,
     out int size);

  [CCode (cname = "colourtrans_generate_table")]
  public static int GenerateTable
    (OS.Mode source_mode,
     OS.Palette source_palette,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags,
     char* workspace,
     AsmRoutine transfer_fn);

  [CCode (cname = "xcolourtrans_generate_table_for_sprite")]
  public static OS.Error* XGenerateTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags,
     char* workspace,
     AsmRoutine transfer_fn,
     out int size);

  [CCode (cname = "colourtrans_generate_table_for_sprite")]
  public static int GenerateTableForSprite
    (OSSpriteOp.Area source_area,
     OSSpriteOp.Id source_id,
     OS.Mode dest_mode,
     OS.Palette dest_palette,
     OSSpriteOp.TransTab trans_tab,
     TableFlags flags,
     char* workspace,
     AsmRoutine transfer_fn);

  [CCode (cname = "xservice_calibration_changed")]
  public static OS.Error* XService_CalibrationChanged ();

  [CCode (cname = "service_calibration_changed")]
  public static void Service_CalibrationChanged ();

  [CCode (cname = "xservice_invalidate_cache")]
  public static OS.Error* XService_InvalidateCache ();

  [CCode (cname = "service_invalidate_cache")]
  public static void Service_InvalidateCache ();

}
