/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSArgs;
using OSLib.Buffer;

[CCode (cheader_filename = "oslib/devicefs.h")]
namespace OSLib.DeviceFS
{
  [CCode (cname = "devicefs_flags")]
  public enum Flags
  {
    [CCode (cname = "devicefs_DEVICE_BUFFERED")]
    DeviceBuffered,
    [CCode (cname = "devicefs_DEVICE_USES_PATH_VAR")]
    DeviceUsesPathVar
  }

  [CCode (cname = "devicefs_d", has_type_id = false)]
  public struct D : bits
  {
  }

  [CCode (cname = "devicefs_s", has_type_id = false)]
  public struct S : bits
  {
  }

  [CCode (cname = "devicefs_h", has_type_id = false)]
  public struct H : bits
  {
  }

  [CCode (cname = "devicefs_device")]
  public struct Device
  {
    [CCode (cname = "name_offset")]
    public unowned int NameOffset;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "rx_flags")]
    public unowned bits RxFlags;
    [CCode (cname = "rx_buffer_size")]
    public unowned int RxBufferSize;
    [CCode (cname = "tx_flags")]
    public unowned bits TxFlags;
    [CCode (cname = "tx_buffer_size")]
    public unowned int TxBufferSize;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
  }

  [CCode (cname = "devicefs_device_list")]
  public struct DeviceList
  {
    [CCode (cname = "d", array_length = false)]
    public unowned Device[] D;
  }

  [CCode (cname = "devicefs_register_flags")]
  public enum RegisterFlags
  {
    [CCode (cname = "devicefs_REGISTER_BLOCK_DEVICE")]
    RegisterBlockDevice,
    [CCode (cname = "devicefs_REGISTER_FULL_DUPLEX")]
    RegisterFullDuplex
  }

  [CCode (cname = "devicefs_initialise_flags")]
  public enum InitialiseFlags
  {
    [CCode (cname = "devicefs_OPEN_FOR_RX")]
    OpenForRx,
    [CCode (cname = "devicefs_OPEN_FOR_TX")]
    OpenForTx
  }


  [CCode (cname = "xdevicefs_register")]
  public static OS.Error* XRegister
    (RegisterFlags flags,
     DeviceList devices,
     AsmRoutine driver,
     int handle,
     char* workspace,
     string validation,
     int rx_limit,
     int tx_limit,
     out D d);

  [CCode (cname = "devicefs_register")]
  public static D Register
    (RegisterFlags flags,
     DeviceList devices,
     AsmRoutine driver,
     int handle,
     char* workspace,
     string validation,
     int rx_limit,
     int tx_limit);

  [CCode (cname = "xdevicefs_deregister")]
  public static OS.Error* XDeregister
    (D d);

  [CCode (cname = "devicefs_deregister")]
  public static void Deregister
    (D d);

  [CCode (cname = "xdevicefs_register_objects")]
  public static OS.Error* XRegisterObjects
    (D d,
     DeviceList devices);

  [CCode (cname = "devicefs_register_objects")]
  public static void RegisterObjects
    (D d,
     DeviceList devices);

  [CCode (cname = "xdevicefs_deregister_objects")]
  public static OS.Error* XDeregisterObjects
    (D d,
     string device_name);

  [CCode (cname = "devicefs_deregister_objects")]
  public static void DeregisterObjects
    (D d,
     string device_name);

  [CCode (cname = "xdevicefscalldevice_initialise")]
  public static OS.Error* XDeviceFSCallDevice_Initialise
    (D d,
     S s,
     InitialiseFlags flags,
     char* decoded_special,
     out H h);

  [CCode (cname = "devicefscalldevice_initialise")]
  public static H DeviceFSCallDevice_Initialise
    (D d,
     S s,
     InitialiseFlags flags,
     char* decoded_special);

  [CCode (cname = "xdevicefscalldevice_terminate")]
  public static OS.Error* XDeviceFSCallDevice_Terminate
    (D d,
     H h);

  [CCode (cname = "devicefscalldevice_terminate")]
  public static void DeviceFSCallDevice_Terminate
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_tx_wake_up")]
  public static OS.Error* XDeviceFSCallDevice_TxWakeUp
    (D d,
     H h,
     out bool woken_up);

  [CCode (cname = "devicefscalldevice_tx_wake_up")]
  public static bool DeviceFSCallDevice_TxWakeUp
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_rx_wake_up")]
  public static OS.Error* XDeviceFSCallDevice_RxWakeUp
    (D d,
     H h);

  [CCode (cname = "devicefscalldevice_rx_wake_up")]
  public static void DeviceFSCallDevice_RxWakeUp
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_rx_sleep")]
  public static OS.Error* XDeviceFSCallDevice_RxSleep
    (D d,
     H h);

  [CCode (cname = "devicefscalldevice_rx_sleep")]
  public static void DeviceFSCallDevice_RxSleep
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_enum_dir")]
  public static OS.Error* XDeviceFSCallDevice_EnumDir
    (D d,
     string file_name);

  [CCode (cname = "devicefscalldevice_enum_dir")]
  public static void DeviceFSCallDevice_EnumDir
    (D d,
     string file_name);

  [CCode (cname = "xdevicefscalldevice_tx_create_buffer")]
  public static OS.Error* XDeviceFSCallDevice_TxCreateBuffer
    (D d,
     H h,
     Buffer.Flags flags,
     int size,
     Buffer.B b,
     int threshold,
     out Buffer.Flags flags_out,
     out int size_out,
     out Buffer.B b_out,
     out int threshold_out);

  [CCode (cname = "devicefscalldevice_tx_create_buffer")]
  public static Buffer.B DeviceFSCallDevice_TxCreateBuffer
    (D d,
     H h,
     Buffer.Flags flags,
     int size,
     Buffer.B b,
     int threshold,
     out Buffer.Flags flags_out,
     out int size_out,
     out int threshold_out);

  [CCode (cname = "xdevicefscalldevice_rx_create_buffer")]
  public static OS.Error* XDeviceFSCallDevice_RxCreateBuffer
    (D d,
     H h,
     Buffer.Flags flags,
     int size,
     Buffer.B b,
     int threshold,
     out Buffer.Flags flags_out,
     out int size_out,
     out Buffer.B b_out,
     out int threshold_out);

  [CCode (cname = "devicefscalldevice_rx_create_buffer")]
  public static Buffer.B DeviceFSCallDevice_RxCreateBuffer
    (D d,
     H h,
     Buffer.Flags flags,
     int size,
     Buffer.B b,
     int threshold,
     out Buffer.Flags flags_out,
     out int size_out,
     out int threshold_out);

  [CCode (cname = "xdevicefscalldevice_halt")]
  public static OS.Error* XDeviceFSCallDevice_Halt
    (D d,
     H h);

  [CCode (cname = "devicefscalldevice_halt")]
  public static void DeviceFSCallDevice_Halt
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_resume")]
  public static OS.Error* XDeviceFSCallDevice_Resume
    (D d,
     H h);

  [CCode (cname = "devicefscalldevice_resume")]
  public static void DeviceFSCallDevice_Resume
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_end_of_data")]
  public static OS.Error* XDeviceFSCallDevice_EndOfData
    (D d,
     H h,
     out bool end_of_data);

  [CCode (cname = "devicefscalldevice_end_of_data")]
  public static bool DeviceFSCallDevice_EndOfData
    (D d,
     H h);

  [CCode (cname = "xdevicefscalldevice_stream_created")]
  public static OS.Error* XDeviceFSCallDevice_StreamCreated
    (D d,
     H h,
     Buffer.B b);

  [CCode (cname = "devicefscalldevice_stream_created")]
  public static void DeviceFSCallDevice_StreamCreated
    (D d,
     H h,
     Buffer.B b);

  [CCode (cname = "xdevicefscalldevice_io_ctl")]
  public static OS.Error* XDeviceFSCallDevice_IOCtl
    (D d,
     H h,
     OSArgs.IOCtlParameters parameters);

  [CCode (cname = "devicefscalldevice_io_ctl")]
  public static void DeviceFSCallDevice_IOCtl
    (D d,
     H h,
     OSArgs.IOCtlParameters parameters);

  [CCode (cname = "xdevicefs_threshold")]
  public static OS.Error* XThreshold
    (S s,
     int threshold);

  [CCode (cname = "devicefs_threshold")]
  public static void Threshold
    (S s,
     int threshold);

  [CCode (cname = "xdevicefs_received_char")]
  public static OS.Error* XReceivedChar
    (byte b,
     S s,
     out uint psr);

  [CCode (cname = "devicefs_received_char")]
  public static bits ReceivedChar
    (byte b,
     S s);

  [CCode (cname = "xdevicefs_transmit_char")]
  public static OS.Error* XTransmitChar
    (S s,
     out byte b,
     out uint psr);

  [CCode (cname = "devicefs_transmit_char")]
  public static bits TransmitChar
    (S s,
     out byte b);

  [CCode (cname = "xservice_device_fs_starting")]
  public static OS.Error* XService_DeviceFSStarting ();

  [CCode (cname = "service_device_fs_starting")]
  public static void Service_DeviceFSStarting ();

  [CCode (cname = "xservice_device_fs_dying")]
  public static OS.Error* XService_DeviceFSDying ();

  [CCode (cname = "service_device_fs_dying")]
  public static void Service_DeviceFSDying ();

  [CCode (cname = "xservice_device_dead")]
  public static OS.Error* XService_DeviceDead
    (D d,
     string device_name);

  [CCode (cname = "service_device_dead")]
  public static void Service_DeviceDead
    (D d,
     string device_name);

  [CCode (cname = "xservice_device_fs_close_request")]
  public static OS.Error* XService_DeviceFSCloseRequest
    (OS.F file,
     out bool unclaimed);

  [CCode (cname = "service_device_fs_close_request")]
  public static bool Service_DeviceFSCloseRequest
    (OS.F file);

  [CCode (cname = "xservice_device_fs_close_requestw")]
  public static OS.Error* XService_DeviceFSCloseRequestW
    (OS.FW file,
     out bool unclaimed);

  [CCode (cname = "service_device_fs_close_requestw")]
  public static bool Service_DeviceFSCloseRequestW
    (OS.FW file);

  [CCode (cname = "xupcall_stream_created")]
  public static OS.Error* XUpCall_StreamCreated
    (D d,
     bool tx_stream,
     OS.F file,
     S s);

  [CCode (cname = "upcall_stream_created")]
  public static void UpCall_StreamCreated
    (D d,
     bool tx_stream,
     OS.F file,
     S s);

  [CCode (cname = "xupcall_stream_createdw")]
  public static OS.Error* XUpCall_StreamCreatedW
    (D d,
     bool tx_stream,
     OS.FW file,
     S s);

  [CCode (cname = "upcall_stream_createdw")]
  public static void UpCall_StreamCreatedW
    (D d,
     bool tx_stream,
     OS.FW file,
     S s);

  [CCode (cname = "xupcall_stream_closed")]
  public static OS.Error* XUpCall_StreamClosed
    (D d,
     bool tx_stream,
     OS.F file,
     S s);

  [CCode (cname = "upcall_stream_closed")]
  public static void UpCall_StreamClosed
    (D d,
     bool tx_stream,
     OS.F file,
     S s);

  [CCode (cname = "xupcall_stream_closedw")]
  public static OS.Error* XUpCall_StreamClosedW
    (D d,
     bool tx_stream,
     OS.FW file,
     S s);

  [CCode (cname = "upcall_stream_closedw")]
  public static void UpCall_StreamClosedW
    (D d,
     bool tx_stream,
     OS.FW file,
     S s);

  [CCode (cname = "xupcall_device_rx_data_present")]
  public static OS.Error* XUpCall_DeviceRxDataPresent
    (D d);

  [CCode (cname = "upcall_device_rx_data_present")]
  public static void UpCall_DeviceRxDataPresent
    (D d);

  [CCode (cname = "xupcall_device_thres_above")]
  public static OS.Error* XUpCall_DeviceThresAbove
    (D d);

  [CCode (cname = "upcall_device_thres_above")]
  public static void UpCall_DeviceThresAbove
    (D d);

  [CCode (cname = "xupcall_device_thres_below")]
  public static OS.Error* XUpCall_DeviceThresBelow
    (D d);

  [CCode (cname = "upcall_device_thres_below")]
  public static void UpCall_DeviceThresBelow
    (D d);

  [CCode (cname = "xevent_device_overrun")]
  public static OS.Error* XEvent_DeviceOverrun
    (D d,
     OS.F file);

  [CCode (cname = "event_device_overrun")]
  public static void Event_DeviceOverrun
    (D d,
     OS.F file);

  [CCode (cname = "xevent_device_overrunw")]
  public static OS.Error* XEvent_DeviceOverrunW
    (D d,
     OS.FW file);

  [CCode (cname = "event_device_overrunw")]
  public static void Event_DeviceOverrunW
    (D d,
     OS.FW file);

}
