/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;

[CCode (cheader_filename = "oslib/extedit.h")]
namespace OSLib.ExtEdit
{
  [CCode (cname = "extedit_data_type")]
  public enum DataType
  {
    [CCode (cname = "extedit_FILE_TYPE")]
    FileType,
    [CCode (cname = "extedit_SUB_TYPE")]
    SubType
  }

  [CCode (cname = "extedit_job_handle")]
  public enum JobHandle
  {
    [CCode (cname = "extedit_CLIENT_HANDLE")]
    ClientHandle,
    [CCode (cname = "extedit_EDITOR_HANDLE")]
    EditorHandle
  }

  [CCode (cname = "extedit_flags")]
  public enum Flags
  {
    [CCode (cname = "extedit_CONTINUE_EDITING")]
    ContinueEditing,
    [CCode (cname = "extedit_SELECTION_ONLY")]
    SelectionOnly,
    [CCode (cname = "extedit_READ_ONLY")]
    ReadOnly,
    [CCode (cname = "extedit_IMMEDIATE_EXECUTION")]
    ImmediateExecution,
    [CCode (cname = "extedit_ADJUST_SELECTION")]
    AdjustSelection
  }

  [CCode (cname = "extedit_message_rq")]
  public struct MessageRq
  {
    [CCode (cname = "data_type")]
    public unowned DataType DataType;
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "parent_name")]
    public unowned char ParentName[20];
    [CCode (cname = "leaf_name")]
    public unowned char LeafName[204];
  }

  [CCode (cname = "extedit_message_ack")]
  public struct MessageAck
  {
    [CCode (cname = "data_type")]
    public unowned DataType DataType;
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
  }

  [CCode (cname = "extedit_message_return")]
  public struct MessageReturn
  {
    [CCode (cname = "data_type")]
    public unowned DataType DataType;
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
  }

  [CCode (cname = "extedit_message_abort")]
  public struct MessageAbort
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[4];
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
  }

  [CCode (cname = "extedit_message_data_save")]
  public struct MessageDataSave
  {
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[12];
    [CCode (cname = "est_size")]
    public unowned int EstSize;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "extedit_message_cursor")]
  public struct MessageCursor
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[4];
    [CCode (cname = "job_handle")]
    public unowned JobHandle JobHandle;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "new_cursor_pos")]
    public unowned int NewCursorPos;
    [CCode (cname = "new_sos_pos")]
    public unowned int NewSosPos;
    [CCode (cname = "new_eos_pos")]
    public unowned int NewEosPos;
    [CCode (cname = "old_cursor_pos")]
    public unowned int OldCursorPos;
    [CCode (cname = "old_sos_pos")]
    public unowned int OldSosPos;
    [CCode (cname = "old_eos_pos")]
    public unowned int OldEosPos;
  }

  [CCode (cname="message_EXT_EDIT_RQ")]
  public const bits Message_ExtEditRq;
  [CCode (cname="message_EXT_EDIT_ACK")]
  public const bits Message_ExtEditAck;
  [CCode (cname="message_EXT_EDIT_RETURN")]
  public const bits Message_ExtEditReturn;
  [CCode (cname="message_EXT_EDIT_ABORT")]
  public const bits Message_ExtEditAbort;
  [CCode (cname="message_EXT_EDIT_DATA_SAVE")]
  public const bits Message_ExtEditDataSave;
  [CCode (cname="message_EXT_EDIT_CURSOR")]
  public const bits Message_ExtEditCursor;
  [CCode (cname="extedit_SUB_TYPE_SHIFT")]
  public const int SubTypeShift;
  [CCode (cname="extedit_EDITOR_HANDLE_SHIFT")]
  public const int EditorHandleShift;
  [CCode (cname="extedit_BEFORE_START")]
  public const int BeforeStart;
  [CCode (cname="extedit_AFTER_END")]
  public const int AfterEnd;
  [CCode (cname="extedit_NO_CHANGE")]
  public const int NoChange;

}
