/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/filecore.h")]
namespace OSLib.FileCore
{
  [CCode (cname = "filecore_descriptor")]
  public struct Descriptor
  {
    [CCode (cname = "flags")]
    public unowned byte Flags[3];
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "title_offset")]
    public unowned int TitleOffset;
    [CCode (cname = "boot_text_offset")]
    public unowned int BootTextOffset;
    [CCode (cname = "disc_op_offset")]
    public unowned int DiscOpOffset;
    [CCode (cname = "misc_op_offset")]
    public unowned int MiscOpOffset;
  }

  [CCode (cname = "filecore_disc")]
  public struct Disc
  {
    [CCode (cname = "log2secsize")]
    public unowned byte Log2secsize;
    [CCode (cname = "secspertrack")]
    public unowned byte Secspertrack;
    [CCode (cname = "heads")]
    public unowned byte Heads;
    [CCode (cname = "density")]
    public unowned byte Density;
    [CCode (cname = "idlen")]
    public unowned byte Idlen;
    [CCode (cname = "log2bpmb")]
    public unowned byte Log2bpmb;
    [CCode (cname = "skew")]
    public unowned byte Skew;
    [CCode (cname = "bootoption")]
    public unowned byte Bootoption;
    [CCode (cname = "lowsector")]
    public unowned byte Lowsector;
    [CCode (cname = "nzones")]
    public unowned byte Nzones;
    [CCode (cname = "zone_spare")]
    public unowned short ZoneSpare;
    [CCode (cname = "root")]
    public unowned int Root;
    [CCode (cname = "disc_size")]
    public unowned int DiscSize;
    [CCode (cname = "disc_id")]
    public unowned short DiscId;
    [CCode (cname = "disc_name")]
    public unowned char DiscName[10];
    [CCode (cname = "disctype")]
    public unowned int Disctype;
    [CCode (cname = "disc_size_hi")]
    public unowned int DiscSizeHi;
    [CCode (cname = "share_size")]
    public unowned byte ShareSize;
    [CCode (cname = "big_flag")]
    public unowned byte BigFlag;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[22];
  }

  [CCode (cname = "filecore_disc_address")]
  public enum DiscAddress
  {
    Default = 0
  }

  [CCode (cname = "filecore_sector_disc_address")]
  public enum SectorDiscAddress
  {
    Default = 0
  }

  [CCode (cname = "filecore_disc_address64")]
  public struct DiscAddress64
  {
    [CCode (cname = "drive")]
    public unowned byte Drive;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "offset_low")]
    public unowned int OffsetLow;
    [CCode (cname = "offset_high")]
    public unowned int OffsetHigh;
  }

  [CCode (cname = "filecore_format")]
  public struct Format
  {
    [CCode (cname = "sector_size")]
    public unowned int SectorSize;
    [CCode (cname = "gap1_side0")]
    public unowned int Gap1Side0;
    [CCode (cname = "gap1_side1")]
    public unowned int Gap1Side1;
    [CCode (cname = "gap3")]
    public unowned int Gap3;
    [CCode (cname = "secspertrack")]
    public unowned byte Secspertrack;
    [CCode (cname = "density")]
    public unowned byte Density;
    [CCode (cname = "options")]
    public unowned byte Options;
    [CCode (cname = "lowsector")]
    public unowned byte Lowsector;
    [CCode (cname = "interleave")]
    public unowned byte Interleave;
    [CCode (cname = "side_skew")]
    public unowned byte SideSkew;
    [CCode (cname = "track_skew")]
    public unowned byte TrackSkew;
    [CCode (cname = "sector_fill")]
    public unowned byte SectorFill;
    [CCode (cname = "track_count")]
    public unowned int TrackCount;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[36];
  }

  [CCode (cname = "filecore_track_format_base")]
  public struct TrackFormatBase
  {
    [CCode (cname = "sector_size")]
    public unowned int SectorSize;
    [CCode (cname = "gap1_side0")]
    public unowned int Gap1Side0;
    [CCode (cname = "gap1_side1")]
    public unowned int Gap1Side1;
    [CCode (cname = "gap3")]
    public unowned int Gap3;
    [CCode (cname = "secspertrack")]
    public unowned byte Secspertrack;
    [CCode (cname = "density")]
    public unowned byte Density;
    [CCode (cname = "options")]
    public unowned byte Options;
    [CCode (cname = "sector_fill")]
    public unowned byte SectorFill;
    [CCode (cname = "track_count")]
    public unowned int TrackCount;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[12];
  }

  [Compact]
  [CCode (cname = "filecore_track_format", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class TrackFormat
  {
    public TrackFormat(int numEntries);
    [CCode (cname = "sector_size")]
    public unowned int SectorSize;
    [CCode (cname = "gap1_side0")]
    public unowned int Gap1Side0;
    [CCode (cname = "gap1_side1")]
    public unowned int Gap1Side1;
    [CCode (cname = "gap3")]
    public unowned int Gap3;
    [CCode (cname = "secspertrack")]
    public unowned byte Secspertrack;
    [CCode (cname = "density")]
    public unowned byte Density;
    [CCode (cname = "options")]
    public unowned byte Options;
    [CCode (cname = "sector_fill")]
    public unowned byte SectorFill;
    [CCode (cname = "track_count")]
    public unowned int TrackCount;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[12];
    [CCode (cname = "sectors", array_length = false)]
    public unowned bits[] Sectors;
  }

  [CCode (cname = "filecore_defect_list")]
  public struct DefectList
  {
    [CCode (cname = "defect", array_length = false)]
    public unowned int[] Defect;
  }

  [CCode (cname = "filecore_info")]
  public struct Info
  {
    [CCode (cname = "flags")]
    public unowned byte Flags[3];
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "title")]
    public unowned char* Title;
    [CCode (cname = "boot_text")]
    public unowned char* BootText;
    [CCode (cname = "disc_op")]
    public unowned AsmRoutine DiscOp;
    [CCode (cname = "misc_op")]
    public unowned AsmRoutine MiscOp;
  }

  [CCode (cname = "filecore_feature_flags")]
  public enum FeatureFlags
  {
    [CCode (cname = "filecore_FEATURE_NEW_STYLE_ERRORS")]
    FeatureNewStyleErrors
  }

  [CCode (cname="filecore_HARD_DISCS_NEED_FIQ")]
  public const bits HardDiscsNeedFIQ;
  [CCode (cname="filecore_FLOPPIES_NEED_FIQ")]
  public const bits FloppiesNeedFIQ;
  [CCode (cname="filecore_ADDRESS_BY_SECTOR")]
  public const bits AddressBySector;
  [CCode (cname="filecore_USE_SCRATCH_FOR_TEMPORARY")]
  public const bits UseScratchForTemporary;
  [CCode (cname="filecore_HARD_DISCS_CAN_MOUNT")]
  public const bits HardDiscsCanMount;
  [CCode (cname="filecore_HARD_DISCS_CAN_POLL_CHANGE")]
  public const bits HardDiscsCanPollChange;
  [CCode (cname="filecore_FLOPPIES_CAN_EJECT")]
  public const bits FloppiesCanEject;
  [CCode (cname="filecore_HARD_DISCS_CAN_EJECT")]
  public const bits HardDiscsCanEject;
  [CCode (cname="filecore_SUPPORTS_READ_DRIVE_FLAGS")]
  public const bits SupportsReadDriveFlags;
  [CCode (cname="filecore_USE_SECTOR_ADDRESSING")]
  public const bits UseSectorAddressing;
  [CCode (cname="filecore_RETURNS_NEW_STYLE_ERRORS")]
  public const bits ReturnsNewStyleErrors;
  [CCode (cname="filecore_LOW_SECTOR_SEQUENCE_SIDES")]
  public const bits LowSectorSequenceSides;
  [CCode (cname="filecore_LOW_SECTOR_DOUBLE_STEP")]
  public const bits LowSectorDoubleStep;
  [CCode (cname="filecore_DISC_ADDRESS_OFFSET")]
  public const bits DiscAddressOffset;
  [CCode (cname="filecore_DISC_ADDRESS_DRIVE")]
  public const bits DiscAddressDrive;
  [CCode (cname="filecore_DISC_ADDRESS_OFFSET_SHIFT")]
  public const int DiscAddressOffsetShift;
  [CCode (cname="filecore_DISC_ADDRESS_DRIVE_SHIFT")]
  public const int DiscAddressDriveShift;
  [CCode (cname="filecore_DISC_OP_GIVEN_ALTERNATIVE_DEFECT_LIST")]
  public const bits DiscOpGivenAlternativeDefectList;
  [CCode (cname="filecore_DISC_OP_GIVEN_SECTOR_LIST")]
  public const bits DiscOpGivenSectorList;
  [CCode (cname="filecore_DISC_OP_IGNORE_ESCAPE")]
  public const bits DiscOpIgnoreEscape;
  [CCode (cname="filecore_DISC_OP_IGNORE_TIMEOUT")]
  public const bits DiscOpIgnoreTimeout;
  [CCode (cname="filecore_DISC_OP_ALTERNATIVE_RECORD")]
  public const bits DiscOpAlternativeRecord;
  [CCode (cname="filecore_DISC_OP_ALTERNATIVE_RECORD_SHIFT")]
  public const int DiscOpAlternativeRecordShift;
  [CCode (cname="filecore_CREATE_FLOPPY_COUNT")]
  public const bits CreateFloppyCount;
  [CCode (cname="filecore_CREATE_FLOPPY_COUNT_SHIFT")]
  public const int CreateFloppyCountShift;
  [CCode (cname="filecore_CREATE_HARD_DISC_COUNT")]
  public const bits CreateHardDiscCount;
  [CCode (cname="filecore_CREATE_HARD_DISC_COUNT_SHIFT")]
  public const int CreateHardDiscCountShift;
  [CCode (cname="filecore_CREATE_DEFAULT_DRIVE")]
  public const bits CreateDefaultDrive;
  [CCode (cname="filecore_CREATE_DEFAULT_DRIVE_SHIFT")]
  public const int CreateDefaultDriveShift;
  [CCode (cname="filecore_CREATE_NO_DIR_STATE")]
  public const bits CreateNoDirState;
  [CCode (cname="filecore_FLOPPY_OLD_STRUCTURE")]
  public const bits FloppyOldStructure;
  [CCode (cname="filecore_FLOPPY_OLD_MAP")]
  public const bits FloppyOldMap;
  [CCode (cname="filecore_FORMATL")]
  public const int FormatL;
  [CCode (cname="filecore_FORMATD")]
  public const int FormatD;
  [CCode (cname="filecore_FORMATE")]
  public const int FormatE;
  [CCode (cname="filecore_FORMATF")]
  public const int FormatF;
  [CCode (cname="filecore_POLL_NOT_CHANGED")]
  public const bits PollNotChanged;
  [CCode (cname="filecore_POLL_MAY_BE_CHANGED")]
  public const bits PollMayBeChanged;
  [CCode (cname="filecore_POLL_CHANGED")]
  public const bits PollChanged;
  [CCode (cname="filecore_POLL_EMPTY")]
  public const bits PollEmpty;
  [CCode (cname="filecore_POLL_READY")]
  public const bits PollReady;
  [CCode (cname="filecore_POLL_FORTY_TRACK")]
  public const bits PollFortyTrack;
  [CCode (cname="filecore_POLL_EMPTY_WORKS")]
  public const bits PollEmptyWorks;
  [CCode (cname="filecore_POLL_CHANGED_WORKS")]
  public const bits PollChangedWorks;
  [CCode (cname="filecore_POLL_HIGH_DENSITY")]
  public const bits PollHighDensity;
  [CCode (cname="filecore_POLL_DENSITY_SENSE_WORKS")]
  public const bits PollDensitySenseWorks;
  [CCode (cname="filecore_POLL_READY_WORKS")]
  public const bits PollReadyWorks;

  [CCode (cname = "xfilecorediscop_verify")]
  public static OS.Error* XFileCoreDiscOp_Verify
    (bits flags,
     DiscAddress disc_addr,
     int size,
     char* instance,
     out DiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "filecorediscop_verify")]
  public static int FileCoreDiscOp_Verify
    (bits flags,
     DiscAddress disc_addr,
     int size,
     char* instance,
     out DiscAddress next_disc_addr);

  [CCode (cname = "xfilecorediscop_read_sectors")]
  public static OS.Error* XFileCoreDiscOp_ReadSectors
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "filecorediscop_read_sectors")]
  public static int FileCoreDiscOp_ReadSectors
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecorediscop_write_sectors")]
  public static OS.Error* XFileCoreDiscOp_WriteSectors
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "filecorediscop_write_sectors")]
  public static int FileCoreDiscOp_WriteSectors
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecorediscop_read_track")]
  public static OS.Error* XFileCoreDiscOp_ReadTrack
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop_read_track")]
  public static void FileCoreDiscOp_ReadTrack
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop_read_id")]
  public static OS.Error* XFileCoreDiscOp_ReadId
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop_read_id")]
  public static void FileCoreDiscOp_ReadId
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop_write_track")]
  public static OS.Error* XFileCoreDiscOp_WriteTrack
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop_write_track")]
  public static void FileCoreDiscOp_WriteTrack
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop_format_track")]
  public static OS.Error* XFileCoreDiscOp_FormatTrack
    (bits flags,
     DiscAddress disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "filecorediscop_format_track")]
  public static void FileCoreDiscOp_FormatTrack
    (bits flags,
     DiscAddress disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "xfilecorediscop_seek")]
  public static OS.Error* XFileCoreDiscOp_Seek
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop_seek")]
  public static void FileCoreDiscOp_Seek
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecorediscop_restore")]
  public static OS.Error* XFileCoreDiscOp_Restore
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop_restore")]
  public static void FileCoreDiscOp_Restore
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecorediscop_step_in")]
  public static OS.Error* XFileCoreDiscOp_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "filecorediscop_step_in")]
  public static void FileCoreDiscOp_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecorediscop_step_out")]
  public static OS.Error* XFileCoreDiscOp_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "filecorediscop_step_out")]
  public static void FileCoreDiscOp_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecorediscop_read_sectors_via_cache")]
  public static OS.Error* XFileCoreDiscOp_ReadSectorsViaCache
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "filecorediscop_read_sectors_via_cache")]
  public static int FileCoreDiscOp_ReadSectorsViaCache
    (bits flags,
     DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out DiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xfilecorediscop_specify")]
  public static OS.Error* XFileCoreDiscOp_Specify
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop_specify")]
  public static void FileCoreDiscOp_Specify
    (bits flags,
     DiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecore_create")]
  public static OS.Error* XCreate
    (Descriptor descriptor,
     char* module_base,
     char* word,
     bits options,
     int dir_cache_size,
     int file_cache_buffer_count,
     bits map_sizes,
     out char* instance,
     out AsmRoutine floppy_done,
     out AsmRoutine hard_disc_done,
     out AsmRoutine release_fiq);

  [CCode (cname = "filecore_create")]
  public static void Create
    (Descriptor descriptor,
     char* module_base,
     char* word,
     bits options,
     int dir_cache_size,
     int file_cache_buffer_count,
     bits map_sizes,
     out char* instance,
     out AsmRoutine floppy_done,
     out AsmRoutine hard_disc_done,
     out AsmRoutine release_fiq);

  [CCode (cname = "xfilecore_drives")]
  public static OS.Error* XDrives
    (char* instance,
     out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "filecore_drives")]
  public static void Drives
    (char* instance,
     out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "xfilecore_free_space")]
  public static OS.Error* XFreeSpace
    (string disc_name,
     char* instance,
     out int free,
     out int max);

  [CCode (cname = "filecore_free_space")]
  public static int FreeSpace
    (string disc_name,
     char* instance,
     out int max);

  [CCode (cname = "xfilecore_floppy_structure")]
  public static OS.Error* XFloppyStructure
    (char* buffer,
     Disc disc,
     bits flags,
     DefectList defect_list,
     out int used);

  [CCode (cname = "filecore_floppy_structure")]
  public static int FloppyStructure
    (char* buffer,
     Disc disc,
     bits flags,
     DefectList defect_list);

  [CCode (cname = "xfilecore_describe_disc")]
  public static OS.Error* XDescribeDisc
    (string disc_name,
     Disc disc,
     char* instance);

  [CCode (cname = "filecore_describe_disc")]
  public static void DescribeDisc
    (string disc_name,
     Disc disc,
     char* instance);

  [CCode (cname = "xfilecore_discard_read_sectors_cache")]
  public static OS.Error* XDiscardReadSectorsCache
    (int cache_handle);

  [CCode (cname = "filecore_discard_read_sectors_cache")]
  public static void DiscardReadSectorsCache
    (int cache_handle);

  [CCode (cname = "xfilecore_disc_format")]
  public static OS.Error* XDiscFormat
    (Format buffer,
     int vet_format_swi,
     int vet_format_handle,
     int format);

  [CCode (cname = "filecore_disc_format")]
  public static void DiscFormat
    (Format buffer,
     int vet_format_swi,
     int vet_format_handle,
     int format);

  [CCode (cname = "xfilecore_layout_structure")]
  public static OS.Error* XLayoutStructure
    (Disc disc,
     DefectList defect_list,
     string disc_name,
     OS.F image_file);

  [CCode (cname = "filecore_layout_structure")]
  public static void LayoutStructure
    (Disc disc,
     DefectList defect_list,
     string disc_name,
     OS.F image_file);

  [CCode (cname = "xfilecore_layout_structurew")]
  public static OS.Error* XLayoutStructureW
    (Disc disc,
     DefectList defect_list,
     string disc_name,
     OS.FW image_file);

  [CCode (cname = "filecore_layout_structurew")]
  public static void LayoutStructureW
    (Disc disc,
     DefectList defect_list,
     string disc_name,
     OS.FW image_file);

  [CCode (cname = "xfilecoremiscop_mount")]
  public static OS.Error* XFileCoreMiscOp_Mount
    (int drive_no,
     DiscAddress disc_addr,
     char* buffer,
     int size,
     Disc disc,
     char* instance);

  [CCode (cname = "filecoremiscop_mount")]
  public static void FileCoreMiscOp_Mount
    (int drive_no,
     DiscAddress disc_addr,
     char* buffer,
     int size,
     Disc disc,
     char* instance);

  [CCode (cname = "xfilecoremiscop_poll_changed")]
  public static OS.Error* XFileCoreMiscOp_PollChanged
    (int drive_no,
     int sequence_no,
     char* instance,
     out int sequence_no_out,
     out bits result);

  [CCode (cname = "filecoremiscop_poll_changed")]
  public static void FileCoreMiscOp_PollChanged
    (int drive_no,
     int sequence_no,
     char* instance,
     out int sequence_no_out,
     out bits result);

  [CCode (cname = "xfilecoremiscop_lock_drive")]
  public static OS.Error* XFileCoreMiscOp_LockDrive
    (int drive_no,
     char* instance);

  [CCode (cname = "filecoremiscop_lock_drive")]
  public static void FileCoreMiscOp_LockDrive
    (int drive_no,
     char* instance);

  [CCode (cname = "xfilecoremiscop_unlock_drive")]
  public static OS.Error* XFileCoreMiscOp_UnlockDrive
    (int drive_no,
     char* instance);

  [CCode (cname = "filecoremiscop_unlock_drive")]
  public static void FileCoreMiscOp_UnlockDrive
    (int drive_no,
     char* instance);

  [CCode (cname = "xfilecoremiscop_poll_period")]
  public static OS.Error* XFileCoreMiscOp_PollPeriod
    (string disc_name,
     char* instance,
     out int poll_period,
     out string media_type);

  [CCode (cname = "filecoremiscop_poll_period")]
  public static void FileCoreMiscOp_PollPeriod
    (string disc_name,
     char* instance,
     out int poll_period,
     out string media_type);

  [CCode (cname = "xfilecoremiscop_eject_disc")]
  public static OS.Error* XFileCoreMiscOp_EjectDisc
    (int drive_no,
     char* instance);

  [CCode (cname = "filecoremiscop_eject_disc")]
  public static void FileCoreMiscOp_EjectDisc
    (int drive_no,
     char* instance);

  [CCode (cname = "xfilecoremiscop_read_info")]
  public static OS.Error* XFileCoreMiscOp_ReadInfo
    (char* instance,
     out Info info);

  [CCode (cname = "filecoremiscop_read_info")]
  public static void FileCoreMiscOp_ReadInfo
    (char* instance,
     out Info info);

  [CCode (cname = "xfilecoremiscop_drive_status")]
  public static OS.Error* XFileCoreMiscOp_DriveStatus
    (int drive_no,
     char* instance,
     out bool locked);

  [CCode (cname = "filecoremiscop_drive_status")]
  public static bool FileCoreMiscOp_DriveStatus
    (int drive_no,
     char* instance);

  [CCode (cname = "xfilecoresectorop_verify")]
  public static OS.Error* XFileCoreSectorOp_Verify
    (bits flags,
     SectorDiscAddress disc_addr,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "filecoresectorop_verify")]
  public static int FileCoreSectorOp_Verify
    (bits flags,
     SectorDiscAddress disc_addr,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr);

  [CCode (cname = "xfilecoresectorop_read_sectors")]
  public static OS.Error* XFileCoreSectorOp_ReadSectors
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "filecoresectorop_read_sectors")]
  public static int FileCoreSectorOp_ReadSectors
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecoresectorop_write_sectors")]
  public static OS.Error* XFileCoreSectorOp_WriteSectors
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "filecoresectorop_write_sectors")]
  public static int FileCoreSectorOp_WriteSectors
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecoresectorop_read_track")]
  public static OS.Error* XFileCoreSectorOp_ReadTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecoresectorop_read_track")]
  public static void FileCoreSectorOp_ReadTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecoresectorop_read_id")]
  public static OS.Error* XFileCoreSectorOp_ReadId
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecoresectorop_read_id")]
  public static void FileCoreSectorOp_ReadId
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecoresectorop_write_track")]
  public static OS.Error* XFileCoreSectorOp_WriteTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecoresectorop_write_track")]
  public static void FileCoreSectorOp_WriteTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecoresectorop_format_track")]
  public static OS.Error* XFileCoreSectorOp_FormatTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "filecoresectorop_format_track")]
  public static void FileCoreSectorOp_FormatTrack
    (bits flags,
     SectorDiscAddress disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "xfilecoresectorop_seek")]
  public static OS.Error* XFileCoreSectorOp_Seek
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecoresectorop_seek")]
  public static void FileCoreSectorOp_Seek
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecoresectorop_restore")]
  public static OS.Error* XFileCoreSectorOp_Restore
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecoresectorop_restore")]
  public static void FileCoreSectorOp_Restore
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecoresectorop_step_in")]
  public static OS.Error* XFileCoreSectorOp_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "filecoresectorop_step_in")]
  public static void FileCoreSectorOp_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecoresectorop_step_out")]
  public static OS.Error* XFileCoreSectorOp_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "filecoresectorop_step_out")]
  public static void FileCoreSectorOp_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecoresectorop_read_sectors_via_cache")]
  public static OS.Error* XFileCoreSectorOp_ReadSectorsViaCache
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "filecoresectorop_read_sectors_via_cache")]
  public static int FileCoreSectorOp_ReadSectorsViaCache
    (bits flags,
     SectorDiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out SectorDiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xfilecoresectorop_specify")]
  public static OS.Error* XFileCoreSectorOp_Specify
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "filecoresectorop_specify")]
  public static void FileCoreSectorOp_Specify
    (bits flags,
     SectorDiscAddress disc_addr,
     char* instance);

  [CCode (cname = "xfilecore_free_space64")]
  public static OS.Error* XFreeSpace64
    (string disc_name,
     char* instance,
     out bits free_lo,
     out int free_hi,
     out int max);

  [CCode (cname = "filecore_free_space64")]
  public static void FreeSpace64
    (string disc_name,
     char* instance,
     out bits free_lo,
     out int free_hi,
     out int max);

  [CCode (cname = "xfilecorediscop64_verify")]
  public static OS.Error* XFileCoreDiscOp64_Verify
    (bits flags,
     DiscAddress64 disc_addr,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out int unverified);

  [CCode (cname = "filecorediscop64_verify")]
  public static int FileCoreDiscOp64_Verify
    (bits flags,
     DiscAddress64 disc_addr,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr);

  [CCode (cname = "xfilecorediscop64_read_sectors")]
  public static OS.Error* XFileCoreDiscOp64_ReadSectors
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "filecorediscop64_read_sectors")]
  public static int FileCoreDiscOp64_ReadSectors
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecorediscop64_write_sectors")]
  public static OS.Error* XFileCoreDiscOp64_WriteSectors
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "filecorediscop64_write_sectors")]
  public static int FileCoreDiscOp64_WriteSectors
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data);

  [CCode (cname = "xfilecorediscop64_read_track")]
  public static OS.Error* XFileCoreDiscOp64_ReadTrack
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop64_read_track")]
  public static void FileCoreDiscOp64_ReadTrack
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop64_read_id")]
  public static OS.Error* XFileCoreDiscOp64_ReadId
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop64_read_id")]
  public static void FileCoreDiscOp64_ReadId
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop64_write_track")]
  public static OS.Error* XFileCoreDiscOp64_WriteTrack
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "filecorediscop64_write_track")]
  public static void FileCoreDiscOp64_WriteTrack
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     char* instance);

  [CCode (cname = "xfilecorediscop64_format_track")]
  public static OS.Error* XFileCoreDiscOp64_FormatTrack
    (bits flags,
     DiscAddress64 disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "filecorediscop64_format_track")]
  public static void FileCoreDiscOp64_FormatTrack
    (bits flags,
     DiscAddress64 disc_addr,
     TrackFormat track_format,
     char* instance);

  [CCode (cname = "xfilecorediscop64_seek")]
  public static OS.Error* XFileCoreDiscOp64_Seek
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop64_seek")]
  public static void FileCoreDiscOp64_Seek
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "xfilecorediscop64_restore")]
  public static OS.Error* XFileCoreDiscOp64_Restore
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop64_restore")]
  public static void FileCoreDiscOp64_Restore
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "xfilecorediscop64_step_in")]
  public static OS.Error* XFileCoreDiscOp64_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "filecorediscop64_step_in")]
  public static void FileCoreDiscOp64_StepIn
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecorediscop64_step_out")]
  public static OS.Error* XFileCoreDiscOp64_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "filecorediscop64_step_out")]
  public static void FileCoreDiscOp64_StepOut
    (bits flags,
     char* instance);

  [CCode (cname = "xfilecorediscop64_read_sectors_via_cache")]
  public static OS.Error* XFileCoreDiscOp64_ReadSectorsViaCache
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "filecorediscop64_read_sectors_via_cache")]
  public static int FileCoreDiscOp64_ReadSectorsViaCache
    (bits flags,
     DiscAddress64 disc_addr,
     char* data,
     int size,
     int cache_handle,
     char* instance,
     out DiscAddress64 next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xfilecorediscop64_specify")]
  public static OS.Error* XFileCoreDiscOp64_Specify
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "filecorediscop64_specify")]
  public static void FileCoreDiscOp64_Specify
    (bits flags,
     DiscAddress64 disc_addr,
     char* instance);

  [CCode (cname = "xfilecore_features")]
  public static OS.Error* XFeatures
    (out FeatureFlags features);

  [CCode (cname = "filecore_features")]
  public static FeatureFlags Features ();

  [CCode (cname = "xservice_identify_disc")]
  public static OS.Error* XService_IdentifyDisc
    (string format_name,
     int size,
     Disc disc,
     int cache_handle,
     char* instance,
     out bool unclaimed,
     out bits file_type,
     out int cache_handle_out);

  [CCode (cname = "service_identify_disc")]
  public static bool Service_IdentifyDisc
    (string format_name,
     int size,
     Disc disc,
     int cache_handle,
     char* instance,
     out bits file_type,
     out int cache_handle_out);

}
