/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/fileinfo.h")]
namespace OSLib.FileInfo
{
  [CCode (cname = "fileinfo_flags")]
  public enum Flags
  {
    [CCode (cname = "fileinfo_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "fileinfo_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted
  }

  [CCode (cname = "fileinfo_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "modified")]
    public unowned bool Modified;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned Toolbox.MsgReference FileName;
    [CCode (cname = "file_size")]
    public unowned int FileSize;
    [CCode (cname = "date_and_time")]
    public unowned OS.DateAndTime DateAndTime;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "fileinfo_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "modified")]
    public unowned bool Modified;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned Toolbox.MsgReference FileName;
    [CCode (cname = "file_size")]
    public unowned int FileSize;
    [CCode (cname = "date_and_time")]
    public unowned OS.DateAndTime DateAndTime;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "fileinfo_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "fileinfo_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "fileinfo_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "fileinfo_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname="class_FILE_INFO")]
  public const Toolbox.Class Class_FileInfo;
  [CCode (cname="error_FILE_INFO_TASKS_ACTIVE")]
  public const bits Error_FileInfoTasksActive;
  [CCode (cname="error_FILE_INFO_ALLOC_FAILED")]
  public const bits Error_FileInfoAllocFailed;
  [CCode (cname="error_FILE_INFO_SHORT_BUFFER")]
  public const bits Error_FileInfoShortBuffer;
  [CCode (cname="error_FILE_INFO_NO_SUCH_TASK")]
  public const bits Error_FileInfoNoSuchTask;
  [CCode (cname="error_FILE_INFO_NO_SUCH_METHOD")]
  public const bits Error_FileInfoNoSuchMethod;
  [CCode (cname="error_FILE_INFO_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_FileInfoNoSuchMiscOpMethod;
  [CCode (cname="action_FILE_INFO_ABOUT_TO_BE_SHOWN")]
  public const bits Action_FileInfoAboutToBeShown;
  [CCode (cname="action_FILE_INFO_DIALOGUE_COMPLETED")]
  public const bits Action_FileInfoDialogueCompleted;

  [CCode (cname = "xfileinfo_get_window_id")]
  public static OS.Error* XGetWindowId
    (bits flags,
     Toolbox.O file_info,
     out Toolbox.O window);

  [CCode (cname = "fileinfo_get_window_id")]
  public static Toolbox.O GetWindowId
    (bits flags,
     Toolbox.O file_info);

  [CCode (cname = "xfileinfo_set_modified")]
  public static OS.Error* XSetModified
    (bits flags,
     Toolbox.O file_info,
     bool modified);

  [CCode (cname = "fileinfo_set_modified")]
  public static void SetModified
    (bits flags,
     Toolbox.O file_info,
     bool modified);

  [CCode (cname = "xfileinfo_get_modified")]
  public static OS.Error* XGetModified
    (bits flags,
     Toolbox.O file_info,
     out bool modified);

  [CCode (cname = "fileinfo_get_modified")]
  public static bool GetModified
    (bits flags,
     Toolbox.O file_info);

  [CCode (cname = "xfileinfo_set_file_type")]
  public static OS.Error* XSetFileType
    (bits flags,
     Toolbox.O file_info,
     bits file_type);

  [CCode (cname = "fileinfo_set_file_type")]
  public static void SetFileType
    (bits flags,
     Toolbox.O file_info,
     bits file_type);

  [CCode (cname = "xfileinfo_get_file_type")]
  public static OS.Error* XGetFileType
    (bits flags,
     Toolbox.O file_info,
     out bits file_type);

  [CCode (cname = "fileinfo_get_file_type")]
  public static bits GetFileType
    (bits flags,
     Toolbox.O file_info);

  [CCode (cname = "xfileinfo_set_file_name")]
  public static OS.Error* XSetFileName
    (bits flags,
     Toolbox.O file_info,
     string file_name);

  [CCode (cname = "fileinfo_set_file_name")]
  public static void SetFileName
    (bits flags,
     Toolbox.O file_info,
     string file_name);

  [CCode (cname = "xfileinfo_get_file_name")]
  public static OS.Error* XGetFileName
    (bits flags,
     Toolbox.O file_info,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "fileinfo_get_file_name")]
  public static int GetFileName
    (bits flags,
     Toolbox.O file_info,
     string buffer,
     int size);

  [CCode (cname = "xfileinfo_set_file_size")]
  public static OS.Error* XSetFileSize
    (bits flags,
     Toolbox.O file_info,
     int file_size);

  [CCode (cname = "fileinfo_set_file_size")]
  public static void SetFileSize
    (bits flags,
     Toolbox.O file_info,
     int file_size);

  [CCode (cname = "xfileinfo_get_file_size")]
  public static OS.Error* XGetFileSize
    (bits flags,
     Toolbox.O file_info,
     out int file_size);

  [CCode (cname = "fileinfo_get_file_size")]
  public static int GetFileSize
    (bits flags,
     Toolbox.O file_info);

  [CCode (cname = "xfileinfo_set_date")]
  public static OS.Error* XSetDate
    (bits flags,
     Toolbox.O file_info,
     OS.DateAndTime date_and_time);

  [CCode (cname = "fileinfo_set_date")]
  public static void SetDate
    (bits flags,
     Toolbox.O file_info,
     OS.DateAndTime date_and_time);

  [CCode (cname = "xfileinfo_get_date")]
  public static OS.Error* XGetDate
    (bits flags,
     Toolbox.O file_info,
     OS.DateAndTime date_and_time);

  [CCode (cname = "fileinfo_get_date")]
  public static void GetDate
    (bits flags,
     Toolbox.O file_info,
     OS.DateAndTime date_and_time);

  [CCode (cname = "xfileinfo_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O file_info,
     string title);

  [CCode (cname = "fileinfo_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O file_info,
     string title);

  [CCode (cname = "xfileinfo_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O file_info,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "fileinfo_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O file_info,
     string buffer,
     int size);

}
