/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.FileSwitch;
using OSLib.FilerAction;

[CCode (cheader_filename = "oslib/filer.h")]
namespace OSLib.Filer
{
  [CCode (cname = "filer_control")]
  public enum Control
  {
    [CCode (cname = "filer_CONTROL_ACKNOWLEDGE")]
    Acknowledge,
    [CCode (cname = "filer_CONTROL_VERBOSE_ON")]
    VerboseOn,
    [CCode (cname = "filer_CONTROL_VERBOSE_OFF")]
    VerboseOff
  }

  [CCode (cname = "filer_message_open_dir")]
  public struct MessageOpenDir
  {
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "dir_name")]
    public unowned char DirName[228];
  }

  [CCode (cname = "filer_message_open_dir_at")]
  public struct MessageOpenDirAt
  {
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "at")]
    public unowned OS.Coord At;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "display")]
    public unowned byte Display;
    [CCode (cname = "dir_name")]
    public unowned char DirName[211];
  }

  [CCode (cname = "filer_message_selection_dir")]
  public struct MessageSelectionDir
  {
    [CCode (cname = "dir_name")]
    public unowned char DirName[236];
  }

  [CCode (cname = "filer_message_add_selection")]
  public struct MessageAddSelection
  {
    [CCode (cname = "leaf_list")]
    public unowned char LeafList[236];
  }

  [CCode (cname = "filer_message_action")]
  public struct MessageAction
  {
    [CCode (cname = "operation")]
    public unowned FilerAction.Operation Operation;
    [CCode (cname = "options")]
    public unowned FilerAction.Flags Options;
    [CCode (cname = "data.destination_dir_name")]
    public unowned char Data_DestinationDirName[228];
    [CCode (cname = "data.access.access")]
    public unowned short Data_Access_Access;
    [CCode (cname = "data.access.mask")]
    public unowned short Data_Access_Mask;
    [CCode (cname = "data.file_type")]
    public unowned bits Data_FileType;
    [CCode (cname = "data.find_leaf")]
    public unowned char Data_FindLeaf[228];
  }

  [CCode (cname = "filer_message_control_action")]
  public struct MessageControlAction
  {
    [CCode (cname = "control")]
    public unowned Control Control;
  }

  [CCode (cname = "filer_message_selection")]
  public struct MessageSelection
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "display")]
    public unowned bits Display;
    [CCode (cname = "col0")]
    public unowned int Col0;
    [CCode (cname = "row0")]
    public unowned int Row0;
    [CCode (cname = "col1")]
    public unowned int Col1;
    [CCode (cname = "row1")]
    public unowned int Row1;
  }

  [CCode (cname = "filer_message_device_dir")]
  public struct MessageDeviceDir
  {
    [CCode (cname = "path_name")]
    public unowned char PathName[236];
  }

  [CCode (cname = "filer_full_message_open_dir")]
  public struct FullMessageOpenDir : Wimp.MessageHeaderBase
  {
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "dir_name")]
    public unowned char DirName[228];
  }

  [CCode (cname = "filer_full_message_open_dir_at")]
  public struct FullMessageOpenDirAt : Wimp.MessageHeaderBase
  {
    [CCode (cname = "fs_no")]
    public unowned FileSwitch.FSNo FsNo;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "at")]
    public unowned OS.Coord At;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "display")]
    public unowned byte Display;
    [CCode (cname = "dir_name")]
    public unowned char DirName[211];
  }

  [CCode (cname = "filer_full_message_selection_dir")]
  public struct FullMessageSelectionDir : Wimp.MessageHeaderBase
  {
    [CCode (cname = "dir_name")]
    public unowned char DirName[236];
  }

  [CCode (cname = "filer_full_message_add_selection")]
  public struct FullMessageAddSelection : Wimp.MessageHeaderBase
  {
    [CCode (cname = "leaf_list")]
    public unowned char LeafList[236];
  }

  [CCode (cname = "filer_full_message_action")]
  public struct FullMessageAction : Wimp.MessageHeaderBase
  {
    [CCode (cname = "operation")]
    public unowned FilerAction.Operation Operation;
    [CCode (cname = "options")]
    public unowned FilerAction.Flags Options;
    [CCode (cname = "data.destination_dir_name")]
    public unowned char Data_DestinationDirName[228];
    [CCode (cname = "data.access.access")]
    public unowned short Data_Access_Access;
    [CCode (cname = "data.access.mask")]
    public unowned short Data_Access_Mask;
    [CCode (cname = "data.file_type")]
    public unowned bits Data_FileType;
    [CCode (cname = "data.find_leaf")]
    public unowned char Data_FindLeaf[228];
  }

  [CCode (cname = "filer_full_message_control_action")]
  public struct FullMessageControlAction : Wimp.MessageHeaderBase
  {
    [CCode (cname = "control")]
    public unowned Control Control;
  }

  [CCode (cname = "filer_full_message_selection")]
  public struct FullMessageSelection : Wimp.MessageHeaderBase
  {
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "display")]
    public unowned bits Display;
    [CCode (cname = "col0")]
    public unowned int Col0;
    [CCode (cname = "row0")]
    public unowned int Row0;
    [CCode (cname = "col1")]
    public unowned int Col1;
    [CCode (cname = "row1")]
    public unowned int Row1;
  }

  [CCode (cname = "filer_full_message_device_dir")]
  public struct FullMessageDeviceDir : Wimp.MessageHeaderBase
  {
    [CCode (cname = "path_name")]
    public unowned char PathName[236];
  }

  [CCode (cname = "filer_format")]
  public struct Format
  {
    [CCode (cname = "next")]
    public unowned Format* Next;
    [CCode (cname = "menu_text")]
    public unowned string MenuText;
    [CCode (cname = "help_text")]
    public unowned string HelpText;
    [CCode (cname = "disc_format_swi")]
    public unowned int DiscFormatSwi;
    [CCode (cname = "disc_format_handle")]
    public unowned int DiscFormatHandle;
    [CCode (cname = "layout_structure_swi")]
    public unowned int LayoutStructureSwi;
    [CCode (cname = "layout_structure_handle")]
    public unowned int LayoutStructureHandle;
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname="error_FILER_NO_RECURSION")]
  public const bits Error_FilerNoRecursion;
  [CCode (cname="error_FILER_NO_TEMPLATE")]
  public const bits Error_FilerNoTemplate;
  [CCode (cname="error_FILER_FAILED_SAVE")]
  public const bits Error_FilerFailedSave;
  [CCode (cname="error_FILER_BAD_PATH")]
  public const bits Error_FilerBadPath;
  [CCode (cname="message_FILER_OPEN_DIR")]
  public const bits Message_FilerOpenDir;
  [CCode (cname="message_FILER_CLOSE_DIR")]
  public const bits Message_FilerCloseDir;
  [CCode (cname="message_FILER_OPEN_DIR_AT")]
  public const bits Message_FilerOpenDirAt;
  [CCode (cname="message_FILER_SELECTION_DIR")]
  public const bits Message_FilerSelectionDir;
  [CCode (cname="message_FILER_ADD_SELECTION")]
  public const bits Message_FilerAddSelection;
  [CCode (cname="message_FILER_ACTION")]
  public const bits Message_FilerAction;
  [CCode (cname="message_FILER_CONTROL_ACTION")]
  public const bits Message_FilerControlAction;
  [CCode (cname="message_FILER_SELECTION")]
  public const bits Message_FilerSelection;
  [CCode (cname="message_FILER_DEVICE_DIR")]
  public const bits Message_FilerDeviceDir;
  [CCode (cname="filer_FORMAT_NATIVE")]
  public const bits FormatNative;
  [CCode (cname="filer_DIR_NONCANONICAL")]
  public const bits DirNoncanonical;
  [CCode (cname="filer_DISPLAY_LARGE_ICONS")]
  public const bits DisplayLargeIcons;
  [CCode (cname="filer_DISPLAY_SMALL_ICONS")]
  public const bits DisplaySmallIcons;
  [CCode (cname="filer_DISPLAY_FULL_INFO")]
  public const bits DisplayFullInfo;
  [CCode (cname="filer_DISPLAY")]
  public const bits Display;
  [CCode (cname="filer_DISPLAY_SHIFT")]
  public const int DisplayShift;
  [CCode (cname="filer_SORT_BY_NAME")]
  public const bits SortByName;
  [CCode (cname="filer_SORT_BY_SIZE")]
  public const bits SortBySize;
  [CCode (cname="filer_SORT_BY_TYPE")]
  public const bits SortByType;
  [CCode (cname="filer_SORT_BY_DATE")]
  public const bits SortByDate;
  [CCode (cname="filer_SORT")]
  public const bits Sort;
  [CCode (cname="filer_SORT_SHIFT")]
  public const int SortShift;
  [CCode (cname="filer_GIVEN_DISPLAY")]
  public const bits GivenDisplay;
  [CCode (cname="filer_GIVEN_SORT")]
  public const bits GivenSort;

  [CCode (cname = "xservice_start_filer")]
  public static OS.Error* XService_StartFiler
    (Wimp.T task,
     out string command,
     out bool unclaimed);

  [CCode (cname = "service_start_filer")]
  public static bool Service_StartFiler
    (Wimp.T task,
     out string command);

  [CCode (cname = "xservice_started_filer")]
  public static OS.Error* XService_StartedFiler ();

  [CCode (cname = "service_started_filer")]
  public static void Service_StartedFiler ();

  [CCode (cname = "xservice_filer_dying")]
  public static OS.Error* XService_FilerDying ();

  [CCode (cname = "service_filer_dying")]
  public static void Service_FilerDying ();

  [CCode (cname = "xservice_enumerate_formats")]
  public static OS.Error* XService_EnumerateFormats
    (Format format_list,
     out Format format_list_out);

  [CCode (cname = "service_enumerate_formats")]
  public static void Service_EnumerateFormats
    (Format format_list,
     out Format format_list_out);

  [CCode (cname = "xservice_disc_dismounted")]
  public static OS.Error* XService_DiscDismounted
    (string disc_name);

  [CCode (cname = "service_disc_dismounted")]
  public static void Service_DiscDismounted
    (string disc_name);

}
