/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/filter.h")]
namespace OSLib.Filter
{
  [CCode (cname="filter_NAME_LIMIT")]
  public const int NameLimit;

  [CCode (cname = "xfilter_register_pre_filter")]
  public static OS.Error* XRegisterPreFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_register_pre_filter")]
  public static void RegisterPreFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_register_post_filter")]
  public static OS.Error* XRegisterPostFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task,
     Wimp.PollFlags mask);

  [CCode (cname = "filter_register_post_filter")]
  public static void RegisterPostFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task,
     Wimp.PollFlags mask);

  [CCode (cname = "xfilter_de_register_pre_filter")]
  public static OS.Error* XDeRegisterPreFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_de_register_pre_filter")]
  public static void DeRegisterPreFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_de_register_post_filter")]
  public static OS.Error* XDeRegisterPostFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task,
     Wimp.PollFlags mask);

  [CCode (cname = "filter_de_register_post_filter")]
  public static void DeRegisterPostFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task,
     Wimp.PollFlags mask);

  [CCode (cname = "xfilter_register_rect_filter")]
  public static OS.Error* XRegisterRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_register_rect_filter")]
  public static void RegisterRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_de_register_rect_filter")]
  public static OS.Error* XDeRegisterRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_de_register_rect_filter")]
  public static void DeRegisterRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_register_copy_filter")]
  public static OS.Error* XRegisterCopyFilter
    (string filter_name,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "filter_register_copy_filter")]
  public static void RegisterCopyFilter
    (string filter_name,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "xfilter_de_register_copy_filter")]
  public static OS.Error* XDeRegisterCopyFilter
    (string filter_name,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "filter_de_register_copy_filter")]
  public static void DeRegisterCopyFilter
    (string filter_name,
     AsmRoutine code,
     char* handle);

  [CCode (cname = "xfilter_register_post_rect_filter")]
  public static OS.Error* XRegisterPostRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_register_post_rect_filter")]
  public static void RegisterPostRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_de_register_post_rect_filter")]
  public static OS.Error* XDeRegisterPostRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_de_register_post_rect_filter")]
  public static void DeRegisterPostRectFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_register_post_icon_filter")]
  public static OS.Error* XRegisterPostIconFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_register_post_icon_filter")]
  public static void RegisterPostIconFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xfilter_de_register_post_icon_filter")]
  public static OS.Error* XDeRegisterPostIconFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "filter_de_register_post_icon_filter")]
  public static void DeRegisterPostIconFilter
    (string filter_name,
     AsmRoutine code,
     char* handle,
     Wimp.T task);

  [CCode (cname = "xservice_filter_manager_installed")]
  public static OS.Error* XService_FilterManagerInstalled
    (int version);

  [CCode (cname = "service_filter_manager_installed")]
  public static void Service_FilterManagerInstalled
    (int version);

  [CCode (cname = "xservice_filter_manager_dying")]
  public static OS.Error* XService_FilterManagerDying ();

  [CCode (cname = "service_filter_manager_dying")]
  public static void Service_FilterManagerDying ();

}
