/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/fontdbox.h")]
namespace OSLib.FontDbox
{
  [CCode (cname = "fontdbox_flags")]
  public enum Flags
  {
    [CCode (cname = "fontdbox_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "fontdbox_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "fontdbox_INCLUDE_SYSTEM_FONT")]
    IncludeSystemFont
  }

  [CCode (cname = "fontdbox_font")]
  public struct Font
  {
    [CCode (cname = "font")]
    public unowned Toolbox.StringReference Font;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "aspect")]
    public unowned int Aspect;
  }

  [CCode (cname = "fontdbox_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "initial")]
    public unowned Font Initial;
    [CCode (cname = "try_string")]
    public unowned Toolbox.MsgReference TryString;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "fontdbox_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "initial")]
    public unowned Font Initial;
    [CCode (cname = "try_string")]
    public unowned Toolbox.MsgReference TryString;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "fontdbox_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "fontdbox_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "fontdbox_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "fontdbox_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "fontdbox_action_apply_font")]
  public struct ActionApplyFont
  {
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "aspect")]
    public unowned int Aspect;
    [CCode (cname = "font")]
    public unowned char Font[204];
  }

  [CCode (cname = "fontdbox_action_apply_font_block")]
  public struct ActionApplyFontBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "aspect")]
    public unowned int Aspect;
    [CCode (cname = "font")]
    public unowned char Font[204];
  }

  [CCode (cname="class_FONT_DBOX")]
  public const Toolbox.Class Class_FontDbox;
  [CCode (cname="fontdbox_SET_HEIGHT")]
  public const bits SetHeight;
  [CCode (cname="fontdbox_SET_ASPECT")]
  public const bits SetAspect;
  [CCode (cname="action_FONT_DBOX_ABOUT_TO_BE_SHOWN")]
  public const bits Action_FontDboxAboutToBeShown;
  [CCode (cname="action_FONT_DBOX_DIALOGUE_COMPLETED")]
  public const bits Action_FontDboxDialogueCompleted;
  [CCode (cname="action_FONT_DBOX_APPLY_FONT")]
  public const bits Action_FontDboxApplyFont;
  [CCode (cname="error_FONT_DBOX_TASKS_ACTIVE")]
  public const bits Error_FontDboxTasksActive;
  [CCode (cname="error_FONT_DBOX_ALLOC_FAILED")]
  public const bits Error_FontDboxAllocFailed;
  [CCode (cname="error_FONT_DBOX_SHORT_BUFFER")]
  public const bits Error_FontDboxShortBuffer;
  [CCode (cname="error_FONT_DBOX_NO_SUCH_TASK")]
  public const bits Error_FontDboxNoSuchTask;
  [CCode (cname="error_FONT_DBOX_NO_SUCH_METHOD")]
  public const bits Error_FontDboxNoSuchMethod;
  [CCode (cname="error_FONT_DBOX_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_FontDboxNoSuchMiscOpMethod;
  [CCode (cname="error_FONT_DBOX_NO_SUCH_FONT")]
  public const bits Error_FontDboxNoSuchFont;
  [CCode (cname="error_FONT_DBOX_NO_FONTS")]
  public const bits Error_FontDboxNoFonts;
  [CCode (cname="error_FONT_DBOX_OUT_OF_MESSAGE_SPACE")]
  public const bits Error_FontDboxOutOfMessageSpace;

  [CCode (cname = "xfontdbox_get_window_id")]
  public static OS.Error* XGetWindowId
    (bits flags,
     Toolbox.O font_dbox,
     out Toolbox.O window);

  [CCode (cname = "fontdbox_get_window_id")]
  public static Toolbox.O GetWindowId
    (bits flags,
     Toolbox.O font_dbox);

  [CCode (cname = "xfontdbox_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O font_dbox,
     string font_identifier);

  [CCode (cname = "fontdbox_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O font_dbox,
     string font_identifier);

  [CCode (cname = "xfontdbox_get_font")]
  public static OS.Error* XGetFont
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "fontdbox_get_font")]
  public static int GetFont
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size);

  [CCode (cname = "xfontdbox_set_size")]
  public static OS.Error* XSetSize
    (bits flags,
     Toolbox.O font_dbox,
     int height,
     int aspect);

  [CCode (cname = "fontdbox_set_size")]
  public static void SetSize
    (bits flags,
     Toolbox.O font_dbox,
     int height,
     int aspect);

  [CCode (cname = "xfontdbox_get_size")]
  public static OS.Error* XGetSize
    (bits flags,
     Toolbox.O font_dbox,
     out int height,
     out int aspect);

  [CCode (cname = "fontdbox_get_size")]
  public static void GetSize
    (bits flags,
     Toolbox.O font_dbox,
     out int height,
     out int aspect);

  [CCode (cname = "xfontdbox_set_try_string")]
  public static OS.Error* XSetTryString
    (bits flags,
     Toolbox.O font_dbox,
     string try_string);

  [CCode (cname = "fontdbox_set_try_string")]
  public static void SetTryString
    (bits flags,
     Toolbox.O font_dbox,
     string try_string);

  [CCode (cname = "xfontdbox_get_try_string")]
  public static OS.Error* XGetTryString
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "fontdbox_get_try_string")]
  public static int GetTryString
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size);

  [CCode (cname = "xfontdbox_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O font_dbox,
     string title);

  [CCode (cname = "fontdbox_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O font_dbox,
     string title);

  [CCode (cname = "xfontdbox_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "fontdbox_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O font_dbox,
     string buffer,
     int size);

}
