/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/freeway.h")]
namespace OSLib.Freeway
{
  [CCode (cname = "freeway_object_type")]
  public enum ObjectType
  {
    [CCode (cname = "freeway_TYPE_DISC")]
    TypeDisc,
    [CCode (cname = "freeway_TYPE_PRINTER")]
    TypePrinter,
    [CCode (cname = "freeway_TYPE_HOST")]
    TypeHost
  }

  [CCode (cname = "freeway_interest_flags")]
  public enum InterestFlags
  {
    [CCode (cname = "freeway_INTEREST_REGISTER")]
    InterestRegister,
    [CCode (cname = "freeway_INTEREST_DEREGISTER")]
    InterestDeregister,
    [CCode (cname = "freeway_INTEREST_UNAUTHENTICATED")]
    InterestUnauthenticated,
    [CCode (cname = "freeway_INTEREST_AUTHENTICATED")]
    InterestAuthenticated
  }

  [CCode (cname = "freeway_object_flags")]
  public enum ObjectFlags
  {
    [CCode (cname = "freeway_OBJECT_ADD")]
    ObjectAdd,
    [CCode (cname = "freeway_OBJECT_REMOVE")]
    ObjectRemove,
    [CCode (cname = "freeway_OBJECT_UNAUTHENTICATED")]
    ObjectUnauthenticated,
    [CCode (cname = "freeway_OBJECT_AUTHENTICATED")]
    ObjectAuthenticated
  }

  [CCode (cname = "freeway_read_flags")]
  public enum ReadFlags
  {
    [CCode (cname = "freeway_READ_GIVEN_AUTHENTICATION")]
    ReadGivenAuthentication
  }

  [CCode (cname = "freeway_enumerate_flags")]
  public enum EnumerateFlags
  {
    [CCode (cname = "freeway_ENUMERATE_AUTHENTICATED")]
    EnumerateAuthenticated
  }

  [CCode (cname = "freeway_starting_flags")]
  public enum StartingFlags
  {
    Default = 0
  }

  [CCode (cname = "freeway_terminating_flags")]
  public enum TerminatingFlags
  {
    Default = 0
  }

  [CCode (cname = "freeway_upcall_reason")]
  public enum UpcallReason
  {
    [CCode (cname = "freeway_OBJECT_ADDED")]
    ObjectAdded,
    [CCode (cname = "freeway_OBJECT_REMOVED")]
    ObjectRemoved,
    [CCode (cname = "freeway_OBJECT_CHANGED")]
    ObjectChanged,
    [CCode (cname = "freeway_OBJECT_DELETED")]
    ObjectDeleted
  }

  [CCode (cname="freeway_NO_MORE")]
  public const int NoMore;

  [CCode (cname = "xfreeway_register")]
  public static OS.Error* XRegister
    (InterestFlags flags,
     ObjectType type,
     int authentication);

  [CCode (cname = "freeway_register")]
  public static void Register
    (InterestFlags flags,
     ObjectType type,
     int authentication);

  [CCode (cname = "xfreeway_write")]
  public static OS.Error* XWrite
    (ObjectFlags flags,
     ObjectType type,
     string obj_name,
     int size,
     string desc,
     int authentication);

  [CCode (cname = "freeway_write")]
  public static void Write
    (ObjectFlags flags,
     ObjectType type,
     string obj_name,
     int size,
     string desc,
     int authentication);

  [CCode (cname = "xfreeway_read")]
  public static OS.Error* XRead
    (ReadFlags flags,
     ObjectType type,
     string obj_name,
     int size,
     string desc,
     int authentication,
     out int size_out,
     out bits ip_addr);

  [CCode (cname = "freeway_read")]
  public static int Read
    (ReadFlags flags,
     ObjectType type,
     string obj_name,
     int size,
     string desc,
     int authentication,
     out bits ip_addr);

  [CCode (cname = "xfreeway_enumerate")]
  public static OS.Error* XEnumerate
    (EnumerateFlags flags,
     ObjectType type,
     int obj_name_size,
     string obj_name,
     int desc_size,
     string desc,
     int context,
     int authentication,
     out int obj_name_size_out,
     out int desc_size_out,
     out bits ip_addr,
     out int context_out);

  [CCode (cname = "freeway_enumerate")]
  public static int Enumerate
    (EnumerateFlags flags,
     ObjectType type,
     int obj_name_size,
     string obj_name,
     int desc_size,
     string desc,
     int context,
     int authentication,
     out int obj_name_size_out,
     out int desc_size_out,
     out bits ip_addr);

  [CCode (cname = "xservice_freeway_starting")]
  public static OS.Error* XService_FreewayStarting
    (StartingFlags flags);

  [CCode (cname = "service_freeway_starting")]
  public static void Service_FreewayStarting
    (StartingFlags flags);

  [CCode (cname = "xservice_freeway_terminating")]
  public static OS.Error* XService_FreewayTerminating
    (TerminatingFlags flags);

  [CCode (cname = "service_freeway_terminating")]
  public static void Service_FreewayTerminating
    (TerminatingFlags flags);

  [CCode (cname = "xupcall_freeway")]
  public static OS.Error* XUpCall_Freeway
    (UpcallReason reason,
     ObjectType type,
     string obj_name,
     int desc_size,
     string desc,
     bits ip_addr);

  [CCode (cname = "upcall_freeway")]
  public static void UpCall_Freeway
    (UpcallReason reason,
     ObjectType type,
     string obj_name,
     int desc_size,
     string desc,
     bits ip_addr);

}
