/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/hal.h")]
namespace OSLib.HAL
{
  [CCode (cname = "hal_device")]
  public enum Device
  {
    Default = 0
  }

  [CCode (cname = "hal_timer")]
  public enum Timer
  {
    Default = 0
  }


  [CCode (cname = "xhal_irq_enable")]
  public static OS.Error* XIRQEnable
    (Device device,
     out bool was_enabled);

  [CCode (cname = "hal_irq_enable")]
  public static bool IRQEnable
    (Device device);

  [CCode (cname = "xhal_irq_disable")]
  public static OS.Error* XIRQDisable
    (Device device,
     out bool was_enabled);

  [CCode (cname = "hal_irq_disable")]
  public static bool IRQDisable
    (Device device);

  [CCode (cname = "xhal_irq_clear")]
  public static OS.Error* XIRQClear
    (Device device);

  [CCode (cname = "hal_irq_clear")]
  public static void IRQClear
    (Device device);

  [CCode (cname = "xhal_irq_source")]
  public static OS.Error* XIRQSource
    (out Device device);

  [CCode (cname = "hal_irq_source")]
  public static Device IRQSource ();

  [CCode (cname = "xhal_irq_status")]
  public static OS.Error* XIRQStatus
    (Device device,
     out bool requested);

  [CCode (cname = "hal_irq_status")]
  public static bool IRQStatus
    (Device device);

  [CCode (cname = "xhal_fiq_enable")]
  public static OS.Error* XFIQEnable
    (Device device,
     out bool was_enabled);

  [CCode (cname = "hal_fiq_enable")]
  public static bool FIQEnable
    (Device device);

  [CCode (cname = "xhal_fiq_disable")]
  public static OS.Error* XFIQDisable
    (Device device,
     out bool was_enabled);

  [CCode (cname = "hal_fiq_disable")]
  public static bool FIQDisable
    (Device device);

  [CCode (cname = "xhal_fiq_disable_all")]
  public static OS.Error* XFIQDisableAll
    (Device device);

  [CCode (cname = "hal_fiq_disable_all")]
  public static void FIQDisableAll
    (Device device);

  [CCode (cname = "xhal_fiq_clear")]
  public static OS.Error* XFIQClear
    (Device device);

  [CCode (cname = "hal_fiq_clear")]
  public static void FIQClear
    (Device device);

  [CCode (cname = "xhal_fiq_source")]
  public static OS.Error* XFIQSource
    (out Device device);

  [CCode (cname = "hal_fiq_source")]
  public static Device FIQSource ();

  [CCode (cname = "xhal_fiq_status")]
  public static OS.Error* XFIQStatus
    (Device device,
     out bool requested);

  [CCode (cname = "hal_fiq_status")]
  public static bool FIQStatus
    (Device device);

  [CCode (cname = "xhal_timers")]
  public static OS.Error* XTimers
    (out int timers);

  [CCode (cname = "hal_timers")]
  public static int Timers ();

  [CCode (cname = "xhal_timer_device")]
  public static OS.Error* XTimerDevice
    (Timer timer,
     out Device device);

  [CCode (cname = "hal_timer_device")]
  public static Device TimerDevice
    (Timer timer);

  [CCode (cname = "xhal_timer_granularity")]
  public static OS.Error* XTimerGranularity
    (Timer timer,
     out int granularity);

  [CCode (cname = "hal_timer_granularity")]
  public static int TimerGranularity
    (Timer timer);

  [CCode (cname = "xhal_timer_max_period")]
  public static OS.Error* XTimerMaxPeriod
    (Timer timer,
     out int max_period);

  [CCode (cname = "hal_timer_max_period")]
  public static int TimerMaxPeriod
    (Timer timer);

  [CCode (cname = "xhal_timer_set_period")]
  public static OS.Error* XTimerSetPeriod
    (Timer timer,
     int period);

  [CCode (cname = "hal_timer_set_period")]
  public static void TimerSetPeriod
    (Timer timer,
     int period);

  [CCode (cname = "xhal_timer_period")]
  public static OS.Error* XTimerPeriod
    (Timer timer,
     out int period);

  [CCode (cname = "hal_timer_period")]
  public static int TimerPeriod
    (Timer timer);

  [CCode (cname = "xhal_timer_read_countdown")]
  public static OS.Error* XTimerReadCountdown
    (Timer timer,
     out int time_remaining);

  [CCode (cname = "hal_timer_read_countdown")]
  public static int TimerReadCountdown
    (Timer timer);

  [CCode (cname = "xhal_counter_rate")]
  public static OS.Error* XCounterRate
    (out int rate);

  [CCode (cname = "hal_counter_rate")]
  public static int CounterRate ();

  [CCode (cname = "xhal_counter_period")]
  public static OS.Error* XCounterPeriod
    (out int period);

  [CCode (cname = "hal_counter_period")]
  public static int CounterPeriod ();

  [CCode (cname = "xhal_counter_read")]
  public static OS.Error* XCounterRead
    (out int period);

  [CCode (cname = "hal_counter_read")]
  public static int CounterRead ();

  [CCode (cname = "xhal_counter_delay")]
  public static OS.Error* XCounterDelay
    (int delay);

  [CCode (cname = "hal_counter_delay")]
  public static void CounterDelay
    (int delay);

}
