/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;

[CCode (cheader_filename = "oslib/iconbar.h")]
namespace OSLib.Iconbar
{
  [CCode (cname = "iconbar_flags")]
  public enum Flags
  {
    [CCode (cname = "iconbar_GENERATE_SELECT_ABOUT_TO_BE_SHOWN")]
    GenerateSelectAboutToBeShown,
    [CCode (cname = "iconbar_GENERATE_ADJUST_ABOUT_TO_BE_SHOWN")]
    GenerateAdjustAboutToBeShown,
    [CCode (cname = "iconbar_SELECT_IS_MENU")]
    SelectIsMenu,
    [CCode (cname = "iconbar_ADJUST_IS_MENU")]
    AdjustIsMenu,
    [CCode (cname = "iconbar_HAS_TEXT")]
    HasText,
    [CCode (cname = "iconbar_GENERATE_SELECT_CLICKED")]
    GenerateSelectClicked,
    [CCode (cname = "iconbar_GENERATE_ADJUST_CLICKED")]
    GenerateAdjustClicked,
    [CCode (cname = "iconbar_SHOW_SELECT_CENTERED")]
    ShowSelectCentered,
    [CCode (cname = "iconbar_SHOW_ADJUST_CENTERED")]
    ShowAdjustCentered,
    [CCode (cname = "iconbar_SHOW_SELECT_AT_POINTER")]
    ShowSelectAtPointer,
    [CCode (cname = "iconbar_SHOW_ADJUST_AT_POINTER")]
    ShowAdjustAtPointer
  }

  [CCode (cname = "iconbar_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "position")]
    public unowned int Position;
    [CCode (cname = "priority")]
    public unowned int Priority;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "sprite_limit")]
    public unowned int SpriteLimit;
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "menu")]
    public unowned Toolbox.StringReference Menu;
    [CCode (cname = "select_action")]
    public unowned bits SelectAction;
    [CCode (cname = "adjust_action")]
    public unowned bits AdjustAction;
    [CCode (cname = "select_show")]
    public unowned Toolbox.StringReference SelectShow;
    [CCode (cname = "adjust_show")]
    public unowned Toolbox.StringReference AdjustShow;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
  }

  [CCode (cname = "iconbar_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "position")]
    public unowned int Position;
    [CCode (cname = "priority")]
    public unowned int Priority;
    [CCode (cname = "sprite_name")]
    public unowned Toolbox.StringReference SpriteName;
    [CCode (cname = "sprite_limit")]
    public unowned int SpriteLimit;
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "menu")]
    public unowned Toolbox.StringReference Menu;
    [CCode (cname = "select_action")]
    public unowned bits SelectAction;
    [CCode (cname = "adjust_action")]
    public unowned bits AdjustAction;
    [CCode (cname = "select_show")]
    public unowned Toolbox.StringReference SelectShow;
    [CCode (cname = "adjust_show")]
    public unowned Toolbox.StringReference AdjustShow;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
  }

  [CCode (cname = "iconbar_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "iconbar_action_clicked_block")]
  public enum ActionClickedBlock
  {
    Default = 0
  }

  [CCode (cname = "iconbar_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "iconbar_action_select_about_to_be_shown")]
  public struct ActionSelectAboutToBeShown
  {
    [CCode (cname = "o")]
    public unowned Toolbox.O O;
  }

  [CCode (cname = "iconbar_action_adjust_about_to_be_shown")]
  public struct ActionAdjustAboutToBeShown
  {
    [CCode (cname = "o")]
    public unowned Toolbox.O O;
  }

  [CCode (cname = "iconbar_action_select_about_to_be_shown_block")]
  public struct ActionSelectAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "o")]
    public unowned Toolbox.O O;
  }

  [CCode (cname = "iconbar_action_adjust_about_to_be_shown_block")]
  public struct ActionAdjustAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "o")]
    public unowned Toolbox.O O;
  }

  [CCode (cname = "iconbar_set_button")]
  public enum SetButton
  {
    [CCode (cname = "iconbar_SELECT")]
    Select,
    [CCode (cname = "iconbar_ADJUST")]
    Adjust
  }

  [CCode (cname="class_ICONBAR")]
  public const Toolbox.Class Class_Iconbar;
  [CCode (cname="error_ICONBAR_ALLOC_FAILED")]
  public const bits Error_IconbarAllocFailed;
  [CCode (cname="error_ICONBAR_SHORT_BUFFER")]
  public const bits Error_IconbarShortBuffer;
  [CCode (cname="error_ICONBAR_BAD_OBJECT_VERSION")]
  public const bits Error_IconbarBadObjectVersion;
  [CCode (cname="error_ICONBAR_BAD_FLAGS")]
  public const bits Error_IconbarBadFlags;
  [CCode (cname="error_ICONBAR_NO_SUCH_TASK")]
  public const bits Error_IconbarNoSuchTask;
  [CCode (cname="error_ICONBAR_NO_SUCH_METHOD")]
  public const bits Error_IconbarNoSuchMethod;
  [CCode (cname="error_ICONBAR_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_IconbarNoSuchMiscOpMethod;
  [CCode (cname="error_ICONBAR_WRONG_SHOW_TYPE")]
  public const bits Error_IconbarWrongShowType;
  [CCode (cname="error_ICONBAR_NO_TEXT")]
  public const bits Error_IconbarNoText;
  [CCode (cname="error_ICONBAR_TASKS_ACTIVE")]
  public const bits Error_IconbarTasksActive;
  [CCode (cname="action_ICONBAR_CLICKED")]
  public const bits Action_IconbarClicked;
  [CCode (cname="action_ICONBAR_SELECT_ABOUT_TO_BE_SHOWN")]
  public const bits Action_IconbarSelectAboutToBeShown;
  [CCode (cname="action_ICONBAR_ADJUST_ABOUT_TO_BE_SHOWN")]
  public const bits Action_IconbarAdjustAboutToBeShown;
  [CCode (cname="iconbar_CLICKED_ADJUST")]
  public const bits ClickedAdjust;
  [CCode (cname="iconbar_CLICKED_SELECT")]
  public const bits ClickedSelect;

  [CCode (cname = "xiconbar_get_icon_handle")]
  public static OS.Error* XGetIconHandle
    (bits flags,
     Toolbox.O iconbar,
     out Wimp.I i);

  [CCode (cname = "iconbar_get_icon_handle")]
  public static Wimp.I GetIconHandle
    (bits flags,
     Toolbox.O iconbar);

  [CCode (cname = "xiconbar_set_menu")]
  public static OS.Error* XSetMenu
    (bits flags,
     Toolbox.O iconbar,
     Toolbox.O menu);

  [CCode (cname = "iconbar_set_menu")]
  public static void SetMenu
    (bits flags,
     Toolbox.O iconbar,
     Toolbox.O menu);

  [CCode (cname = "xiconbar_get_menu")]
  public static OS.Error* XGetMenu
    (bits flags,
     Toolbox.O iconbar,
     out Toolbox.O menu);

  [CCode (cname = "iconbar_get_menu")]
  public static Toolbox.O GetMenu
    (bits flags,
     Toolbox.O iconbar);

  [CCode (cname = "xiconbar_set_action")]
  public static OS.Error* XSetAction
    (SetButton flags,
     Toolbox.O iconbar,
     bits select_action,
     bits adjust_action);

  [CCode (cname = "iconbar_set_action")]
  public static void SetAction
    (SetButton flags,
     Toolbox.O iconbar,
     bits select_action,
     bits adjust_action);

  [CCode (cname = "xiconbar_get_action")]
  public static OS.Error* XGetAction
    (bits flags,
     Toolbox.O iconbar,
     out int select_action,
     out int adjust_action);

  [CCode (cname = "iconbar_get_action")]
  public static void GetAction
    (bits flags,
     Toolbox.O iconbar,
     out int select_action,
     out int adjust_action);

  [CCode (cname = "xiconbar_set_show")]
  public static OS.Error* XSetShow
    (SetButton flags,
     Toolbox.O iconbar,
     Toolbox.O select,
     Toolbox.O adjust);

  [CCode (cname = "iconbar_set_show")]
  public static void SetShow
    (SetButton flags,
     Toolbox.O iconbar,
     Toolbox.O select,
     Toolbox.O adjust);

  [CCode (cname = "xiconbar_get_show")]
  public static OS.Error* XGetShow
    (bits flags,
     Toolbox.O iconbar,
     out Toolbox.O select,
     out Toolbox.O adjust);

  [CCode (cname = "iconbar_get_show")]
  public static void GetShow
    (bits flags,
     Toolbox.O iconbar,
     out Toolbox.O select,
     out Toolbox.O adjust);

  [CCode (cname = "xiconbar_set_help_message")]
  public static OS.Error* XSetHelpMessage
    (bits flags,
     Toolbox.O iconbar,
     string help_message);

  [CCode (cname = "iconbar_set_help_message")]
  public static void SetHelpMessage
    (bits flags,
     Toolbox.O iconbar,
     string help_message);

  [CCode (cname = "xiconbar_get_help_message")]
  public static OS.Error* XGetHelpMessage
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "iconbar_get_help_message")]
  public static int GetHelpMessage
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size);

  [CCode (cname = "xiconbar_set_text")]
  public static OS.Error* XSetText
    (bits flags,
     Toolbox.O iconbar,
     string text);

  [CCode (cname = "iconbar_set_text")]
  public static void SetText
    (bits flags,
     Toolbox.O iconbar,
     string text);

  [CCode (cname = "xiconbar_get_text")]
  public static OS.Error* XGetText
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "iconbar_get_text")]
  public static int GetText
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size);

  [CCode (cname = "xiconbar_set_sprite")]
  public static OS.Error* XSetSprite
    (bits flags,
     Toolbox.O iconbar,
     string sprite_name);

  [CCode (cname = "iconbar_set_sprite")]
  public static void SetSprite
    (bits flags,
     Toolbox.O iconbar,
     string sprite_name);

  [CCode (cname = "xiconbar_get_sprite")]
  public static OS.Error* XGetSprite
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "iconbar_get_sprite")]
  public static int GetSprite
    (bits flags,
     Toolbox.O iconbar,
     string buffer,
     int size);

}
