/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.PDriver;

[CCode (cheader_filename = "oslib/imagefilerender.h")]
namespace OSLib.ImageFileRender
{
  [CCode (cname = "imagefilerender_sequence_number")]
  public enum SequenceNumber
  {
    [CCode (cname = "imagefilerender_DEFAULT_IMAGE")]
    DefaultImage
  }

  [CCode (cname = "imagefilerender_fit_block")]
  public struct FitBlock
  {
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "border")]
    public unowned int Border;
    [CCode (cname = "angle")]
    public unowned int Angle;
  }

  [CCode (cname = "imagefilerender_transformation_data")]
  public struct TransformationData
  {
    [CCode (cname = "to_fit")]
    public unowned FitBlock ToFit;
    [CCode (cname = "scaled")]
    public unowned OS.Factors Scaled;
    [CCode (cname = "transformed")]
    public unowned OS.Trfm Transformed;
  }

  [CCode (cname = "imagefilerender_query_type")]
  public enum QueryType
  {
    [CCode (cname = "imagefilerender_BASE_DETAIL_QUERY")]
    BaseDetailQuery
  }

  [CCode (cname = "imagefilerender_image_flags")]
  public enum ImageFlags
  {
    [CCode (cname = "imagefilerender_IMAGE_IS_SOLID")]
    ImageIsSolid
  }

  [CCode (cname = "imagefilerender_base_details")]
  public struct BaseDetails
  {
    [CCode (cname = "sequence")]
    public unowned SequenceNumber Sequence;
    [CCode (cname = "xdpi")]
    public unowned int Xdpi;
    [CCode (cname = "ydpi")]
    public unowned int Ydpi;
    [CCode (cname = "colour_type")]
    public unowned OSSpriteOp.ModeWord ColourType;
    [CCode (cname = "image_flags")]
    public unowned ImageFlags ImageFlags;
  }

  [CCode (cname = "imagefilerender_flags")]
  public enum Flags
  {
    [CCode (cname = "imagefilerender_TRANSFORMATION_TYPE")]
    TransformationType,
    [CCode (cname = "imagefilerender_COLOUR_MAPPING_SUPPLIED")]
    ColourMappingSupplied,
    [CCode (cname = "imagefilerender_IGNORE_DOCUMENT_ORIGIN")]
    IgnoreDocumentOrigin,
    [CCode (cname = "imagefilerender_RETURN_OS_UNITS")]
    ReturnOSUnits,
    [CCode (cname = "imagefilerender_QUALITY")]
    Quality
  }

  [CCode (cname = "imagefilerender_renderer_flags")]
  public enum RendererFlags
  {
    [CCode (cname = "imagefilerender_TRANSFORMATION_CAPABILITIES")]
    TransformationCapabilities,
    [CCode (cname = "imagefilerender_SUPPORTS_COLOUR_MAPPING")]
    SupportsColourMapping,
    [CCode (cname = "imagefilerender_SUPPORTS_IRREGULAR_SHAPES")]
    SupportsIrregularShapes,
    [CCode (cname = "imagefilerender_MAXIMUM_QUALITY_LEVEL")]
    MaximumQualityLevel,
    [CCode (cname = "imagefilerender_DEFAULT_QUALITY_LEVEL")]
    DefaultQualityLevel
  }

  [CCode (cname = "imagefilerender_magic", has_type_id = false)]
  public struct Magic : bits
  {
  }

  [CCode (cname = "imagefilerender_renderer")]
  public struct Renderer
  {
    [CCode (cname = "api_version")]
    public unowned int ApiVersion;
    [CCode (cname = "flags")]
    public unowned RendererFlags Flags;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "magic")]
    public unowned Magic Magic;
    [CCode (cname = "name")]
    public unowned char* Name;
    [CCode (cname = "workspace")]
    public unowned char* Workspace;
    [CCode (cname = "start")]
    public unowned AsmRoutine Start;
    [CCode (cname = "stop")]
    public unowned AsmRoutine Stop;
    [CCode (cname = "render")]
    public unowned AsmRoutine Render;
    [CCode (cname = "bbox")]
    public unowned AsmRoutine Bbox;
    [CCode (cname = "declare_fonts")]
    public unowned AsmRoutine DeclareFonts;
    [CCode (cname = "info")]
    public unowned AsmRoutine Info;
  }

  [CCode (cname = "imagefilerender_image_descriptor")]
  public struct ImageDescriptor
  {
    [CCode (cname = "data")]
    public unowned char* Data;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "extension_data")]
    public unowned char* ExtensionData;
    [CCode (cname = "sequence")]
    public unowned SequenceNumber Sequence;
    [CCode (cname = "private_data")]
    public unowned char* PrivateData;
  }

  [CCode (cname = "imagefilerender_rendering_descriptor")]
  public struct RenderingDescriptor
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "mapping")]
    public unowned OSSpriteOp.ColourMapping Mapping;
  }

  [CCode (cname = "imagefilerender_bbox_descriptor")]
  public struct BBoxDescriptor
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
  }

  [CCode (cname = "imagefilerender_declare_fonts_descriptor")]
  public struct DeclareFontsDescriptor
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "pdriver_flags")]
    public unowned PDriver.FontFlags PdriverFlags;
  }

  [CCode (cname = "imagefilerender_info_descriptor")]
  public struct InfoDescriptor
  {
    [CCode (cname = "query")]
    public unowned QueryType Query;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "data")]
    public unowned char* Data;
  }

  [CCode (cname="imagefilerender_TRANSFORMATION_TYPE_SHIFT")]
  public const int TransformationTypeShift;
  [CCode (cname="imagefilerender_RENDER_TO_FIT")]
  public const int RenderToFit;
  [CCode (cname="imagefilerender_RENDER_SCALED")]
  public const int RenderScaled;
  [CCode (cname="imagefilerender_RENDER_TRANSFORMED")]
  public const int RenderTransformed;
  [CCode (cname="imagefilerender_QUALITY_SHIFT")]
  public const int QualityShift;
  [CCode (cname="imagefilerender_DEFAULT_QUALITY")]
  public const int DefaultQuality;
  [CCode (cname="imagefilerender_LOWEST_QUALITY")]
  public const int LowestQuality;
  [CCode (cname="imagefilerender_HIGHEST_QUALITY")]
  public const int HighestQuality;
  [CCode (cname="imagefilerender_TRANSFORMATION_CAPABILITIES_SHIFT")]
  public const int TransformationCapabilitiesShift;
  [CCode (cname="imagefilerender_NO_TRANSFORMATION")]
  public const int NoTransformation;
  [CCode (cname="imagefilerender_FIXED_ASPECT_RATIO")]
  public const int FixedAspectRatio;
  [CCode (cname="imagefilerender_ANY_ASPECT_RATIO")]
  public const int AnyAspectRatio;
  [CCode (cname="imagefilerender_ANY_TRANSFORMATION")]
  public const int AnyTransformation;
  [CCode (cname="imagefilerender_MAXIMUM_QUALITY_LEVEL_SHIFT")]
  public const int MaximumQualityLevelShift;
  [CCode (cname="imagefilerender_DEFAULT_QUALITY_LEVEL_SHIFT")]
  public const int DefaultQualityLevelShift;

  [CCode (cname = "ximagefilerender_render")]
  public static OS.Error* XRender
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     int xorigin,
     int yorigin,
     TransformationData transformation_data,
     OSSpriteOp.ColourMapping colour_mapping);

  [CCode (cname = "imagefilerender_render")]
  public static void Render
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     int xorigin,
     int yorigin,
     TransformationData transformation_data,
     OSSpriteOp.ColourMapping colour_mapping);

  [CCode (cname = "ximagefilerender_bbox")]
  public static OS.Error* XBBox
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     TransformationData transformation_data,
     OS.Box bbox);

  [CCode (cname = "imagefilerender_bbox")]
  public static void BBox
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     TransformationData transformation_data,
     OS.Box bbox);

  [CCode (cname = "ximagefilerender_transform")]
  public static OS.Error* XTransform
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     TransformationData transformation_data,
     OS.Trfm trfm,
     out int xdpi,
     out int ydpi);

  [CCode (cname = "imagefilerender_transform")]
  public static void Transform
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     TransformationData transformation_data,
     OS.Trfm trfm,
     out int xdpi,
     out int ydpi);

  [CCode (cname = "ximagefilerender_declare_fonts")]
  public static OS.Error* XDeclareFonts
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     PDriver.FontFlags pdriver_flags);

  [CCode (cname = "imagefilerender_declare_fonts")]
  public static void DeclareFonts
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     PDriver.FontFlags pdriver_flags);

  [CCode (cname = "ximagefilerender_info")]
  public static OS.Error* XInfo
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     QueryType query,
     char* query_data,
     int query_data_size,
     out int size_used);

  [CCode (cname = "imagefilerender_info")]
  public static int Info
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     QueryType query,
     char* query_data,
     int query_data_size);

  [CCode (cname = "ximagefilerender_info_base_details")]
  public static OS.Error* XInfoBaseDetails
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     BaseDetails base_details,
     int base_details_size,
     out int size_used);

  [CCode (cname = "imagefilerender_info_base_details")]
  public static int InfoBaseDetails
    (Flags flags,
     bits file_type,
     char* data,
     int size,
     char* extension_data,
     SequenceNumber sequence,
     BaseDetails base_details,
     int base_details_size);

  [CCode (cname = "ximagefilerender_renderer_info")]
  public static OS.Error* XRendererInfo
    (Flags flags,
     bits file_type,
     Magic magic,
     out Renderer definition,
     out char* name);

  [CCode (cname = "imagefilerender_renderer_info")]
  public static Renderer RendererInfo
    (Flags flags,
     bits file_type,
     Magic magic,
     out char* name);

  [CCode (cname = "ximagefilerender_register")]
  public static OS.Error* XRegister
    (Flags flags,
     Renderer definition,
     out int api_version);

  [CCode (cname = "imagefilerender_register")]
  public static int Register
    (Flags flags,
     Renderer definition);

  [CCode (cname = "ximagefilerender_deregister")]
  public static OS.Error* XDeregister
    (Flags flags,
     bits file_type,
     char* name,
     Magic magic,
     out int api_version);

  [CCode (cname = "imagefilerender_deregister")]
  public static int Deregister
    (Flags flags,
     bits file_type,
     char* name,
     Magic magic);

  [CCode (cname = "xservice_image_file_render_started")]
  public static OS.Error* XService_ImageFileRenderStarted
    (int api_version);

  [CCode (cname = "service_image_file_render_started")]
  public static void Service_ImageFileRenderStarted
    (int api_version);

  [CCode (cname = "xservice_image_file_render_dying")]
  public static OS.Error* XService_ImageFileRenderDying
    (int api_version);

  [CCode (cname = "service_image_file_render_dying")]
  public static void Service_ImageFileRenderDying
    (int api_version);

  [CCode (cname = "xservice_image_file_render_renderer_changed")]
  public static OS.Error* XService_ImageFileRenderRendererChanged
    (int api_version,
     bits file_type);

  [CCode (cname = "service_image_file_render_renderer_changed")]
  public static void Service_ImageFileRenderRendererChanged
    (int api_version,
     bits file_type);

}
