/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/inversetable.h")]
namespace OSLib.InverseTable
{
  [CCode (cname = "inversetable_colour")]
  public enum Colour
  {
    [CCode (cname = "inversetable_R")]
    R,
    [CCode (cname = "inversetable_G")]
    G,
    [CCode (cname = "inversetable_B")]
    B
  }

  [CCode (cname="inversetable_RSHIFT")]
  public const int RShift;
  [CCode (cname="inversetable_GSHIFT")]
  public const int GShift;
  [CCode (cname="inversetable_BSHIFT")]
  public const int BShift;

  [CCode (cname = "xinversetable_calculate")]
  public static OS.Error* XCalculate
    (out Colour gcol_to_colour,
     out OS.GCOL colour_to_gcol);

  [CCode (cname = "inversetable_calculate")]
  public static void Calculate
    (out Colour gcol_to_colour,
     out OS.GCOL colour_to_gcol);

  [CCode (cname = "xinversetable_sprite_table_for_sprite")]
  public static OS.Error* XSpriteTableForSprite
    (Colour gcol_to_colour_in,
     OS.GCOL colour_to_gcol_in,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     out Colour gcol_to_colour_out,
     out OS.GCOL colour_to_gcol_out);

  [CCode (cname = "inversetable_sprite_table_for_sprite")]
  public static void SpriteTableForSprite
    (Colour gcol_to_colour_in,
     OS.GCOL colour_to_gcol_in,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     out Colour gcol_to_colour_out,
     out OS.GCOL colour_to_gcol_out);

  [CCode (cname = "xinversetable_sprite_table_for_mode")]
  public static OS.Error* XSpriteTableForMode
    (Colour gcol_to_colour_in,
     OS.GCOL colour_to_gcol_in,
     OS.Mode mode,
     out Colour gcol_to_colour_out,
     out OS.GCOL colour_to_gcol_out);

  [CCode (cname = "inversetable_sprite_table_for_mode")]
  public static void SpriteTableForMode
    (Colour gcol_to_colour_in,
     OS.GCOL colour_to_gcol_in,
     OS.Mode mode,
     out Colour gcol_to_colour_out,
     out OS.GCOL colour_to_gcol_out);

}
