/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;

[CCode (cheader_filename = "oslib/menu.h")]
namespace OSLib.Menu
{
  [CCode (cname = "menu_flags")]
  public enum Flags
  {
    [CCode (cname = "menu_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "menu_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "menu_GENERATE_HAS_BEEN_HIDDEN")]
    GenerateHasBeenHidden
  }

  [CCode (cname = "menu_entry_flags")]
  public enum EntryFlags
  {
    [CCode (cname = "menu_ENTRY_TICKED")]
    EntryTicked,
    [CCode (cname = "menu_ENTRY_SEPARATE")]
    EntrySeparate,
    [CCode (cname = "menu_ENTRY_FADED")]
    EntryFaded,
    [CCode (cname = "menu_ENTRY_IS_SPRITE")]
    EntryIsSprite,
    [CCode (cname = "menu_ENTRY_SUB_MENU")]
    EntrySubMenu,
    [CCode (cname = "menu_ENTRY_GENERATE_SUB_MENU_ACTION")]
    EntryGenerateSubMenuAction,
    [CCode (cname = "menu_ENTRY_IS_MENU")]
    EntryIsMenu
  }

  [CCode (cname = "menu_entry_object")]
  public struct EntryObject
  {
    [CCode (cname = "flags")]
    public unowned EntryFlags Flags;
    [CCode (cname = "cmp")]
    public unowned Toolbox.C Cmp;
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "click_object_name")]
    public unowned Toolbox.StringReference ClickObjectName;
    [CCode (cname = "sub_menu_object_name")]
    public unowned Toolbox.StringReference SubMenuObjectName;
    [CCode (cname = "sub_menu_action")]
    public unowned bits SubMenuAction;
    [CCode (cname = "click_action")]
    public unowned bits ClickAction;
    [CCode (cname = "help")]
    public unowned Toolbox.MsgReference Help;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
  }

  [CCode (cname = "menu_object_base")]
  public struct ObjectBase
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "help")]
    public unowned Toolbox.MsgReference Help;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
  }

  [Compact]
  [CCode (cname = "menu_object", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Object
  {
    public Object(int numEntries);
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "help")]
    public unowned Toolbox.MsgReference Help;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
    [CCode (cname = "entries", array_length = false)]
    public unowned EntryObject[] Entries;
  }

  [CCode (cname = "menu_object_template_base")]
  public struct ObjectTemplateBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "help")]
    public unowned Toolbox.MsgReference Help;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
  }

  [CCode (cname = "menu_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "help")]
    public unowned Toolbox.MsgReference Help;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "show_action")]
    public unowned bits ShowAction;
    [CCode (cname = "hide_action")]
    public unowned bits HideAction;
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
    [CCode (cname = "entries", array_length = false)]
    public unowned EntryObject[] Entries;
  }

  [CCode (cname = "menu_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "menu_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "menu_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "menu_action_has_been_hidden_block")]
  public enum ActionHasBeenHiddenBlock
  {
    Default = 0
  }

  [CCode (cname = "menu_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "menu_action_sub_menu")]
  public struct ActionSubMenu
  {
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
  }

  [CCode (cname = "menu_action_sub_menu_block")]
  public struct ActionSubMenuBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
  }

  [CCode (cname = "menu_action_selection_block")]
  public enum ActionSelectionBlock
  {
    Default = 0
  }

  [CCode (cname = "menu_show_flags")]
  public enum ShowFlags
  {
    [CCode (cname = "menu_SHOW_PERSISTENT")]
    ShowPersistent,
    [CCode (cname = "menu_SHOW_TRANSIENT")]
    ShowTransient
  }

  [CCode (cname = "menu_add_flags")]
  public enum AddFlags
  {
    [CCode (cname = "menu_ADD_BEFORE")]
    AddBefore
  }

  [CCode (cname = "menu_add_at")]
  public enum AddAt
  {
    [CCode (cname = "menu_ADD_AT_START")]
    Start,
    [CCode (cname = "menu_ADD_AT_END")]
    End
  }

  [CCode (cname="class_MENU")]
  public const Toolbox.Class Class_Menu;
  [CCode (cname="action_MENU_ABOUT_TO_BE_SHOWN")]
  public const bits Action_MenuAboutToBeShown;
  [CCode (cname="action_MENU_DIALOGUE_COMPLETED")]
  public const bits Action_MenuDialogueCompleted;
  [CCode (cname="action_MENU_HAS_BEEN_HIDDEN")]
  public const bits Action_MenuHasBeenHidden;
  [CCode (cname="action_MENU_SUB_MENU")]
  public const bits Action_MenuSubMenu;
  [CCode (cname="action_MENU_SELECTION")]
  public const bits Action_MenuSelection;
  [CCode (cname="error_MENU_TASKS_ACTIVE")]
  public const bits Error_MenuTasksActive;
  [CCode (cname="error_MENU_ALLOC_FAILED")]
  public const bits Error_MenuAllocFailed;
  [CCode (cname="error_MENU_SHORT_BUFFER")]
  public const bits Error_MenuShortBuffer;
  [CCode (cname="error_MENU_NO_SUCH_TASK")]
  public const bits Error_MenuNoSuchTask;
  [CCode (cname="error_MENU_NO_SUCH_METHOD")]
  public const bits Error_MenuNoSuchMethod;
  [CCode (cname="error_MENU_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_MenuNoSuchMiscOpMethod;
  [CCode (cname="error_MENU_NO_SUCH_COMPONENT")]
  public const bits Error_MenuNoSuchComponent;
  [CCode (cname="error_MENU_SPRITE_NOT_TEXT")]
  public const bits Error_MenuSpriteNotText;
  [CCode (cname="error_MENU_TEXT_NOT_SPRITE")]
  public const bits Error_MenuTextNotSprite;
  [CCode (cname="error_MENU_NO_TOP_MENU")]
  public const bits Error_MenuNoTopMenu;
  [CCode (cname="error_MENU_UNKNOWN_SUB_MENU")]
  public const bits Error_MenuUnknownSubMenu;
  [CCode (cname="error_MENU_NO_SPRITE_NAME")]
  public const bits Error_MenuNoSpriteName;

  [CCode (cname = "xmenu_update_tree")]
  public static OS.Error* XUpdateTree
    (bits flags);

  [CCode (cname = "menu_update_tree")]
  public static void UpdateTree
    (bits flags);

  [CCode (cname = "xmenu_set_tick")]
  public static OS.Error* XSetTick
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bool tick);

  [CCode (cname = "menu_set_tick")]
  public static void SetTick
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bool tick);

  [CCode (cname = "xmenu_get_tick")]
  public static OS.Error* XGetTick
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out bool tick);

  [CCode (cname = "menu_get_tick")]
  public static bool GetTick
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_set_fade")]
  public static OS.Error* XSetFade
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bool fade);

  [CCode (cname = "menu_set_fade")]
  public static void SetFade
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bool fade);

  [CCode (cname = "xmenu_get_fade")]
  public static OS.Error* XGetFade
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out bool fade);

  [CCode (cname = "menu_get_fade")]
  public static bool GetFade
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_set_entry_text")]
  public static OS.Error* XSetEntryText
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string text);

  [CCode (cname = "menu_set_entry_text")]
  public static void SetEntryText
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string text);

  [CCode (cname = "xmenu_get_entry_text")]
  public static OS.Error* XGetEntryText
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "menu_get_entry_text")]
  public static int GetEntryText
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size);

  [CCode (cname = "xmenu_set_entry_sprite")]
  public static OS.Error* XSetEntrySprite
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string sprite_name);

  [CCode (cname = "menu_set_entry_sprite")]
  public static void SetEntrySprite
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string sprite_name);

  [CCode (cname = "xmenu_get_entry_sprite")]
  public static OS.Error* XGetEntrySprite
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "menu_get_entry_sprite")]
  public static int GetEntrySprite
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size);

  [CCode (cname = "xmenu_set_sub_menu_show")]
  public static OS.Error* XSetSubMenuShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     Toolbox.O sub_menu);

  [CCode (cname = "menu_set_sub_menu_show")]
  public static void SetSubMenuShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     Toolbox.O sub_menu);

  [CCode (cname = "xmenu_get_sub_menu_show")]
  public static OS.Error* XGetSubMenuShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out Toolbox.O sub_menu);

  [CCode (cname = "menu_get_sub_menu_show")]
  public static Toolbox.O GetSubMenuShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_set_sub_menu_action")]
  public static OS.Error* XSetSubMenuAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bits action);

  [CCode (cname = "menu_set_sub_menu_action")]
  public static void SetSubMenuAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bits action);

  [CCode (cname = "xmenu_get_sub_menu_action")]
  public static OS.Error* XGetSubMenuAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out bits action);

  [CCode (cname = "menu_get_sub_menu_action")]
  public static bits GetSubMenuAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_set_click_show")]
  public static OS.Error* XSetClickShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     Toolbox.O show,
     ShowFlags show_flags);

  [CCode (cname = "menu_set_click_show")]
  public static void SetClickShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     Toolbox.O show,
     ShowFlags show_flags);

  [CCode (cname = "xmenu_get_click_show")]
  public static OS.Error* XGetClickShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out Toolbox.O show,
     out ShowFlags show_flags);

  [CCode (cname = "menu_get_click_show")]
  public static Toolbox.O GetClickShow
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out ShowFlags show_flags);

  [CCode (cname = "xmenu_set_click_action")]
  public static OS.Error* XSetClickAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bits action);

  [CCode (cname = "menu_set_click_action")]
  public static void SetClickAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     bits action);

  [CCode (cname = "xmenu_get_click_action")]
  public static OS.Error* XGetClickAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     out bits action);

  [CCode (cname = "menu_get_click_action")]
  public static bits GetClickAction
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_set_help_message")]
  public static OS.Error* XSetHelpMessage
    (bits flags,
     Toolbox.O menu,
     string help);

  [CCode (cname = "menu_set_help_message")]
  public static void SetHelpMessage
    (bits flags,
     Toolbox.O menu,
     string help);

  [CCode (cname = "xmenu_get_help_message")]
  public static OS.Error* XGetHelpMessage
    (bits flags,
     Toolbox.O menu,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "menu_get_help_message")]
  public static int GetHelpMessage
    (bits flags,
     Toolbox.O menu,
     string buffer,
     int size);

  [CCode (cname = "xmenu_set_entry_help_message")]
  public static OS.Error* XSetEntryHelpMessage
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string help);

  [CCode (cname = "menu_set_entry_help_message")]
  public static void SetEntryHelpMessage
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string help);

  [CCode (cname = "xmenu_get_entry_help_message")]
  public static OS.Error* XGetEntryHelpMessage
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "menu_get_entry_help_message")]
  public static int GetEntryHelpMessage
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry,
     string buffer,
     int size);

  [CCode (cname = "xmenu_add_entry")]
  public static OS.Error* XAddEntry
    (AddFlags flags,
     Toolbox.O menu,
     AddAt at,
     EntryObject entry,
     out Toolbox.C entry_out);

  [CCode (cname = "menu_add_entry")]
  public static Toolbox.C AddEntry
    (AddFlags flags,
     Toolbox.O menu,
     AddAt at,
     EntryObject entry);

  [CCode (cname = "xmenu_remove_entry")]
  public static OS.Error* XRemoveEntry
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "menu_remove_entry")]
  public static void RemoveEntry
    (bits flags,
     Toolbox.O menu,
     Toolbox.C entry);

  [CCode (cname = "xmenu_get_height")]
  public static OS.Error* XGetHeight
    (bits flags,
     Toolbox.O menu,
     out int height);

  [CCode (cname = "menu_get_height")]
  public static int GetHeight
    (bits flags,
     Toolbox.O menu);

  [CCode (cname = "xmenu_get_width")]
  public static OS.Error* XGetWidth
    (bits flags,
     Toolbox.O menu,
     out int width);

  [CCode (cname = "menu_get_width")]
  public static int GetWidth
    (bits flags,
     Toolbox.O menu);

  [CCode (cname = "xmenu_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O menu,
     string title);

  [CCode (cname = "menu_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O menu,
     string title);

  [CCode (cname = "xmenu_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O menu,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "menu_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O menu,
     string buffer,
     int size);

}
