/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/messagetrans.h")]
namespace OSLib.MessageTrans
{
  [CCode (cname = "messagetrans_control_block")]
  public struct ControlBlock
  {
    [CCode (cname = "cb")]
    public unowned int Cb[4];
  }

  [CCode (cname = "messagetrans_file_flags")]
  public enum FileFlags
  {
    [CCode (cname = "messagetrans_DIRECT_ACCESS")]
    DirectAccess
  }

  [CCode (cname="error_MESSAGE_TRANS_SYNTAX")]
  public const bits Error_MessageTransSyntax;
  [CCode (cname="error_MESSAGE_TRANS_FILE_OPEN")]
  public const bits Error_MessageTransFileOpen;
  [CCode (cname="error_MESSAGE_TRANS_TOKEN_NOT_FOUND")]
  public const bits Error_MessageTransTokenNotFound;
  [CCode (cname="error_MESSAGE_TRANS_RECURSE")]
  public const bits Error_MessageTransRecurse;

  [CCode (cname = "xmessagetrans_file_info")]
  public static OS.Error* XFileInfo
    (string file_name,
     out FileFlags flags,
     out int size);

  [CCode (cname = "messagetrans_file_info")]
  public static void FileInfo
    (string file_name,
     out FileFlags flags,
     out int size);

  [CCode (cname = "xmessagetrans_open_file")]
  public static OS.Error* XOpenFile
    (ControlBlock cb,
     string file_name,
     string buffer);

  [CCode (cname = "messagetrans_open_file")]
  public static void OpenFile
    (ControlBlock cb,
     string file_name,
     string buffer);

  [CCode (cname = "xmessagetrans_lookup")]
  public static OS.Error* XLookup
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3,
     out string result,
     out int used);

  [CCode (cname = "messagetrans_lookup")]
  public static string Lookup
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3,
     out int used);

  [CCode (cname = "xmessagetrans_make_menus")]
  public static OS.Error* XMakeMenus
    (ControlBlock cb,
     char* menus,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "messagetrans_make_menus")]
  public static int MakeMenus
    (ControlBlock cb,
     char* menus,
     string buffer,
     int size);

  [CCode (cname = "xmessagetrans_close_file")]
  public static OS.Error* XCloseFile
    (ControlBlock cb);

  [CCode (cname = "messagetrans_close_file")]
  public static void CloseFile
    (ControlBlock cb);

  [CCode (cname = "xmessagetrans_enumerate_tokens")]
  public static OS.Error* XEnumerateTokens
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     int context,
     out bool more,
     out int used,
     out int context_out);

  [CCode (cname = "messagetrans_enumerate_tokens")]
  public static bool EnumerateTokens
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     int context,
     out int used,
     out int context_out);

  [CCode (cname = "xmessagetrans_error_lookup")]
  public static OS.Error* XErrorLookup
    (OS.Error error,
     ControlBlock cb,
     OS.Error buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3);

  [CCode (cname = "messagetrans_error_lookup")]
  public static void ErrorLookup
    (OS.Error error,
     ControlBlock cb,
     OS.Error buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3);

  [CCode (cname = "xmessagetrans_gs_lookup")]
  public static OS.Error* XGSLookup
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3,
     out string result,
     out int used);

  [CCode (cname = "messagetrans_gs_lookup")]
  public static string GSLookup
    (ControlBlock cb,
     string token,
     string buffer,
     int size,
     string arg0,
     string arg1,
     string arg2,
     string arg3,
     out int used);

  [CCode (cname = "xmessagetrans_copy_error")]
  public static OS.Error* XCopyError
    (OS.Error error,
     out OS.Error copy);

  [CCode (cname = "messagetrans_copy_error")]
  public static OS.Error CopyError
    (OS.Error error);

  [CCode (cname = "xmessagetrans_dictionary")]
  public static OS.Error* XDictionary
    (out char* dictionary);

  [CCode (cname = "messagetrans_dictionary")]
  public static char* Dictionary ();

  [CCode (cname = "xservice_message_file_closed")]
  public static OS.Error* XService_MessageFileClosed
    (ControlBlock cb);

  [CCode (cname = "service_message_file_closed")]
  public static void Service_MessageFileClosed
    (ControlBlock cb);

}
