/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/mimemap.h")]
namespace OSLib.MimeMap
{
  [CCode (cname = "mimemap_format")]
  public enum Format
  {
    [CCode (cname = "mimemap_FORMAT_FILETYPE")]
    Filetype,
    [CCode (cname = "mimemap_FORMAT_FILETYPE_NAME")]
    FiletypeName,
    [CCode (cname = "mimemap_FORMAT_MIME_TYPE")]
    MIMEType,
    [CCode (cname = "mimemap_FORMAT_EXTENSION")]
    Extension
  }


  [CCode (cname = "xmimemaptranslate_file_type_to_anything")]
  public static OS.Error* XMimeMapTranslate_FileTypeToAnything
    (bits file_type,
     Format output_format,
     char* output);

  [CCode (cname = "mimemaptranslate_file_type_to_anything")]
  public static void MimeMapTranslate_FileTypeToAnything
    (bits file_type,
     Format output_format,
     char* output);

  [CCode (cname = "xmimemaptranslate_anything_to_file_type")]
  public static OS.Error* XMimeMapTranslate_AnythingToFileType
    (Format input_format,
     char* input,
     out bits file_type);

  [CCode (cname = "mimemaptranslate_anything_to_file_type")]
  public static bits MimeMapTranslate_AnythingToFileType
    (Format input_format,
     char* input);

  [CCode (cname = "xmimemaptranslate_filetype_to_filetype_name")]
  public static OS.Error* XMimeMapTranslate_FiletypeToFiletypeName
    (bits file_type,
     string file_type_name);

  [CCode (cname = "mimemaptranslate_filetype_to_filetype_name")]
  public static void MimeMapTranslate_FiletypeToFiletypeName
    (bits file_type,
     string file_type_name);

  [CCode (cname = "xmimemaptranslate_filetype_to_mime_type")]
  public static OS.Error* XMimeMapTranslate_FiletypeToMIMEType
    (bits file_type,
     string mime_type);

  [CCode (cname = "mimemaptranslate_filetype_to_mime_type")]
  public static void MimeMapTranslate_FiletypeToMIMEType
    (bits file_type,
     string mime_type);

  [CCode (cname = "xmimemaptranslate_filetype_to_extension")]
  public static OS.Error* XMimeMapTranslate_FiletypeToExtension
    (bits file_type,
     string extension);

  [CCode (cname = "mimemaptranslate_filetype_to_extension")]
  public static void MimeMapTranslate_FiletypeToExtension
    (bits file_type,
     string extension);

  [CCode (cname = "xmimemaptranslate_filetype_name_to_filetype")]
  public static OS.Error* XMimeMapTranslate_FiletypeNameToFiletype
    (string file_type_name,
     out bits file_type);

  [CCode (cname = "mimemaptranslate_filetype_name_to_filetype")]
  public static bits MimeMapTranslate_FiletypeNameToFiletype
    (string file_type_name);

  [CCode (cname = "xmimemaptranslate_filetype_name_to_mime_type")]
  public static OS.Error* XMimeMapTranslate_FiletypeNameToMIMEType
    (string file_type_name,
     string mime_type);

  [CCode (cname = "mimemaptranslate_filetype_name_to_mime_type")]
  public static void MimeMapTranslate_FiletypeNameToMIMEType
    (string file_type_name,
     string mime_type);

  [CCode (cname = "xmimemaptranslate_filetype_name_to_extension")]
  public static OS.Error* XMimeMapTranslate_FiletypeNameToExtension
    (string file_type_name,
     string extension);

  [CCode (cname = "mimemaptranslate_filetype_name_to_extension")]
  public static void MimeMapTranslate_FiletypeNameToExtension
    (string file_type_name,
     string extension);

  [CCode (cname = "xmimemaptranslate_mime_type_to_filetype")]
  public static OS.Error* XMimeMapTranslate_MIMETypeToFiletype
    (string mime_type,
     out bits file_type);

  [CCode (cname = "mimemaptranslate_mime_type_to_filetype")]
  public static bits MimeMapTranslate_MIMETypeToFiletype
    (string mime_type);

  [CCode (cname = "xmimemaptranslate_mime_type_to_filetype_name")]
  public static OS.Error* XMimeMapTranslate_MIMETypeToFiletypeName
    (string mime_type,
     string file_type_name);

  [CCode (cname = "mimemaptranslate_mime_type_to_filetype_name")]
  public static void MimeMapTranslate_MIMETypeToFiletypeName
    (string mime_type,
     string file_type_name);

  [CCode (cname = "xmimemaptranslate_mime_type_to_extension")]
  public static OS.Error* XMimeMapTranslate_MIMETypeToExtension
    (string mime_type,
     string extension);

  [CCode (cname = "mimemaptranslate_mime_type_to_extension")]
  public static void MimeMapTranslate_MIMETypeToExtension
    (string mime_type,
     string extension);

  [CCode (cname = "xmimemaptranslate_extension_to_filetype")]
  public static OS.Error* XMimeMapTranslate_ExtensionToFiletype
    (string extension,
     out bits file_type);

  [CCode (cname = "mimemaptranslate_extension_to_filetype")]
  public static bits MimeMapTranslate_ExtensionToFiletype
    (string extension);

  [CCode (cname = "xmimemaptranslate_extension_to_filetype_name")]
  public static OS.Error* XMimeMapTranslate_ExtensionToFiletypeName
    (string extension,
     string file_type_name);

  [CCode (cname = "mimemaptranslate_extension_to_filetype_name")]
  public static void MimeMapTranslate_ExtensionToFiletypeName
    (string extension,
     string file_type_name);

  [CCode (cname = "xmimemaptranslate_extension_to_mime_type")]
  public static OS.Error* XMimeMapTranslate_ExtensionToMIMEType
    (string extension,
     string mime_type);

  [CCode (cname = "mimemaptranslate_extension_to_mime_type")]
  public static void MimeMapTranslate_ExtensionToMIMEType
    (string extension,
     string mime_type);

}
