/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/nfs.h")]
namespace OSLib.NFS
{
  [CCode (cname="nfs_NOBODY_ID")]
  public const int NobodyId;
  [CCode (cname="nfs_NOBODY_GROUP")]
  public const int NobodyGroup;

  [CCode (cname = "xnfs_mount")]
  public static OS.Error* XMount
    (string file_server,
     string mount_name,
     string path);

  [CCode (cname = "nfs_mount")]
  public static void Mount
    (string file_server,
     string mount_name,
     string path);

  [CCode (cname = "xnfs_mount_list")]
  public static OS.Error* XMountList
    (string file_server,
     string context,
     out string context_out);

  [CCode (cname = "nfs_mount_list")]
  public static string MountList
    (string file_server,
     string context);

  [CCode (cname = "xnfs_set_user")]
  public static OS.Error* XSetUser
    (string name_server,
     string user_name,
     string password);

  [CCode (cname = "nfs_set_user")]
  public static void SetUser
    (string name_server,
     string user_name,
     string password);

  [CCode (cname = "xnfs_dismount")]
  public static OS.Error* XDismount
    (string file_server,
     string mount_name);

  [CCode (cname = "nfs_dismount")]
  public static void Dismount
    (string file_server,
     string mount_name);

  [CCode (cname = "xnfs_mount_info")]
  public static OS.Error* XMountInfo
    (string mount_name,
     out string file_server,
     out string path,
     out string user_name,
     out string name_server);

  [CCode (cname = "nfs_mount_info")]
  public static void MountInfo
    (string mount_name,
     out string file_server,
     out string path,
     out string user_name,
     out string name_server);

  [CCode (cname = "xnfs_free_space")]
  public static OS.Error* XFreeSpace
    (string mount_name,
     out int free,
     out int size,
     out int unused);

  [CCode (cname = "nfs_free_space")]
  public static int FreeSpace
    (string mount_name,
     out int size,
     out int unused);

  [CCode (cname = "xservice_nfs")]
  public static OS.Error* XService_NFS ();

  [CCode (cname = "service_nfs")]
  public static void Service_NFS ();

}
