/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/osargs.h")]
namespace OSLib.OSArgs
{
  [CCode (cname = "osargs_stream_status")]
  public enum StreamStatus
  {
    [CCode (cname = "osargs_STREAM_INTERACTIVE")]
    StreamInteractive,
    [CCode (cname = "osargs_STREAM_SUPPORTS_GBPB")]
    StreamSupportsGBPB,
    [CCode (cname = "osargs_STREAM_DIR")]
    StreamDir,
    [CCode (cname = "osargs_STREAM_READABLE")]
    StreamReadable,
    [CCode (cname = "osargs_STREAM_WRITABLE")]
    StreamWritable,
    [CCode (cname = "osargs_STREAM_WRITTEN")]
    StreamWritten,
    [CCode (cname = "osargs_STREAM_EOF_ERROR_ON_NEXT_READ")]
    StreamEOFErrorOnNextRead,
    [CCode (cname = "osargs_STREAM_UNBUFFERED")]
    StreamUnbuffered,
    [CCode (cname = "osargs_STREAM_UNALLOCATED")]
    StreamUnallocated,
    [CCode (cname = "osargs_STREAM_CRITICAL")]
    StreamCritical,
    [CCode (cname = "osargs_STREAM_DATA_LOST")]
    StreamDataLost,
    [CCode (cname = "osargs_STREAM_IMAGE_FILE_BUSY")]
    StreamImageFileBusy
  }

  [CCode (cname = "osargs_io_ctl_operation")]
  public enum IOCtlOperation
  {
    [CCode (cname = "osargs_IO_CTL_REASON")]
    IOCtlReason,
    [CCode (cname = "osargs_IO_CTL_GROUP")]
    IOCtlGroup,
    [CCode (cname = "osargs_IO_CTL_READ_VALUE")]
    IOCtlReadValue,
    [CCode (cname = "osargs_IO_CTL_WRITE_VALUE")]
    IOCtlWriteValue,
    [CCode (cname = "osargs_IO_CTL_BAUD_RATE")]
    IOCtlBaudRate,
    [CCode (cname = "osargs_IO_CTL_DATA_FORMAT")]
    IOCtlDataFormat,
    [CCode (cname = "osargs_IO_CTL_HANDSHAKING")]
    IOCtlHandshaking,
    [CCode (cname = "osargs_IO_CTL_BUFFER_SIZE")]
    IOCtlBufferSize,
    [CCode (cname = "osargs_IO_CTL_BUFFER_THRESHOLDS")]
    IOCtlBufferThresholds,
    [CCode (cname = "osargs_IO_CTL_CONTROL_LINES")]
    IOCtlControlLines,
    [CCode (cname = "osargs_IO_CTL_FIFO_TRIGGER_LEVEL")]
    IOCtlFIFOTriggerLevel,
    [CCode (cname = "osargs_IO_CTL_SUPPORTED_BAUD_RATE_COUNT")]
    IOCtlSupportedBaudRateCount,
    [CCode (cname = "osargs_IO_CTL_SUPPORTED_BAUD_RATE")]
    IOCtlSupportedBaudRate
  }

  [CCode (cname = "osargs_io_ctl_parameters")]
  public struct IOCtlParameters
  {
    [CCode (cname = "operation")]
    public unowned IOCtlOperation Operation;
    [CCode (cname = "value")]
    public unowned int Value;
  }

  [CCode (cname="osargs_IO_CTL_REASON_SHIFT")]
  public const int IOCtlReasonShift;
  [CCode (cname="osargs_IO_CTL_GROUP_SHIFT")]
  public const int IOCtlGroupShift;

  [CCode (cname = "xosargs_read_ptr")]
  public static OS.Error* XReadPtr
    (OS.F file,
     out int ptr);

  [CCode (cname = "osargs_read_ptr")]
  public static int ReadPtr
    (OS.F file);

  [CCode (cname = "xosargs_read_ptrw")]
  public static OS.Error* XReadPtrW
    (OS.FW file,
     out int ptr);

  [CCode (cname = "osargs_read_ptrw")]
  public static int ReadPtrW
    (OS.FW file);

  [CCode (cname = "xosargs_read_temporary_fs")]
  public static OS.Error* XReadTemporaryFS
    (out FileSwitch.FSNo temp_fs_no);

  [CCode (cname = "osargs_read_temporary_fs")]
  public static FileSwitch.FSNo ReadTemporaryFS ();

  [CCode (cname = "xosargs_set_ptr")]
  public static OS.Error* XSetPtr
    (OS.F file,
     int ptr);

  [CCode (cname = "osargs_set_ptr")]
  public static void SetPtr
    (OS.F file,
     int ptr);

  [CCode (cname = "xosargs_set_ptrw")]
  public static OS.Error* XSetPtrW
    (OS.FW file,
     int ptr);

  [CCode (cname = "osargs_set_ptrw")]
  public static void SetPtrW
    (OS.FW file,
     int ptr);

  [CCode (cname = "xosargs_read_ext")]
  public static OS.Error* XReadExt
    (OS.F file,
     out int ext);

  [CCode (cname = "osargs_read_ext")]
  public static int ReadExt
    (OS.F file);

  [CCode (cname = "xosargs_read_extw")]
  public static OS.Error* XReadExtW
    (OS.FW file,
     out int ext);

  [CCode (cname = "osargs_read_extw")]
  public static int ReadExtW
    (OS.FW file);

  [CCode (cname = "xosargs_set_ext")]
  public static OS.Error* XSetExt
    (OS.F file,
     int ext);

  [CCode (cname = "osargs_set_ext")]
  public static void SetExt
    (OS.F file,
     int ext);

  [CCode (cname = "xosargs_set_extw")]
  public static OS.Error* XSetExtW
    (OS.FW file,
     int ext);

  [CCode (cname = "osargs_set_extw")]
  public static void SetExtW
    (OS.FW file,
     int ext);

  [CCode (cname = "xosargs_read_allocation")]
  public static OS.Error* XReadAllocation
    (OS.F file,
     out int allocation);

  [CCode (cname = "osargs_read_allocation")]
  public static int ReadAllocation
    (OS.F file);

  [CCode (cname = "xosargs_read_allocationw")]
  public static OS.Error* XReadAllocationW
    (OS.FW file,
     out int allocation);

  [CCode (cname = "osargs_read_allocationw")]
  public static int ReadAllocationW
    (OS.FW file);

  [CCode (cname = "xosargs_read_eof_status")]
  public static OS.Error* XReadEOFStatus
    (OS.F file,
     out bool eof_status);

  [CCode (cname = "osargs_read_eof_status")]
  public static bool ReadEOFStatus
    (OS.F file);

  [CCode (cname = "xosargs_read_eof_statusw")]
  public static OS.Error* XReadEOFStatusW
    (OS.FW file,
     out bool eof_status);

  [CCode (cname = "osargs_read_eof_statusw")]
  public static bool ReadEOFStatusW
    (OS.FW file);

  [CCode (cname = "xosargs_set_allocation")]
  public static OS.Error* XSetAllocation
    (OS.F file,
     int allocation,
     out int allocation_out);

  [CCode (cname = "osargs_set_allocation")]
  public static int SetAllocation
    (OS.F file,
     int allocation);

  [CCode (cname = "xosargs_set_allocationw")]
  public static OS.Error* XSetAllocationW
    (OS.FW file,
     int allocation,
     out int allocation_out);

  [CCode (cname = "osargs_set_allocationw")]
  public static int SetAllocationW
    (OS.FW file,
     int allocation);

  [CCode (cname = "xosargs_read_path")]
  public static OS.Error* XReadPath
    (OS.F file,
     string buffer,
     int size,
     out int spare);

  [CCode (cname = "osargs_read_path")]
  public static int ReadPath
    (OS.F file,
     string buffer,
     int size);

  [CCode (cname = "xosargs_read_pathw")]
  public static OS.Error* XReadPathW
    (OS.FW file,
     string buffer,
     int size,
     out int spare);

  [CCode (cname = "osargs_read_pathw")]
  public static int ReadPathW
    (OS.FW file,
     string buffer,
     int size);

  [CCode (cname = "xosargs_image_stamped")]
  public static OS.Error* XImageStamped
    (OS.F file,
     int image_stamp);

  [CCode (cname = "osargs_image_stamped")]
  public static void ImageStamped
    (OS.F file,
     int image_stamp);

  [CCode (cname = "xosargs_image_stampedw")]
  public static OS.Error* XImageStampedW
    (OS.FW file,
     int image_stamp);

  [CCode (cname = "osargs_image_stampedw")]
  public static void ImageStampedW
    (OS.FW file,
     int image_stamp);

  [CCode (cname = "xosargs_io_ctl")]
  public static OS.Error* XIOCtl
    (OS.F file,
     IOCtlParameters parameters);

  [CCode (cname = "osargs_io_ctl")]
  public static void IOCtl
    (OS.F file,
     IOCtlParameters parameters);

  [CCode (cname = "xosargs_read_info")]
  public static OS.Error* XReadInfo
    (OS.F file,
     out StreamStatus stream_status,
     out FileSwitch.FSInfo fs_info);

  [CCode (cname = "osargs_read_info")]
  public static void ReadInfo
    (OS.F file,
     out StreamStatus stream_status,
     out FileSwitch.FSInfo fs_info);

  [CCode (cname = "xosargs_read_infow")]
  public static OS.Error* XReadInfoW
    (OS.FW file,
     out StreamStatus stream_status,
     out FileSwitch.FSInfo fs_info);

  [CCode (cname = "osargs_read_infow")]
  public static void ReadInfoW
    (OS.FW file,
     out StreamStatus stream_status,
     out FileSwitch.FSInfo fs_info);

  [CCode (cname = "xosargs_ensure")]
  public static OS.Error* XEnsure
    (OS.F file);

  [CCode (cname = "osargs_ensure")]
  public static void Ensure
    (OS.F file);

  [CCode (cname = "xosargs_ensurew")]
  public static OS.Error* XEnsureW
    (OS.FW file);

  [CCode (cname = "osargs_ensurew")]
  public static void EnsureW
    (OS.FW file);

}
