/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osbyte.h")]
namespace OSLib.OSByte
{
  [CCode (cname = "osbyte_op")]
  public enum Op
  {
    [CCode (cname = "osbyte_VERSION")]
    Version,
    [CCode (cname = "osbyte_USER_FLAG")]
    UserFlag,
    [CCode (cname = "osbyte_INPUT_STREAM")]
    InputStream,
    [CCode (cname = "osbyte_OUTPUT_STREAMS")]
    OutputStreams,
    [CCode (cname = "osbyte_INTERPRETATION_ARROWS")]
    InterpretationArrows,
    [CCode (cname = "osbyte_PRINTER")]
    Printer,
    [CCode (cname = "osbyte_IGNORE_CHAR")]
    IgnoreChar,
    [CCode (cname = "osbyte_SERIAL_RECEIVE_RATE")]
    SerialReceiveRate,
    [CCode (cname = "osbyte_SERIAL_TRANSMIT_RATE")]
    SerialTransmitRate,
    [CCode (cname = "osbyte_ON_DURATION")]
    OnDuration,
    [CCode (cname = "osbyte_OFF_DURATION")]
    OffDuration,
    [CCode (cname = "osbyte_KEYBOARD_DELAY")]
    KeyboardDelay,
    [CCode (cname = "osbyte_KEYBOARD_RATE")]
    KeyboardRate,
    [CCode (cname = "osbyte_DISABLE_EVENT")]
    DisableEvent,
    [CCode (cname = "osbyte_ENABLE_EVENT")]
    EnableEvent,
    [CCode (cname = "osbyte_FLUSH_BUFFERS")]
    FlushBuffers,
    [CCode (cname = "osbyte_CHANNEL_COUNT")]
    ChannelCount,
    [CCode (cname = "osbyte_CONVERT_CHANNEL")]
    ConvertChannel,
    [CCode (cname = "osbyte_RESET_FUNCTION_KEYS")]
    ResetFunctionKeys,
    [CCode (cname = "osbyte_AWAIT_VSYNC")]
    AwaitVSync,
    [CCode (cname = "osbyte_RESET_SYSTEM_FONT")]
    ResetSystemFont,
    [CCode (cname = "osbyte_FLUSH_BUFFER")]
    FlushBuffer,
    [CCode (cname = "osbyte_INCREMENT_ROM_SEMAPHORE")]
    IncrementROMSemaphore,
    [CCode (cname = "osbyte_DECREMENT_ROM_SEMAPHORE")]
    DecrementROMSemaphore,
    [CCode (cname = "osbyte_RESET_SYSTEM_CHARS")]
    ResetSystemChars,
    [CCode (cname = "osbyte_ECONET_POLL_TRANSMIT")]
    EconetPollTransmit,
    [CCode (cname = "osbyte_ECONET_POLL_RECEIVE")]
    EconetPollReceive,
    [CCode (cname = "osbyte_ECONET_ABANDON_RECEIVE")]
    EconetAbandonReceive,
    [CCode (cname = "osbyte_ECONET_ROFF")]
    EconetROff,
    [CCode (cname = "osbyte_BANKED_RAM_PRESENCE")]
    BankedRAMPresence,
    [CCode (cname = "osbyte_BANKED_RAM_USE")]
    BankedRAMUse,
    [CCode (cname = "osbyte_COUNTRY_NUMBER")]
    CountryNumber,
    [CCode (cname = "osbyte_ALPHABET_NUMBER")]
    AlphabetNumber,
    [CCode (cname = "osbyte_SELECT_POINTER")]
    SelectPointer,
    [CCode (cname = "osbyte_WRITE_BUS_STATE")]
    WriteBusState,
    [CCode (cname = "osbyte_WRITE_SHADOW_STATE")]
    WriteShadowState,
    [CCode (cname = "osbyte_SELECT_TEMPORARY_FS")]
    SelectTemporaryFS,
    [CCode (cname = "osbyte_OUTPUT_SCREEN_BANK")]
    OutputScreenBank,
    [CCode (cname = "osbyte_DISPLAY_SCREEN_BANK")]
    DisplayScreenBank,
    [CCode (cname = "osbyte_SHADOW_STATE")]
    ShadowState,
    [CCode (cname = "osbyte_VDU_STATE")]
    VDUState,
    [CCode (cname = "osbyte_UPDATE_KEYBOARD")]
    UpdateKeyboard,
    [CCode (cname = "osbyte_CLOSE_SPOOL_AND_EXEC")]
    CloseSpoolAndExec,
    [CCode (cname = "osbyte_LOCK_REPEAT")]
    LockRepeat,
    [CCode (cname = "osbyte_SCAN_KEYBOARD")]
    ScanKeyboard,
    [CCode (cname = "osbyte_SCAN_KEYBOARD_LIMITED")]
    ScanKeyboardLimited,
    [CCode (cname = "osbyte_PRINTER_DORMANT")]
    PrinterDormant,
    [CCode (cname = "osbyte_CLEAR_ESCAPE")]
    ClearEscape,
    [CCode (cname = "osbyte_SET_ESCAPE")]
    SetEscape,
    [CCode (cname = "osbyte_ACKNOWLEDGE_ESCAPE")]
    AcknowledgeEscape,
    [CCode (cname = "osbyte_READ_EOF_STATUS")]
    ReadEOFStatus,
    [CCode (cname = "osbyte_BUFFER_OP")]
    BufferOp,
    [CCode (cname = "osbyte_IN_KEY")]
    InKey,
    [CCode (cname = "osbyte_READ_HIGH_ORDER_ADDRESS")]
    ReadHighOrderAddress,
    [CCode (cname = "osbyte_READ_HWM")]
    ReadHWM,
    [CCode (cname = "osbyte_READ_TOP")]
    ReadTop,
    [CCode (cname = "osbyte_READ_TOP_FOR_MODE")]
    ReadTopForMode,
    [CCode (cname = "osbyte_TEXT_CURSOR_POSITION")]
    TextCursorPosition,
    [CCode (cname = "osbyte_SCREEN_CHAR")]
    ScreenChar,
    [CCode (cname = "osbyte_RUN")]
    Run,
    [CCode (cname = "osbyte_CASSETTE_MOTOR")]
    CassetteMotor,
    [CCode (cname = "osbyte_BUFFER_INSERT")]
    BufferInsert,
    [CCode (cname = "osbyte_OPT")]
    Opt,
    [CCode (cname = "osbyte_SELECT_CFS")]
    SelectCFS,
    [CCode (cname = "osbyte_SELECT_ROMFS")]
    SelectROMFS,
    [CCode (cname = "osbyte_ROM_ENTER")]
    ROMEnter,
    [CCode (cname = "osbyte_SERVICE_CALL")]
    ServiceCall,
    [CCode (cname = "osbyte_TV")]
    TV,
    [CCode (cname = "osbyte_BUFFER_REMOVE")]
    BufferRemove,
    [CCode (cname = "osbyte_READ_FRED")]
    ReadFred,
    [CCode (cname = "osbyte_WRITE_FRED")]
    WriteFred,
    [CCode (cname = "osbyte_READ_JIM")]
    ReadJim,
    [CCode (cname = "osbyte_WRITE_JIM")]
    WriteJim,
    [CCode (cname = "osbyte_READ_SHEILA")]
    ReadSheila,
    [CCode (cname = "osbyte_WRITE_SHEILA")]
    WriteSheila,
    [CCode (cname = "osbyte_BUFFER_STATUS")]
    BufferStatus,
    [CCode (cname = "osbyte_TYPE_CHAR")]
    TypeChar,
    [CCode (cname = "osbyte_WRITE_VDU_CONTROL")]
    WriteVDUControl,
    [CCode (cname = "osbyte_WRITE_VDU_PALETTE")]
    WriteVDUPalette,
    [CCode (cname = "osbyte_SERIAL_STATE")]
    SerialState,
    [CCode (cname = "osbyte_WRITE_TUBE")]
    WriteTube,
    [CCode (cname = "osbyte_READ_VDU_VARIABLE")]
    ReadVDUVariable,
    [CCode (cname = "osbyte_READ_CMOS")]
    ReadCMOS,
    [CCode (cname = "osbyte_WRITE_CMOS")]
    WriteCMOS,
    [CCode (cname = "osbyte_GRAPHICS_INFO")]
    GraphicsInfo,
    [CCode (cname = "osbyte_CPU_TYPE")]
    CPUType,
    [CCode (cname = "osbyte_OUTPUT_CURSOR_POSITION")]
    OutputCursorPosition,
    [CCode (cname = "osbyte_READ_VARIABLE_START")]
    ReadVariableStart,
    [CCode (cname = "osbyte_READ_ROM_POINTER_TABLE")]
    ReadROMPointerTable,
    [CCode (cname = "osbyte_READ_ROM_INFO_TABLE")]
    ReadROMInfoTable,
    [CCode (cname = "osbyte_READ_KEYBOARD_TABLE")]
    ReadKeyboardTable,
    [CCode (cname = "osbyte_READ_VDU_VARIABLE_START")]
    ReadVDUVariableStart
  }

  [CCode (cname = "osbyte_var")]
  public enum Var
  {
    [CCode (cname = "osbyte_VAR_VSYNC_TIMER")]
    VSyncTimer,
    [CCode (cname = "osbyte_VAR_INPUT_STREAM")]
    InputStream,
    [CCode (cname = "osbyte_VAR_KEYBOARD_SEMAPHORE")]
    KeyboardSemaphore,
    [CCode (cname = "osbyte_VAR_ROM_SEMAPHORE")]
    ROMSemaphore,
    [CCode (cname = "osbyte_VAR_HWM")]
    HWM,
    [CCode (cname = "osbyte_VAR_SERIAL_INTERPRETATION")]
    SerialInterpretation,
    [CCode (cname = "osbyte_VAR_IGNORE_STATE")]
    IgnoreState,
    [CCode (cname = "osbyte_VAR_CASSETTE_ROM_SWITCH")]
    CassetteROMSwitch,
    [CCode (cname = "osbyte_VAR_VDU_CONTROL")]
    VDUControl,
    [CCode (cname = "osbyte_VAR_VDU_PALETTE")]
    VDUPalette,
    [CCode (cname = "osbyte_VAR_ROM_LAST_RESET")]
    ROMLastReset,
    [CCode (cname = "osbyte_VAR_ROM_BASIC")]
    ROM_BASIC,
    [CCode (cname = "osbyte_VAR_CURRENT_CHANNEL")]
    CurrentChannel,
    [CCode (cname = "osbyte_VAR_CHANNEL_COUNT")]
    ChannelCount,
    [CCode (cname = "osbyte_VAR_CONVERSION_RESOLUTION")]
    ConversionResolution,
    [CCode (cname = "osbyte_VAR_SERIAL_BUSY")]
    SerialBusy,
    [CCode (cname = "osbyte_VAR_SERIAL_STATE")]
    SerialState,
    [CCode (cname = "osbyte_VAR_FLASH_COUNTER")]
    FlashCounter,
    [CCode (cname = "osbyte_VAR_OFF_DURATION")]
    OffDuration,
    [CCode (cname = "osbyte_VAR_ON_DURATION")]
    OnDuration,
    [CCode (cname = "osbyte_VAR_KEYBOARD_DELAY")]
    KeyboardDelay,
    [CCode (cname = "osbyte_VAR_KEYBOARD_RATE")]
    KeyboardRate,
    [CCode (cname = "osbyte_VAR_EXEC_HANDLE")]
    ExecHandle,
    [CCode (cname = "osbyte_VAR_SPOOL_HANDLE")]
    SpoolHandle,
    [CCode (cname = "osbyte_VAR_RESET_EFFECTS")]
    ResetEffects,
    [CCode (cname = "osbyte_VAR_KEYBOARD_DISABLE")]
    KeyboardDisable,
    [CCode (cname = "osbyte_VAR_KEYBOARD_STATE")]
    KeyboardState,
    [CCode (cname = "osbyte_VAR_SERIAL_THRESHOLD")]
    SerialThreshold,
    [CCode (cname = "osbyte_VAR_SERIAL_BUFFER_STATE")]
    SerialBufferState,
    [CCode (cname = "osbyte_VAR_SERIAL_DESTINATION")]
    SerialDestination,
    [CCode (cname = "osbyte_VAR_ECONET_OS_CALL_STATUS")]
    EconetOSCallStatus,
    [CCode (cname = "osbyte_VAR_ECONET_INPUT_STATUS")]
    EconetInputStatus,
    [CCode (cname = "osbyte_VAR_ECONET_OUTPUT_STATUS")]
    EconetOutputStatus,
    [CCode (cname = "osbyte_VAR_SOUND_SUPPRESSION")]
    SoundSuppression,
    [CCode (cname = "osbyte_VAR_BELL_CHANNEL")]
    BellChannel,
    [CCode (cname = "osbyte_VAR_BELL_VOLUME")]
    BellVolume,
    [CCode (cname = "osbyte_VAR_BELL_FREQUENCY")]
    BellFrequency,
    [CCode (cname = "osbyte_VAR_BELL_DURATION")]
    BellDuration,
    [CCode (cname = "osbyte_VAR_BANNER_SUPPRESS")]
    BannerSuppress,
    [CCode (cname = "osbyte_VAR_PENDING_KEY_LENGTH")]
    PendingKeyLength,
    [CCode (cname = "osbyte_VAR_PAGE_LENGTH")]
    PageLength,
    [CCode (cname = "osbyte_VAR_VDU_QUEUE_LENGTH")]
    VDUQueueLength,
    [CCode (cname = "osbyte_VAR_TAB_CHAR")]
    TabChar,
    [CCode (cname = "osbyte_VAR_ESCAPE_CHAR")]
    EscapeChar,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP4")]
    InterpretationGroup4,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP5")]
    InterpretationGroup5,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP6")]
    InterpretationGroup6,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP7")]
    InterpretationGroup7,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP0")]
    InterpretationGroup0,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP1")]
    InterpretationGroup1,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP2")]
    InterpretationGroup2,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_GROUP3")]
    InterpretationGroup3,
    [CCode (cname = "osbyte_VAR_ESCAPE_STATE")]
    EscapeState,
    [CCode (cname = "osbyte_VAR_ESCAPE_EFFECTS")]
    EscapeEffects,
    [CCode (cname = "osbyte_VAR_IRQ_MASK_USER")]
    IRQMaskUser,
    [CCode (cname = "osbyte_VAR_IRQ_MASK_SERIAL")]
    IRQMaskSerial,
    [CCode (cname = "osbyte_VAR_IRQ_MASK_SYSTEM")]
    IRQMaskSystem,
    [CCode (cname = "osbyte_VAR_TUBE")]
    Tube,
    [CCode (cname = "osbyte_VAR_OUTPUT_STREAMS")]
    OutputStreams,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_ARROWS")]
    InterpretationArrows,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_KEYPAD")]
    InterpretationKeypad,
    [CCode (cname = "osbyte_VAR_SHADOW_STATE")]
    ShadowState,
    [CCode (cname = "osbyte_VAR_COUNTRY_NUMBER")]
    CountryNumber,
    [CCode (cname = "osbyte_VAR_USER_FLAG")]
    UserFlag,
    [CCode (cname = "osbyte_VAR_SERIAL_RATE")]
    SerialRate,
    [CCode (cname = "osbyte_VAR_INTERVAL_TIMER_STATE")]
    IntervalTimerState,
    [CCode (cname = "osbyte_VAR_SOFT_KEY_STATE")]
    SoftKeyState,
    [CCode (cname = "osbyte_VAR_PRINTER")]
    Printer,
    [CCode (cname = "osbyte_VAR_IGNORE_CHAR")]
    IgnoreChar,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_BREAK")]
    InterpretationBreak,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_BREAK1")]
    InterpretationBreak1,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_BREAK2")]
    InterpretationBreak2,
    [CCode (cname = "osbyte_VAR_OUTPUT_SCREEN_BANK")]
    OutputScreenBank,
    [CCode (cname = "osbyte_VAR_DISPLAY_SCREEN_BANK")]
    DisplayScreenBank,
    [CCode (cname = "osbyte_VAR_ROM_CURRENT")]
    ROMCurrent,
    [CCode (cname = "osbyte_VAR_LAST_RESET")]
    LastReset,
    [CCode (cname = "osbyte_VAR_INTERPRETATION_SHIFTING")]
    InterpretationShifting,
    [CCode (cname = "osbyte_VAR_BOOT_OPTION")]
    BootOption
  }

  [CCode (cname="osbyte_OP_JOYSTICK_STATE")]
  public const int OpJoystickState;
  [CCode (cname="osbyte_OP_CHANNEL_POSITION1")]
  public const int OpChannelPosition1;
  [CCode (cname="osbyte_OP_CHANNEL_POSITION2")]
  public const int OpChannelPosition2;
  [CCode (cname="osbyte_OP_CHANNEL_POSITION3")]
  public const int OpChannelPosition3;
  [CCode (cname="osbyte_OP_CHANNEL_POSITION4")]
  public const int OpChannelPosition4;
  [CCode (cname="osbyte_OP_POINTER_XPOSITION")]
  public const int OpPointerXPosition;
  [CCode (cname="osbyte_OP_POINTER_YPOSITION")]
  public const int OpPointerYPosition;
  [CCode (cname="osbyte_OP_MOUSE_BUFFER_USED")]
  public const int OpMouseBufferUsed;
  [CCode (cname="osbyte_OP_PRINTER_BUFFER_FREE")]
  public const int OpPrinterBufferFree;
  [CCode (cname="osbyte_OP_SERIAL_OUTPUT_BUFFER_FREE")]
  public const int OpSerialOutputBufferFree;
  [CCode (cname="osbyte_OP_SERIAL_INPUT_BUFFER_USED")]
  public const int OpSerialInputBufferUsed;
  [CCode (cname="osbyte_OP_KEYBOARD_BUFFER_USED")]
  public const int OpKeyboardBufferUsed;
  [CCode (cname="os_RESET_SOFT")]
  public const int ResetSoft;
  [CCode (cname="os_RESET_POWER_ON")]
  public const int ResetPowerOn;
  [CCode (cname="os_RESET_HARD")]
  public const int ResetHard;
  [CCode (cname="osbyte_CONFIGURE_STATION")]
  public const int ConfigureStation;
  [CCode (cname="osbyte_CONFIGURE_FS_STATION")]
  public const int ConfigureFSStation;
  [CCode (cname="osbyte_CONFIGURE_FS_NET")]
  public const int ConfigureFSNet;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME0")]
  public const int ConfigureFSName0;
  [CCode (cname="osbyte_CONFIGURE_PS_STATION")]
  public const int ConfigurePSStation;
  [CCode (cname="osbyte_CONFIGURE_PS_NET")]
  public const int ConfigurePSNet;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME0")]
  public const int ConfigurePSName0;
  [CCode (cname="osbyte_CONFIGURE_FILE_SYSTEM")]
  public const int ConfigureFileSystem;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG0")]
  public const int ConfigureUnplug0;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG1")]
  public const int ConfigureUnplug1;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG8")]
  public const int ConfigureUnplug8;
  [CCode (cname="osbyte_CONFIGURE_MODE")]
  public const int ConfigureMode;
  [CCode (cname="osbyte_CONFIGURE_MODE_SHIFT")]
  public const int ConfigureModeShift;
  [CCode (cname="osbyte_CONFIGURE_MODE_MASK")]
  public const bits ConfigureModeMask;
  [CCode (cname="osbyte_CONFIGURE_TV_INTERLACE")]
  public const int ConfigureTVInterlace;
  [CCode (cname="osbyte_CONFIGURE_TV_INTERLACE_MASK")]
  public const bits ConfigureTVInterlaceMask;
  [CCode (cname="osbyte_CONFIGURE_TV_VERTICAL_ADJUST")]
  public const int ConfigureTVVerticalAdjust;
  [CCode (cname="osbyte_CONFIGURE_TV_VERTICAL_ADJUST_SHIFT")]
  public const int ConfigureTVVerticalAdjustShift;
  [CCode (cname="osbyte_CONFIGURE_TV_VERTICAL_ADJUST_MASK")]
  public const bits ConfigureTVVerticalAdjustMask;
  [CCode (cname="osbyte_CONFIGURE_DRIVE")]
  public const int ConfigureDrive;
  [CCode (cname="osbyte_CONFIGURE_DRIVE_SHIFT")]
  public const int ConfigureDriveShift;
  [CCode (cname="osbyte_CONFIGURE_DRIVE_MASK")]
  public const bits ConfigureDriveMask;
  [CCode (cname="osbyte_CONFIGURE_CAPS")]
  public const int ConfigureCaps;
  [CCode (cname="osbyte_CONFIGURE_CAPS_SHIFT")]
  public const int ConfigureCapsShift;
  [CCode (cname="osbyte_CONFIGURE_CAPS_MASK")]
  public const bits ConfigureCapsMask;
  [CCode (cname="osbyte_CONFIGURE_DIR")]
  public const int ConfigureDir;
  [CCode (cname="osbyte_CONFIGURE_DIR_MASK")]
  public const bits ConfigureDirMask;
  [CCode (cname="osbyte_CONFIGURE_DELAY")]
  public const int ConfigureDelay;
  [CCode (cname="osbyte_CONFIGURE_REPEAT")]
  public const int ConfigureRepeat;
  [CCode (cname="osbyte_CONFIGURE_IGNORE")]
  public const int ConfigureIgnore;
  [CCode (cname="osbyte_CONFIGURE_NO_IGNORE")]
  public const int ConfigureNoIgnore;
  [CCode (cname="osbyte_CONFIGURE_NO_IGNORE_MASK")]
  public const bits ConfigureNoIgnoreMask;
  [CCode (cname="osbyte_CONFIGURE_BAUD")]
  public const int ConfigureBaud;
  [CCode (cname="osbyte_CONFIGURE_BAUD_SHIFT")]
  public const int ConfigureBaudShift;
  [CCode (cname="osbyte_CONFIGURE_BAUD_MASK")]
  public const bits ConfigureBaudMask;
  [CCode (cname="osbyte_CONFIGURE_PRINT")]
  public const int ConfigurePrint;
  [CCode (cname="osbyte_CONFIGURE_PRINT_SHIFT")]
  public const int ConfigurePrintShift;
  [CCode (cname="osbyte_CONFIGURE_PRINT_MASK")]
  public const bits ConfigurePrintMask;
  [CCode (cname="osbyte_CONFIGURE_LOUD")]
  public const int ConfigureLoud;
  [CCode (cname="osbyte_CONFIGURE_LOUD_MASK")]
  public const bits ConfigureLoudMask;
  [CCode (cname="osbyte_CONFIGURE_NO_SCROLL")]
  public const int ConfigureNoScroll;
  [CCode (cname="osbyte_CONFIGURE_NO_SCROLL_MASK")]
  public const bits ConfigureNoScrollMask;
  [CCode (cname="osbyte_CONFIGURE_BOOT")]
  public const int ConfigureBoot;
  [CCode (cname="osbyte_CONFIGURE_BOOT_MASK")]
  public const bits ConfigureBootMask;
  [CCode (cname="osbyte_CONFIGURE_DATA")]
  public const int ConfigureData;
  [CCode (cname="osbyte_CONFIGURE_DATA_SHIFT")]
  public const int ConfigureDataShift;
  [CCode (cname="osbyte_CONFIGURE_DATA_MASK")]
  public const bits ConfigureDataMask;
  [CCode (cname="osbyte_CONFIGURE_NET_FILER_SORT")]
  public const int ConfigureNetFilerSort;
  [CCode (cname="osbyte_CONFIGURE_NET_FILER_SORT_MASK")]
  public const bits ConfigureNetFilerSortMask;
  [CCode (cname="osbyte_CONFIGURE_LIBRARY")]
  public const int ConfigureLibrary;
  [CCode (cname="osbyte_CONFIGURE_LIBRARY_MASK")]
  public const bits ConfigureLibraryMask;
  [CCode (cname="osbyte_CONFIGURE_NET_FILER_DISPLAY")]
  public const int ConfigureNetFilerDisplay;
  [CCode (cname="osbyte_CONFIGURE_NET_FILER_DISPLAY_SHIFT")]
  public const int ConfigureNetFilerDisplayShift;
  [CCode (cname="osbyte_CONFIGURE_NET_FILER_DISPLAY_MASK")]
  public const bits ConfigureNetFilerDisplayMask;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG2")]
  public const int ConfigureUnplug2;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG3")]
  public const int ConfigureUnplug3;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXTENSION0")]
  public const int ConfigureUnplugExtension0;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXTENSION1")]
  public const int ConfigureUnplugExtension1;
  [CCode (cname="osbyte_CONFIGURE_WIMP_DOUBLE_CLICK_MOVE")]
  public const int ConfigureWimpDoubleClickMove;
  [CCode (cname="osbyte_CONFIGURE_WIMP_AUTO_MENU_DELAY")]
  public const int ConfigureWimpAutoMenuDelay;
  [CCode (cname="osbyte_CONFIGURE_TERRITORY")]
  public const int ConfigureTerritory;
  [CCode (cname="osbyte_CONFIGURE_PRINTER_BUFFER_SIZE")]
  public const int ConfigurePrinterBufferSize;
  [CCode (cname="osbyte_CONFIGURE_ADFS_SPINDOWN")]
  public const int ConfigureADFSSpindown;
  [CCode (cname="osbyte_CONFIGURE_WIMP_MENU_DRAG_DELAY")]
  public const int ConfigureWimpMenuDragDelay;
  [CCode (cname="osbyte_CONFIGURE_TRUNCATE")]
  public const int ConfigureTruncate;
  [CCode (cname="osbyte_CONFIGURE_TRUNCATE_MASK")]
  public const bits ConfigureTruncateMask;
  [CCode (cname="osbyte_CONFIGURE_DRAG_ASPRITE")]
  public const int ConfigureDragASprite;
  [CCode (cname="osbyte_CONFIGURE_DRAG_ASPRITE_MASK")]
  public const bits ConfigureDragASpriteMask;
  [CCode (cname="osbyte_CONFIGURE_NO_FILER_ACTION")]
  public const int ConfigureNoFilerAction;
  [CCode (cname="osbyte_CONFIGURE_NO_FILER_ACTION_MASK")]
  public const bits ConfigureNoFilerActionMask;
  [CCode (cname="osbyte_CONFIGURE_DESKTOP_DITHER")]
  public const int ConfigureDesktopDither;
  [CCode (cname="osbyte_CONFIGURE_DESKTOP_DITHER_MASK")]
  public const bits ConfigureDesktopDitherMask;
  [CCode (cname="osbyte_CONFIGURE_NO_OBSCURE_ICON_BAR")]
  public const int ConfigureNoObscureIconBar;
  [CCode (cname="osbyte_CONFIGURE_NO_OBSCURE_ICON_BAR_MASK")]
  public const bits ConfigureNoObscureIconBarMask;
  [CCode (cname="osbyte_CONFIGURE_SHUTDOWN_TYPE")]
  public const int ConfigureShutdownType;
  [CCode (cname="osbyte_CONFIGURE_SHUTDOWN_TYPE_SHIFT")]
  public const int ConfigureShutdownTypeShift;
  [CCode (cname="osbyte_CONFIGURE_SHUTDOWN_TYPE_MASK")]
  public const bits ConfigureShutdownTypeMask;
  [CCode (cname="osbyte_CONFIGURE_MOUSE_TYPE")]
  public const int ConfigureMouseType;
  [CCode (cname="osbyte_CONFIGURE_USER0")]
  public const int ConfigureUser0;
  [CCode (cname="osbyte_CONFIGURE_THIRD_PARTY0")]
  public const int ConfigureThirdParty0;
  [CCode (cname="osbyte_CONFIGURE_PODULE_EXTRA0")]
  public const int ConfigurePoduleExtra0;
  [CCode (cname="osbyte_CONFIGURE_RISCIX")]
  public const int ConfigureRISCIX;
  [CCode (cname="osbyte_CONFIGURE_SHARE_NET")]
  public const int ConfigureShareNet;
  [CCode (cname="osbyte_CONFIGURE_SHARE_MARKER")]
  public const int ConfigureShareMarker;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DISCS")]
  public const int ConfigureShareDiscs;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE4")]
  public const int ConfigureShareDrive4;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE4_MASK")]
  public const bits ConfigureShareDrive4Mask;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE4")]
  public const int ConfigureProtectDrive4;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE4_MASK")]
  public const bits ConfigureProtectDrive4Mask;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE5")]
  public const int ConfigureShareDrive5;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE5_MASK")]
  public const bits ConfigureShareDrive5Mask;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE5")]
  public const int ConfigureProtectDrive5;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE5_MASK")]
  public const bits ConfigureProtectDrive5Mask;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE6")]
  public const int ConfigureShareDrive6;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE6_MASK")]
  public const bits ConfigureShareDrive6Mask;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE6")]
  public const int ConfigureProtectDrive6;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE6_MASK")]
  public const bits ConfigureProtectDrive6Mask;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE7")]
  public const int ConfigureShareDrive7;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DRIVE7_MASK")]
  public const bits ConfigureShareDrive7Mask;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE7")]
  public const int ConfigureProtectDrive7;
  [CCode (cname="osbyte_CONFIGURE_PROTECT_DRIVE7_MASK")]
  public const bits ConfigureProtectDrive7Mask;
  [CCode (cname="osbyte_CONFIGURE_SHARE_DISCS_LIMIT")]
  public const int ConfigureShareDiscsLimit;
  [CCode (cname="osbyte_CONFIGURE_PODULE0")]
  public const int ConfigurePodule0;
  [CCode (cname="osbyte_CONFIGURE_PODULE1")]
  public const int ConfigurePodule1;
  [CCode (cname="osbyte_CONFIGURE_PODULE2")]
  public const int ConfigurePodule2;
  [CCode (cname="osbyte_CONFIGURE_PODULE3")]
  public const int ConfigurePodule3;
  [CCode (cname="osbyte_CONFIGURE_YEAR0")]
  public const int ConfigureYear0;
  [CCode (cname="osbyte_CONFIGURE_YEAR1")]
  public const int ConfigureYear1;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG9")]
  public const int ConfigureUnplug9;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG10")]
  public const int ConfigureUnplug10;
  [CCode (cname="osbyte_CONFIGURE_DUMP_FORMAT")]
  public const int ConfigureDumpFormat;
  [CCode (cname="osbyte_CONFIGURE_DUMP_FORMAT_SHIFT")]
  public const int ConfigureDumpFormatShift;
  [CCode (cname="osbyte_CONFIGURE_DUMP_FORMAT_MASK")]
  public const bits ConfigureDumpFormatMask;
  [CCode (cname="osbyte_CONFIGURE_TUBE")]
  public const int ConfigureTube;
  [CCode (cname="osbyte_CONFIGURE_TUBE_MASK")]
  public const bits ConfigureTubeMask;
  [CCode (cname="osbyte_CONFIGURE_HOST_PODULE")]
  public const int ConfigureHostPodule;
  [CCode (cname="osbyte_CONFIGURE_HOST_PODULE_SHIFT")]
  public const int ConfigureHostPoduleShift;
  [CCode (cname="osbyte_CONFIGURE_HOST_PODULE_MASK")]
  public const bits ConfigureHostPoduleMask;
  [CCode (cname="osbyte_CONFIGURE_SOUND")]
  public const int ConfigureSound;
  [CCode (cname="osbyte_CONFIGURE_SOUND_SHIFT")]
  public const int ConfigureSoundShift;
  [CCode (cname="osbyte_CONFIGURE_SOUND_MASK")]
  public const bits ConfigureSoundMask;
  [CCode (cname="osbyte_CONFIGURE_SOUND_QUALITY")]
  public const int ConfigureSoundQuality;
  [CCode (cname="osbyte_CONFIGURE_SOUND_QUALITY_MASK")]
  public const bits ConfigureSoundQualityMask;
  [CCode (cname="osbyte_CONFIGURE_SYNC")]
  public const int ConfigureSync;
  [CCode (cname="osbyte_CONFIGURE_SYNC_SHIFT")]
  public const int ConfigureSyncShift;
  [CCode (cname="osbyte_CONFIGURE_SYNC_MASK")]
  public const bits ConfigureSyncMask;
  [CCode (cname="osbyte_CONFIGURE_MODE_TOP_BIT")]
  public const int ConfigureModeTopBit;
  [CCode (cname="osbyte_CONFIGURE_MODE_TOP_BIT_MASK")]
  public const bits ConfigureModeTopBitMask;
  [CCode (cname="osbyte_CONFIGURE_MONITOR_NOT_CONFIGURED")]
  public const int ConfigureMonitorNotConfigured;
  [CCode (cname="osbyte_CONFIGURE_MONITOR_NOT_CONFIGURED_MASK")]
  public const bits ConfigureMonitorNotConfiguredMask;
  [CCode (cname="osbyte_CONFIGURE_MONITOR_TYPE")]
  public const int ConfigureMonitorType;
  [CCode (cname="osbyte_CONFIGURE_MONITOR_TYPE_SHIFT")]
  public const int ConfigureMonitorTypeShift;
  [CCode (cname="osbyte_CONFIGURE_MONITOR_TYPE_MASK")]
  public const bits ConfigureMonitorTypeMask;
  [CCode (cname="osbyte_CONFIGURE_FONT_SIZE")]
  public const int ConfigureFontSize;
  [CCode (cname="osbyte_CONFIGURE_FLOPPIES")]
  public const int ConfigureFloppies;
  [CCode (cname="osbyte_CONFIGURE_FLOPPIES_SHIFT")]
  public const int ConfigureFloppiesShift;
  [CCode (cname="osbyte_CONFIGURE_FLOPPIES_MASK")]
  public const bits ConfigureFloppiesMask;
  [CCode (cname="osbyte_CONFIGURE_HARD_DISCS")]
  public const int ConfigureHardDiscs;
  [CCode (cname="osbyte_CONFIGURE_HARD_DISCS_SHIFT")]
  public const int ConfigureHardDiscsShift;
  [CCode (cname="osbyte_CONFIGURE_HARD_DISCS_MASK")]
  public const bits ConfigureHardDiscsMask;
  [CCode (cname="osbyte_CONFIGURE_IDE_DISCS")]
  public const int ConfigureIDEDiscs;
  [CCode (cname="osbyte_CONFIGURE_IDE_DISCS_SHIFT")]
  public const int ConfigureIDEDiscsShift;
  [CCode (cname="osbyte_CONFIGURE_IDE_DISCS_MASK")]
  public const bits ConfigureIDEDiscsMask;
  [CCode (cname="osbyte_CONFIGURE_STEP0")]
  public const int ConfigureStep0;
  [CCode (cname="osbyte_CONFIGURE_STEP0_SHIFT")]
  public const int ConfigureStep0Shift;
  [CCode (cname="osbyte_CONFIGURE_STEP0_MASK")]
  public const bits ConfigureStep0Mask;
  [CCode (cname="osbyte_CONFIGURE_STEP1")]
  public const int ConfigureStep1;
  [CCode (cname="osbyte_CONFIGURE_STEP1_SHIFT")]
  public const int ConfigureStep1Shift;
  [CCode (cname="osbyte_CONFIGURE_STEP1_MASK")]
  public const bits ConfigureStep1Mask;
  [CCode (cname="osbyte_CONFIGURE_STEP2")]
  public const int ConfigureStep2;
  [CCode (cname="osbyte_CONFIGURE_STEP2_SHIFT")]
  public const int ConfigureStep2Shift;
  [CCode (cname="osbyte_CONFIGURE_STEP2_MASK")]
  public const bits ConfigureStep2Mask;
  [CCode (cname="osbyte_CONFIGURE_STEP3")]
  public const int ConfigureStep3;
  [CCode (cname="osbyte_CONFIGURE_STEP3_SHIFT")]
  public const int ConfigureStep3Shift;
  [CCode (cname="osbyte_CONFIGURE_STEP3_MASK")]
  public const bits ConfigureStep3Mask;
  [CCode (cname="osbyte_CONFIGURE_ADFS_BUFFERS")]
  public const int ConfigureADFSBuffers;
  [CCode (cname="osbyte_CONFIGURE_CDFS_FLAGS")]
  public const int ConfigureCDFSFlags;
  [CCode (cname="osbyte_CONFIGURE_TIME_ZONE")]
  public const int ConfigureTimeZone;
  [CCode (cname="osbyte_CONFIGURE_NEW_LOOK")]
  public const int ConfigureNewLook;
  [CCode (cname="osbyte_CONFIGURE_NEW_LOOK_MASK")]
  public const bits ConfigureNewLookMask;
  [CCode (cname="osbyte_CONFIGURE_WIMP_FONT")]
  public const int ConfigureWimpFont;
  [CCode (cname="osbyte_CONFIGURE_WIMP_FONT_SHIFT")]
  public const int ConfigureWimpFontShift;
  [CCode (cname="osbyte_CONFIGURE_WIMP_FONT_MASK")]
  public const bits ConfigureWimpFontMask;
  [CCode (cname="osbyte_CONFIGURE_NO_TILE")]
  public const int ConfigureNoTile;
  [CCode (cname="osbyte_CONFIGURE_NO_TILE_MASK")]
  public const bits ConfigureNoTileMask;
  [CCode (cname="osbyte_CONFIGURE_PRINTER")]
  public const int ConfigurePrinter;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG11")]
  public const int ConfigureUnplug11;
  [CCode (cname="osbyte_CONFIGURE_TWIN_FLAGS")]
  public const int ConfigureTwinFlags;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG12")]
  public const int ConfigureUnplug12;
  [CCode (cname="osbyte_CONFIGURE_SCREEN_SIZE")]
  public const int ConfigureScreenSize;
  [CCode (cname="osbyte_CONFIGURE_RAMFS_SIZE")]
  public const int ConfigureRAMFSSize;
  [CCode (cname="osbyte_CONFIGURE_SYSTEM_SIZE")]
  public const int ConfigureSystemSize;
  [CCode (cname="osbyte_CONFIGURE_RMA_SIZE")]
  public const int ConfigureRMASize;
  [CCode (cname="osbyte_CONFIGURE_SPRITE_SIZE")]
  public const int ConfigureSpriteSize;
  [CCode (cname="osbyte_CONFIGURE_SOUND_DEFAULT")]
  public const int ConfigureSoundDefault;
  [CCode (cname="osbyte_CONFIGURE_SOUND_DEFAULT_SHIFT")]
  public const int ConfigureSoundDefaultShift;
  [CCode (cname="osbyte_CONFIGURE_SOUND_DEFAULT_MASK")]
  public const bits ConfigureSoundDefaultMask;
  [CCode (cname="osbyte_CONFIGURE_SOUND_LOUDNESS")]
  public const int ConfigureSoundLoudness;
  [CCode (cname="osbyte_CONFIGURE_SOUND_LOUDNESS_SHIFT")]
  public const int ConfigureSoundLoudnessShift;
  [CCode (cname="osbyte_CONFIGURE_SOUND_LOUDNESS_MASK")]
  public const bits ConfigureSoundLoudnessMask;
  [CCode (cname="osbyte_CONFIGURE_SOUND_ON")]
  public const int ConfigureSoundOn;
  [CCode (cname="osbyte_CONFIGURE_SOUND_ON_MASK")]
  public const bits ConfigureSoundOnMask;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_FLAGS0")]
  public const int ConfigureBASICEditorFlags0;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_FLAGS1")]
  public const int ConfigureBASICEditorFlags1;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_FLAGS2")]
  public const int ConfigureBASICEditorFlags2;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_FLAGS3")]
  public const int ConfigureBASICEditorFlags3;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME1")]
  public const int ConfigurePSName1;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME2")]
  public const int ConfigurePSName2;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME3")]
  public const int ConfigurePSName3;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME4")]
  public const int ConfigurePSName4;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME5")]
  public const int ConfigurePSName5;
  [CCode (cname="osbyte_CONFIGURE_PS_NAME_LIMIT")]
  public const int ConfigurePSNameLimit;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME1")]
  public const int ConfigureFSName1;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME2")]
  public const int ConfigureFSName2;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME3")]
  public const int ConfigureFSName3;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME4")]
  public const int ConfigureFSName4;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME5")]
  public const int ConfigureFSName5;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME6")]
  public const int ConfigureFSName6;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME7")]
  public const int ConfigureFSName7;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME8")]
  public const int ConfigureFSName8;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME9")]
  public const int ConfigureFSName9;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME10")]
  public const int ConfigureFSName10;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME11")]
  public const int ConfigureFSName11;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME12")]
  public const int ConfigureFSName12;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME13")]
  public const int ConfigureFSName13;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME14")]
  public const int ConfigureFSName14;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME15")]
  public const int ConfigureFSName15;
  [CCode (cname="osbyte_CONFIGURE_FS_NAME_LIMIT")]
  public const int ConfigureFSNameLimit;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG4")]
  public const int ConfigureUnplug4;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG5")]
  public const int ConfigureUnplug5;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG6")]
  public const int ConfigureUnplug6;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG7")]
  public const int ConfigureUnplug7;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXPANSION_CARD0")]
  public const int ConfigureUnplugExpansionCard0;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXPANSION_CARD1")]
  public const int ConfigureUnplugExpansionCard1;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXPANSION_CARD2")]
  public const int ConfigureUnplugExpansionCard2;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG_EXPANSION_CARD3")]
  public const int ConfigureUnplugExpansionCard3;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_WILD_CARD0")]
  public const int ConfigureBASICEditorWildCard0;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_WILD_CARD1")]
  public const int ConfigureBASICEditorWildCard1;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_WILD_CARD2")]
  public const int ConfigureBASICEditorWildCard2;
  [CCode (cname="osbyte_CONFIGURE_BASIC_EDITOR_WILD_CARD3")]
  public const int ConfigureBASICEditorWildCard3;
  [CCode (cname="osbyte_CONFIGURE_LANGUAGE")]
  public const int ConfigureLanguage;
  [CCode (cname="osbyte_CONFIGURE_COUNTRY")]
  public const int ConfigureCountry;
  [CCode (cname="osbyte_CONFIGURE_VFS")]
  public const int ConfigureVFS;
  [CCode (cname="osbyte_CONFIGURE_ROMFS_OPT")]
  public const int ConfigureROMFSOpt;
  [CCode (cname="osbyte_CONFIGURE_ROMFS_OPT_SHIFT")]
  public const int ConfigureROMFSOptShift;
  [CCode (cname="osbyte_CONFIGURE_ROMFS_OPT_MASK")]
  public const bits ConfigureROMFSOptMask;
  [CCode (cname="osbyte_CONFIGURE_ICON_CACHE")]
  public const int ConfigureIconCache;
  [CCode (cname="osbyte_CONFIGURE_ICON_CACHE_MASK")]
  public const bits ConfigureIconCacheMask;
  [CCode (cname="osbyte_CONFIGURE_BLANK_TIME")]
  public const int ConfigureBlankTime;
  [CCode (cname="osbyte_CONFIGURE_BLANK_TIME_SHIFT")]
  public const int ConfigureBlankTimeShift;
  [CCode (cname="osbyte_CONFIGURE_BLANK_TIME_MASK")]
  public const bits ConfigureBlankTimeMask;
  [CCode (cname="osbyte_CONFIGURE_UNBLANK_ON_WRITE")]
  public const int ConfigureUnblankOnWrite;
  [CCode (cname="osbyte_CONFIGURE_UNBLANK_ON_WRITE_MASK")]
  public const bits ConfigureUnblankOnWriteMask;
  [CCode (cname="osbyte_CONFIGURE_NO_LONG_TEST")]
  public const int ConfigureNoLongTest;
  [CCode (cname="osbyte_CONFIGURE_NO_LONG_TEST_MASK")]
  public const bits ConfigureNoLongTestMask;
  [CCode (cname="osbyte_CONFIGURE_ADFS_DISC_SIZE0")]
  public const int ConfigureADFSDiscSize0;
  [CCode (cname="osbyte_CONFIGURE_ADFS_DISC_SIZE1")]
  public const int ConfigureADFSDiscSize1;
  [CCode (cname="osbyte_CONFIGURE_ADFS_DISC_SIZE2")]
  public const int ConfigureADFSDiscSize2;
  [CCode (cname="osbyte_CONFIGURE_ADFS_DISC_SIZE3")]
  public const int ConfigureADFSDiscSize3;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_PEEK")]
  public const int ConfigureProtectionPeek;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_PEEK_MASK")]
  public const bits ConfigureProtectionPeekMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_POKE")]
  public const int ConfigureProtectionPoke;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_POKE_MASK")]
  public const bits ConfigureProtectionPokeMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_JSR")]
  public const int ConfigureProtectionJSR;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_JSR_MASK")]
  public const bits ConfigureProtectionJSRMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_USER_RPC")]
  public const int ConfigureProtectionUserRPC;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_USER_RPC_MASK")]
  public const bits ConfigureProtectionUserRPCMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_OS_PROC")]
  public const int ConfigureProtectionOSProc;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_OS_PROC_MASK")]
  public const bits ConfigureProtectionOSProcMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_HALT")]
  public const int ConfigureProtectionHalt;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_HALT_MASK")]
  public const bits ConfigureProtectionHaltMask;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_GET_REGS")]
  public const int ConfigureProtectionGetRegs;
  [CCode (cname="osbyte_CONFIGURE_PROTECTION_GET_REGS_MASK")]
  public const bits ConfigureProtectionGetRegsMask;
  [CCode (cname="osbyte_CONFIGURE_MOUSE_STEP")]
  public const int ConfigureMouseStep;
  [CCode (cname="osbyte_CONFIGURE_BOOT_NET")]
  public const int ConfigureBootNet;
  [CCode (cname="osbyte_CONFIGURE_BOOT_NET_MASK")]
  public const bits ConfigureBootNetMask;
  [CCode (cname="osbyte_CONFIGURE_DYNAMIC_STATION")]
  public const int ConfigureDynamicStation;
  [CCode (cname="osbyte_CONFIGURE_DYNAMIC_STATION_MASK")]
  public const bits ConfigureDynamicStationMask;
  [CCode (cname="osbyte_CONFIGURE_CMOS_RESET")]
  public const int ConfigureCMOSReset;
  [CCode (cname="osbyte_CONFIGURE_CMOS_RESET_MASK")]
  public const bits ConfigureCMOSResetMask;
  [CCode (cname="osbyte_CONFIGURE_POWER_SAVE")]
  public const int ConfigurePowerSave;
  [CCode (cname="osbyte_CONFIGURE_POWER_SAVE_MASK")]
  public const bits ConfigurePowerSaveMask;
  [CCode (cname="osbyte_CONFIGURE_AUTO_WIMP_MODE")]
  public const int ConfigureAutoWimpMode;
  [CCode (cname="osbyte_CONFIGURE_AUTO_WIMP_MODE_MASK")]
  public const bits ConfigureAutoWimpModeMask;
  [CCode (cname="osbyte_CONFIGURE_NO_CACHE")]
  public const int ConfigureNoCache;
  [CCode (cname="osbyte_CONFIGURE_NO_CACHE_MASK")]
  public const bits ConfigureNoCacheMask;
  [CCode (cname="osbyte_CONFIGURE_BROADCAST_LOADER")]
  public const int ConfigureBroadcastLoader;
  [CCode (cname="osbyte_CONFIGURE_BROADCAST_LOADER_MASK")]
  public const bits ConfigureBroadcastLoaderMask;
  [CCode (cname="osbyte_CONFIGURE_HOURGLASS_COLOUR")]
  public const int ConfigureHourglassColour;
  [CCode (cname="osbyte_CONFIGURE_HOURGLASS_COLOUR_MASK")]
  public const bits ConfigureHourglassColourMask;
  [CCode (cname="osbyte_CONFIGURE_WIMP_MODE")]
  public const int ConfigureWimpMode;
  [CCode (cname="osbyte_CONFIGURE_WIMP_FLAGS")]
  public const int ConfigureWimpFlags;
  [CCode (cname="osbyte_CONFIGURE_FILER_DISPLAY")]
  public const int ConfigureFilerDisplay;
  [CCode (cname="osbyte_CONFIGURE_FILER_DISPLAY_SHIFT")]
  public const int ConfigureFilerDisplayShift;
  [CCode (cname="osbyte_CONFIGURE_FILER_DISPLAY_MASK")]
  public const bits ConfigureFilerDisplayMask;
  [CCode (cname="osbyte_CONFIGURE_FILER_SORT")]
  public const int ConfigureFilerSort;
  [CCode (cname="osbyte_CONFIGURE_FILER_SORT_SHIFT")]
  public const int ConfigureFilerSortShift;
  [CCode (cname="osbyte_CONFIGURE_FILER_SORT_MASK")]
  public const bits ConfigureFilerSortMask;
  [CCode (cname="osbyte_CONFIGURE_FILER_FORCE")]
  public const int ConfigureFilerForce;
  [CCode (cname="osbyte_CONFIGURE_FILER_FORCE_MASK")]
  public const bits ConfigureFilerForceMask;
  [CCode (cname="osbyte_CONFIGURE_FILER_CONFIRM")]
  public const int ConfigureFilerConfirm;
  [CCode (cname="osbyte_CONFIGURE_FILER_CONFIRM_MASK")]
  public const bits ConfigureFilerConfirmMask;
  [CCode (cname="osbyte_CONFIGURE_FILER_VERBOSE")]
  public const int ConfigureFilerVerbose;
  [CCode (cname="osbyte_CONFIGURE_FILER_VERBOSE_MASK")]
  public const bits ConfigureFilerVerboseMask;
  [CCode (cname="osbyte_CONFIGURE_FILER_NEWER")]
  public const int ConfigureFilerNewer;
  [CCode (cname="osbyte_CONFIGURE_FILER_NEWER_MASK")]
  public const bits ConfigureFilerNewerMask;
  [CCode (cname="osbyte_CONFIGURE_ADFS_DIR_CACHE")]
  public const int ConfigureADFSDirCache;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX")]
  public const int ConfigureFontMax;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX1")]
  public const int ConfigureFontMax1;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX2")]
  public const int ConfigureFontMax2;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX3")]
  public const int ConfigureFontMax3;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX4")]
  public const int ConfigureFontMax4;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX5")]
  public const int ConfigureFontMax5;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX6")]
  public const int ConfigureFontMax6;
  [CCode (cname="osbyte_CONFIGURE_FONT_MAX7")]
  public const int ConfigureFontMax7;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISCS")]
  public const int ConfigureSCSIFSDiscs;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISCS_SHIFT")]
  public const int ConfigureSCSIFSDiscsShift;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISCS_MASK")]
  public const bits ConfigureSCSIFSDiscsMask;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DRIVE")]
  public const int ConfigureSCSIFSDrive;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DRIVE_SHIFT")]
  public const int ConfigureSCSIFSDriveShift;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DRIVE_MASK")]
  public const bits ConfigureSCSIFSDriveMask;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_BUFFERS")]
  public const int ConfigureSCSIFSBuffers;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DIR_CACHE")]
  public const int ConfigureSCSIFSDirCache;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISC_SIZE0")]
  public const int ConfigureSCSIFSDiscSize0;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISC_SIZE1")]
  public const int ConfigureSCSIFSDiscSize1;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISC_SIZE2")]
  public const int ConfigureSCSIFSDiscSize2;
  [CCode (cname="osbyte_CONFIGURE_SCSIFS_DISC_SIZE3")]
  public const int ConfigureSCSIFSDiscSize3;
  [CCode (cname="osbyte_CONFIGURE_ROM_ALARM")]
  public const int ConfigureROMAlarm;
  [CCode (cname="osbyte_CONFIGURE_ROM_ALARM_MASK")]
  public const bits ConfigureROMAlarmMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_CALC")]
  public const int ConfigureROMCalc;
  [CCode (cname="osbyte_CONFIGURE_ROM_CALC_MASK")]
  public const bits ConfigureROMCalcMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_CHARS")]
  public const int ConfigureROMChars;
  [CCode (cname="osbyte_CONFIGURE_ROM_CHARS_MASK")]
  public const bits ConfigureROMCharsMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_CONFIGURE")]
  public const int ConfigureROMConfigure;
  [CCode (cname="osbyte_CONFIGURE_ROM_CONFIGURE_MASK")]
  public const bits ConfigureROMConfigureMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_DRAW")]
  public const int ConfigureROMDraw;
  [CCode (cname="osbyte_CONFIGURE_ROM_DRAW_MASK")]
  public const bits ConfigureROMDrawMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_EDIT")]
  public const int ConfigureROMEdit;
  [CCode (cname="osbyte_CONFIGURE_ROM_EDIT_MASK")]
  public const bits ConfigureROMEditMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_HELP")]
  public const int ConfigureROMHelp;
  [CCode (cname="osbyte_CONFIGURE_ROM_HELP_MASK")]
  public const bits ConfigureROMHelpMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_PAINT")]
  public const int ConfigureROMPaint;
  [CCode (cname="osbyte_CONFIGURE_ROM_PAINT_MASK")]
  public const bits ConfigureROMPaintMask;
  [CCode (cname="osbyte_CONFIGURE_ROM_BAT_MAN")]
  public const int ConfigureROMBatMan;
  [CCode (cname="osbyte_CONFIGURE_ROM_BAT_MAN_MASK")]
  public const bits ConfigureROMBatManMask;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG13")]
  public const int ConfigureUnplug13;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG14")]
  public const int ConfigureUnplug14;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG15")]
  public const int ConfigureUnplug15;
  [CCode (cname="osbyte_CONFIGURE_ALARM_FORMAT")]
  public const int ConfigureAlarmFormat;
  [CCode (cname="osbyte_CONFIGURE_ALARM_FORMAT_SHIFT")]
  public const int ConfigureAlarmFormatShift;
  [CCode (cname="osbyte_CONFIGURE_ALARM_FORMAT_MASK")]
  public const bits ConfigureAlarmFormatMask;
  [CCode (cname="osbyte_CONFIGURE_ALARM_CONFIRM")]
  public const int ConfigureAlarmConfirm;
  [CCode (cname="osbyte_CONFIGURE_ALARM_CONFIRM_MASK")]
  public const bits ConfigureAlarmConfirmMask;
  [CCode (cname="osbyte_CONFIGURE_ALARM_AUTO_SAVE")]
  public const int ConfigureAlarmAutoSave;
  [CCode (cname="osbyte_CONFIGURE_ALARM_AUTO_SAVE_MASK")]
  public const bits ConfigureAlarmAutoSaveMask;
  [CCode (cname="osbyte_CONFIGURE_ALARM_WORKING_WEEK")]
  public const int ConfigureAlarmWorkingWeek;
  [CCode (cname="osbyte_CONFIGURE_ALARM_WORKING_WEEK_MASK")]
  public const bits ConfigureAlarmWorkingWeekMask;
  [CCode (cname="osbyte_CONFIGURE_ALARM_SILENT")]
  public const int ConfigureAlarmSilent;
  [CCode (cname="osbyte_CONFIGURE_ALARM_SILENT_MASK")]
  public const bits ConfigureAlarmSilentMask;
  [CCode (cname="osbyte_CONFIGURE_DST")]
  public const int ConfigureDST;
  [CCode (cname="osbyte_CONFIGURE_DST_MASK")]
  public const bits ConfigureDSTMask;
  [CCode (cname="osbyte_CONFIGURE_WIMP_DRAG_DELAY")]
  public const int ConfigureWimpDragDelay;
  [CCode (cname="osbyte_CONFIGURE_WIMP_DRAG_MOVE")]
  public const int ConfigureWimpDragMove;
  [CCode (cname="osbyte_CONFIGURE_WIMP_DOUBLE_CLICK_DELAY")]
  public const int ConfigureWimpDoubleClickDelay;
  [CCode (cname="osbyte_CONFIGURE_RISCIX32")]
  public const int ConfigureRISCIX32;
  [CCode (cname="osbyte_CONFIGURE_PRINTER_NAME")]
  public const int ConfigurePrinterName;
  [CCode (cname="osbyte_CONFIGURE_LCD")]
  public const int ConfigureLCD;
  [CCode (cname="osbyte_CONFIGURE_UNPLUG16")]
  public const int ConfigureUnplug16;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK0")]
  public const int ConfigureFSLock0;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK1")]
  public const int ConfigureFSLock1;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK2")]
  public const int ConfigureFSLock2;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK3")]
  public const int ConfigureFSLock3;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK4")]
  public const int ConfigureFSLock4;
  [CCode (cname="osbyte_CONFIGURE_FS_LOCK5")]
  public const int ConfigureFSLock5;
  [CCode (cname="osbyte_CONFIGURE_CHECKSUM")]
  public const int ConfigureChecksum;
  [CCode (cname="osbyte_CAPS_SHIFT")]
  public const bits CapsShift;
  [CCode (cname="osbyte_CAPS_NONE")]
  public const bits CapsNone;
  [CCode (cname="osbyte_CAPS_LOCK")]
  public const bits CapsLock;
  [CCode (cname="osbyte_SHUTDOWN_UNKNOWN")]
  public const bits ShutdownUnknown;
  [CCode (cname="osbyte_SHUTDOWN_FAILED")]
  public const bits ShutdownFailed;
  [CCode (cname="osbyte_SHUTDOWN_POWER_LOST")]
  public const bits ShutdownPowerLost;
  [CCode (cname="osbyte_SHUTDOWN_UNDEFINED")]
  public const bits ShutdownUndefined;
  [CCode (cname="osbyte_MOUSE_QUADRATURE")]
  public const bits MouseQuadrature;
  [CCode (cname="osbyte_MOUSE_MICROSOFT_SERIAL")]
  public const bits MouseMicrosoftSerial;
  [CCode (cname="osbyte_MOUSE_MSC_SERIAL")]
  public const bits MouseMSCSerial;
  [CCode (cname="osbyte_SYNC_VERTICAL")]
  public const bits SyncVertical;
  [CCode (cname="osbyte_SYNC_COMPOSITE")]
  public const bits SyncComposite;
  [CCode (cname="osbyte_SYNC_AUTO")]
  public const bits SyncAuto;
  [CCode (cname="osbyte_MONITOR_TYPE_AUTO")]
  public const bits MonitorTypeAuto;
  [CCode (cname="osbyte_BLANK_OFF")]
  public const bits BlankOff;
  [CCode (cname="osbyte_BLANK30S")]
  public const bits Blank30s;
  [CCode (cname="osbyte_BLANK1MIN")]
  public const bits Blank1min;
  [CCode (cname="osbyte_BLANK2MIN")]
  public const bits Blank2min;
  [CCode (cname="osbyte_BLANK5MIN")]
  public const bits Blank5min;
  [CCode (cname="osbyte_BLANK10MIN")]
  public const bits Blank10min;
  [CCode (cname="osbyte_BLANK15MIN")]
  public const bits Blank15min;
  [CCode (cname="osbyte_BLANK30MIN")]
  public const bits Blank30min;
  [CCode (cname="osbyte_DISPLAY_LARGE_ICONS")]
  public const bits DisplayLargeIcons;
  [CCode (cname="osbyte_DISPLAY_SMALL_ICONS")]
  public const bits DisplaySmallIcons;
  [CCode (cname="osbyte_DISPLAY_FULL_INFO")]
  public const bits DisplayFullInfo;
  [CCode (cname="osbyte_SORT_BY_NAME")]
  public const bits SortByName;
  [CCode (cname="osbyte_SORT_BY_NUMBER")]
  public const bits SortByNumber;
  [CCode (cname="osbyte_SORT_BY_TYPE")]
  public const bits SortByType;
  [CCode (cname="osbyte_SORT_BY_SIZE")]
  public const bits SortBySize;
  [CCode (cname="osbyte_SORT_BY_DATE")]
  public const bits SortByDate;

  [CCode (cname = "xos_byte")]
  public static OS.Error* XByte
    (Op op,
     int r1,
     int r2,
     out int r1_out,
     out int r2_out);

  [CCode (cname = "os_byte")]
  public static void Byte
    (Op op,
     int r1,
     int r2,
     out int r1_out,
     out int r2_out);

  [CCode (cname = "xbytev")]
  public static OS.Error* XByteV
    (Op op,
     int r1,
     int r2,
     out int r1_out,
     out int r2_out);

  [CCode (cname = "bytev")]
  public static void ByteV
    (Op op,
     int r1,
     int r2,
     out int r1_out,
     out int r2_out);

  [CCode (cname = "xosbyte")]
  public static OS.Error* XOSByte
    (Op op,
     int r1,
     int r2);

  [CCode (cname = "osbyte")]
  public static void OSByte
    (Op op,
     int r1,
     int r2);

  [CCode (cname = "xosbyte1")]
  public static OS.Error* XOSByte1
    (Op op,
     int r1,
     int r2,
     out int r1_out);

  [CCode (cname = "osbyte1")]
  public static int OSByte1
    (Op op,
     int r1,
     int r2);

  [CCode (cname = "xosbyte2")]
  public static OS.Error* XOSByte2
    (Op op,
     int r1,
     int r2,
     out int r2_out);

  [CCode (cname = "osbyte2")]
  public static int OSByte2
    (Op op,
     int r1,
     int r2);

  [CCode (cname = "xosbyte_read")]
  public static OS.Error* XRead
    (Var var,
     out int value);

  [CCode (cname = "osbyte_read")]
  public static int Read
    (Var var);

  [CCode (cname = "xosbyte_write")]
  public static OS.Error* XWrite
    (Var var,
     int value);

  [CCode (cname = "osbyte_write")]
  public static void Write
    (Var var,
     int value);

}
