/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/osgbpb.h")]
namespace OSLib.OSGBPB
{
  [CCode (cname = "osgbpb_name_base")]
  public struct NameBase
  {
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [Compact]
  [CCode (cname = "osgbpb_name", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Name
  {
    public Name(int numEntries);
    [CCode (cname = "count")]
    public unowned byte Count;
    [CCode (cname = "c", array_length = false)]
    public unowned char[] C;
  }

  [CCode (cname = "osgbpb_disc_name")]
  public struct DiscName
  {
    [CCode (cname = "name")]
    public unowned Name Name;
  }

  [CCode (cname = "osgbpb_dir_name")]
  public struct DirName
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved;
    [CCode (cname = "name")]
    public unowned Name Name;
  }

  [CCode (cname = "osgbpb_name_list")]
  public struct NameList
  {
    [CCode (cname = "names", array_length = false)]
    public unowned Name[] Names;
  }

  [CCode (cname = "osgbpb_info_base")]
  public struct InfoBase
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
  }

  [Compact]
  [CCode (cname = "osgbpb_info", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Info
  {
    public Info(int numEntries);
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
    [CCode (cname = "name", array_length = false)]
    public unowned char[] Name;
  }

  [CCode (cname = "osgbpb_system_info_base")]
  public struct SystemInfoBase
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
    [CCode (cname = "sin")]
    public unowned int Sin;
    [CCode (cname = "stamp")]
    public unowned OS.DateAndTime Stamp;
  }

  [Compact]
  [CCode (cname = "osgbpb_system_info", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class SystemInfo
  {
    public SystemInfo(int numEntries);
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
    [CCode (cname = "sin")]
    public unowned int Sin;
    [CCode (cname = "stamp")]
    public unowned OS.DateAndTime Stamp;
    [CCode (cname = "name", array_length = false)]
    public unowned char[] Name;
  }

  [CCode (cname = "osgbpb_info_stamped_base")]
  public struct InfoStampedBase
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
  }

  [Compact]
  [CCode (cname = "osgbpb_info_stamped", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class InfoStamped
  {
    public InfoStamped(int numEntries);
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "attr")]
    public unowned FileSwitch.Attr Attr;
    [CCode (cname = "obj_type")]
    public unowned FileSwitch.ObjectType ObjType;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "name", array_length = false)]
    public unowned char[] Name;
  }

  [CCode (cname = "osgbpb_string_list")]
  public struct StringList
  {
    [CCode (cname = "names", array_length = false)]
    public unowned string[] Names;
  }

  [CCode (cname = "osgbpb_info_list")]
  public struct InfoList
  {
    [CCode (cname = "info", array_length = false)]
    public unowned Info[] Info;
  }

  [CCode (cname = "osgbpb_system_info_list")]
  public struct SystemInfoList
  {
    [CCode (cname = "info", array_length = false)]
    public unowned SystemInfo[] Info;
  }

  [CCode (cname = "osgbpb_info_stamped_list")]
  public struct InfoStampedList
  {
    [CCode (cname = "info", array_length = false)]
    public unowned InfoStamped[] Info;
  }

  [CCode (cname = "osgbpb_privilege")]
  public enum Privilege
  {
    [CCode (cname = "osgbpb_PRIVILEGE_OWNER")]
    Owner,
    [CCode (cname = "osgbpb_PRIVILEGE_PUBLIC")]
    Public
  }

  [CCode (cname="osgbpb_NO_MORE")]
  public const int NoMore;

  [CCode (cname = "xosgbpb_write_at")]
  public static OS.Error* XWriteAt
    (OS.F file,
     char* data,
     int size,
     int ptr,
     out int unwritten);

  [CCode (cname = "osgbpb_write_at")]
  public static int WriteAt
    (OS.F file,
     char* data,
     int size,
     int ptr);

  [CCode (cname = "xosgbpb_write_atw")]
  public static OS.Error* XWriteAtW
    (OS.FW file,
     char* data,
     int size,
     int ptr,
     out int unwritten);

  [CCode (cname = "osgbpb_write_atw")]
  public static int WriteAtW
    (OS.FW file,
     char* data,
     int size,
     int ptr);

  [CCode (cname = "xosgbpb_write")]
  public static OS.Error* XWrite
    (OS.F file,
     char* data,
     int size,
     out int unwritten);

  [CCode (cname = "osgbpb_write")]
  public static int Write
    (OS.F file,
     char* data,
     int size);

  [CCode (cname = "xosgbpb_writew")]
  public static OS.Error* XWriteW
    (OS.FW file,
     char* data,
     int size,
     out int unwritten);

  [CCode (cname = "osgbpb_writew")]
  public static int WriteW
    (OS.FW file,
     char* data,
     int size);

  [CCode (cname = "xosgbpb_read_at")]
  public static OS.Error* XReadAt
    (OS.F file,
     char* buffer,
     int size,
     int ptr,
     out int unread);

  [CCode (cname = "osgbpb_read_at")]
  public static int ReadAt
    (OS.F file,
     char* buffer,
     int size,
     int ptr);

  [CCode (cname = "xosgbpb_read_atw")]
  public static OS.Error* XReadAtW
    (OS.FW file,
     char* buffer,
     int size,
     int ptr,
     out int unread);

  [CCode (cname = "osgbpb_read_atw")]
  public static int ReadAtW
    (OS.FW file,
     char* buffer,
     int size,
     int ptr);

  [CCode (cname = "xosgbpb_read")]
  public static OS.Error* XRead
    (OS.F file,
     char* buffer,
     int size,
     out int unread);

  [CCode (cname = "osgbpb_read")]
  public static int Read
    (OS.F file,
     char* buffer,
     int size);

  [CCode (cname = "xosgbpb_readw")]
  public static OS.Error* XReadW
    (OS.FW file,
     char* buffer,
     int size,
     out int unread);

  [CCode (cname = "osgbpb_readw")]
  public static int ReadW
    (OS.FW file,
     char* buffer,
     int size);

  [CCode (cname = "xosgbpb_read_disc_name")]
  public static OS.Error* XReadDiscName
    (DiscName disc_name);

  [CCode (cname = "osgbpb_read_disc_name")]
  public static void ReadDiscName
    (DiscName disc_name);

  [CCode (cname = "xosgbpb_read_csd_name")]
  public static OS.Error* XReadCSDName
    (DirName csd_name);

  [CCode (cname = "osgbpb_read_csd_name")]
  public static void ReadCSDName
    (DirName csd_name);

  [CCode (cname = "xosgbpb_read_lib_name")]
  public static OS.Error* XReadLibName
    (DirName lib_name);

  [CCode (cname = "osgbpb_read_lib_name")]
  public static void ReadLibName
    (DirName lib_name);

  [CCode (cname = "xosgbpb_csd_entries")]
  public static OS.Error* XCSDEntries
    (NameList name_list,
     int count,
     int context,
     out int unread,
     out int context_out);

  [CCode (cname = "osgbpb_csd_entries")]
  public static int CSDEntries
    (NameList name_list,
     int count,
     int context,
     out int unread);

  [CCode (cname = "xosgbpb_dir_entries")]
  public static OS.Error* XDirEntries
    (string dir_name,
     StringList name_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count,
     out int context_out);

  [CCode (cname = "osgbpb_dir_entries")]
  public static int DirEntries
    (string dir_name,
     StringList name_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count);

  [CCode (cname = "xosgbpb_dir_entries_info")]
  public static OS.Error* XDirEntriesInfo
    (string dir_name,
     InfoList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count,
     out int context_out);

  [CCode (cname = "osgbpb_dir_entries_info")]
  public static int DirEntriesInfo
    (string dir_name,
     InfoList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count);

  [CCode (cname = "xosgbpb_dir_entries_system_info")]
  public static OS.Error* XDirEntriesSystemInfo
    (string dir_name,
     SystemInfoList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count,
     out int context_out);

  [CCode (cname = "osgbpb_dir_entries_system_info")]
  public static int DirEntriesSystemInfo
    (string dir_name,
     SystemInfoList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count);

  [CCode (cname = "xosgbpb_dir_entries_info_stamped")]
  public static OS.Error* XDirEntriesInfoStamped
    (string dir_name,
     InfoStampedList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count,
     out int context_out);

  [CCode (cname = "osgbpb_dir_entries_info_stamped")]
  public static int DirEntriesInfoStamped
    (string dir_name,
     InfoStampedList info_list,
     int count,
     int context,
     int size,
     string entries,
     out int read_count);

}
