/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osserialop.h")]
namespace OSLib.OSSerialOp
{
  [CCode (cname = "osserialop_rs423_status")]
  public enum RS423Status
  {
    [CCode (cname = "osserialop_RS423_PARITY_ERROR")]
    RS423ParityError,
    [CCode (cname = "osserialop_RS423_OVERRUN_ERROR")]
    RS423OverrunError,
    [CCode (cname = "osserialop_RS423_FRAMING_ERROR")]
    RS423FramingError
  }

  [CCode (cname = "osserialop_format_flags")]
  public enum FormatFlags
  {
    [CCode (cname = "osserialop_DATA_FORMAT_WORD_SIZE")]
    DataFormatWordSize,
    [CCode (cname = "osserialop_DATA_FORMAT_EXTRA_STOP_BITS")]
    DataFormatExtraStopBits,
    [CCode (cname = "osserialop_DATA_FORMAT_PARITY_ENABLE")]
    DataFormatParityEnable,
    [CCode (cname = "osserialop_DATA_FORMAT_PARITY")]
    DataFormatParity
  }

  [CCode (cname = "osserialop_baud_rate")]
  public enum BaudRate
  {
    [CCode (cname = "osserialop_BAUD_RATE9600")]
    BaudRate9600,
    [CCode (cname = "osserialop_BAUD_RATE75")]
    BaudRate75,
    [CCode (cname = "osserialop_BAUD_RATE150")]
    BaudRate150,
    [CCode (cname = "osserialop_BAUD_RATE300")]
    BaudRate300,
    [CCode (cname = "osserialop_BAUD_RATE1200")]
    BaudRate1200,
    [CCode (cname = "osserialop_BAUD_RATE2400")]
    BaudRate2400,
    [CCode (cname = "osserialop_BAUD_RATE4800")]
    BaudRate4800,
    [CCode (cname = "osserialop_BAUD_RATE19200")]
    BaudRate19200,
    [CCode (cname = "osserialop_BAUD_RATE50")]
    BaudRate50,
    [CCode (cname = "osserialop_BAUD_RATE110")]
    BaudRate110,
    [CCode (cname = "osserialop_BAUD_RATE134PT5")]
    BaudRate134pt5,
    [CCode (cname = "osserialop_BAUD_RATE600")]
    BaudRate600,
    [CCode (cname = "osserialop_BAUD_RATE1800")]
    BaudRate1800,
    [CCode (cname = "osserialop_BAUD_RATE3600")]
    BaudRate3600,
    [CCode (cname = "osserialop_BAUD_RATE7200")]
    BaudRate7200,
    [CCode (cname = "osserialop_BAUD_RATE38400")]
    BaudRate38400,
    [CCode (cname = "osserialop_BAUD_RATE57600")]
    BaudRate57600,
    [CCode (cname = "osserialop_BAUD_RATE115200")]
    BaudRate115200
  }

  [CCode (cname="osserialop_DATA_FORMAT_WORD_SIZE_SHIFT")]
  public const int DataFormatWordSizeShift;
  [CCode (cname="osserialop_DATA_FORMAT_WORD_SIZE8_BIT")]
  public const int DataFormatWordSize8Bit;
  [CCode (cname="osserialop_DATA_FORMAT_WORD_SIZE7_BIT")]
  public const int DataFormatWordSize7Bit;
  [CCode (cname="osserialop_DATA_FORMAT_WORD_SIZE6_BIT")]
  public const int DataFormatWordSize6Bit;
  [CCode (cname="osserialop_DATA_FORMAT_PARITY_SHIFT")]
  public const int DataFormatParityShift;
  [CCode (cname="osserialop_DATA_FORMAT_PARITY_ODD")]
  public const int DataFormatParityOdd;
  [CCode (cname="osserialop_DATA_FORMAT_PARITY_EVEN")]
  public const int DataFormatParityEven;
  [CCode (cname="osserialop_DATA_FORMAT_PARITY1")]
  public const int DataFormatParity1;
  [CCode (cname="osserialop_DATA_FORMAT_PARITY0")]
  public const int DataFormatParity0;

  [CCode (cname = "xosserialop_status")]
  public static OS.Error* XStatus
    (bits eor_mask,
     bits and_mask,
     out bits old_value,
     out bits new_value);

  [CCode (cname = "osserialop_status")]
  public static bits Status
    (bits eor_mask,
     bits and_mask,
     out bits new_value);

  [CCode (cname = "xosserialop_data_format")]
  public static OS.Error* XDataFormat
    (FormatFlags data_format,
     out FormatFlags old_value);

  [CCode (cname = "osserialop_data_format")]
  public static FormatFlags DataFormat
    (FormatFlags data_format);

  [CCode (cname = "xosserialop_break")]
  public static OS.Error* XBreak
    (int period);

  [CCode (cname = "osserialop_break")]
  public static void Break
    (int period);

  [CCode (cname = "xosserialop_bput")]
  public static OS.Error* XBPut
    (byte c,
     out uint psr);

  [CCode (cname = "osserialop_bput")]
  public static bits BPut
    (byte c);

  [CCode (cname = "xosserialop_bget")]
  public static OS.Error* XBGet
    (out byte c,
     out uint psr);

  [CCode (cname = "osserialop_bget")]
  public static bits BGet
    (out byte c);

  [CCode (cname = "xosserialop_rx_baud_rate")]
  public static OS.Error* XRxBaudRate
    (BaudRate rx_rate,
     out BaudRate old_rx_rate);

  [CCode (cname = "osserialop_rx_baud_rate")]
  public static BaudRate RxBaudRate
    (BaudRate rx_rate);

  [CCode (cname = "xosserialop_tx_baud_rate")]
  public static OS.Error* XTxBaudRate
    (BaudRate tx_rate,
     out BaudRate old_tx_rate);

  [CCode (cname = "osserialop_tx_baud_rate")]
  public static BaudRate TxBaudRate
    (BaudRate tx_rate);

  [CCode (cname = "xosserialop_rx_threshold")]
  public static OS.Error* XRxThreshold
    (bits rx_threshold,
     out bits old_rx_threshold);

  [CCode (cname = "osserialop_rx_threshold")]
  public static bits RxThreshold
    (bits rx_threshold);

  [CCode (cname = "xosserialop_enumerate_baud_rates")]
  public static OS.Error* XEnumerateBaudRates
    (out int* table,
     out int table_size);

  [CCode (cname = "osserialop_enumerate_baud_rates")]
  public static void EnumerateBaudRates
    (out int* table,
     out int table_size);

  [CCode (cname = "xevent_rs423_error")]
  public static OS.Error* XEvent_RS423Error
    (RS423Status status,
     byte b);

  [CCode (cname = "event_rs423_error")]
  public static void Event_RS423Error
    (RS423Status status,
     byte b);

}
