/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osspriteop.h")]
namespace OSLib.OSSpriteOp
{
  [CCode (cname = "osspriteop_trans_tab")]
  public struct TransTab
  {
    [CCode (cname = "c", array_length = false)]
    public unowned byte[] C;
  }

  [CCode (cname = "osspriteop_colour_mapping")]
  public struct ColourMapping
  {
    [CCode (cname = "workspace")]
    public unowned char* Workspace;
    [CCode (cname = "function")]
    public unowned AsmRoutine Function;
  }

  [CCode (cname = "osspriteop_area")]
  public struct Area
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "sprite_count")]
    public unowned int SpriteCount;
    [CCode (cname = "first")]
    public unowned int First;
    [CCode (cname = "used")]
    public unowned int Used;
  }

  [CCode (cname = "osspriteop_header")]
  public struct Header
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "name")]
    public unowned char Name[12];
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "left_bit")]
    public unowned int LeftBit;
    [CCode (cname = "right_bit")]
    public unowned int RightBit;
    [CCode (cname = "image")]
    public unowned int Image;
    [CCode (cname = "mask")]
    public unowned int Mask;
    [CCode (cname = "mode")]
    public unowned OS.Mode Mode;
  }

  [CCode (cname = "osspriteop_flags")]
  public enum Flags
  {
    [CCode (cname = "osspriteop_SYSTEM_AREA")]
    SystemArea,
    [CCode (cname = "osspriteop_USER_AREA")]
    UserArea,
    [CCode (cname = "osspriteop_NAME")]
    Name,
    [CCode (cname = "osspriteop_PTR")]
    Ptr
  }

  [CCode (cname = "osspriteop_mode_word")]
  public enum ModeWord
  {
    [CCode (cname = "osspriteop_NEW_STYLE")]
    NewStyle,
    [CCode (cname = "osspriteop_XRES")]
    XRes,
    [CCode (cname = "osspriteop_YRES")]
    YRes,
    [CCode (cname = "osspriteop_TYPE")]
    Type,
    [CCode (cname = "osspriteop_ALPHA_MASK")]
    AlphaMask,
    [CCode (cname = "osspriteop_TYPE_OLD")]
    TypeOld,
    [CCode (cname = "osspriteop_TYPE1BPP")]
    Type1bpp,
    [CCode (cname = "osspriteop_TYPE2BPP")]
    Type2bpp,
    [CCode (cname = "osspriteop_TYPE4BPP")]
    Type4bpp,
    [CCode (cname = "osspriteop_TYPE8BPP")]
    Type8bpp,
    [CCode (cname = "osspriteop_TYPE16BPP")]
    Type16bpp,
    [CCode (cname = "osspriteop_TYPE32BPP")]
    Type32bpp,
    [CCode (cname = "osspriteop_TYPE_CMYK")]
    TypeCMYK,
    [CCode (cname = "osspriteop_TYPE16BPP64K")]
    Type16bpp64K
  }

  [CCode (cname = "osspriteop_id", has_type_id = false)]
  public struct Id : bits
  {
  }

  [CCode (cname = "osspriteop_save_area")]
  public struct SaveArea
  {
    [CCode (cname = "a", array_length = false)]
    public unowned int[] A;
  }

  [CCode (cname = "osspriteop_action")]
  public enum Action
  {
    [CCode (cname = "osspriteop_USE_MASK")]
    UseMask,
    [CCode (cname = "osspriteop_USE_PALETTE")]
    UsePalette,
    [CCode (cname = "osspriteop_GIVEN_WIDE_ENTRIES")]
    GivenWideEntries,
    [CCode (cname = "osspriteop_DITHERED")]
    Dithered,
    [CCode (cname = "osspriteop_GIVEN_COLOUR_MAPPING")]
    GivenColourMapping,
    [CCode (cname = "osspriteop_TRANSLUCENCY")]
    Translucency
  }

  [CCode (cname = "osspriteop_pointer_flags")]
  public enum PointerFlags
  {
    [CCode (cname = "osspriteop_POINTER_SHAPE")]
    PointerShape,
    [CCode (cname = "osspriteop_POINTER_IGNORE_SHAPE_DATA")]
    PointerIgnoreShapeData,
    [CCode (cname = "osspriteop_POINTER_IGNORE_PALETTE")]
    PointerIgnorePalette,
    [CCode (cname = "osspriteop_POINTER_IGNORE_SHAPE")]
    PointerIgnoreShape
  }

  [CCode (cname = "osspriteopremovealphachannel_flags")]
  public enum OSSpriteOpRemoveAlphaChannel_Flags
  {
    [CCode (cname = "osspriteopremovealphachannelflags_REMOVE_MASK")]
    RemoveMask
  }

  [CCode (cname = "osspriteopaddalphachannel_flags")]
  public enum OSSpriteOpAddAlphaChannel_Flags
  {
    [CCode (cname = "osspriteopaddalphachannelflags_ADD")]
    Add
  }

  [CCode (cname = "osspriteop_trfm_flags")]
  public enum TrfmFlags
  {
    [CCode (cname = "osspriteop_DESTINATION_COORDS")]
    DestinationCoords,
    [CCode (cname = "osspriteop_SOURCE_RECTANGLE")]
    SourceRectangle
  }

  [CCode (cname = "servicespritereregister_type")]
  public enum ServiceSpriteReregister_Type
  {
    [CCode (cname = "servicespritereregistertype_SOFTWARE_EXT")]
    SoftwareExt,
    [CCode (cname = "servicespritereregistertype_ACCELERATION_EXT")]
    AccelerationExt,
    [CCode (cname = "servicespritereregistertype_MONITOR_OR_VECTOR_EXT")]
    MonitorOrVectorExt
  }

  [CCode (cname="osspriteop_NAME_LIMIT")]
  public const int NameLimit;
  [CCode (cname="osspriteop_XRES_SHIFT")]
  public const int XResShift;
  [CCode (cname="osspriteop_YRES_SHIFT")]
  public const int YResShift;
  [CCode (cname="osspriteop_TYPE_SHIFT")]
  public const int TypeShift;
  [CCode (cname="osspriteop_UNSPECIFIED")]
  public static Area* Unspecified;
  [CCode (cname="error_SPRITE_OP_NO_WORK_SPACE")]
  public const bits Error_SpriteOpNoWorkSpace;
  [CCode (cname="error_SPRITE_OP_NOT_GRAPHICS")]
  public const bits Error_SpriteOpNotGraphics;
  [CCode (cname="error_SPRITE_OP_NO_ROOM")]
  public const bits Error_SpriteOpNoRoom;
  [CCode (cname="error_SPRITE_OP_NO_SPRITES")]
  public const bits Error_SpriteOpNoSprites;
  [CCode (cname="error_SPRITE_OP_NOT_ENOUGH_ROOM")]
  public const bits Error_SpriteOpNotEnoughRoom;
  [CCode (cname="error_SPRITE_OP_DOESNT_EXIST")]
  public const bits Error_SpriteOpDoesntExist;
  [CCode (cname="error_SPRITE_OP_BAD_SPRITE_FILE")]
  public const bits Error_SpriteOpBadSpriteFile;
  [CCode (cname="error_SPRITE_OP_NO_ROOM_TO_MERGE")]
  public const bits Error_SpriteOpNoRoomToMerge;
  [CCode (cname="error_SPRITE_OP_INVALID_SECOND_PTR")]
  public const bits Error_SpriteOpInvalidSecondPtr;
  [CCode (cname="error_SPRITE_OP_INVALID_ROW_OR_COL")]
  public const bits Error_SpriteOpInvalidRowOrCol;
  [CCode (cname="error_SPRITE_OP_INVALID_HEIGHT")]
  public const bits Error_SpriteOpInvalidHeight;
  [CCode (cname="error_SPRITE_OP_INVALID_WIDTH")]
  public const bits Error_SpriteOpInvalidWidth;
  [CCode (cname="error_SPRITE_OP_NO_ROOM_TO_INSERT")]
  public const bits Error_SpriteOpNoRoomToInsert;
  [CCode (cname="error_SPRITE_OP_SPRITE_ALREADY_EXISTS")]
  public const bits Error_SpriteOpSpriteAlreadyExists;
  [CCode (cname="error_SPRITE_OP_INVALID_SPRITE_MODE")]
  public const bits Error_SpriteOpInvalidSpriteMode;
  [CCode (cname="error_SPRITE_OP_BAD_REASON_CODE")]
  public const bits Error_SpriteOpBadReasonCode;
  [CCode (cname="error_SPRITE_OP_CANT_DO_SYSTEM")]
  public const bits Error_SpriteOpCantDoSystem;
  [CCode (cname="error_SPRITE_OP_BAD_TRANSLATION")]
  public const bits Error_SpriteOpBadTranslation;
  [CCode (cname="error_SPRITE_OP_BAD_GREY_SCALE")]
  public const bits Error_SpriteOpBadGreyScale;
  [CCode (cname="error_SPRITE_OP_BAD_POINTER_SHAPE")]
  public const bits Error_SpriteOpBadPointerShape;
  [CCode (cname="error_SPRITE_OP_BAD_APPEND")]
  public const bits Error_SpriteOpBadAppend;
  [CCode (cname="error_SPRITE_OP_CANT_IN_TELETEXT")]
  public const bits Error_SpriteOpCantInTeletext;
  [CCode (cname="error_SPRITE_OP_INVALID_SAVE_AREA")]
  public const bits Error_SpriteOpInvalidSaveArea;
  [CCode (cname="error_SPRITE_OP_SPRITE_IS_CURRENT_DEST")]
  public const bits Error_SpriteOpSpriteIsCurrentDest;
  [CCode (cname="error_SPRITE_OP_BAD_FLAGS")]
  public const bits Error_SpriteOpBadFlags;
  [CCode (cname="error_SPRITE_OP_BAD_COORD_BLOCK")]
  public const bits Error_SpriteOpBadCoordBlock;
  [CCode (cname="error_SPRITE_OP_BAD_SOURCE_RECTANGLE")]
  public const bits Error_SpriteOpBadSourceRectangle;
  [CCode (cname="error_SPRITE_OP_BAD_TRANSFORMATION")]
  public const bits Error_SpriteOpBadTransformation;
  [CCode (cname="error_SPRITE_OP_BAD_DEPTH")]
  public const bits Error_SpriteOpBadDepth;
  [CCode (cname="error_SPRITE_OP_BAD_SWITCH_DEPTH")]
  public const bits Error_SpriteOpBadSwitchDepth;
  [CCode (cname="error_SPRITE_OP_BAD_COLOUR_MAP")]
  public const bits Error_SpriteOpBadColourMap;
  [CCode (cname="error_SPRITE_OP_BAD_JPEG_UNSUPPORTED")]
  public const bits Error_SpriteOpBadJPEGUnsupported;
  [CCode (cname="error_SPRITE_OP_BAD_JPEG_PROGRESSIVE")]
  public const bits Error_SpriteOpBadJPEGProgressive;
  [CCode (cname="error_SPRITE_OP_BAD_TYPE")]
  public const bits Error_SpriteOpBadType;
  [CCode (cname="osspriteop_TRANSLUCENCY_SHIFT")]
  public const int TranslucencyShift;
  [CCode (cname="osspriteop_POINTER_SHAPE_SHIFT")]
  public const int PointerShapeShift;
  [CCode (cname="osspriteopremovealphachannelflags_CUT_OFF_POINT_SHIFT")]
  public const int OSSpriteOpRemoveAlphaChannelFlags_CutOffPointShift;
  [CCode (cname="osspriteopremovealphachannelflags_CUT_OFF_POINT_MASK")]
  public const int OSSpriteOpRemoveAlphaChannelFlags_CutOffPointMask;

  [CCode (cname = "xosspriteop_screen_save")]
  public static OS.Error* XScreenSave
    (string file_name,
     bool save_palette);

  [CCode (cname = "osspriteop_screen_save")]
  public static void ScreenSave
    (string file_name,
     bool save_palette);

  [CCode (cname = "xosspriteop_screen_load")]
  public static OS.Error* XScreenLoad
    (string file_name);

  [CCode (cname = "osspriteop_screen_load")]
  public static void ScreenLoad
    (string file_name);

  [CCode (cname = "xosspriteop_read_area_cb")]
  public static OS.Error* XReadAreaCB
    (Flags flags,
     Area area,
     out int size,
     out int count,
     out int first,
     out int used);

  [CCode (cname = "osspriteop_read_area_cb")]
  public static void ReadAreaCB
    (Flags flags,
     Area area,
     out int size,
     out int count,
     out int first,
     out int used);

  [CCode (cname = "xosspriteop_clear_sprites")]
  public static OS.Error* XClearSprites
    (Flags flags,
     Area area);

  [CCode (cname = "osspriteop_clear_sprites")]
  public static void ClearSprites
    (Flags flags,
     Area area);

  [CCode (cname = "xosspriteop_load_sprite_file")]
  public static OS.Error* XLoadSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "osspriteop_load_sprite_file")]
  public static void LoadSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "xosspriteop_merge_sprite_file")]
  public static OS.Error* XMergeSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "osspriteop_merge_sprite_file")]
  public static void MergeSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "xosspriteop_save_sprite_file")]
  public static OS.Error* XSaveSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "osspriteop_save_sprite_file")]
  public static void SaveSpriteFile
    (Flags flags,
     Area area,
     string file_name);

  [CCode (cname = "xosspriteop_return_name")]
  public static OS.Error* XReturnName
    (Flags flags,
     Area area,
     string buffer,
     int size,
     int sprite_no,
     out int used);

  [CCode (cname = "osspriteop_return_name")]
  public static int ReturnName
    (Flags flags,
     Area area,
     string buffer,
     int size,
     int sprite_no);

  [CCode (cname = "xosspriteop_get_sprite")]
  public static OS.Error* XGetSprite
    (Flags flags,
     Area area,
     string sprite_name,
     bool get_palette,
     out Header header);

  [CCode (cname = "osspriteop_get_sprite")]
  public static Header GetSprite
    (Flags flags,
     Area area,
     string sprite_name,
     bool get_palette);

  [CCode (cname = "xosspriteop_create_sprite")]
  public static OS.Error* XCreateSprite
    (Flags flags,
     Area area,
     string sprite_name,
     bool create_palette,
     int width,
     int height,
     OS.Mode mode);

  [CCode (cname = "osspriteop_create_sprite")]
  public static void CreateSprite
    (Flags flags,
     Area area,
     string sprite_name,
     bool create_palette,
     int width,
     int height,
     OS.Mode mode);

  [CCode (cname = "xosspriteop_get_sprite_user_coords")]
  public static OS.Error* XGetSpriteUserCoords
    (Flags flags,
     Area area,
     string sprite_name,
     bool get_palette,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "osspriteop_get_sprite_user_coords")]
  public static void GetSpriteUserCoords
    (Flags flags,
     Area area,
     string sprite_name,
     bool get_palette,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "xosspriteop_verify_area")]
  public static OS.Error* XVerifyArea
    (Flags flags,
     Area area);

  [CCode (cname = "osspriteop_verify_area")]
  public static void VerifyArea
    (Flags flags,
     Area area);

  [CCode (cname = "xosspriteop_select_sprite")]
  public static OS.Error* XSelectSprite
    (Flags flags,
     Area area,
     Id id,
     out Header header);

  [CCode (cname = "osspriteop_select_sprite")]
  public static Header SelectSprite
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_delete_sprite")]
  public static OS.Error* XDeleteSprite
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_delete_sprite")]
  public static void DeleteSprite
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_rename_sprite")]
  public static OS.Error* XRenameSprite
    (Flags flags,
     Area area,
     Id id,
     string sprite_name);

  [CCode (cname = "osspriteop_rename_sprite")]
  public static void RenameSprite
    (Flags flags,
     Area area,
     Id id,
     string sprite_name);

  [CCode (cname = "xosspriteop_copy_sprite")]
  public static OS.Error* XCopySprite
    (Flags flags,
     Area area,
     Id id,
     string sprite_name);

  [CCode (cname = "osspriteop_copy_sprite")]
  public static void CopySprite
    (Flags flags,
     Area area,
     Id id,
     string sprite_name);

  [CCode (cname = "xosspriteop_put_sprite")]
  public static OS.Error* XPutSprite
    (Flags flags,
     Area area,
     Id id,
     OS.Action action);

  [CCode (cname = "osspriteop_put_sprite")]
  public static void PutSprite
    (Flags flags,
     Area area,
     Id id,
     OS.Action action);

  [CCode (cname = "xosspriteop_create_mask")]
  public static OS.Error* XCreateMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_create_mask")]
  public static void CreateMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_remove_mask")]
  public static OS.Error* XRemoveMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_remove_mask")]
  public static void RemoveMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_insert_row")]
  public static OS.Error* XInsertRow
    (Flags flags,
     Area area,
     Id id,
     int row);

  [CCode (cname = "osspriteop_insert_row")]
  public static void InsertRow
    (Flags flags,
     Area area,
     Id id,
     int row);

  [CCode (cname = "xosspriteop_delete_row")]
  public static OS.Error* XDeleteRow
    (Flags flags,
     Area area,
     Id id,
     int row);

  [CCode (cname = "osspriteop_delete_row")]
  public static void DeleteRow
    (Flags flags,
     Area area,
     Id id,
     int row);

  [CCode (cname = "xosspriteop_flip_about_xaxis")]
  public static OS.Error* XFlipAboutXAxis
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_flip_about_xaxis")]
  public static void FlipAboutXAxis
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_put_sprite_user_coords")]
  public static OS.Error* XPutSpriteUserCoords
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Action action);

  [CCode (cname = "osspriteop_put_sprite_user_coords")]
  public static void PutSpriteUserCoords
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Action action);

  [CCode (cname = "xosspriteop_append_sprite")]
  public static OS.Error* XAppendSprite
    (Flags flags,
     Area area,
     Id id0,
     Id id1,
     bool append_vertically);

  [CCode (cname = "osspriteop_append_sprite")]
  public static void AppendSprite
    (Flags flags,
     Area area,
     Id id0,
     Id id1,
     bool append_vertically);

  [CCode (cname = "xosspriteop_set_pointer_shape")]
  public static OS.Error* XSetPointerShape
    (Flags flags,
     Area area,
     Id id,
     PointerFlags pointer_flags,
     int xactive,
     int yactive,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "osspriteop_set_pointer_shape")]
  public static void SetPointerShape
    (Flags flags,
     Area area,
     Id id,
     PointerFlags pointer_flags,
     int xactive,
     int yactive,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "xosspriteop_read_palette_size")]
  public static OS.Error* XReadPaletteSize
    (Flags flags,
     Area area,
     Id id,
     out int size,
     out OS.SpritePalette palette,
     out OS.Mode mode);

  [CCode (cname = "osspriteop_read_palette_size")]
  public static void ReadPaletteSize
    (Flags flags,
     Area area,
     Id id,
     out int size,
     out OS.SpritePalette palette,
     out OS.Mode mode);

  [CCode (cname = "xosspriteop_remove_palette")]
  public static OS.Error* XRemovePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_remove_palette")]
  public static void RemovePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_create_palette")]
  public static OS.Error* XCreatePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_create_palette")]
  public static void CreatePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_create_true_palette")]
  public static OS.Error* XCreateTruePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_create_true_palette")]
  public static void CreateTruePalette
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_remove_alpha_channel")]
  public static OS.Error* XRemoveAlphaChannel
    (Flags flags,
     Area area,
     Id id,
     OSSpriteOpRemoveAlphaChannel_Flags aflags);

  [CCode (cname = "osspriteop_remove_alpha_channel")]
  public static void RemoveAlphaChannel
    (Flags flags,
     Area area,
     Id id,
     OSSpriteOpRemoveAlphaChannel_Flags aflags);

  [CCode (cname = "xosspriteop_add_alpha_channel")]
  public static OS.Error* XAddAlphaChannel
    (Flags flags,
     Area area,
     Id id,
     OSSpriteOpAddAlphaChannel_Flags aflags);

  [CCode (cname = "osspriteop_add_alpha_channel")]
  public static void AddAlphaChannel
    (Flags flags,
     Area area,
     Id id,
     OSSpriteOpAddAlphaChannel_Flags aflags);

  [CCode (cname = "xosspriteop_read_sprite_info")]
  public static OS.Error* XReadSpriteInfo
    (Flags flags,
     Area area,
     Id id,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "osspriteop_read_sprite_info")]
  public static void ReadSpriteInfo
    (Flags flags,
     Area area,
     Id id,
     out int width,
     out int height,
     out bool mask,
     out OS.Mode mode);

  [CCode (cname = "xosspriteop_read_pixel_colour")]
  public static OS.Error* XReadPixelColour
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out OS.GCOL gcol,
     out OS.Tint tint);

  [CCode (cname = "osspriteop_read_pixel_colour")]
  public static void ReadPixelColour
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out OS.GCOL gcol,
     out OS.Tint tint);

  [CCode (cname = "xosspriteop_read_pixel_colour_deep")]
  public static OS.Error* XReadPixelColourDeep
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out bits gcol,
     out OS.Tint tint);

  [CCode (cname = "osspriteop_read_pixel_colour_deep")]
  public static void ReadPixelColourDeep
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out bits gcol,
     out OS.Tint tint);

  [CCode (cname = "xosspriteop_write_pixel_colour")]
  public static OS.Error* XWritePixelColour
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.GCOL gcol,
     OS.Tint tint);

  [CCode (cname = "osspriteop_write_pixel_colour")]
  public static void WritePixelColour
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.GCOL gcol,
     OS.Tint tint);

  [CCode (cname = "xosspriteop_write_pixel_colour_deep")]
  public static OS.Error* XWritePixelColourDeep
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     bits gcol,
     OS.Tint tint);

  [CCode (cname = "osspriteop_write_pixel_colour_deep")]
  public static void WritePixelColourDeep
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     bits gcol,
     OS.Tint tint);

  [CCode (cname = "xosspriteop_read_pixel_mask")]
  public static OS.Error* XReadPixelMask
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out bool solid);

  [CCode (cname = "osspriteop_read_pixel_mask")]
  public static bool ReadPixelMask
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y);

  [CCode (cname = "xosspriteop_read_pixel_mask_alpha")]
  public static OS.Error* XReadPixelMaskAlpha
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     out int opacity);

  [CCode (cname = "osspriteop_read_pixel_mask_alpha")]
  public static int ReadPixelMaskAlpha
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y);

  [CCode (cname = "xosspriteop_write_pixel_mask")]
  public static OS.Error* XWritePixelMask
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     bool solid);

  [CCode (cname = "osspriteop_write_pixel_mask")]
  public static void WritePixelMask
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     bool solid);

  [CCode (cname = "xosspriteop_write_pixel_mask_alpha")]
  public static OS.Error* XWritePixelMaskAlpha
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     int opacity);

  [CCode (cname = "osspriteop_write_pixel_mask_alpha")]
  public static void WritePixelMaskAlpha
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     int opacity);

  [CCode (cname = "xosspriteop_insert_column")]
  public static OS.Error* XInsertColumn
    (Flags flags,
     Area area,
     Id id,
     int column);

  [CCode (cname = "osspriteop_insert_column")]
  public static void InsertColumn
    (Flags flags,
     Area area,
     Id id,
     int column);

  [CCode (cname = "xosspriteop_delete_column")]
  public static OS.Error* XDeleteColumn
    (Flags flags,
     Area area,
     Id id,
     int column);

  [CCode (cname = "osspriteop_delete_column")]
  public static void DeleteColumn
    (Flags flags,
     Area area,
     Id id,
     int column);

  [CCode (cname = "xosspriteop_flip_about_yaxis")]
  public static OS.Error* XFlipAboutYAxis
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_flip_about_yaxis")]
  public static void FlipAboutYAxis
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_plot_mask")]
  public static OS.Error* XPlotMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_plot_mask")]
  public static void PlotMask
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_plot_mask_user_coords")]
  public static OS.Error* XPlotMaskUserCoords
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y);

  [CCode (cname = "osspriteop_plot_mask_user_coords")]
  public static void PlotMaskUserCoords
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y);

  [CCode (cname = "xosspriteop_plot_mask_scaled")]
  public static OS.Error* XPlotMaskScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "osspriteop_plot_mask_scaled")]
  public static void PlotMaskScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "xosspriteop_paint_char_scaled")]
  public static OS.Error* XPaintCharScaled
    (char c,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "osspriteop_paint_char_scaled")]
  public static void PaintCharScaled
    (char c,
     int x,
     int y,
     OS.Factors factors);

  [CCode (cname = "xosspriteop_put_sprite_scaled")]
  public static OS.Error* XPutSpriteScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "osspriteop_put_sprite_scaled")]
  public static void PutSpriteScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "xosspriteop_put_sprite_scaled_mapped")]
  public static OS.Error* XPutSpriteScaledMapped
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     ColourMapping mapping);

  [CCode (cname = "osspriteop_put_sprite_scaled_mapped")]
  public static void PutSpriteScaledMapped
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     ColourMapping mapping);

  [CCode (cname = "xosspriteop_put_sprite_grey_scaled")]
  public static OS.Error* XPutSpriteGreyScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "osspriteop_put_sprite_grey_scaled")]
  public static void PutSpriteGreyScaled
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "xosspriteop_remove_left_hand_wastage")]
  public static OS.Error* XRemoveLeftHandWastage
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "osspriteop_remove_left_hand_wastage")]
  public static void RemoveLeftHandWastage
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_plot_mask_trfm")]
  public static OS.Error* XPlotMaskTrfm
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     OS.Trfm trfm);

  [CCode (cname = "osspriteop_plot_mask_trfm")]
  public static void PlotMaskTrfm
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     OS.Trfm trfm);

  [CCode (cname = "xosspriteop_put_sprite_trfm")]
  public static OS.Error* XPutSpriteTrfm
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     Action action,
     OS.Trfm trfm,
     TransTab trans_tab);

  [CCode (cname = "osspriteop_put_sprite_trfm")]
  public static void PutSpriteTrfm
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     Action action,
     OS.Trfm trfm,
     TransTab trans_tab);

  [CCode (cname = "xosspriteop_put_sprite_trfm_mapped")]
  public static OS.Error* XPutSpriteTrfmMapped
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     Action action,
     OS.Trfm trfm,
     ColourMapping mapping);

  [CCode (cname = "osspriteop_put_sprite_trfm_mapped")]
  public static void PutSpriteTrfmMapped
    (Flags flags,
     Area area,
     Id id,
     TrfmFlags trfm_flags,
     OS.Box source_rect,
     Action action,
     OS.Trfm trfm,
     ColourMapping mapping);

  [CCode (cname = "xosspriteop_insert_delete_rows")]
  public static OS.Error* XInsertDeleteRows
    (Flags flags,
     Area area,
     Id id,
     int row_no,
     int row_count);

  [CCode (cname = "osspriteop_insert_delete_rows")]
  public static void InsertDeleteRows
    (Flags flags,
     Area area,
     Id id,
     int row_no,
     int row_count);

  [CCode (cname = "xosspriteop_insert_delete_columns")]
  public static OS.Error* XInsertDeleteColumns
    (Flags flags,
     Area area,
     Id id,
     int column_no,
     int column_count);

  [CCode (cname = "osspriteop_insert_delete_columns")]
  public static void InsertDeleteColumns
    (Flags flags,
     Area area,
     Id id,
     int column_no,
     int column_count);

  [CCode (cname = "xosspriteop_switch_output_to_sprite")]
  public static OS.Error* XSwitchOutputToSprite
    (Flags flags,
     Area area,
     Id id,
     SaveArea save_area,
     out int context0,
     out int context1,
     out int context2,
     out int context3);

  [CCode (cname = "osspriteop_switch_output_to_sprite")]
  public static void SwitchOutputToSprite
    (Flags flags,
     Area area,
     Id id,
     SaveArea save_area,
     out int context0,
     out int context1,
     out int context2,
     out int context3);

  [CCode (cname = "xosspriteop_switch_output_to_mask")]
  public static OS.Error* XSwitchOutputToMask
    (Flags flags,
     Area area,
     Id id,
     SaveArea save_area,
     out int context0,
     out int context1,
     out int context2,
     out int context3);

  [CCode (cname = "osspriteop_switch_output_to_mask")]
  public static void SwitchOutputToMask
    (Flags flags,
     Area area,
     Id id,
     SaveArea save_area,
     out int context0,
     out int context1,
     out int context2,
     out int context3);

  [CCode (cname = "xosspriteop_read_save_area_size")]
  public static OS.Error* XReadSaveAreaSize
    (Flags flags,
     Area area,
     Id id,
     out int size);

  [CCode (cname = "osspriteop_read_save_area_size")]
  public static int ReadSaveAreaSize
    (Flags flags,
     Area area,
     Id id);

  [CCode (cname = "xosspriteop_unswitch_output")]
  public static OS.Error* XUnswitchOutput
    (int context0,
     int context1,
     int context2,
     int context3);

  [CCode (cname = "osspriteop_unswitch_output")]
  public static void UnswitchOutput
    (int context0,
     int context1,
     int context2,
     int context3);

  [CCode (cname = "xosspriteop_plot_tiled_sprite")]
  public static OS.Error* XPlotTiledSprite
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "osspriteop_plot_tiled_sprite")]
  public static void PlotTiledSprite
    (Flags flags,
     Area area,
     Id id,
     int x,
     int y,
     Action action,
     OS.Factors factors,
     TransTab trans_tab);

  [CCode (cname = "xservice_switching_output_to_sprite")]
  public static OS.Error* XService_SwitchingOutputToSprite ();

  [CCode (cname = "service_switching_output_to_sprite")]
  public static void Service_SwitchingOutputToSprite ();

  [CCode (cname = "xservice_sprite_reregister")]
  public static OS.Error* XService_SpriteReregister
    (ServiceSpriteReregister_Type type);

  [CCode (cname = "service_sprite_reregister")]
  public static void Service_SpriteReregister
    (ServiceSpriteReregister_Type type);

}
