/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/optionbutton.h")]
namespace OSLib.OptionButton
{
  [CCode (cname = "optionbutton_gadget")]
  public struct Gadget
  {
    [CCode (cname = "label")]
    public unowned Toolbox.MsgReference Label;
    [CCode (cname = "label_limit")]
    public unowned int LabelLimit;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "optionbutton_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "label")]
    public unowned Toolbox.MsgReference Label;
    [CCode (cname = "label_limit")]
    public unowned int LabelLimit;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "optionbutton_action_state_changed")]
  public struct ActionStateChanged
  {
    [CCode (cname = "on")]
    public unowned bool On;
  }

  [CCode (cname = "optionbutton_action_state_changed_block")]
  public struct ActionStateChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "on")]
    public unowned bool On;
  }

  [CCode (cname="class_OPTION_BUTTON")]
  public const Toolbox.Class Class_OptionButton;
  [CCode (cname="optionbutton_GENERATE_STATE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateStateChanged;
  [CCode (cname="optionbutton_ON")]
  public const OSLib.Gadget.Flags On;
  [CCode (cname="action_OPTION_BUTTON_STATE_CHANGED")]
  public const bits Action_OptionButtonStateChanged;
  [CCode (cname="optionbutton_STATE_CHANGED_ADJUST")]
  public const bits StateChangedAdjust;
  [CCode (cname="optionbutton_STATE_CHANGED_SELECT")]
  public const bits StateChangedSelect;

  [CCode (cname = "xoptionbutton_set_label")]
  public static OS.Error* XSetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     string label);

  [CCode (cname = "optionbutton_set_label")]
  public static void SetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     string label);

  [CCode (cname = "xoptionbutton_get_label")]
  public static OS.Error* XGetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "optionbutton_get_label")]
  public static int GetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     string buffer,
     int size);

  [CCode (cname = "xoptionbutton_set_action")]
  public static OS.Error* XSetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     bits action);

  [CCode (cname = "optionbutton_set_action")]
  public static void SetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     bits action);

  [CCode (cname = "xoptionbutton_get_action")]
  public static OS.Error* XGetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     out bits action);

  [CCode (cname = "optionbutton_get_action")]
  public static bits GetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button);

  [CCode (cname = "xoptionbutton_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     bool on);

  [CCode (cname = "optionbutton_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     bool on);

  [CCode (cname = "xoptionbutton_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button,
     out bool on);

  [CCode (cname = "optionbutton_get_state")]
  public static bool GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C option_button);

}
