/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/pdumper.h")]
namespace OSLib.PDumper
{
  [CCode (cname = "pdumper_job_flags")]
  public enum JobFlags
  {
    Default = 0
  }

  [CCode (cname = "pdumper_device")]
  public enum Device
  {
    [CCode (cname = "pdumper_DEVICE_SPRITE_DEVICE")]
    SpriteDevice,
    [CCode (cname = "pdumper_DEVICE_GENERIC_DOT_MATRIX")]
    GenericDotMatrix,
    [CCode (cname = "pdumper_DEVICE_LASERJET")]
    Laserjet,
    [CCode (cname = "pdumper_DEVICE_IMAGEWRITER")]
    Imagewriter,
    [CCode (cname = "pdumper_DEVICE24_PIN")]
    Device24Pin,
    [CCode (cname = "pdumper_DEVICE_DESKJET")]
    Deskjet
  }

  [CCode (cname = "pdumper_strip_type")]
  public enum StripType
  {
    [CCode (cname = "pdumper_STRIP_MONOCHROME")]
    StripMonochrome,
    [CCode (cname = "pdumper_STRIP_GREY_SCALE")]
    StripGreyScale,
    [CCode (cname = "pdumper_STRIP8_BIT")]
    Strip8Bit,
    [CCode (cname = "pdumper_STRIP24_BIT_MULTIPLE")]
    Strip24BitMultiple,
    [CCode (cname = "pdumper_STRIP16_BIT")]
    Strip16Bit,
    [CCode (cname = "pdumper_STRIP24_BIT")]
    Strip24Bit,
    [CCode (cname = "pdumper_PASS_NUMBER")]
    PassNumber
  }

  [CCode (cname = "pdumper_features")]
  public enum Features
  {
    [CCode (cname = "pdumper_FEATURE_HALFTONE_GREY")]
    FeatureHalftoneGrey,
    [CCode (cname = "pdumper_FEATURE_DIFFUSED_GREY")]
    FeatureDiffusedGrey,
    [CCode (cname = "pdumper_FEATURE_HALFTONE_COLOUR")]
    FeatureHalftoneColour,
    [CCode (cname = "pdumper_FEATURE_DIFFUSED_COLOUR")]
    FeatureDiffusedColour
  }

  [CCode (cname = "pdumper_format")]
  public enum Format
  {
    [CCode (cname = "pdumper_FORMAT_HALFTONE_GREY")]
    HalftoneGrey,
    [CCode (cname = "pdumper_FORMAT_DIFFUSED_GREY")]
    DiffusedGrey,
    [CCode (cname = "pdumper_FORMAT_HALFTONE_COLOUR")]
    HalftoneColour,
    [CCode (cname = "pdumper_FORMAT_DIFFUSED_COLOUR")]
    DiffusedColour,
    [CCode (cname = "pdumper_FORMAT_HALFTONE_MULTIPASS_COLOUR")]
    HalftoneMultipassColour,
    [CCode (cname = "pdumper_FORMAT_DIFFUSED_MULTIPASS_COLOUR")]
    DiffusedMultipassColour,
    [CCode (cname = "pdumper_FORMAT_HALFTONE16_BPP_COLOUR")]
    Halftone16BppColour,
    [CCode (cname = "pdumper_FORMAT_DIFFUSED16_BPP_COLOUR")]
    Diffused16BppColour,
    [CCode (cname = "pdumper_FORMAT_HALFTONE32_BPP_COLOUR")]
    Halftone32BppColour,
    [CCode (cname = "pdumper_FORMAT_DIFFUSED32_BPP_COLOUR")]
    Diffused32BppColour
  }

  [CCode (cname = "pdumper_halftone_flags")]
  public enum HalftoneFlags
  {
    [CCode (cname = "pdumper_HALFTONE_XRESOLUTION")]
    HalftoneXResolution,
    [CCode (cname = "pdumper_HALFTONE_YRESOLUTION")]
    HalftoneYResolution
  }

  [CCode (cname="error_PDUMPER_UNDECLARED")]
  public const bits Error_PDumperUndeclared;
  [CCode (cname="error_PDUMPER_TOO_OLD")]
  public const bits Error_PDumperTooOld;
  [CCode (cname="error_PDUMPER_DUPLICATE_MODULE")]
  public const bits Error_PDumperDuplicateModule;
  [CCode (cname="error_PDUMPER_BAD_CALL")]
  public const bits Error_PDumperBadCall;
  [CCode (cname="error_PDUMPER_BAD_STRIP")]
  public const bits Error_PDumperBadStrip;
  [CCode (cname="error_PDUMPER_BAD_PALETTE")]
  public const bits Error_PDumperBadPalette;
  [CCode (cname="error_PDUMPER_NOT_LINKED")]
  public const bits Error_PDumperNotLinked;
  [CCode (cname="error_PDUMPER_RESERVED")]
  public const bits Error_PDumperReserved;
  [CCode (cname="error_PDUMPER_BAD_OUTPUT_TYPE")]
  public const bits Error_PDumperBadOutputType;
  [CCode (cname="error_PDUMPER_BLOCK_NOT_FOUND")]
  public const bits Error_PDumperBlockNotFound;
  [CCode (cname="error_PDUMPER_IN_USE")]
  public const bits Error_PDumperInUse;
  [CCode (cname="pdumper_HALFTONE_XRESOLUTION_SHIFT")]
  public const int HalftoneXResolutionShift;
  [CCode (cname="pdumper_HALFTONE_YRESOLUTION_SHIFT")]
  public const int HalftoneYResolutionShift;

  [CCode (cname = "xpdumper_info")]
  public static OS.Error* XInfo
    (out int version,
     out Features features);

  [CCode (cname = "pdumper_info")]
  public static void Info
    (out int version,
     out Features features);

  [CCode (cname = "xpdumper_claim")]
  public static OS.Error* XClaim
    (int* anchor,
     int size,
     bits tag,
     out char* blk);

  [CCode (cname = "pdumper_claim")]
  public static char* Claim
    (int* anchor,
     int size,
     bits tag);

  [CCode (cname = "xpdumper_free")]
  public static OS.Error* XFree
    (int* anchor,
     char* blk);

  [CCode (cname = "pdumper_free")]
  public static void Free
    (int* anchor,
     char* blk);

  [CCode (cname = "xpdumper_find")]
  public static OS.Error* XFind
    (int* anchor,
     bits tag,
     out char* blk);

  [CCode (cname = "pdumper_find")]
  public static char* Find
    (int* anchor,
     bits tag);

  [CCode (cname = "xpdumper_start_job")]
  public static OS.Error* XStartJob
    (int* anchor,
     JobFlags flags,
     string palette_file_name);

  [CCode (cname = "pdumper_start_job")]
  public static void StartJob
    (int* anchor,
     JobFlags flags,
     string palette_file_name);

  [CCode (cname = "xpdumper_tidy_job")]
  public static OS.Error* XTidyJob
    (int* anchor,
     int* tags,
     bool end_of_document);

  [CCode (cname = "pdumper_tidy_job")]
  public static void TidyJob
    (int* anchor,
     int* tags,
     bool end_of_document);

  [CCode (cname = "xpdumper_set_colour")]
  public static OS.Error* XSetColour
    (int* anchor,
     OS.Colour colour,
     StripType strip_type_and_pass_no,
     HalftoneFlags halftone_info,
     out int col);

  [CCode (cname = "pdumper_set_colour")]
  public static int SetColour
    (int* anchor,
     OS.Colour colour,
     StripType strip_type_and_pass_no,
     HalftoneFlags halftone_info);

  [CCode (cname = "xpdumper_prepare_strip")]
  public static OS.Error* XPrepareStrip
    (int* anchor,
     char* image,
     Format format,
     int width,
     int height,
     int stride,
     HalftoneFlags halftone_info);

  [CCode (cname = "pdumper_prepare_strip")]
  public static void PrepareStrip
    (int* anchor,
     char* image,
     Format format,
     int width,
     int height,
     int stride,
     HalftoneFlags halftone_info);

  [CCode (cname = "xpdumper_lookup_error")]
  public static OS.Error* XLookupError
    (OS.Error error,
     string arg0);

  [CCode (cname = "pdumper_lookup_error")]
  public static void LookupError
    (OS.Error error,
     string arg0);

  [CCode (cname = "xpdumper_copy_filename")]
  public static OS.Error* XCopyFilename
    (string buffer,
     int size,
     string file_name);

  [CCode (cname = "pdumper_copy_filename")]
  public static void CopyFilename
    (string buffer,
     int size,
     string file_name);

  [CCode (cname = "xservice_pdumper_starting")]
  public static OS.Error* XService_PDumperStarting ();

  [CCode (cname = "service_pdumper_starting")]
  public static void Service_PDumperStarting ();

  [CCode (cname = "xservice_pdumper_dying")]
  public static OS.Error* XService_PDumperDying ();

  [CCode (cname = "service_pdumper_dying")]
  public static void Service_PDumperDying ();

}
