/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/plugin.h")]
namespace OSLib.PlugIn
{
  [CCode (cname = "plugin_p", has_type_id = false)]
  public struct P : bits
  {
  }

  [CCode (cname = "plugin_b", has_type_id = false)]
  public struct B : bits
  {
  }

  [CCode (cname = "plugin_s", has_type_id = false)]
  public struct S : bits
  {
  }

  [CCode (cname = "plugin_bs", has_type_id = false)]
  public struct BS : bits
  {
  }

  [CCode (cname = "plugin_notify_data")]
  public enum NotifyData
  {
    Default = 0
  }

  [CCode (cname = "plugin_open_flags")]
  public enum OpenFlags
  {
    [CCode (cname = "plugin_OPEN_AS_HELPER")]
    OpenAsHelper
  }

  [CCode (cname = "plugin_opening_flags")]
  public enum OpeningFlags
  {
    [CCode (cname = "plugin_OPENING_ACCEPTS_INPUT_FOCUS")]
    OpeningAcceptsInputFocus,
    [CCode (cname = "plugin_OPENING_WANTS_CODE_FETCHING")]
    OpeningWantsCodeFetching,
    [CCode (cname = "plugin_OPENING_WANTS_DATA_FETCHING")]
    OpeningWantsDataFetching,
    [CCode (cname = "plugin_OPENING_WILL_DELETE_PARAMETERS")]
    OpeningWillDeleteParameters,
    [CCode (cname = "plugin_OPENING_STILL_BUSY")]
    OpeningStillBusy,
    [CCode (cname = "plugin_OPENING_SUPPORTS_EXTENDED_ACTIONS")]
    OpeningSupportsExtendedActions,
    [CCode (cname = "plugin_OPENING_HAS_HELPER_WINDOW")]
    OpeningHasHelperWindow
  }

  [CCode (cname = "plugin_close_flags")]
  public enum CloseFlags
  {
    [CCode (cname = "plugin_CLOSE_REQUEST_EXIT")]
    CloseRequestExit
  }

  [CCode (cname = "plugin_closed_flags")]
  public enum ClosedFlags
  {
    [CCode (cname = "plugin_CLOSED_WILL_EXIT")]
    ClosedWillExit,
    [CCode (cname = "plugin_CLOSED_SELF_INITIATED")]
    ClosedSelfInitiated,
    [CCode (cname = "plugin_CLOSED_WITH_ERROR")]
    ClosedWithError
  }

  [CCode (cname = "plugin_stream_new_flags")]
  public enum StreamNewFlags
  {
    [CCode (cname = "plugin_STREAM_NEW_TYPE")]
    StreamNewType,
    [CCode (cname = "plugin_STREAM_NEW_SEEKABLE")]
    StreamNewSeekable
  }

  [CCode (cname = "plugin_stream_write_flags")]
  public enum StreamWriteFlags
  {
    [CCode (cname = "plugin_STREAM_WRITE_DATA_TYPE")]
    StreamWriteDataType
  }

  [CCode (cname = "plugin_url_access_flags")]
  public enum URLAccessFlags
  {
    [CCode (cname = "plugin_URL_ACCESS_NOTIFY_COMPLETION")]
    URLAccessNotifyCompletion,
    [CCode (cname = "plugin_URL_ACCESS_USE_POST")]
    URLAccessUsePOST,
    [CCode (cname = "plugin_URL_ACCESS_POST_FILE")]
    URLAccessPOSTFile
  }

  [CCode (cname = "plugin_busy_flags")]
  public enum BusyFlags
  {
    [CCode (cname = "plugin_BUSY_IS_BUSY")]
    BusyIsBusy,
    [CCode (cname = "plugin_BUSY_HAS_NEW_STATE")]
    BusyHasNewState
  }

  [CCode (cname = "plugin_stream_destroy_reason")]
  public enum StreamDestroyReason
  {
    [CCode (cname = "plugin_STREAM_DESTROY_FINISHED")]
    StreamDestroyFinished,
    [CCode (cname = "plugin_STREAM_DESTROY_ERROR")]
    StreamDestroyError,
    [CCode (cname = "plugin_STREAM_DESTROY_USER_REQUEST")]
    StreamDestroyUserRequest
  }

  [CCode (cname = "plugin_notify_reason")]
  public enum NotifyReason
  {
    [CCode (cname = "plugin_NOTIFY_FINISHED")]
    NotifyFinished,
    [CCode (cname = "plugin_NOTIFY_ERROR")]
    NotifyError,
    [CCode (cname = "plugin_NOTIFY_USER_REQUEST")]
    NotifyUserRequest
  }

  [CCode (cname = "plugin_state")]
  public enum State
  {
    [CCode (cname = "plugin_STATE_STOP")]
    Stop,
    [CCode (cname = "plugin_STATE_PLAY")]
    Play,
    [CCode (cname = "plugin_STATE_PAUSE")]
    Pause,
    [CCode (cname = "plugin_STATE_FAST_FORWARD")]
    FastForward,
    [CCode (cname = "plugin_STATE_REWIND")]
    Rewind,
    [CCode (cname = "plugin_STATE_RECORD")]
    Record,
    [CCode (cname = "plugin_STATE_MUTE")]
    Mute,
    [CCode (cname = "plugin_STATE_UN_MUTE")]
    UnMute
  }

  [CCode (cname = "plugin_message_open")]
  public struct MessageOpen
  {
    [CCode (cname = "flags")]
    public unowned OpenFlags Flags;
    [CCode (cname = "reserved")]
    public unowned bits Reserved;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "parent_window")]
    public unowned Wimp.W ParentWindow;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "filename")]
    public unowned OS.StringValue Filename;
  }

  [CCode (cname = "plugin_message_opening")]
  public struct MessageOpening
  {
    [CCode (cname = "flags")]
    public unowned OpeningFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_message_close")]
  public struct MessageClose
  {
    [CCode (cname = "flags")]
    public unowned CloseFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_message_closed")]
  public struct MessageClosed
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "error_number")]
    public unowned bits ErrorNumber;
    [CCode (cname = "error_text")]
    public unowned char ErrorText[220];
  }

  [CCode (cname = "plugin_message_reshape")]
  public struct MessageReshape
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "parent_window")]
    public unowned Wimp.W ParentWindow;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
  }

  [CCode (cname = "plugin_message_reshape_request")]
  public struct MessageReshapeRequest
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "size")]
    public unowned OS.Coord Size;
  }

  [CCode (cname = "plugin_message_focus")]
  public struct MessageFocus
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_message_unlock")]
  public struct MessageUnlock
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
  }

  [CCode (cname = "plugin_message_stream_new")]
  public struct MessageStreamNew
  {
    [CCode (cname = "flags")]
    public unowned StreamNewFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "mime_type")]
    public unowned OS.StringValue MimeType;
    [CCode (cname = "target_window")]
    public unowned OS.StringValue TargetWindow;
  }

  [CCode (cname = "plugin_message_stream_destroy")]
  public struct MessageStreamDestroy
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "reason")]
    public unowned StreamDestroyReason Reason;
  }

  [CCode (cname = "plugin_message_stream_write")]
  public struct MessageStreamWrite
  {
    [CCode (cname = "flags")]
    public unowned StreamWriteFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "offset")]
    public unowned int Offset;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "data")]
    public unowned char* Data;
  }

  [CCode (cname = "plugin_message_stream_written")]
  public struct MessageStreamWritten
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "length")]
    public unowned int Length;
  }

  [CCode (cname = "plugin_message_stream_as_file")]
  public struct MessageStreamAsFile
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "filename")]
    public unowned OS.StringValue Filename;
  }

  [CCode (cname = "plugin_message_url_access")]
  public struct MessageURLAccess
  {
    [CCode (cname = "flags")]
    public unowned URLAccessFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "target_window")]
    public unowned OS.StringValue TargetWindow;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "filename_or_data")]
    public unowned OS.StringValue FilenameOrData;
  }

  [CCode (cname = "plugin_message_notify")]
  public struct MessageNotify
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "reason")]
    public unowned NotifyReason Reason;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
  }

  [CCode (cname = "plugin_message_status")]
  public struct MessageStatus
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "message")]
    public unowned OS.StringValue Message;
  }

  [CCode (cname = "plugin_message_busy")]
  public struct MessageBusy
  {
    [CCode (cname = "flags")]
    public unowned BusyFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "new_state")]
    public unowned State NewState;
  }

  [CCode (cname = "plugin_message_action")]
  public struct MessageAction
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "new_state")]
    public unowned State NewState;
  }

  [CCode (cname = "plugin_message_abort")]
  public struct MessageAbort
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_full_message_open")]
  public struct FullMessageOpen : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned OpenFlags Flags;
    [CCode (cname = "reserved")]
    public unowned bits Reserved;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "parent_window")]
    public unowned Wimp.W ParentWindow;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "filename")]
    public unowned OS.StringValue Filename;
  }

  [CCode (cname = "plugin_full_message_opening")]
  public struct FullMessageOpening : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned OpeningFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_full_message_close")]
  public struct FullMessageClose : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned CloseFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_full_message_closed")]
  public struct FullMessageClosed : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "error_number")]
    public unowned bits ErrorNumber;
    [CCode (cname = "error_text")]
    public unowned char ErrorText[220];
  }

  [CCode (cname = "plugin_full_message_reshape")]
  public struct FullMessageReshape : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "parent_window")]
    public unowned Wimp.W ParentWindow;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
  }

  [CCode (cname = "plugin_full_message_reshape_request")]
  public struct FullMessageReshapeRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "shape")]
    public unowned OS.Coord Shape;
  }

  [CCode (cname = "plugin_full_message_focus")]
  public struct FullMessageFocus : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname = "plugin_full_message_unlock")]
  public struct FullMessageUnlock : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
  }

  [CCode (cname = "plugin_full_message_stream_new")]
  public struct FullMessageStreamNew : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned StreamNewFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "mime_type")]
    public unowned OS.StringValue MimeType;
    [CCode (cname = "target_window")]
    public unowned OS.StringValue TargetWindow;
  }

  [CCode (cname = "plugin_full_message_stream_destroy")]
  public struct FullMessageStreamDestroy : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "reason")]
    public unowned StreamDestroyReason Reason;
  }

  [CCode (cname = "plugin_full_message_stream_write")]
  public struct FullMessageStreamWrite : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned StreamWriteFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "offset")]
    public unowned int Offset;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "data")]
    public unowned char* Data;
  }

  [CCode (cname = "plugin_full_message_stream_written")]
  public struct FullMessageStreamWritten : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "length")]
    public unowned int Length;
  }

  [CCode (cname = "plugin_full_message_stream_as_file")]
  public struct FullMessageStreamAsFile : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "stream")]
    public unowned S Stream;
    [CCode (cname = "browser_stream")]
    public unowned BS BrowserStream;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "end")]
    public unowned int End;
    [CCode (cname = "last_modified_date")]
    public unowned int LastModifiedDate;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "filename")]
    public unowned OS.StringValue Filename;
  }

  [CCode (cname = "plugin_full_message_url_access")]
  public struct FullMessageURLAccess : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned URLAccessFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "target_window")]
    public unowned OS.StringValue TargetWindow;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
    [CCode (cname = "length")]
    public unowned int Length;
    [CCode (cname = "filename_or_data")]
    public unowned OS.StringValue FilenameOrData;
  }

  [CCode (cname = "plugin_full_message_notify")]
  public struct FullMessageNotify : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "url")]
    public unowned OS.StringValue Url;
    [CCode (cname = "reason")]
    public unowned NotifyReason Reason;
    [CCode (cname = "notify_data")]
    public unowned NotifyData NotifyData;
  }

  [CCode (cname = "plugin_full_message_status")]
  public struct FullMessageStatus : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "message")]
    public unowned OS.StringValue Message;
  }

  [CCode (cname = "plugin_full_message_busy")]
  public struct FullMessageBusy : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned BusyFlags Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "new_state")]
    public unowned State NewState;
  }

  [CCode (cname = "plugin_full_message_action")]
  public struct FullMessageAction : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
    [CCode (cname = "new_state")]
    public unowned State NewState;
  }

  [CCode (cname = "plugin_full_message_abort")]
  public struct FullMessageAbort : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "plugin")]
    public unowned P Plugin;
    [CCode (cname = "browser")]
    public unowned B Browser;
  }

  [CCode (cname="message_PLUG_IN_OPEN")]
  public const bits Message_PlugInOpen;
  [CCode (cname="message_PLUG_IN_OPENING")]
  public const bits Message_PlugInOpening;
  [CCode (cname="message_PLUG_IN_CLOSE")]
  public const bits Message_PlugInClose;
  [CCode (cname="message_PLUG_IN_CLOSED")]
  public const bits Message_PlugInClosed;
  [CCode (cname="message_PLUG_IN_RESHAPE")]
  public const bits Message_PlugInReshape;
  [CCode (cname="message_PLUG_IN_RESHAPE_REQUEST")]
  public const bits Message_PlugInReshapeRequest;
  [CCode (cname="message_PLUG_IN_FOCUS")]
  public const bits Message_PlugInFocus;
  [CCode (cname="message_PLUG_IN_UNLOCK")]
  public const bits Message_PlugInUnlock;
  [CCode (cname="message_PLUG_IN_STREAM_NEW")]
  public const bits Message_PlugInStreamNew;
  [CCode (cname="message_PLUG_IN_STREAM_DESTROY")]
  public const bits Message_PlugInStreamDestroy;
  [CCode (cname="message_PLUG_IN_STREAM_WRITE")]
  public const bits Message_PlugInStreamWrite;
  [CCode (cname="message_PLUG_IN_STREAM_WRITTEN")]
  public const bits Message_PlugInStreamWritten;
  [CCode (cname="message_PLUG_IN_STREAM_AS_FILE")]
  public const bits Message_PlugInStreamAsFile;
  [CCode (cname="message_PLUG_IN_URL_ACCESS")]
  public const bits Message_PlugInURLAccess;
  [CCode (cname="message_PLUG_IN_NOTIFY")]
  public const bits Message_PlugInNotify;
  [CCode (cname="message_PLUG_IN_STATUS")]
  public const bits Message_PlugInStatus;
  [CCode (cname="message_PLUG_IN_BUSY")]
  public const bits Message_PlugInBusy;
  [CCode (cname="message_PLUG_IN_ACTION")]
  public const bits Message_PlugInAction;
  [CCode (cname="message_PLUG_IN_ABORT")]
  public const bits Message_PlugInAbort;
  [CCode (cname="plugin_STREAM_NEW_TYPE_SHIFT")]
  public const int StreamNewTypeShift;
  [CCode (cname="plugin_STREAM_NEW_TYPE_NORMAL")]
  public const int StreamNewTypeNormal;
  [CCode (cname="plugin_STREAM_NEW_TYPE_SEEK_ONLY")]
  public const int StreamNewTypeSeekOnly;
  [CCode (cname="plugin_STREAM_NEW_TYPE_AS_FILE")]
  public const int StreamNewTypeAsFile;
  [CCode (cname="plugin_STREAM_NEW_TYPE_AS_FILE_ONLY")]
  public const int StreamNewTypeAsFileOnly;
  [CCode (cname="plugin_STREAM_WRITE_DATA_TYPE_SHIFT")]
  public const int StreamWriteDataTypeShift;
  [CCode (cname="plugin_STREAM_WRITE_DATA_TYPE_STRING")]
  public const int StreamWriteDataTypeString;
  [CCode (cname="plugin_STREAM_WRITE_DATA_TYPE_ANCHOR")]
  public const int StreamWriteDataTypeAnchor;
  [CCode (cname="plugin_STREAM_WRITE_DATA_TYPE_FILE_HANDLE")]
  public const int StreamWriteDataTypeFileHandle;

}
