/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/proginfo.h")]
namespace OSLib.ProgInfo
{
  [CCode (cname = "proginfo_flags")]
  public enum Flags
  {
    [CCode (cname = "proginfo_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "proginfo_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "proginfo_INCLUDE_LICENCE_TYPE")]
    IncludeLicenceType,
    [CCode (cname = "proginfo_INCLUDE_WEB_PAGE_BUTTON")]
    IncludeWebPageButton,
    [CCode (cname = "proginfo_GENERATE_WEB_PAGE_CLICKED")]
    GenerateWebPageClicked
  }

  [CCode (cname = "proginfo_licence")]
  public enum Licence
  {
    [CCode (cname = "proginfo_LICENCE_PUBLIC_DOMAIN")]
    PublicDomain,
    [CCode (cname = "proginfo_LICENCE_SINGLE_USER")]
    SingleUser,
    [CCode (cname = "proginfo_LICENCE_SINGLE_MACHINE")]
    SingleMachine,
    [CCode (cname = "proginfo_LICENCE_SITE")]
    Site,
    [CCode (cname = "proginfo_LICENCE_NETWORK")]
    Network,
    [CCode (cname = "proginfo_LICENCE_AUTHORITY")]
    Authority
  }

  [CCode (cname = "proginfo_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "purpose")]
    public unowned Toolbox.MsgReference Purpose;
    [CCode (cname = "author")]
    public unowned Toolbox.MsgReference Author;
    [CCode (cname = "licence_type")]
    public unowned Licence LicenceType;
    [CCode (cname = "version")]
    public unowned Toolbox.MsgReference Version;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
    [CCode (cname = "uri")]
    public unowned Toolbox.MsgReference Uri;
    [CCode (cname = "uri_action")]
    public unowned bits UriAction;
  }

  [CCode (cname = "proginfo_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "purpose")]
    public unowned Toolbox.MsgReference Purpose;
    [CCode (cname = "author")]
    public unowned Toolbox.MsgReference Author;
    [CCode (cname = "licence_type")]
    public unowned Licence LicenceType;
    [CCode (cname = "prog_version")]
    public unowned Toolbox.MsgReference ProgVersion;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
    [CCode (cname = "uri")]
    public unowned Toolbox.MsgReference Uri;
    [CCode (cname = "uri_action")]
    public unowned bits UriAction;
  }

  [CCode (cname = "proginfo_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "proginfo_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "proginfo_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "proginfo_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "proginfo_action_launch_web_page_block")]
  public enum ActionLaunchWebPageBlock
  {
    Default = 0
  }

  [CCode (cname="class_PROG_INFO")]
  public const Toolbox.Class Class_ProgInfo;
  [CCode (cname="action_PROG_INFO_ABOUT_TO_BE_SHOWN")]
  public const bits Action_ProgInfoAboutToBeShown;
  [CCode (cname="action_PROG_INFO_DIALOGUE_COMPLETED")]
  public const bits Action_ProgInfoDialogueCompleted;
  [CCode (cname="action_PROG_INFO_LAUNCH_WEB_PAGE")]
  public const bits Action_ProgInfoLaunchWebPage;
  [CCode (cname="error_PROG_INFO_TASKS_ACTIVE")]
  public const bits Error_ProgInfoTasksActive;
  [CCode (cname="error_PROG_INFO_ALLOC_FAILED")]
  public const bits Error_ProgInfoAllocFailed;
  [CCode (cname="error_PROG_INFO_SHORT_BUFFER")]
  public const bits Error_ProgInfoShortBuffer;
  [CCode (cname="error_PROG_INFO_NO_SUCH_TASK")]
  public const bits Error_ProgInfoNoSuchTask;
  [CCode (cname="error_PROG_INFO_NO_SUCH_METHOD")]
  public const bits Error_ProgInfoNoSuchMethod;
  [CCode (cname="error_PROG_INFO_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_ProgInfoNoSuchMiscOpMethod;

  [CCode (cname = "xproginfo_get_window_id")]
  public static OS.Error* XGetWindowID
    (bits flags,
     Toolbox.O prog_info,
     out Toolbox.O window);

  [CCode (cname = "proginfo_get_window_id")]
  public static Toolbox.O GetWindowID
    (bits flags,
     Toolbox.O prog_info);

  [CCode (cname = "xproginfo_set_version")]
  public static OS.Error* XSetVersion
    (bits flags,
     Toolbox.O prog_info,
     string version);

  [CCode (cname = "proginfo_set_version")]
  public static void SetVersion
    (bits flags,
     Toolbox.O prog_info,
     string version);

  [CCode (cname = "xproginfo_get_version")]
  public static OS.Error* XGetVersion
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "proginfo_get_version")]
  public static int GetVersion
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size);

  [CCode (cname = "xproginfo_set_licence_type")]
  public static OS.Error* XSetLicenceType
    (bits flags,
     Toolbox.O prog_info,
     Licence licence_type);

  [CCode (cname = "proginfo_set_licence_type")]
  public static void SetLicenceType
    (bits flags,
     Toolbox.O prog_info,
     Licence licence_type);

  [CCode (cname = "xproginfo_get_licence_type")]
  public static OS.Error* XGetLicenceType
    (bits flags,
     Toolbox.O prog_info,
     out Licence licence_type);

  [CCode (cname = "proginfo_get_licence_type")]
  public static Licence GetLicenceType
    (bits flags,
     Toolbox.O prog_info);

  [CCode (cname = "xproginfo_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O prog_info,
     string title);

  [CCode (cname = "proginfo_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O prog_info,
     string title);

  [CCode (cname = "xproginfo_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "proginfo_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size);

  [CCode (cname = "xproginfo_set_uri")]
  public static OS.Error* XSetURI
    (bits flags,
     Toolbox.O prog_info,
     string uri);

  [CCode (cname = "proginfo_set_uri")]
  public static void SetURI
    (bits flags,
     Toolbox.O prog_info,
     string uri);

  [CCode (cname = "xproginfo_get_uri")]
  public static OS.Error* XGetURI
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "proginfo_get_uri")]
  public static int GetURI
    (bits flags,
     Toolbox.O prog_info,
     string buffer,
     int size);

  [CCode (cname = "xproginfo_get_web_event")]
  public static OS.Error* XGetWebEvent
    (bits flags,
     Toolbox.O prog_info,
     out bits action);

  [CCode (cname = "proginfo_get_web_event")]
  public static bits GetWebEvent
    (bits flags,
     Toolbox.O prog_info);

  [CCode (cname = "xproginfo_set_web_event")]
  public static OS.Error* XSetWebEvent
    (bits flags,
     Toolbox.O prog_info,
     bits action);

  [CCode (cname = "proginfo_set_web_event")]
  public static void SetWebEvent
    (bits flags,
     Toolbox.O prog_info,
     bits action);

}
