/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/radiobutton.h")]
namespace OSLib.RadioButton
{
  [CCode (cname = "radiobutton_gadget")]
  public struct Gadget
  {
    [CCode (cname = "group")]
    public unowned int Group;
    [CCode (cname = "label")]
    public unowned Toolbox.MsgReference Label;
    [CCode (cname = "label_limit")]
    public unowned int LabelLimit;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "radiobutton_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "group")]
    public unowned int Group;
    [CCode (cname = "label")]
    public unowned Toolbox.MsgReference Label;
    [CCode (cname = "label_limit")]
    public unowned int LabelLimit;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "radiobutton_action_state_changed")]
  public struct ActionStateChanged
  {
    [CCode (cname = "on")]
    public unowned bool On;
    [CCode (cname = "previous_on")]
    public unowned Toolbox.C PreviousOn;
  }

  [CCode (cname = "radiobutton_action_state_changed_block")]
  public struct ActionStateChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "on")]
    public unowned bool On;
    [CCode (cname = "previous_on")]
    public unowned Toolbox.C PreviousOn;
  }

  [CCode (cname="class_RADIO_BUTTON")]
  public const Toolbox.Class Class_RadioButton;
  [CCode (cname="radiobutton_GENERATE_STATE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateStateChanged;
  [CCode (cname="radiobutton_GENERATE_SET_STATE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateSetStateChanged;
  [CCode (cname="radiobutton_ON")]
  public const OSLib.Gadget.Flags On;
  [CCode (cname="action_RADIO_BUTTON_STATE_CHANGED")]
  public const bits Action_RadioButtonStateChanged;
  [CCode (cname="radiobutton_STATE_CHANGED_ADJUST")]
  public const bits StateChangedAdjust;
  [CCode (cname="radiobutton_STATE_CHANGED_SELECT")]
  public const bits StateChangedSelect;

  [CCode (cname = "xradiobutton_set_label")]
  public static OS.Error* XSetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     string label);

  [CCode (cname = "radiobutton_set_label")]
  public static void SetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     string label);

  [CCode (cname = "xradiobutton_get_label")]
  public static OS.Error* XGetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "radiobutton_get_label")]
  public static int GetLabel
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     string buffer,
     int size);

  [CCode (cname = "xradiobutton_set_action")]
  public static OS.Error* XSetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     bits action);

  [CCode (cname = "radiobutton_set_action")]
  public static void SetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     bits action);

  [CCode (cname = "xradiobutton_get_action")]
  public static OS.Error* XGetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     out bits action);

  [CCode (cname = "radiobutton_get_action")]
  public static bits GetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button);

  [CCode (cname = "xradiobutton_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     bool on);

  [CCode (cname = "radiobutton_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     bool on);

  [CCode (cname = "xradiobutton_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     out bool on,
     out Toolbox.C on_button);

  [CCode (cname = "radiobutton_get_state")]
  public static bool GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C radio_button,
     out Toolbox.C on_button);

}
