/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileCore;

[CCode (cheader_filename = "oslib/ramfs.h")]
namespace OSLib.RamFS
{

  [CCode (cname = "xramfsdiscop_verify")]
  public static OS.Error* XRamFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out int unverified);

  [CCode (cname = "ramfsdiscop_verify")]
  public static int RamFSDiscOp_Verify
    (bits flags,
     FileCore.DiscAddress disc_addr,
     int size,
     out FileCore.DiscAddress next_disc_addr);

  [CCode (cname = "xramfsdiscop_read_sectors")]
  public static OS.Error* XRamFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread);

  [CCode (cname = "ramfsdiscop_read_sectors")]
  public static int RamFSDiscOp_ReadSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xramfsdiscop_write_sectors")]
  public static OS.Error* XRamFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unwritten);

  [CCode (cname = "ramfsdiscop_write_sectors")]
  public static int RamFSDiscOp_WriteSectors
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data);

  [CCode (cname = "xramfsdiscop_read_track")]
  public static OS.Error* XRamFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "ramfsdiscop_read_track")]
  public static void RamFSDiscOp_ReadTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xramfsdiscop_read_id")]
  public static OS.Error* XRamFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "ramfsdiscop_read_id")]
  public static void RamFSDiscOp_ReadId
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xramfsdiscop_write_track")]
  public static OS.Error* XRamFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "ramfsdiscop_write_track")]
  public static void RamFSDiscOp_WriteTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data);

  [CCode (cname = "xramfsdiscop_format_track")]
  public static OS.Error* XRamFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat track_format);

  [CCode (cname = "ramfsdiscop_format_track")]
  public static void RamFSDiscOp_FormatTrack
    (bits flags,
     FileCore.DiscAddress disc_addr,
     FileCore.TrackFormat track_format);

  [CCode (cname = "xramfsdiscop_seek")]
  public static OS.Error* XRamFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "ramfsdiscop_seek")]
  public static void RamFSDiscOp_Seek
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xramfsdiscop_restore")]
  public static OS.Error* XRamFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "ramfsdiscop_restore")]
  public static void RamFSDiscOp_Restore
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xramfsdiscop_step_in")]
  public static OS.Error* XRamFSDiscOp_StepIn
    (bits flags);

  [CCode (cname = "ramfsdiscop_step_in")]
  public static void RamFSDiscOp_StepIn
    (bits flags);

  [CCode (cname = "xramfsdiscop_step_out")]
  public static OS.Error* XRamFSDiscOp_StepOut
    (bits flags);

  [CCode (cname = "ramfsdiscop_step_out")]
  public static void RamFSDiscOp_StepOut
    (bits flags);

  [CCode (cname = "xramfsdiscop_read_sectors_via_cache")]
  public static OS.Error* XRamFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int unread,
     out int cache_handle_out);

  [CCode (cname = "ramfsdiscop_read_sectors_via_cache")]
  public static int RamFSDiscOp_ReadSectorsViaCache
    (bits flags,
     FileCore.DiscAddress disc_addr,
     char* data,
     int size,
     int cache_handle,
     out FileCore.DiscAddress next_disc_addr,
     out char* next_data,
     out int cache_handle_out);

  [CCode (cname = "xramfsdiscop_specify")]
  public static OS.Error* XRamFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "ramfsdiscop_specify")]
  public static void RamFSDiscOp_Specify
    (bits flags,
     FileCore.DiscAddress disc_addr);

  [CCode (cname = "xramfs_drives")]
  public static OS.Error* XDrives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "ramfs_drives")]
  public static void Drives
    (out int default_drive,
     out int floppy_count,
     out int hard_disc_count);

  [CCode (cname = "xramfs_free_space")]
  public static OS.Error* XFreeSpace
    (string disc_name,
     out int free,
     out int max);

  [CCode (cname = "ramfs_free_space")]
  public static int FreeSpace
    (string disc_name,
     out int max);

  [CCode (cname = "xramfs_describe_disc")]
  public static OS.Error* XDescribeDisc
    (string disc_name,
     FileCore.Disc disc);

  [CCode (cname = "ramfs_describe_disc")]
  public static void DescribeDisc
    (string disc_name,
     FileCore.Disc disc);

}
