/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Socket;

[CCode (cheader_filename = "oslib/resolver.h")]
namespace OSLib.Resolver
{
  [CCode (cname = "resolver_host_details")]
  public struct HostDetails
  {
    [CCode (cname = "name")]
    public unowned string Name;
    [CCode (cname = "aliases")]
    public unowned string* Aliases;
    [CCode (cname = "address_type")]
    public unowned Socket.AF AddressType;
    [CCode (cname = "address_size")]
    public unowned int AddressSize;
    [CCode (cname = "addresses")]
    public unowned byte** Addresses;
  }

  [CCode (cname = "resolvercachecontrol_reason")]
  public enum ResolverCacheControl_Reason
  {
    [CCode (cname = "resolvercachecontrolreason_FLUSH_FAILED")]
    FlushFailed,
    [CCode (cname = "resolvercachecontrolreason_FLUSH_ALL")]
    FlushAll,
    [CCode (cname = "resolvercachecontrolreason_FLUSH_HOST")]
    FlushHost,
    [CCode (cname = "resolvercachecontrolreason_REREAD_CONFIG")]
    RereadConfig,
    [CCode (cname = "resolvercachecontrolreason_NO_CACHE_FAILED")]
    NoCacheFailed,
    [CCode (cname = "resolvercachecontrolreason_DO_CACHE_FAILED")]
    DoCacheFailed
  }


  [CCode (cname = "xresolver_get_host_by_name")]
  public static OS.Error* XGetHostByName
    (string host_name,
     out int errno,
     out HostDetails host_details);

  [CCode (cname = "resolver_get_host_by_name")]
  public static HostDetails GetHostByName
    (string host_name,
     out int errno);

  [CCode (cname = "xresolver_get_host")]
  public static OS.Error* XGetHost
    (char* host_name,
     out int errno,
     out HostDetails host_details);

  [CCode (cname = "resolver_get_host")]
  public static HostDetails GetHost
    (char* host_name,
     out int errno);

  [CCode (cname = "xresolver_cache_control")]
  public static OS.Error* XCacheControl
    (ResolverCacheControl_Reason reason);

  [CCode (cname = "resolver_cache_control")]
  public static void CacheControl
    (ResolverCacheControl_Reason reason);

  [CCode (cname = "xeventinternet_dns_resolve_complete")]
  public static OS.Error* XEventInternet_DNSResolveComplete
    (char* host_name,
     int ip_address,
     int errno,
     HostDetails host_details);

  [CCode (cname = "eventinternet_dns_resolve_complete")]
  public static void EventInternet_DNSResolveComplete
    (char* host_name,
     int ip_address,
     int errno,
     HostDetails host_details);

}
