/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/scsi.h")]
namespace OSLib.SCSI
{
  [CCode (cname = "scsi_software_features")]
  public enum SoftwareFeatures
  {
    Default = 0
  }

  [CCode (cname = "scsi_hardware_features")]
  public enum HardwareFeatures
  {
    Default = 0
  }

  [CCode (cname = "scsi_device_type")]
  public enum DeviceType
  {
    Default = 0
  }

  [CCode (cname = "scsi_device_flags")]
  public enum DeviceFlags
  {
    Default = 0
  }

  [CCode (cname = "scsi_device_version")]
  public enum DeviceVersion
  {
    Default = 0
  }

  [CCode (cname = "scsi_device_description")]
  public struct DeviceDescription
  {
    [CCode (cname = "type")]
    public unowned DeviceType Type;
    [CCode (cname = "flags")]
    public unowned DeviceFlags Flags;
    [CCode (cname = "version")]
    public unowned DeviceVersion Version;
    [CCode (cname = "reserved0")]
    public unowned byte Reserved0;
    [CCode (cname = "length")]
    public unowned byte Length;
    [CCode (cname = "reserved1")]
    public unowned byte Reserved1;
    [CCode (cname = "reserved2")]
    public unowned byte Reserved2;
    [CCode (cname = "reserved3")]
    public unowned byte Reserved3;
    [CCode (cname = "block_count")]
    public unowned int BlockCount;
    [CCode (cname = "block_size")]
    public unowned int BlockSize;
  }

  [CCode (cname = "scsi_error_response_level")]
  public enum ErrorResponseLevel
  {
    [CCode (cname = "scsi_CHECK_NO_CHANGE")]
    CheckNoChange,
    [CCode (cname = "scsi_CHECK_ERROR")]
    CheckError,
    [CCode (cname = "scsi_CHECK_SENSE")]
    CheckSense,
    [CCode (cname = "scsi_CHECK_RETRY")]
    CheckRetry
  }

  [CCode (cname = "scsi_queue_behaviour")]
  public enum QueueBehaviour
  {
    Default = 0
  }

  [CCode (cname = "scsi_disconnection")]
  public enum Disconnection
  {
    [CCode (cname = "scsi_DISCONNECT_NO_CHANGE")]
    DisconnectNoChange,
    [CCode (cname = "scsi_DISCONNECT_ALLOWED")]
    DisconnectAllowed,
    [CCode (cname = "scsi_DISCONNECT_INHIBITED_BY_BLOCKING")]
    DisconnectInhibitedByBlocking
  }

  [CCode (cname = "scsi_op_block")]
  public struct OpBlock
  {
    [CCode (cname = "o", array_length = false)]
    public unowned int[] O;
  }

  [CCode (cname = "scsi_device_status")]
  public enum DeviceStatus
  {
    [CCode (cname = "scsi_DEVICE_STATUS_IDLE")]
    Idle,
    [CCode (cname = "scsi_DEVICE_STATUS_BUSY")]
    Busy
  }

  [CCode (cname = "scsi_card_address_list")]
  public struct CardAddressList
  {
    [CCode (cname = "addr", array_length = false)]
    public unowned char*[] Addr;
  }


  [CCode (cname = "xscsi_version")]
  public static OS.Error* XVersion
    (out bits software_version,
     out SoftwareFeatures software_features,
     out bits hardware_version,
     out HardwareFeatures hardware_features);

  [CCode (cname = "scsi_version")]
  public static void Version
    (out bits software_version,
     out SoftwareFeatures software_features,
     out bits hardware_version,
     out HardwareFeatures hardware_features);

  [CCode (cname = "xscsiinitialise_reset_bus")]
  public static OS.Error* XSCSIInitialise_ResetBus
    (int host_id);

  [CCode (cname = "scsiinitialise_reset_bus")]
  public static void SCSIInitialise_ResetBus
    (int host_id);

  [CCode (cname = "xscsiinitialise_reset_device")]
  public static OS.Error* XSCSIInitialise_ResetDevice
    (int device_id,
     int access_key);

  [CCode (cname = "scsiinitialise_reset_device")]
  public static void SCSIInitialise_ResetDevice
    (int device_id,
     int access_key);

  [CCode (cname = "xscsiinitialise_determine_device")]
  public static OS.Error* XSCSIInitialise_DetermineDevice
    (int device_id,
     DeviceDescription description);

  [CCode (cname = "scsiinitialise_determine_device")]
  public static void SCSIInitialise_DetermineDevice
    (int device_id,
     DeviceDescription description);

  [CCode (cname = "xscsiinitialise_enumerate_device")]
  public static OS.Error* XSCSIInitialise_EnumerateDevice
    (int device_id,
     string description,
     int size);

  [CCode (cname = "scsiinitialise_enumerate_device")]
  public static void SCSIInitialise_EnumerateDevice
    (int device_id,
     string description,
     int size);

  [CCode (cname = "xscsicontrol_abort_device")]
  public static OS.Error* XSCSIControl_AbortDevice
    (int device_id,
     int access_key);

  [CCode (cname = "scsicontrol_abort_device")]
  public static void SCSIControl_AbortDevice
    (int device_id,
     int access_key);

  [CCode (cname = "xscsicontrol_abort_op")]
  public static OS.Error* XSCSIControl_AbortOp
    (int device_id,
     int op_id,
     int access_key);

  [CCode (cname = "scsicontrol_abort_op")]
  public static void SCSIControl_AbortOp
    (int device_id,
     int op_id,
     int access_key);

  [CCode (cname = "xscsicontrol_set_timeout")]
  public static OS.Error* XSCSIControl_SetTimeout
    (int device_id,
     int timeout,
     int access_key,
     out int old_timeout);

  [CCode (cname = "scsicontrol_set_timeout")]
  public static int SCSIControl_SetTimeout
    (int device_id,
     int timeout,
     int access_key);

  [CCode (cname = "xscsicontrol_set_error_response")]
  public static OS.Error* XSCSIControl_SetErrorResponse
    (int device_id,
     ErrorResponseLevel level,
     int access_key,
     out int old_level);

  [CCode (cname = "scsicontrol_set_error_response")]
  public static int SCSIControl_SetErrorResponse
    (int device_id,
     ErrorResponseLevel level,
     int access_key);

  [CCode (cname = "xscsicontrol_set_queue_behaviour")]
  public static OS.Error* XSCSIControl_SetQueueBehaviour
    (int device_id,
     QueueBehaviour behaviour,
     int access_key,
     out int old_level);

  [CCode (cname = "scsicontrol_set_queue_behaviour")]
  public static int SCSIControl_SetQueueBehaviour
    (int device_id,
     QueueBehaviour behaviour,
     int access_key);

  [CCode (cname = "xscsicontrol_disconnect")]
  public static OS.Error* XSCSIControl_Disconnect
    (int device_id,
     Disconnection disconnection,
     out int old_disconnection,
     out int access_key);

  [CCode (cname = "scsicontrol_disconnect")]
  public static int SCSIControl_Disconnect
    (int device_id,
     Disconnection disconnection,
     out int access_key);

  [CCode (cname = "xscsi_op")]
  public static OS.Error* XOp
    (bits op,
     int block_size,
     OpBlock block,
     char* start_addr,
     int length,
     int timeout,
     AsmRoutine callback_code,
     char* workspace,
     int access_key,
     out int op_id,
     out char* next_addr,
     out int undone);

  [CCode (cname = "scsi_op")]
  public static int Op
    (bits op,
     int block_size,
     OpBlock block,
     char* start_addr,
     int length,
     int timeout,
     AsmRoutine callback_code,
     char* workspace,
     int access_key,
     out int op_id,
     out char* next_addr);

  [CCode (cname = "xscsistatus_check_device")]
  public static OS.Error* XSCSIStatus_CheckDevice
    (int device_id,
     out DeviceStatus status);

  [CCode (cname = "scsistatus_check_device")]
  public static DeviceStatus SCSIStatus_CheckDevice
    (int device_id);

  [CCode (cname = "xscsireserve_claim")]
  public static OS.Error* XSCSIReserve_Claim
    (int device_id,
     AsmRoutine release_code,
     char* workspace,
     int access_key);

  [CCode (cname = "scsireserve_claim")]
  public static void SCSIReserve_Claim
    (int device_id,
     AsmRoutine release_code,
     char* workspace,
     int access_key);

  [CCode (cname = "xscsireserve_force_claim")]
  public static OS.Error* XSCSIReserve_ForceClaim
    (int device_id,
     AsmRoutine release_code,
     char* workspace,
     int access_key);

  [CCode (cname = "scsireserve_force_claim")]
  public static void SCSIReserve_ForceClaim
    (int device_id,
     AsmRoutine release_code,
     char* workspace,
     int access_key);

  [CCode (cname = "xscsireserve_release")]
  public static OS.Error* XSCSIReserve_Release
    (int device_id,
     int access_key);

  [CCode (cname = "scsireserve_release")]
  public static void SCSIReserve_Release
    (int device_id,
     int access_key);

  [CCode (cname = "xscsi_list")]
  public static OS.Error* XList
    (CardAddressList list);

  [CCode (cname = "scsi_list")]
  public static void List
    (CardAddressList list);

}
