/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/scale.h")]
namespace OSLib.Scale
{
  [CCode (cname = "scale_flags")]
  public enum Flags
  {
    [CCode (cname = "scale_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "scale_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "scale_INCLUDE_SCALE_TO_FIT")]
    IncludeScaleToFit
  }

  [CCode (cname = "scale_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
    [CCode (cname = "std_value")]
    public unowned int StdValue[4];
  }

  [CCode (cname = "scale_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
    [CCode (cname = "std_value")]
    public unowned int StdValue[4];
  }

  [CCode (cname = "scale_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "scale_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "scale_action_apply_factor")]
  public struct ActionApplyFactor
  {
    [CCode (cname = "percent")]
    public unowned int Percent;
  }

  [CCode (cname = "scale_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "scale_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "scale_action_apply_factor_block")]
  public struct ActionApplyFactorBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "percent")]
    public unowned int Percent;
  }

  [CCode (cname="class_SCALE")]
  public const Toolbox.Class Class_Scale;
  [CCode (cname="action_SCALE_ABOUT_TO_BE_SHOWN")]
  public const bits Action_ScaleAboutToBeShown;
  [CCode (cname="action_SCALE_DIALOGUE_COMPLETED")]
  public const bits Action_ScaleDialogueCompleted;
  [CCode (cname="action_SCALE_APPLY_FACTOR")]
  public const bits Action_ScaleApplyFactor;
  [CCode (cname="error_SCALE_TASKS_ACTIVE")]
  public const bits Error_ScaleTasksActive;
  [CCode (cname="error_SCALE_ALLOC_FAILED")]
  public const bits Error_ScaleAllocFailed;
  [CCode (cname="error_SCALE_SHORT_BUFFER")]
  public const bits Error_ScaleShortBuffer;
  [CCode (cname="error_SCALE_NO_SUCH_TASK")]
  public const bits Error_ScaleNoSuchTask;
  [CCode (cname="error_SCALE_NO_SUCH_METHOD")]
  public const bits Error_ScaleNoSuchMethod;
  [CCode (cname="error_SCALE_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_ScaleNoSuchMiscOpMethod;
  [CCode (cname="scale_SET_LOWER_BOUND")]
  public const bits SetLowerBound;
  [CCode (cname="scale_SET_UPPER_BOUND")]
  public const bits SetUpperBound;
  [CCode (cname="scale_SET_STEP_SIZE")]
  public const bits SetStepSize;

  [CCode (cname = "xscale_get_window_id")]
  public static OS.Error* XGetWindowID
    (bits flags,
     Toolbox.O scale,
     out Toolbox.O window);

  [CCode (cname = "scale_get_window_id")]
  public static Toolbox.O GetWindowID
    (bits flags,
     Toolbox.O scale);

  [CCode (cname = "xscale_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O scale,
     int value);

  [CCode (cname = "scale_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O scale,
     int value);

  [CCode (cname = "xscale_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O scale,
     out int value);

  [CCode (cname = "scale_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O scale);

  [CCode (cname = "xscale_set_bounds")]
  public static OS.Error* XSetBounds
    (bits flags,
     Toolbox.O scale,
     int lower_bound,
     int upper_bound,
     int step_size);

  [CCode (cname = "scale_set_bounds")]
  public static void SetBounds
    (bits flags,
     Toolbox.O scale,
     int lower_bound,
     int upper_bound,
     int step_size);

  [CCode (cname = "xscale_get_bounds")]
  public static OS.Error* XGetBounds
    (bits flags,
     Toolbox.O scale,
     out int lower_bound,
     out int upper_bound,
     out int step_size);

  [CCode (cname = "scale_get_bounds")]
  public static void GetBounds
    (bits flags,
     Toolbox.O scale,
     out int lower_bound,
     out int upper_bound,
     out int step_size);

  [CCode (cname = "xscale_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O scale,
     string title);

  [CCode (cname = "scale_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O scale,
     string title);

  [CCode (cname = "xscale_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O scale,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "scale_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O scale,
     string buffer,
     int size);

}
