/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/scrollbar.h")]
namespace OSLib.Scrollbar
{
  [CCode (cname = "scrollbar_gadget")]
  public struct Gadget
  {
    [CCode (cname = "type")]
    public unowned int Type;
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "value")]
    public unowned int Value;
    [CCode (cname = "visible_length")]
    public unowned int VisibleLength;
    [CCode (cname = "line_increment")]
    public unowned int LineIncrement;
    [CCode (cname = "page_increment")]
    public unowned int PageIncrement;
  }

  [CCode (cname = "scrollbar_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "type")]
    public unowned int Type;
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "value")]
    public unowned int Value;
    [CCode (cname = "visible_length")]
    public unowned int VisibleLength;
    [CCode (cname = "line_increment")]
    public unowned int LineIncrement;
    [CCode (cname = "page_increment")]
    public unowned int PageIncrement;
  }

  [CCode (cname = "scrollbar_position_changed")]
  public struct PositionChanged
  {
    [CCode (cname = "new_position")]
    public unowned bits NewPosition;
    [CCode (cname = "direction")]
    public unowned int Direction;
  }

  [CCode (cname = "scrollbar_action_position_changed")]
  public struct ActionPositionChanged
  {
    [CCode (cname = "new_position")]
    public unowned bits NewPosition;
    [CCode (cname = "direction")]
    public unowned int Direction;
  }

  [CCode (cname = "scrollbar_action_position_changed_block")]
  public struct ActionPositionChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "new_position")]
    public unowned bits NewPosition;
    [CCode (cname = "direction")]
    public unowned int Direction;
  }

  [CCode (cname = "scrollbar_state")]
  public enum State
  {
    [CCode (cname = "scrollbar_HAS_VERTICAL_SCROLLBAR")]
    HasVerticalScrollbar,
    [CCode (cname = "scrollbar_HAS_HORIZONTAL_SCROLLBAR")]
    HasHorizontalScrollbar
  }

  [CCode (cname = "scrollbar_bounds_flags")]
  public enum BoundsFlags
  {
    [CCode (cname = "scrollbar_BOUNDS_LOWER")]
    BoundsLower,
    [CCode (cname = "scrollbar_BOUNDS_UPPER")]
    BoundsUpper,
    [CCode (cname = "scrollbar_BOUNDS_VISIBLE_LENGTH")]
    BoundsVisibleLength
  }

  [CCode (cname = "scrollbar_increments_flags")]
  public enum IncrementsFlags
  {
    [CCode (cname = "scrollbar_INCREMENTS_LINE")]
    IncrementsLine,
    [CCode (cname = "scrollbar_INCREMENTS_PAGE")]
    IncrementsPage
  }

  [CCode (cname="class_SCROLLBAR")]
  public const Toolbox.Class Class_Scrollbar;
  [CCode (cname="action_SCROLLBAR_POSITION_CHANGED")]
  public const bits Action_ScrollbarPositionChanged;

  [CCode (cname = "xscrollbar_get_state")]
  public static OS.Error* XGetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out State state);

  [CCode (cname = "scrollbar_get_state")]
  public static State GetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_set_state")]
  public static OS.Error* XSetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     State state);

  [CCode (cname = "scrollbar_set_state")]
  public static void SetState
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     State state);

  [CCode (cname = "xscrollbar_set_bounds")]
  public static OS.Error* XSetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int lower_bound,
     int upper_bound,
     int visible_length);

  [CCode (cname = "scrollbar_set_bounds")]
  public static void SetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int lower_bound,
     int upper_bound,
     int visible_length);

  [CCode (cname = "xscrollbar_set_lower_bound")]
  public static OS.Error* XSetLowerBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int lower_bound);

  [CCode (cname = "scrollbar_set_lower_bound")]
  public static void SetLowerBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int lower_bound);

  [CCode (cname = "xscrollbar_set_upper_bound")]
  public static OS.Error* XSetUpperBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int upper_bound);

  [CCode (cname = "scrollbar_set_upper_bound")]
  public static void SetUpperBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int upper_bound);

  [CCode (cname = "xscrollbar_set_visible_length")]
  public static OS.Error* XSetVisibleLength
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int visible_length);

  [CCode (cname = "scrollbar_set_visible_length")]
  public static void SetVisibleLength
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int visible_length);

  [CCode (cname = "xscrollbar_get_bounds")]
  public static OS.Error* XGetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int lower_bound,
     out int upper_bound,
     out int visible_length);

  [CCode (cname = "scrollbar_get_bounds")]
  public static void GetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int lower_bound,
     out int upper_bound,
     out int visible_length);

  [CCode (cname = "xscrollbar_get_lower_bound")]
  public static OS.Error* XGetLowerBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int lower_bound);

  [CCode (cname = "scrollbar_get_lower_bound")]
  public static int GetLowerBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_get_upper_bound")]
  public static OS.Error* XGetUpperBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int upper_bound);

  [CCode (cname = "scrollbar_get_upper_bound")]
  public static int GetUpperBound
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_get_visible_length")]
  public static OS.Error* XGetVisibleLength
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int visible_length);

  [CCode (cname = "scrollbar_get_visible_length")]
  public static int GetVisibleLength
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int value);

  [CCode (cname = "scrollbar_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int value);

  [CCode (cname = "xscrollbar_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int value);

  [CCode (cname = "scrollbar_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_set_increments")]
  public static OS.Error* XSetIncrements
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int line_increment,
     int page_increment);

  [CCode (cname = "scrollbar_set_increments")]
  public static void SetIncrements
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int line_increment,
     int page_increment);

  [CCode (cname = "xscrollbar_set_line_increment")]
  public static OS.Error* XSetLineIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int line_increment);

  [CCode (cname = "scrollbar_set_line_increment")]
  public static void SetLineIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int line_increment);

  [CCode (cname = "xscrollbar_set_page_increment")]
  public static OS.Error* XSetPageIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int page_increment);

  [CCode (cname = "scrollbar_set_page_increment")]
  public static void SetPageIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     int page_increment);

  [CCode (cname = "xscrollbar_get_increments")]
  public static OS.Error* XGetIncrements
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int line_increment,
     out int page_increment);

  [CCode (cname = "scrollbar_get_increments")]
  public static void GetIncrements
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int line_increment,
     out int page_increment);

  [CCode (cname = "xscrollbar_get_line_increment")]
  public static OS.Error* XGetLineIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int line_increment);

  [CCode (cname = "scrollbar_get_line_increment")]
  public static int GetLineIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_get_page_increment")]
  public static OS.Error* XGetPageIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out int page_increment);

  [CCode (cname = "scrollbar_get_page_increment")]
  public static int GetPageIncrement
    (IncrementsFlags flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

  [CCode (cname = "xscrollbar_set_event")]
  public static OS.Error* XSetEvent
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     bits event_code);

  [CCode (cname = "scrollbar_set_event")]
  public static void SetEvent
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     bits event_code);

  [CCode (cname = "xscrollbar_get_event")]
  public static OS.Error* XGetEvent
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar,
     out bits event_code);

  [CCode (cname = "scrollbar_get_event")]
  public static bits GetEvent
    (bits flags,
     Toolbox.O window,
     Toolbox.C scroll_bar);

}
