/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/sharedclibrary.h")]
namespace OSLib.SharedCLibrary
{
  [CCode (cname = "sharedclibrary_stub")]
  public struct Stub
  {
    [CCode (cname = "chunk_id")]
    public unowned int ChunkId;
    [CCode (cname = "entry_vector")]
    public unowned char** EntryVector;
    [CCode (cname = "entry_vector_limit")]
    public unowned char** EntryVectorLimit;
    [CCode (cname = "static_data")]
    public unowned char* StaticData;
    [CCode (cname = "static_data_limit")]
    public unowned char* StaticDataLimit;
  }

  [CCode (cname = "sharedclibrary_stub_list")]
  public struct StubList
  {
    [CCode (cname = "stub", array_length = false)]
    public unowned Stub[] Stub;
  }

  [CCode (cname = "kernel_unwind_block")]
  public struct Kernel_UnwindBlock
  {
    [CCode (cname = "r4")]
    public unowned int R4;
    [CCode (cname = "r5")]
    public unowned int R5;
    [CCode (cname = "r6")]
    public unowned int R6;
    [CCode (cname = "r7")]
    public unowned int R7;
    [CCode (cname = "r8")]
    public unowned int R8;
    [CCode (cname = "r9")]
    public unowned int R9;
    [CCode (cname = "fp")]
    public unowned int Fp;
    [CCode (cname = "sp")]
    public unowned int Sp;
    [CCode (cname = "pc")]
    public unowned int Pc;
    [CCode (cname = "sl")]
    public unowned int Sl;
    [CCode (cname = "f4")]
    public unowned int F4[3];
    [CCode (cname = "f5")]
    public unowned int F5[3];
    [CCode (cname = "f6")]
    public unowned int F6[3];
    [CCode (cname = "f7")]
    public unowned int F7[3];
  }

  [CCode (cname="sharedclibrary_ROOT_STACK_SIZE_SHIFT")]
  public const int RootStackSizeShift;

  [CCode (cname = "xsharedclibrary_lib_init_apcs_r")]
  public static OS.Error* XLibInitAPCS_R
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "sharedclibrary_lib_init_apcs_r")]
  public static void LibInitAPCS_R
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "xsharedclibrary_lib_init_module")]
  public static OS.Error* XLibInitModule
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     char* clear,
     char* static_data,
     char* static_data_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "sharedclibrary_lib_init_module")]
  public static void LibInitModule
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     char* clear,
     char* static_data,
     char* static_data_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "xsharedclibrary_lib_init_apcs_32")]
  public static OS.Error* XLibInitAPCS_32
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "sharedclibrary_lib_init_apcs_32")]
  public static void LibInitAPCS_32
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "xsharedclibrary_lib_init_module_apcs_32")]
  public static OS.Error* XLibInitModuleAPCS_32
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     char* clear,
     char* static_data,
     char* static_data_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "sharedclibrary_lib_init_module_apcs_32")]
  public static void LibInitModuleAPCS_32
    (StubList stub_list,
     char* workspace,
     char* workspace_limit,
     char* clear,
     char* static_data,
     char* static_data_limit,
     int shifted_root_stack_size,
     out char* stack,
     out char* stack_limit,
     out int version);

  [CCode (cname = "xservice_apcs_backtrace_backtrace_request")]
  public static OS.Error* XService_APCSBacktraceBacktraceRequest
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     Kernel_UnwindBlock unwind,
     string language,
     int error_number,
     char* fault_address,
     char* app_def,
     out bool unclaimed);

  [CCode (cname = "service_apcs_backtrace_backtrace_request")]
  public static bool Service_APCSBacktraceBacktraceRequest
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     Kernel_UnwindBlock unwind,
     string language,
     int error_number,
     char* fault_address,
     char* app_def);

  [CCode (cname = "xservice_apcs_backtrace_register_dump")]
  public static OS.Error* XService_APCSBacktraceRegisterDump
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     char* app_def,
     out bool unclaimed);

  [CCode (cname = "service_apcs_backtrace_register_dump")]
  public static bool Service_APCSBacktraceRegisterDump
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     char* app_def);

  [CCode (cname = "xservice_apcs_backtrace_backtrace_dump")]
  public static OS.Error* XService_APCSBacktraceBacktraceDump
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     Kernel_UnwindBlock unwind,
     string language,
     char* app_def,
     out bool unclaimed);

  [CCode (cname = "service_apcs_backtrace_backtrace_dump")]
  public static bool Service_APCSBacktraceBacktraceDump
    (OS.F output,
     OS.RegisterBlock initial_register_set,
     Kernel_UnwindBlock unwind,
     string language,
     char* app_def);

  [CCode (cname = "xservice_apcs_backtrace_decode_name")]
  public static OS.Error* XService_APCSBacktraceDecodeName
    (string encoded_name,
     string language,
     out bool unclaimed,
     out string decoded_name);

  [CCode (cname = "service_apcs_backtrace_decode_name")]
  public static bool Service_APCSBacktraceDecodeName
    (string encoded_name,
     string language,
     out string decoded_name);

  [CCode (cname = "xservice_apcs_backtrace_mini_dump_created")]
  public static OS.Error* XService_APCSBacktraceMiniDumpCreated
    (string minidump_filename,
     string executable_name,
     out bool unclaimed);

  [CCode (cname = "service_apcs_backtrace_mini_dump_created")]
  public static bool Service_APCSBacktraceMiniDumpCreated
    (string minidump_filename,
     string executable_name);

}
