/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Wimp;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/slider.h")]
namespace OSLib.Slider
{
  [CCode (cname = "slider_gadget")]
  public struct Gadget
  {
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "initial_value")]
    public unowned int InitialValue;
  }

  [CCode (cname = "slider_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "lower_bound")]
    public unowned int LowerBound;
    [CCode (cname = "upper_bound")]
    public unowned int UpperBound;
    [CCode (cname = "step_size")]
    public unowned int StepSize;
    [CCode (cname = "initial_value")]
    public unowned int InitialValue;
  }

  [CCode (cname = "slider_action_value_changed")]
  public struct ActionValueChanged
  {
    [CCode (cname = "value")]
    public unowned int Value;
  }

  [CCode (cname = "slider_action_value_changed_block")]
  public struct ActionValueChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "value")]
    public unowned int Value;
  }

  [CCode (cname = "slider_bounds_flags")]
  public enum BoundsFlags
  {
    [CCode (cname = "slider_BOUND_LOWER")]
    BoundLower,
    [CCode (cname = "slider_BOUND_UPPER")]
    BoundUpper,
    [CCode (cname = "slider_BOUND_STEP")]
    BoundStep
  }

  [CCode (cname="class_SLIDER")]
  public const Toolbox.Class Class_Slider;
  [CCode (cname="slider_GENERATE_VALUE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateValueChanged;
  [CCode (cname="slider_GENERATE_VALUE_CHANGED_BY_DRAGGING")]
  public const OSLib.Gadget.Flags GenerateValueChangedByDragging;
  [CCode (cname="slider_GENERATE_SET_VALUE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateSetValueChanged;
  [CCode (cname="slider_VERTICAL")]
  public const OSLib.Gadget.Flags Vertical;
  [CCode (cname="slider_DRAGGABLE")]
  public const OSLib.Gadget.Flags Draggable;
  [CCode (cname="slider_KNOB_COLOUR")]
  public const OSLib.Gadget.Flags KnobColour;
  [CCode (cname="slider_KNOB_COLOUR_SHIFT")]
  public const int KnobColourShift;
  [CCode (cname="slider_WELL_COLOUR")]
  public const OSLib.Gadget.Flags WellColour;
  [CCode (cname="slider_WELL_COLOUR_SHIFT")]
  public const int WellColourShift;
  [CCode (cname="action_SLIDER_VALUE_CHANGED")]
  public const bits Action_SliderValueChanged;
  [CCode (cname="slider_VALUE_CHANGING")]
  public const bits ValueChanging;
  [CCode (cname="slider_VALUE_CHANGED_BY_DRAGGING")]
  public const bits ValueChangedByDragging;
  [CCode (cname="slider_VALUE_CHANGED")]
  public const bits ValueChanged;

  [CCode (cname = "xslider_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     int value);

  [CCode (cname = "slider_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     int value);

  [CCode (cname = "xslider_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     out int value);

  [CCode (cname = "slider_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider);

  [CCode (cname = "xslider_set_bounds")]
  public static OS.Error* XSetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C slider,
     int lower_bound,
     int upper_bound,
     int step_size);

  [CCode (cname = "slider_set_bounds")]
  public static void SetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C slider,
     int lower_bound,
     int upper_bound,
     int step_size);

  [CCode (cname = "xslider_get_bounds")]
  public static OS.Error* XGetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C slider,
     out int lower_bound,
     out int upper_bound,
     out int step_size);

  [CCode (cname = "slider_get_bounds")]
  public static void GetBounds
    (BoundsFlags flags,
     Toolbox.O window,
     Toolbox.C slider,
     out int lower_bound,
     out int upper_bound,
     out int step_size);

  [CCode (cname = "xslider_set_colour")]
  public static OS.Error* XSetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     Wimp.Colour bar_colour,
     Wimp.Colour background_colour);

  [CCode (cname = "slider_set_colour")]
  public static void SetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     Wimp.Colour bar_colour,
     Wimp.Colour background_colour);

  [CCode (cname = "xslider_get_colour")]
  public static OS.Error* XGetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     out Wimp.Colour bar_colour,
     out Wimp.Colour background_colour);

  [CCode (cname = "slider_get_colour")]
  public static void GetColour
    (bits flags,
     Toolbox.O window,
     Toolbox.C slider,
     out Wimp.Colour bar_colour,
     out Wimp.Colour background_colour);

}
