/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/socket.h")]
namespace OSLib.Socket
{
  [CCode (cname = "socket_af")]
  public enum AF
  {
    [CCode (cname = "socket_AF_INET")]
    Inet
  }

  [CCode (cname = "socket_sock")]
  public enum Sock
  {
    [CCode (cname = "socket_SOCK_STREAM")]
    Stream,
    [CCode (cname = "socket_SOCK_DGRAM")]
    DGram,
    [CCode (cname = "socket_SOCK_RAW")]
    Raw
  }

  [CCode (cname = "socket_ipproto")]
  public enum Ipproto
  {
    [CCode (cname = "socket_IPPROTO_IP")]
    IP,
    [CCode (cname = "socket_IPPROTO_ICMP")]
    ICMP,
    [CCode (cname = "socket_IPPROTO_TCP")]
    TCP,
    [CCode (cname = "socket_IPPROTO_UDP")]
    UDP,
    [CCode (cname = "socket_IPPROTO_RAW")]
    Raw
  }

  [CCode (cname = "socket_level")]
  public enum Level
  {
    [CCode (cname = "socket_SOL_SOCKET")]
    SolSocket
  }

  [CCode (cname = "socket_sockaddr_in")]
  public struct SockaddrIn
  {
    [CCode (cname = "af")]
    public unowned AF Af;
    [CCode (cname = "port")]
    public unowned short Port;
    [CCode (cname = "addr")]
    public unowned bits Addr;
    [CCode (cname = "data")]
    public unowned byte Data[8];
  }

  [CCode (cname = "socket_sockaddr")]
  public struct Sockaddr
  {
    [CCode (cname = "sockaddr.af")]
    public unowned AF Sockaddr_Af;
    [CCode (cname = "sockaddr.data")]
    public unowned byte Sockaddr_Data[14];
    [CCode (cname = "sockaddr_in")]
    public unowned SockaddrIn SockaddrIn;
  }

  [CCode (cname = "socket_linger")]
  public struct Linger
  {
    [CCode (cname = "onoff")]
    public unowned int Onoff;
    [CCode (cname = "linger")]
    public unowned int Linger;
  }

  [CCode (cname = "socket_timeval")]
  public struct Timeval
  {
    [CCode (cname = "sec")]
    public unowned int Sec;
    [CCode (cname = "usec")]
    public unowned int Usec;
  }

  [CCode (cname = "socket_so")]
  public enum So
  {
    [CCode (cname = "socket_SO_ACCEPTCONN")]
    Acceptconn,
    [CCode (cname = "socket_SO_REUSEADDR")]
    Reuseaddr,
    [CCode (cname = "socket_SO_KEEPALIVE")]
    Keepalive,
    [CCode (cname = "socket_SO_DONTROUTE")]
    Dontroute,
    [CCode (cname = "socket_SO_BROADCAST")]
    Broadcast,
    [CCode (cname = "socket_SO_USELOOPBACK")]
    Useloopback,
    [CCode (cname = "socket_SO_LINGER")]
    Linger,
    [CCode (cname = "socket_SO_OOBINLINE")]
    Oobinline,
    [CCode (cname = "socket_SO_SNDBUF")]
    Sndbuf,
    [CCode (cname = "socket_SO_RCVBUF")]
    Rcvbuf,
    [CCode (cname = "socket_SO_SNDLOWAT")]
    Sndlowat,
    [CCode (cname = "socket_SO_RCVLOWAT")]
    Rcvlowat,
    [CCode (cname = "socket_SO_SNDTIMEO")]
    Sndtimeo,
    [CCode (cname = "socket_SO_RCVTIMEO")]
    Rcvtimeo,
    [CCode (cname = "socket_SO_ERROR")]
    Error,
    [CCode (cname = "socket_SO_TYPE")]
    Type
  }

  [CCode (cname = "socket_if_flags")]
  public enum IFFlags
  {
    [CCode (cname = "socket_IF_UP")]
    IFUp,
    [CCode (cname = "socket_IF_BROADCAST")]
    IFBroadcast,
    [CCode (cname = "socket_IF_DEBUG")]
    IFDebug,
    [CCode (cname = "socket_IF_LOOPBACK")]
    IFLoopback,
    [CCode (cname = "socket_IF_POINT_TO_POINT")]
    IFPointToPoint,
    [CCode (cname = "socket_IF_NO_TRAILERS")]
    IFNoTrailers,
    [CCode (cname = "socket_IF_RUNNING")]
    IFRunning,
    [CCode (cname = "socket_IF_NO_ARP")]
    IFNoARP
  }

  [CCode (cname = "socket_ifreq")]
  public struct Ifreq
  {
    [CCode (cname = "name")]
    public unowned char Name[16];
    [CCode (cname = "u.sockaddr")]
    public unowned Sockaddr U_Sockaddr;
    [CCode (cname = "u.broadaddr")]
    public unowned Sockaddr U_Broadaddr;
    [CCode (cname = "u.flags")]
    public unowned IFFlags U_Flags;
    [CCode (cname = "u.metric")]
    public unowned int U_Metric;
    [CCode (cname = "u.data")]
    public unowned byte* U_Data;
    [CCode (cname = "u.reserved")]
    public unowned byte U_Reserved[16];
  }

  [CCode (cname = "socket_ifconf")]
  public struct Ifconf
  {
    [CCode (cname = "len")]
    public unowned int Len;
    [CCode (cname = "u.buf")]
    public unowned byte* U_Buf;
    [CCode (cname = "u.req")]
    public unowned Ifreq* U_Req;
  }

  [CCode (cname = "socket_pwstruct")]
  public struct Pwstruct
  {
    [CCode (cname = "pollword")]
    public unowned bits* Pollword;
    [CCode (cname = "bit_number")]
    public unowned int BitNumber;
  }

  [CCode (cname = "socket_ioctl_no")]
  public enum IoctlNo
  {
    [CCode (cname = "socket_SIOCSHIWAT")]
    SIOCSHIWAT,
    [CCode (cname = "socket_SIOCGHIWAT")]
    SIOCGHIWAT,
    [CCode (cname = "socket_SIOCSLOWAT")]
    SIOCSLOWAT,
    [CCode (cname = "socket_SIOCGLOWAT")]
    SIOCGLOWAT,
    [CCode (cname = "socket_SIOCATMARK")]
    SIOCATMARK,
    [CCode (cname = "socket_SIOCSIFADDR")]
    SIOCSIFADDR,
    [CCode (cname = "socket_SIOCGIFADDR")]
    SIOCGIFADDR,
    [CCode (cname = "socket_SIOCSIFDSTADDR")]
    SIOCSIFDSTADDR,
    [CCode (cname = "socket_SIOCGIFDSTADDR")]
    SIOCGIFDSTADDR,
    [CCode (cname = "socket_SIOCSIFFLAGS")]
    SIOCSIFFLAGS,
    [CCode (cname = "socket_SIOCGIFFLAGS")]
    SIOCGIFFLAGS,
    [CCode (cname = "socket_SIOCGIFBRDADDR")]
    SIOCGIFBRDADDR,
    [CCode (cname = "socket_SIOCSIFBRDADDR")]
    SIOCSIFBRDADDR,
    [CCode (cname = "socket_SIOCGIFCONF")]
    SIOCGIFCONF,
    [CCode (cname = "socket_SIOCGIFNETMASK")]
    SIOCGIFNETMASK,
    [CCode (cname = "socket_SIOCSIFNETMASK")]
    SIOCSIFNETMASK,
    [CCode (cname = "socket_SIOCGIFMETRIC")]
    SIOCGIFMETRIC,
    [CCode (cname = "socket_SIOCSIFMETRIC")]
    SIOCSIFMETRIC,
    [CCode (cname = "socket_FIOSLEEPTW")]
    FIOSLEEPTW,
    [CCode (cname = "socket_FIONREAD")]
    FIONREAD,
    [CCode (cname = "socket_FIONBIO")]
    FIONBIO,
    [CCode (cname = "socket_FIOASYNC")]
    FIOASYNC,
    [CCode (cname = "socket_FIOPOLLWORD")]
    FIOPOLLWORD
  }

  [CCode (cname = "socket_msg_flags")]
  public enum MsgFlags
  {
    [CCode (cname = "socket_MSG_OOB")]
    MsgOob,
    [CCode (cname = "socket_MSG_PEEK")]
    MsgPeek,
    [CCode (cname = "socket_MSG_DONTROUTE")]
    MsgDontroute
  }

  [CCode (cname = "socket_s", has_type_id = false)]
  public struct S : bits
  {
  }

  [CCode (cname = "socket_iovec")]
  public struct Iovec
  {
    [CCode (cname = "base")]
    public unowned char* Base;
    [CCode (cname = "len")]
    public unowned int Len;
  }

  [CCode (cname = "socket_msghdr")]
  public struct Msghdr
  {
    [CCode (cname = "name")]
    public unowned char* Name;
    [CCode (cname = "namelen")]
    public unowned int Namelen;
    [CCode (cname = "iov")]
    public unowned Iovec* Iov;
    [CCode (cname = "iov_count")]
    public unowned int IovCount;
    [CCode (cname = "accrights")]
    public unowned char* Accrights;
    [CCode (cname = "accrightslen")]
    public unowned int Accrightslen;
  }

  [CCode (cname = "socket_shutdown_type")]
  public enum ShutdownType
  {
    [CCode (cname = "socket_SHUTDOWN_RECV")]
    ShutdownRecv,
    [CCode (cname = "socket_SHUTDOWN_SEND")]
    ShutdownSend,
    [CCode (cname = "socket_SHUTDOWN_ALL")]
    ShutdownAll
  }

  [CCode (cname = "socket_fdset")]
  public enum Fdset
  {
    Default = 0
  }

  [CCode (cname = "socket_event_type")]
  public enum EventType
  {
    [CCode (cname = "socket_ASYNC_EVENT")]
    AsyncEvent,
    [CCode (cname = "socket_URGENT_EVENT")]
    UrgentEvent,
    [CCode (cname = "socket_BROKEN_EVENT")]
    BrokenEvent
  }

  [CCode (cname="socket_EBADF")]
  public const bits EBadf;
  [CCode (cname="socket_EAGAIN")]
  public const bits EAgain;
  [CCode (cname="socket_EFAULT")]
  public const bits EFault;
  [CCode (cname="socket_EINVAL")]
  public const bits EInval;
  [CCode (cname="socket_EWOULDBLOCK")]
  public const bits EWouldblock;
  [CCode (cname="socket_EINPROGRESS")]
  public const bits EInprogress;
  [CCode (cname="socket_EALREADY")]
  public const bits EAlready;
  [CCode (cname="socket_ENOTSOCK")]
  public const bits ENotsock;
  [CCode (cname="socket_EDESTADDRREQ")]
  public const bits EDestaddrreq;
  [CCode (cname="socket_EMSGSIZE")]
  public const bits EMsgsize;
  [CCode (cname="socket_EPROTOTYPE")]
  public const bits EPrototype;
  [CCode (cname="socket_ENOPROTOOPT")]
  public const bits ENoprotoopt;
  [CCode (cname="socket_EPROTONOSUPPORT")]
  public const bits EProtonosupport;
  [CCode (cname="socket_ESOCKTNOSUPPORT")]
  public const bits ESocktnosupport;
  [CCode (cname="socket_EOPNOTSUPP")]
  public const bits EOpnotsupp;
  [CCode (cname="socket_EPFNOSUPPORT")]
  public const bits EPfnosupport;
  [CCode (cname="socket_EAFNOSUPPORT")]
  public const bits EAfnosupport;
  [CCode (cname="socket_EADDRINUSE")]
  public const bits EAddrinuse;
  [CCode (cname="socket_EADDRNOTAVAIL")]
  public const bits EAddrnotavail;
  [CCode (cname="socket_ENETDOWN")]
  public const bits ENetdown;
  [CCode (cname="socket_ENETUNREACH")]
  public const bits ENetunreach;
  [CCode (cname="socket_ENETRESET")]
  public const bits ENetreset;
  [CCode (cname="socket_ECONNABORTED")]
  public const bits EConnaborted;
  [CCode (cname="socket_ECONNRESET")]
  public const bits EConnreset;
  [CCode (cname="socket_ENOBUFS")]
  public const bits ENobufs;
  [CCode (cname="socket_EISCONN")]
  public const bits EIsconn;
  [CCode (cname="socket_ENOTCONN")]
  public const bits ENotconn;
  [CCode (cname="socket_ESHUTDOWN")]
  public const bits EShutdown;
  [CCode (cname="socket_ETOOMANYREFS")]
  public const bits EToomanyrefs;
  [CCode (cname="socket_ETIMEDOUT")]
  public const bits ETimedout;
  [CCode (cname="socket_ECONNREFUSED")]
  public const bits EConnrefused;
  [CCode (cname="socket_EHOSTDOWN")]
  public const bits EHostdown;
  [CCode (cname="socket_EHOSTUNREACH")]
  public const bits EHostunreach;
  [CCode (cname="socket_IF_NAME_LIMIT")]
  public const int IFNameLimit;

  [CCode (cname = "xsocket_creat")]
  public static OS.Error* XCreat
    (AF af,
     Sock sock,
     Ipproto proto,
     out S s);

  [CCode (cname = "socket_creat")]
  public static S Creat
    (AF af,
     Sock sock,
     Ipproto proto);

  [CCode (cname = "xsocket_bind")]
  public static OS.Error* XBind
    (S s,
     Sockaddr sockaddr,
     int namelen);

  [CCode (cname = "socket_bind")]
  public static void Bind
    (S s,
     Sockaddr sockaddr,
     int namelen);

  [CCode (cname = "xsocket_listen")]
  public static OS.Error* XListen
    (S s,
     int backlog);

  [CCode (cname = "socket_listen")]
  public static void Listen
    (S s,
     int backlog);

  [CCode (cname = "xsocket_accept")]
  public static OS.Error* XAccept
    (S s,
     Sockaddr sockaddr,
     int* addrlen,
     out S ns);

  [CCode (cname = "socket_accept")]
  public static S Accept
    (S s,
     Sockaddr sockaddr,
     int* addrlen);

  [CCode (cname = "xsocket_connect")]
  public static OS.Error* XConnect
    (S s,
     Sockaddr sockaddr,
     int namelen);

  [CCode (cname = "socket_connect")]
  public static void Connect
    (S s,
     Sockaddr sockaddr,
     int namelen);

  [CCode (cname = "xsocket_recv")]
  public static OS.Error* XRecv
    (S s,
     char* buf,
     int len,
     MsgFlags flags,
     out int cc);

  [CCode (cname = "socket_recv")]
  public static int Recv
    (S s,
     char* buf,
     int len,
     MsgFlags flags);

  [CCode (cname = "xsocket_recvfrom")]
  public static OS.Error* XRecvfrom
    (S s,
     char* buf,
     int len,
     MsgFlags flags,
     Sockaddr from,
     int* fromlen,
     out int cc);

  [CCode (cname = "socket_recvfrom")]
  public static int Recvfrom
    (S s,
     char* buf,
     int len,
     MsgFlags flags,
     Sockaddr from,
     int* fromlen);

  [CCode (cname = "xsocket_recvmsg")]
  public static OS.Error* XRecvmsg
    (S s,
     Msghdr msg,
     MsgFlags flags,
     out int cc);

  [CCode (cname = "socket_recvmsg")]
  public static int Recvmsg
    (S s,
     Msghdr msg,
     MsgFlags flags);

  [CCode (cname = "xsocket_send")]
  public static OS.Error* XSend
    (S s,
     char* msg,
     int len,
     MsgFlags flags,
     out int cc);

  [CCode (cname = "socket_send")]
  public static int Send
    (S s,
     char* msg,
     int len,
     MsgFlags flags);

  [CCode (cname = "xsocket_sendto")]
  public static OS.Error* XSendto
    (S s,
     char* msg,
     int len,
     MsgFlags flags,
     Sockaddr to,
     int tolen,
     out int cc);

  [CCode (cname = "socket_sendto")]
  public static int Sendto
    (S s,
     char* msg,
     int len,
     MsgFlags flags,
     Sockaddr to,
     int tolen);

  [CCode (cname = "xsocket_sendmsg")]
  public static OS.Error* XSendmsg
    (S s,
     Msghdr msg,
     MsgFlags flags,
     out int cc);

  [CCode (cname = "socket_sendmsg")]
  public static int Sendmsg
    (S s,
     Msghdr msg,
     MsgFlags flags);

  [CCode (cname = "xsocket_shutdown")]
  public static OS.Error* XShutdown
    (S s,
     ShutdownType type);

  [CCode (cname = "socket_shutdown")]
  public static void Shutdown
    (S s,
     ShutdownType type);

  [CCode (cname = "xsocket_setsockopt")]
  public static OS.Error* XSetsockopt
    (S s,
     Level level,
     So optname,
     char* optval,
     int optlen);

  [CCode (cname = "socket_setsockopt")]
  public static void Setsockopt
    (S s,
     Level level,
     So optname,
     char* optval,
     int optlen);

  [CCode (cname = "xsocket_getsockopt")]
  public static OS.Error* XGetsockopt
    (S s,
     Level level,
     So optname,
     char* optval,
     int* optlen);

  [CCode (cname = "socket_getsockopt")]
  public static void Getsockopt
    (S s,
     Level level,
     So optname,
     char* optval,
     int* optlen);

  [CCode (cname = "xsocket_getpeername")]
  public static OS.Error* XGetpeername
    (S s,
     Sockaddr sockaddr,
     int* namelen);

  [CCode (cname = "socket_getpeername")]
  public static void Getpeername
    (S s,
     Sockaddr sockaddr,
     int* namelen);

  [CCode (cname = "xsocket_getsockname")]
  public static OS.Error* XGetsockname
    (S s,
     Sockaddr sockaddr,
     int* namelen);

  [CCode (cname = "socket_getsockname")]
  public static void Getsockname
    (S s,
     Sockaddr sockaddr,
     int* namelen);

  [CCode (cname = "xsocket_close")]
  public static OS.Error* XClose
    (S s);

  [CCode (cname = "socket_close")]
  public static void Close
    (S s);

  [CCode (cname = "xsocket_select")]
  public static OS.Error* XSelect
    (int nfds,
     Fdset readfds,
     Fdset writefds,
     Fdset exceptfds,
     Timeval timeout,
     out int nfound);

  [CCode (cname = "socket_select")]
  public static int Select
    (int nfds,
     Fdset readfds,
     Fdset writefds,
     Fdset exceptfds,
     Timeval timeout);

  [CCode (cname = "xsocket_ioctl")]
  public static OS.Error* XIoctl
    (S s,
     IoctlNo flags,
     char* argp);

  [CCode (cname = "socket_ioctl")]
  public static void Ioctl
    (S s,
     IoctlNo flags,
     char* argp);

  [CCode (cname = "xsocket_read")]
  public static OS.Error* XRead
    (S s,
     char* buf,
     int len,
     out int cc);

  [CCode (cname = "socket_read")]
  public static int Read
    (S s,
     char* buf,
     int len);

  [CCode (cname = "xsocket_write")]
  public static OS.Error* XWrite
    (S s,
     char* msg,
     int len,
     out int cc);

  [CCode (cname = "socket_write")]
  public static int Write
    (S s,
     char* msg,
     int len);

  [CCode (cname = "xsocket_readv")]
  public static OS.Error* XReadv
    (S s,
     Iovec iov,
     int iov_count,
     out int cc);

  [CCode (cname = "socket_readv")]
  public static int Readv
    (S s,
     Iovec iov,
     int iov_count);

  [CCode (cname = "xsocket_writev")]
  public static OS.Error* XWritev
    (S s,
     Iovec iov,
     int iov_count,
     out int cc);

  [CCode (cname = "socket_writev")]
  public static int Writev
    (S s,
     Iovec iov,
     int iov_count);

  [CCode (cname = "xsocket_gettsize")]
  public static OS.Error* XGettsize
    (out int size);

  [CCode (cname = "socket_gettsize")]
  public static int Gettsize ();

  [CCode (cname = "xsocket_version")]
  public static OS.Error* XVersion
    (out int n);

  [CCode (cname = "socket_version")]
  public static int Version ();

  [CCode (cname = "xsocket_filter_deregister")]
  public static OS.Error* XFilterDeregister
    (AF addr_family,
     int prot_hash,
     AsmRoutine routine,
     int priv_word);

  [CCode (cname = "socket_filter_deregister")]
  public static void FilterDeregister
    (AF addr_family,
     int prot_hash,
     AsmRoutine routine,
     int priv_word);

  [CCode (cname = "xsocket_filter_register")]
  public static OS.Error* XFilterRegister
    (AF addr_family,
     int prot_hash,
     AsmRoutine routine,
     int priv_word);

  [CCode (cname = "socket_filter_register")]
  public static void FilterRegister
    (AF addr_family,
     int prot_hash,
     AsmRoutine routine,
     int priv_word);

  [CCode (cname = "xevent_internet")]
  public static OS.Error* XEvent_Internet
    (EventType event_type,
     S s);

  [CCode (cname = "event_internet")]
  public static void Event_Internet
    (EventType event_type,
     S s);

  [CCode (cname = "xeventinternet_async")]
  public static OS.Error* XEventInternet_Async
    (S s);

  [CCode (cname = "eventinternet_async")]
  public static void EventInternet_Async
    (S s);

  [CCode (cname = "xeventinternet_urgent")]
  public static OS.Error* XEventInternet_Urgent
    (S s);

  [CCode (cname = "eventinternet_urgent")]
  public static void EventInternet_Urgent
    (S s);

  [CCode (cname = "xeventinternet_broken")]
  public static OS.Error* XEventInternet_Broken
    (S s);

  [CCode (cname = "eventinternet_broken")]
  public static void EventInternet_Broken
    (S s);

  [CCode (cname = "xeventinternet_rarp_reply")]
  public static OS.Error* XEventInternet_RARPReply
    (bits addr,
     bits req_addr);

  [CCode (cname = "eventinternet_rarp_reply")]
  public static void EventInternet_RARPReply
    (bits addr,
     bits req_addr);

  [CCode (cname = "xservice_internet_status8")]
  public static OS.Error* XService_InternetStatus8
    (char* interface_name,
     char* dib,
     int ip_address,
     char* hw_address,
     out bool unclaimed);

  [CCode (cname = "service_internet_status8")]
  public static bool Service_InternetStatus8
    (char* interface_name,
     char* dib,
     int ip_address,
     char* hw_address);

}
