/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/squash.h")]
namespace OSLib.Squash
{
  [CCode (cname = "squash_input_status")]
  public enum InputStatus
  {
    [CCode (cname = "squash_INPUT_NOT_START")]
    InputNotStart,
    [CCode (cname = "squash_INPUT_NOT_END")]
    InputNotEnd,
    [CCode (cname = "squash_INPUT_ALL_PRESENT")]
    InputAllPresent
  }

  [CCode (cname = "squash_output_status")]
  public enum OutputStatus
  {
    [CCode (cname = "squash_OUTPUT_END")]
    OutputEnd,
    [CCode (cname = "squash_OUTPUT_MORE")]
    OutputMore,
    [CCode (cname = "squash_OUTPUT_NO_MEM")]
    OutputNoMem
  }

  [CCode (cname = "squash_file_base")]
  public struct FileBase
  {
    [CCode (cname = "id")]
    public unowned bits Id;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
  }

  [Compact]
  [CCode (cname = "squash_file", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class File
  {
    public File(int numEntries);
    [CCode (cname = "id")]
    public unowned bits Id;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "data", array_length = false)]
    public unowned byte[] Data;
  }

  [CCode (cname="error_SQUASH_BAD_ADDRESS")]
  public const bits Error_SquashBadAddress;
  [CCode (cname="error_SQUASH_BAD_INPUT")]
  public const bits Error_SquashBadInput;
  [CCode (cname="error_SQUASH_BAD_WORKSPACE")]
  public const bits Error_SquashBadWorkspace;
  [CCode (cname="error_SQUASH_BAD_PARAMETERS")]
  public const bits Error_SquashBadParameters;
  [CCode (cname="squash_FILE_TYPE")]
  public const bits FileType;

  [CCode (cname = "xsquash_compress_return_sizes")]
  public static OS.Error* XCompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "squash_compress_return_sizes")]
  public static void CompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "xsquash_compress")]
  public static OS.Error* XCompress
    (InputStatus input_status,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out OutputStatus output_status,
     out char* input_out,
     out int input_spare,
     out char* output_out,
     out int output_spare);

  [CCode (cname = "squash_compress")]
  public static void Compress
    (InputStatus input_status,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out OutputStatus output_status,
     out char* input_out,
     out int input_spare,
     out char* output_out,
     out int output_spare);

  [CCode (cname = "xsquash_decompress_return_sizes")]
  public static OS.Error* XDecompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "squash_decompress_return_sizes")]
  public static void DecompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "xsquash_decompress")]
  public static OS.Error* XDecompress
    (InputStatus input_status,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out OutputStatus output_status,
     out char* input_out,
     out int input_spare,
     out char* output_out,
     out int output_spare);

  [CCode (cname = "squash_decompress")]
  public static void Decompress
    (InputStatus input_status,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out OutputStatus output_status,
     out char* input_out,
     out int input_spare,
     out char* output_out,
     out int output_spare);

}
