/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/stringset.h")]
namespace OSLib.StringSet
{
  [CCode (cname = "stringset_gadget")]
  public struct Gadget
  {
    [CCode (cname = "string_set")]
    public unowned Toolbox.MsgReference StringSet;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "initial_selected_string")]
    public unowned Toolbox.MsgReference InitialSelectedString;
    [CCode (cname = "selected_string_limit")]
    public unowned int SelectedStringLimit;
    [CCode (cname = "allowable")]
    public unowned Toolbox.MsgReference Allowable;
    [CCode (cname = "allowable_limit")]
    public unowned int AllowableLimit;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
  }

  [CCode (cname = "stringset_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "string_set")]
    public unowned Toolbox.MsgReference StringSet;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "initial_selected_string")]
    public unowned Toolbox.MsgReference InitialSelectedString;
    [CCode (cname = "selected_string_limit")]
    public unowned int SelectedStringLimit;
    [CCode (cname = "allowable")]
    public unowned Toolbox.MsgReference Allowable;
    [CCode (cname = "allowable_limit")]
    public unowned int AllowableLimit;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
  }

  [CCode (cname = "stringset_action_value_changed")]
  public struct ActionValueChanged
  {
    [CCode (cname = "value")]
    public unowned char Value[216];
  }

  [CCode (cname = "stringset_action_value_changed_block")]
  public struct ActionValueChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "value")]
    public unowned char Value[216];
  }

  [CCode (cname = "stringset_action_about_to_be_shown_block")]
  public enum ActionAboutToBeShownBlock
  {
    Default = 0
  }

  [CCode (cname="class_STRING_SET")]
  public const Toolbox.Class Class_StringSet;
  [CCode (cname="stringset_GENERATE_USER_VALUE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateUserValueChanged;
  [CCode (cname="stringset_GENERATE_SET_VALUE_CHANGED")]
  public const OSLib.Gadget.Flags GenerateSetValueChanged;
  [CCode (cname="stringset_WRITABLE")]
  public const OSLib.Gadget.Flags Writable;
  [CCode (cname="stringset_GENERATE_ABOUT_TO_BE_SHOWN")]
  public const OSLib.Gadget.Flags GenerateAboutToBeShown;
  [CCode (cname="stringset_NO_DISPLAY")]
  public const OSLib.Gadget.Flags NoDisplay;
  [CCode (cname="stringset_JUSTIFICATION")]
  public const OSLib.Gadget.Flags Justification;
  [CCode (cname="stringset_JUSTIFICATION_SHIFT")]
  public const int JustificationShift;
  [CCode (cname="stringset_JUSTIFICATION_LEFT")]
  public const bits JustificationLeft;
  [CCode (cname="stringset_JUSTIFICATION_RIGHT")]
  public const bits JustificationRight;
  [CCode (cname="stringset_JUSTIFICATION_CENTRED")]
  public const bits JustificationCentred;
  [CCode (cname="stringset_GET_ALPHANUMERIC_FIELD")]
  public const bits GetAlphanumericField;
  [CCode (cname="stringset_GET_POP_UP_MENU")]
  public const bits GetPopUpMenu;
  [CCode (cname="action_STRING_SET_VALUE_CHANGED")]
  public const bits Action_StringSetValueChanged;
  [CCode (cname="action_STRING_SET_ABOUT_TO_BE_SHOWN")]
  public const bits Action_StringSetAboutToBeShown;
  [CCode (cname="stringset_VALUE_TOO_LONG")]
  public const bits ValueTooLong;

  [CCode (cname = "xstringset_set_available")]
  public static OS.Error* XSetAvailable
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string set);

  [CCode (cname = "stringset_set_available")]
  public static void SetAvailable
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string set);

  [CCode (cname = "xstringset_set_selected_string")]
  public static OS.Error* XSetSelectedString
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string selection);

  [CCode (cname = "stringset_set_selected_string")]
  public static void SetSelectedString
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string selection);

  [CCode (cname = "xstringset_set_selected_index")]
  public static OS.Error* XSetSelectedIndex
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     int index);

  [CCode (cname = "stringset_set_selected_index")]
  public static void SetSelectedIndex
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     int index);

  [CCode (cname = "xstringset_get_selected_string")]
  public static OS.Error* XGetSelectedString
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "stringset_get_selected_string")]
  public static int GetSelectedString
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string buffer,
     int size);

  [CCode (cname = "xstringset_get_selected_index")]
  public static OS.Error* XGetSelectedIndex
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     out int index);

  [CCode (cname = "stringset_get_selected_index")]
  public static int GetSelectedIndex
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set);

  [CCode (cname = "xstringset_set_allowable")]
  public static OS.Error* XSetAllowable
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string allowable);

  [CCode (cname = "stringset_set_allowable")]
  public static void SetAllowable
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     string allowable);

  [CCode (cname = "xstringset_get_components")]
  public static OS.Error* XGetComponents
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     out Toolbox.C alphanumeric_field,
     out Toolbox.C popup_menu);

  [CCode (cname = "stringset_get_components")]
  public static void GetComponents
    (bits flags,
     Toolbox.O window,
     Toolbox.C string_set,
     out Toolbox.C alphanumeric_field,
     out Toolbox.C popup_menu);

}
