/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/taskwindow.h")]
namespace OSLib.TaskWindow
{
  [CCode (cname = "taskwindow_message_data")]
  public struct MessageData
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "data")]
    public unowned char Data[232];
  }

  [CCode (cname = "taskwindow_message_ego")]
  public struct MessageEgo
  {
    [CCode (cname = "txt")]
    public unowned int Txt;
  }

  [CCode (cname = "taskwindow_message_new_task")]
  public struct MessageNewTask
  {
    [CCode (cname = "command")]
    public unowned char Command[236];
  }

  [CCode (cname = "taskwindow_full_message_data")]
  public struct FullMessageData : Wimp.MessageHeaderBase
  {
    [CCode (cname = "data_size")]
    public unowned int DataSize;
    [CCode (cname = "data")]
    public unowned char Data[232];
  }

  [CCode (cname = "taskwindow_full_message_ego")]
  public struct FullMessageEgo : Wimp.MessageHeaderBase
  {
    [CCode (cname = "txt")]
    public unowned int Txt;
  }

  [CCode (cname = "taskwindow_full_message_new_task")]
  public struct FullMessageNewTask : Wimp.MessageHeaderBase
  {
    [CCode (cname = "command")]
    public unowned char Command[236];
  }

  [CCode (cname="error_TASK_WINDOW_CANT_KILL")]
  public const bits Error_TaskWindowCantKill;
  [CCode (cname="error_TASK_WINDOW_BAD_SWI_ENTRY")]
  public const bits Error_TaskWindowBadSWIEntry;
  [CCode (cname="error_TASK_WINDOW_BAD_TASK_HANDLE")]
  public const bits Error_TaskWindowBadTaskHandle;
  [CCode (cname="error_TASK_WINDOW_DYING")]
  public const bits Error_TaskWindowDying;
  [CCode (cname="error_TASK_WINDOW_FILE_SLEEP")]
  public const bits Error_TaskWindowFileSleep;
  [CCode (cname="error_TASK_WINDOW_NO_EDITOR")]
  public const bits Error_TaskWindowNoEditor;
  [CCode (cname="error_TASK_WINDOW_NO_REDIRECTION")]
  public const bits Error_TaskWindowNoRedirection;
  [CCode (cname="message_TASK_WINDOW_INPUT")]
  public const bits Message_TaskWindowInput;
  [CCode (cname="message_TASK_WINDOW_OUTPUT")]
  public const bits Message_TaskWindowOutput;
  [CCode (cname="message_TASK_WINDOW_EGO")]
  public const bits Message_TaskWindowEgo;
  [CCode (cname="message_TASK_WINDOW_MORIO")]
  public const bits Message_TaskWindowMorio;
  [CCode (cname="message_TASK_WINDOW_MORITE")]
  public const bits Message_TaskWindowMorite;
  [CCode (cname="message_TASK_WINDOW_NEW_TASK")]
  public const bits Message_TaskWindowNewTask;
  [CCode (cname="message_TASK_WINDOW_SUSPEND")]
  public const bits Message_TaskWindowSuspend;
  [CCode (cname="message_TASK_WINDOW_RESUME")]
  public const bits Message_TaskWindowResume;

  [CCode (cname = "xtaskwindowtaskinfo_window_task")]
  public static OS.Error* XTaskWindowTaskInfo_WindowTask
    (out bool window_task);

  [CCode (cname = "taskwindowtaskinfo_window_task")]
  public static bool TaskWindowTaskInfo_WindowTask ();

  [CCode (cname = "xupcall_sleep")]
  public static OS.Error* XUpCall_Sleep
    (int* pollword,
     out bool claimed);

  [CCode (cname = "upcall_sleep")]
  public static bool UpCall_Sleep
    (int* pollword);

}
