/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/uri.h")]
namespace OSLib.URI
{
  [CCode (cname = "uri_h", has_type_id = false)]
  public struct H : bits
  {
  }

  [CCode (cname = "uri_process_flags")]
  public enum ProcessFlags
  {
    [CCode (cname = "uri_PROCESS_CHECK_ONLY")]
    ProcessCheckOnly
  }

  [CCode (cname = "uri_return_result_flags")]
  public enum ReturnResultFlags
  {
    [CCode (cname = "uri_RETURN_RESULT_NOT_CLAIMED")]
    ReturnResultNotClaimed
  }

  [CCode (cname = "uri_message_handler_started")]
  public struct MessageHandlerStarted
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "uri_message_handler_dying")]
  public struct MessageHandlerDying
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "uri_message_process")]
  public struct MessageProcess
  {
    [CCode (cname = "flags")]
    public unowned ProcessFlags Flags;
    [CCode (cname = "uri")]
    public unowned string Uri;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_message_return_result")]
  public struct MessageReturnResult
  {
    [CCode (cname = "flags")]
    public unowned ReturnResultFlags Flags;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_message_process_ack")]
  public struct MessageProcessAck
  {
    [CCode (cname = "flags")]
    public unowned ProcessFlags Flags;
    [CCode (cname = "uri")]
    public unowned string Uri;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_full_message_handler_started")]
  public struct FullMessageHandlerStarted : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "uri_full_message_handler_dying")]
  public struct FullMessageHandlerDying : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "uri_full_message_process")]
  public struct FullMessageProcess : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ProcessFlags Flags;
    [CCode (cname = "uri")]
    public unowned string Uri;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_full_message_return_result")]
  public struct FullMessageReturnResult : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ReturnResultFlags Flags;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_full_message_process_ack")]
  public struct FullMessageProcessAck : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ProcessFlags Flags;
    [CCode (cname = "uri")]
    public unowned string Uri;
    [CCode (cname = "handle")]
    public unowned H Handle;
  }

  [CCode (cname = "uri_dispatch_flags")]
  public enum DispatchFlags
  {
    [CCode (cname = "uri_DISPATCH_INFORM_CALLER")]
    DispatchInformCaller,
    [CCode (cname = "uri_DISPATCH_CHECK_ONLY")]
    DispatchCheckOnly,
    [CCode (cname = "uri_DISPATCH_NO_EXTERNAL_STARTUP")]
    DispatchNoExternalStartup
  }

  [CCode (cname = "uri_dispatch_result_flags")]
  public enum DispatchResultFlags
  {
    [CCode (cname = "uri_DISPATCH_REQUEST_REJECTED")]
    DispatchRequestRejected
  }

  [CCode (cname="uri_FILE_TYPE")]
  public const bits FileType;
  [CCode (cname="message_URI_HANDLER_STARTED")]
  public const bits Message_URIHandlerStarted;
  [CCode (cname="message_URI_HANDLER_DYING")]
  public const bits Message_URIHandlerDying;
  [CCode (cname="message_URI_PROCESS")]
  public const bits Message_URIProcess;
  [CCode (cname="message_URI_RETURN_RESULT")]
  public const bits Message_URIReturnResult;
  [CCode (cname="message_URI_PROCESS_ACK")]
  public const bits Message_URIProcessAck;
  [CCode (cname="error_URI_NO_MEMORY")]
  public const bits Error_URINoMemory;
  [CCode (cname="error_URI_BAD_URI")]
  public const bits Error_URIBadURI;
  [CCode (cname="error_URI_BAD_HANDLE")]
  public const bits Error_URIBadHandle;
  [CCode (cname="error_URI_BAD_FILE")]
  public const bits Error_URIBadFile;

  [CCode (cname = "xuri_version")]
  public static OS.Error* XVersion
    (bits flags,
     out int version);

  [CCode (cname = "uri_version")]
  public static int Version
    (bits flags);

  [CCode (cname = "xuri_dispatch")]
  public static OS.Error* XDispatch
    (DispatchFlags flags,
     string uri,
     Wimp.T source_task,
     out DispatchResultFlags flags_out,
     out Wimp.T handler_task,
     out H handle);

  [CCode (cname = "uri_dispatch")]
  public static H Dispatch
    (DispatchFlags flags,
     string uri,
     Wimp.T source_task,
     out DispatchResultFlags flags_out,
     out Wimp.T handler_task);

  [CCode (cname = "xuri_request_uri_size")]
  public static OS.Error* XRequestURISize
    (bits flags,
     H handle,
     out int buffer_size);

  [CCode (cname = "uri_request_uri_size")]
  public static int RequestURISize
    (bits flags,
     H handle);

  [CCode (cname = "xuri_request_uri")]
  public static OS.Error* XRequestURI
    (bits flags,
     string buffer,
     int buffer_size,
     H handle,
     out int length);

  [CCode (cname = "uri_request_uri")]
  public static int RequestURI
    (bits flags,
     string buffer,
     int buffer_size,
     H handle);

  [CCode (cname = "xuri_invalidate_uri")]
  public static OS.Error* XInvalidateURI
    (bits flags,
     H handle);

  [CCode (cname = "uri_invalidate_uri")]
  public static void InvalidateURI
    (bits flags,
     H handle);

  [CCode (cname = "xservice_uri_handler_started")]
  public static OS.Error* XService_URIHandlerStarted ();

  [CCode (cname = "service_uri_handler_started")]
  public static void Service_URIHandlerStarted ();

  [CCode (cname = "xservice_uri_handler_dying")]
  public static OS.Error* XService_URIHandlerDying ();

  [CCode (cname = "service_uri_handler_dying")]
  public static void Service_URIHandlerDying ();

  [CCode (cname = "xservice_uri_process")]
  public static OS.Error* XService_URIProcess
    (ProcessFlags flags,
     string uri,
     H handle,
     out bool unclaimed);

  [CCode (cname = "service_uri_process")]
  public static bool Service_URIProcess
    (ProcessFlags flags,
     string uri,
     H handle);

  [CCode (cname = "xservice_uri_return_result")]
  public static OS.Error* XService_URIReturnResult
    (ReturnResultFlags flags,
     H handle,
     out bool unclaimed);

  [CCode (cname = "service_uri_return_result")]
  public static bool Service_URIReturnResult
    (ReturnResultFlags flags,
     H handle);

}
