/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Font;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/wimpreadsysinfo.h")]
namespace OSLib.WimpReadSysInfo
{
  [CCode (cname = "wimpreadsysinfo_state")]
  public enum State
  {
    [CCode (cname = "wimpreadsysinfo_STATE_COMMANDS")]
    Commands,
    [CCode (cname = "wimpreadsysinfo_STATE_DESKTOP")]
    Desktop
  }

  [CCode (cname = "wimpreadsysinfo_direction")]
  public enum Direction
  {
    [CCode (cname = "wimpreadsysinfo_DIR_LEFT_TO_RIGHT")]
    DirLeftToRight,
    [CCode (cname = "wimpreadsysinfo_DIR_RIGHT_TO_LEFT")]
    DirRightToLeft
  }

  [CCode (cname = "wimpreadsysinfotextselection_flags")]
  public enum WimpReadSysInfoTextSelection_Flags
  {
    [CCode (cname = "wimpreadsysinfotextselectionflags_ENABLED")]
    Enabled,
    [CCode (cname = "wimpreadsysinfotextselectionflags_INSERT_OP_MASK")]
    InsertOpMask,
    [CCode (cname = "wimpreadsysinfotextselectionflags_INSERT_OP_NOP")]
    InsertOpNOP,
    [CCode (cname = "wimpreadsysinfotextselectionflags_INSERT_OP_DEL")]
    InsertOpDel,
    [CCode (cname = "wimpreadsysinfotextselectionflags_INSERT_OP_CLEAR")]
    InsertOpClear,
    [CCode (cname = "wimpreadsysinfotextselectionflags_INSERT_OP_CUT")]
    InsertOpCut,
    [CCode (cname = "wimpreadsysinfotextselectionflags_DEL_OP_MASK")]
    DelOpMask,
    [CCode (cname = "wimpreadsysinfotextselectionflags_DEL_OP_NOP")]
    DelOpNOP,
    [CCode (cname = "wimpreadsysinfotextselectionflags_DEL_OP_DEL")]
    DelOpDel,
    [CCode (cname = "wimpreadsysinfotextselectionflags_DEL_OP_CLEAR")]
    DelOpClear,
    [CCode (cname = "wimpreadsysinfotextselectionflags_DEL_OP_CUT")]
    DelOpCut,
    [CCode (cname = "wimpreadsysinfotextselectionflags_MOVE_OP_MASK")]
    MoveOpMask,
    [CCode (cname = "wimpreadsysinfotextselectionflags_MOVE_OP_NOP")]
    MoveOpNOP,
    [CCode (cname = "wimpreadsysinfotextselectionflags_MOVE_OP_DEL")]
    MoveOpDel,
    [CCode (cname = "wimpreadsysinfotextselectionflags_MOVE_OP_CLEAR")]
    MoveOpClear,
    [CCode (cname = "wimpreadsysinfotextselectionflags_MOVE_OP_CUT")]
    MoveOpCut,
    [CCode (cname = "wimpreadsysinfotextselectionflags_AUTO_SELECT")]
    AutoSelect
  }


  [CCode (cname = "xwimpreadsysinfo_task_count")]
  public static OS.Error* XTaskCount
    (out int task_count);

  [CCode (cname = "wimpreadsysinfo_task_count")]
  public static int TaskCount ();

  [CCode (cname = "xwimpreadsysinfo_wimp_mode")]
  public static OS.Error* XWimpMode
    (out OS.Mode mode);

  [CCode (cname = "wimpreadsysinfo_wimp_mode")]
  public static OS.Mode WimpMode ();

  [CCode (cname = "xwimpreadsysinfo_sprite_suffix")]
  public static OS.Error* XSpriteSuffix
    (out string suffix);

  [CCode (cname = "wimpreadsysinfo_sprite_suffix")]
  public static string SpriteSuffix ();

  [CCode (cname = "xwimpreadsysinfo_desktop_state")]
  public static OS.Error* XDesktopState
    (out State state);

  [CCode (cname = "wimpreadsysinfo_desktop_state")]
  public static State DesktopState ();

  [CCode (cname = "xwimpreadsysinfo_write_dir")]
  public static OS.Error* XWriteDir
    (out Direction write_dir);

  [CCode (cname = "wimpreadsysinfo_write_dir")]
  public static Direction WriteDir ();

  [CCode (cname = "xwimpreadsysinfo_task")]
  public static OS.Error* XTask
    (out Wimp.T task,
     out Wimp.VersionNo version);

  [CCode (cname = "wimpreadsysinfo_task")]
  public static Wimp.T Task
    (out Wimp.VersionNo version);

  [CCode (cname = "xwimpreadsysinfo_version")]
  public static OS.Error* XVersion
    (out Wimp.VersionNo version);

  [CCode (cname = "wimpreadsysinfo_version")]
  public static Wimp.VersionNo Version ();

  [CCode (cname = "xwimpreadsysinfo_font")]
  public static OS.Error* XFont
    (out Font.F font,
     out Font.F symbol_font);

  [CCode (cname = "wimpreadsysinfo_font")]
  public static Font.F Font
    (out Font.F symbol_font);

  [CCode (cname = "xwimpreadsysinfo_tool_sprites")]
  public static OS.Error* XToolSprites
    (out OSSpriteOp.Area toolsprites);

  [CCode (cname = "wimpreadsysinfo_tool_sprites")]
  public static OSSpriteOp.Area ToolSprites ();

  [CCode (cname = "xwimpreadsysinfo_application_limit")]
  public static OS.Error* XApplicationLimit
    (out int size_limit);

  [CCode (cname = "wimpreadsysinfo_application_limit")]
  public static int ApplicationLimit ();

  [CCode (cname = "xwimpreadsysinfo_sprite_pools")]
  public static OS.Error* XSpritePools
    (out OSSpriteOp.Area low_priority,
     out OSSpriteOp.Area high_priority);

  [CCode (cname = "wimpreadsysinfo_sprite_pools")]
  public static void SpritePools
    (out OSSpriteOp.Area low_priority,
     out OSSpriteOp.Area high_priority);

  [CCode (cname = "xwimpreadsysinfo_auto_scroll_pause_delay")]
  public static OS.Error* XAutoScrollPauseDelay
    (out int delay);

  [CCode (cname = "wimpreadsysinfo_auto_scroll_pause_delay")]
  public static int AutoScrollPauseDelay ();

  [CCode (cname = "xwimpreadsysinfo_sprite_pools_extended")]
  public static OS.Error* XSpritePoolsExtended
    (out OSSpriteOp.Area priority,
     out OSSpriteOp.Area high_priority,
     out OSSpriteOp.Area low_priority);

  [CCode (cname = "wimpreadsysinfo_sprite_pools_extended")]
  public static void SpritePoolsExtended
    (out OSSpriteOp.Area priority,
     out OSSpriteOp.Area high_priority,
     out OSSpriteOp.Area low_priority);

  [CCode (cname = "xwimpreadsysinfo_special_highlighting_colours")]
  public static OS.Error* XSpecialHighlightingColours
    (out Wimp.Colour bg,
     out Wimp.Colour fg);

  [CCode (cname = "wimpreadsysinfo_special_highlighting_colours")]
  public static void SpecialHighlightingColours
    (out Wimp.Colour bg,
     out Wimp.Colour fg);

  [CCode (cname = "xwimpreadsysinfo_text_selection")]
  public static OS.Error* XTextSelection
    (out Wimp.Colour bg,
     out Wimp.Colour fg,
     out WimpReadSysInfoTextSelection_Flags flags);

  [CCode (cname = "wimpreadsysinfo_text_selection")]
  public static void TextSelection
    (out Wimp.Colour bg,
     out Wimp.Colour fg,
     out WimpReadSysInfoTextSelection_Flags flags);

  [CCode (cname = "xwimpreadsysinfo_caret_colour")]
  public static OS.Error* XCaretColour
    (out Wimp.Colour def,
     out Wimp.Colour actual);

  [CCode (cname = "wimpreadsysinfo_caret_colour")]
  public static void CaretColour
    (out Wimp.Colour def,
     out Wimp.Colour actual);

  [CCode (cname = "xwimpreadsysinfo_drag_settings")]
  public static OS.Error* XDragSettings
    (out int os_units_move,
     out int delay);

  [CCode (cname = "wimpreadsysinfo_drag_settings")]
  public static void DragSettings
    (out int os_units_move,
     out int delay);

  [CCode (cname = "xwimpreadsysinfo_dbl_click_settings")]
  public static OS.Error* XDblClickSettings
    (out int os_units_move,
     out int delay);

  [CCode (cname = "wimpreadsysinfo_dbl_click_settings")]
  public static void DblClickSettings
    (out int os_units_move,
     out int delay);

  [CCode (cname = "xwimpreadsysinfo_sub_menu_open_settings")]
  public static OS.Error* XSubMenuOpenSettings
    (out int delay_for_open,
     out int delay_mouse_ignore);

  [CCode (cname = "wimpreadsysinfo_sub_menu_open_settings")]
  public static void SubMenuOpenSettings
    (out int delay_for_open,
     out int delay_mouse_ignore);

  [CCode (cname = "xwimpreadsysinfo_icon_bar_scroll_settings")]
  public static OS.Error* XIconBarScrollSettings
    (out int speed,
     out int acceleration);

  [CCode (cname = "wimpreadsysinfo_icon_bar_scroll_settings")]
  public static void IconBarScrollSettings
    (out int speed,
     out int acceleration);

  [CCode (cname = "xwimpreadsysinfo_edge_settings")]
  public static OS.Error* XEdgeSettings
    (out int delay);

  [CCode (cname = "wimpreadsysinfo_edge_settings")]
  public static void EdgeSettings
    (out int delay);

}
